@@ -16,40 +16,64 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include <ctype.h>
-#include "libs/graphics/gfx_common.h"
-#include "libs/graphics/drawable.h"
-#include "libs/sound/trackplayer.h"
-#include "starcon.h"
-#include "commglue.h"
-#include "options.h"
 #include "comm.h"
-#include "libs/sound/sound.h"
+
+#include "build.h"
+#include "commglue.h"
 #include "controls.h"
+#include "encount.h"
 #include "endian_uqm.h"
+#include "gamestr.h"
+#include "options.h"
+#include "oscill.h"
+#include "resinst.h"
+#include "settings.h"
+#include "setup.h"
+#include "sounds.h"
+#include "libs/graphics/drawable.h"
+#include "libs/graphics/gfx_common.h"
+#include "libs/mathlib.h"
+#include "libs/inplib.h"
+#include "libs/sound/sound.h"
+#include "libs/sound/trackplayer.h"
+#include "libs/sound/trackint.h"
+#include "libs/strlib.h"
 
-void InitOscilloscope (int x, int y, int width, int height, FRAME_DESC *f);
-void SetSliderImage (void *f);
-void InitSlider (int x, int y, int width, int height,
-	int bwidth, int bheight, void *f);
-void Oscilloscope (int grab_data);
-void Slider (void);
+#include <ctype.h>
 
 
+static void init_xform_control (void);
+static void uninit_xform_control (void);
+static void xform_complete (void);
+static void xform_PLUT_step (SIZE TDelta);
+static void FlushPLUTXForms (void);
+static int ambient_anim_task (void *data);
+
+static BOOLEAN getLineWithinWidth(TEXT *pText,
+		 const unsigned char **startNext, SIZE maxWidth, COUNT maxChars);
+
 #define MAX_RESPONSES 8
-#define BACKGROUND_VOL speechVolumeScale == 0.0f ? MAX_VOLUME : (MAX_VOLUME >> 1)
-#define FOREGROUND_VOL MAX_VOLUME
+#define BACKGROUND_VOL \
+		(speechVolumeScale == 0.0f ? NORMAL_VOLUME : (NORMAL_VOLUME >> 1))
+#define FOREGROUND_VOL NORMAL_VOLUME
 
 #define SLIDER_Y 107
 #define SLIDER_HEIGHT 15
 
-// #define DEBUG
 
 LOCDATA CommData;
 int cur_comm;
 UNICODE shared_phrase_buf[256];
 volatile BOOLEAN summary = FALSE;
 
+typedef struct response_entry
+{
+	RESPONSE_REF response_ref;
+	TEXT response_text;
+	RESPONSE_FUNC response_func;
+} RESPONSE_ENTRY;
+typedef RESPONSE_ENTRY *PRESPONSE_ENTRY;
+
 typedef struct encounter_state
 {
 	BOOLEAN (*InputFunc) (struct encounter_state *pES);
@@ -57,12 +81,7 @@ typedef struct encounter_state
 
 	COUNT Initialized;
 	BYTE num_responses, cur_response, top_response;
-	struct
-	{
-		RESPONSE_REF response_ref;
-		TEXT response_text;
-		RESPONSE_FUNC response_func;
-	} response_list[MAX_RESPONSES];
+	RESPONSE_ENTRY response_list[MAX_RESPONSES];
 
 	Task AnimTask;
 
@@ -106,63 +125,33 @@ enum
 static int subtitle_state = DONE_SUBTITLE;
 static Mutex subtitle_mutex;
 
-static volatile UNICODE *last_subtitle;
+static const UNICODE * volatile last_subtitle;
 static TFB_Image *subtitle_cache;
 
-/* _count_lines - mostly stolen from add_text, just sees how many lines
-                  a given input string would take to display given the
-                  line wrapping information */
-static int _count_lines (PTEXT pText) 
+/* _count_lines - sees how many lines a given input string would take to
+ * display given the line wrapping information
+ */
+static int _count_lines (PTEXT pText)
 {
-	COUNT maxchars = (COUNT)~0;
-	UNICODE ch, *pStr;
 	SIZE text_width;
-	int num = 0;
+	COUNT maxchars = (COUNT)~0;
+	const unsigned char *pStr;
+	int numLines = 0;
+	BOOLEAN eol;
 
 	text_width = CommData.AlienTextWidth;
 	SetContextFont (CommData.AlienFont);
 
 	pStr = pText->pStr;
-
 	do
 	{
-		++num;
+		++numLines;
 		pText->pStr = pStr;
-		pText->CharCount = 1;
-
-		{
-			BOOLEAN eot;
-			RECT r, old_r;
-			COUNT OldCount;
-
-			GetContextClipRect (&r);
-			eot = FALSE;
-			do
-			{
-				old_r = r;
-				OldCount = pText->CharCount;
-				while (!(eot = (BOOLEAN)(
-						(ch = *++pStr) == '\0'
-						|| ch == '\n'
-						|| ch == '\r'
-						|| (COUNT)(pStr - pText->pStr) >= maxchars
-						)) && ch != ' ')
-					;
-				pText->CharCount = pStr - pText->pStr;
-				TextRect (pText, &r, NULL_PTR);
-			} while (!eot && r.extent.width < text_width);
-
-			if (r.extent.width >= text_width)
-			{
-				pText->CharCount = OldCount;
-				r = old_r;
-			}
-		}
-		pStr = pText->pStr + pText->CharCount;
-	} while ((ch = *pStr++) != '\0' && ch != '\n' && ch != '\r' && maxchars);
+		eol = getLineWithinWidth(pText, &pStr, text_width, maxchars);
+	} while (!eol && maxchars);
 	pText->pStr = pStr;
 
-	return (num);
+	return numLines;
 }
 
 static COORD
@@ -172,10 +161,11 @@ add_text (int status, PTEXT pTextIn)
 	TEXT locText;
 	PTEXT pText;
 	SIZE leading;
-	UNICODE ch, *pStr;
+	const unsigned char *pStr;
 	SIZE text_width;
 	int num_lines = 0;
 	static COORD last_baseline;
+	BOOLEAN eol;
 	
 	BatchGraphics ();
 
@@ -197,7 +187,7 @@ add_text (int status, PTEXT pTextIn)
 			// saves background to extra screen
 			TFB_DrawScreen_Copy (&r, TFB_SCREEN_MAIN, TFB_SCREEN_EXTRA);
 			// fills screen with transparent color
-			TFB_DrawScreen_Rect (&r, 0, 0, 255, TFB_SCREEN_MAIN);
+			TFB_DrawScreen_Rect (&r, 0, 0, 128, TFB_SCREEN_MAIN);
 
 			last_subtitle = pTextIn->pStr;
 		}
@@ -221,22 +211,24 @@ add_text (int status, PTEXT pTextIn)
 		switch (status)
 		{
 			case -3:
-				SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x00, 0x14), 0x01));
+				SetContextForeGroundColor (
+						BUILD_COLOR (MAKE_RGB15 (0x00, 0x00, 0x14), 0x01));
 				break;
 			case -2:
-				SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x14, 0x14), 0x03));
+				SetContextForeGroundColor (
+						BUILD_COLOR (MAKE_RGB15 (0x00, 0x14, 0x14), 0x03));
 				break;
 			case -1:
-				SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x1A, 0x1A, 0x1A), 0x12));
+				SetContextForeGroundColor (
+						BUILD_COLOR (MAKE_RGB15 (0x1A, 0x1A, 0x1A), 0x12));
 				break;
 		}
 
 		maxchars = pTextIn->CharCount;
 		locText = *pTextIn;
 		locText.baseline.x -= 8;
-		locText.CharCount = 1;
-		ch = '*';
-		locText.pStr = &ch;
+		locText.CharCount = (COUNT)~0;
+		locText.pStr = "*";
 		font_DrawText (&locText);
 
 		locText = *pTextIn;
@@ -247,93 +239,65 @@ add_text (int status, PTEXT pTextIn)
 	numchars = 0;
 	pStr = pText->pStr;
 
-	if (status > 0 && (CommData.AlienTextTemplate.valign & (VALIGN_MIDDLE|VALIGN_BOTTOM))) {
+	if (status > 0 && (CommData.AlienTextTemplate.valign & (
+				VALIGN_MIDDLE|VALIGN_BOTTOM))) {
 		num_lines = _count_lines(pText);
 		if (CommData.AlienTextTemplate.valign == VALIGN_BOTTOM)
 			pText->baseline.y -= (leading * num_lines);
 		else if (CommData.AlienTextTemplate.valign == VALIGN_MIDDLE)
 			pText->baseline.y -= ((leading * num_lines) / 2);
-		if (pText->baseline.y < 0) pText->baseline.y = 0;
+		if (pText->baseline.y < 0)
+			pText->baseline.y = 0;
 	}
 
 	do
 	{
 		pText->pStr = pStr;
-		pText->CharCount = 1;
 		pText->baseline.y += leading;
 
+		eol = getLineWithinWidth(pText, &pStr, text_width, maxchars);
+
+		maxchars -= pText->CharCount;
+		if (maxchars != 0)
+			--maxchars;
+		numchars += pText->CharCount;
+		
+		if (status <= 0)
 		{
-			BOOLEAN eot;
-			RECT r, old_r;
-			COUNT OldCount;
+			if (pText->baseline.y < SIS_SCREEN_HEIGHT)
+				font_DrawText (pText);
 
-			GetContextClipRect (&r);
-			eot = FALSE;
-			do
+			if (status < -4 && pText->baseline.y >= -status - 10)
 			{
-				old_r = r;
-				OldCount = pText->CharCount;
-				while (!(eot = (BOOLEAN)(
-						(ch = *++pStr) == '\0'
-						|| ch == '\n'
-						|| ch == '\r'
-						|| (COUNT)(pStr - pText->pStr) >= maxchars
-						)) && ch != ' ')
-					;
-				pText->CharCount = pStr - pText->pStr;
-				TextRect (pText, &r, NULL_PTR);
-			} while (!eot && r.extent.width < text_width);
-
-			if (r.extent.width >= text_width)
-			{
-				pText->CharCount = OldCount;
-				r = old_r;
-			}
-
-			if (maxchars -= pText->CharCount)
-				--maxchars;
-			numchars += pText->CharCount;
-			
-			if (status <= 0)
-			{
-				if (pText->baseline.y < SIS_SCREEN_HEIGHT)
-					font_DrawText (pText);
-
-				pStr = pText->pStr + pText->CharCount;
-				if(status < -4 && pText->baseline.y >= -status - 10)
-				{
-					++pStr;
-					break;
-				}
-			}
-			else
-			{
-				SetContextForeGroundColor (CommData.AlienTextBColor);
-
-				--pText->baseline.x;
-				font_DrawText (pText);
-
-				++pText->baseline.x;
-				--pText->baseline.y;
-				font_DrawText (pText);
-
-				++pText->baseline.x;
-				++pText->baseline.y;
-				font_DrawText (pText);
-
-				--pText->baseline.x;
-				++pText->baseline.y;
-				font_DrawText (pText);
-
-				SetContextForeGroundColor (CommData.AlienTextFColor);
-
-				--pText->baseline.y;
-				font_DrawText (pText);
-
-				pStr = pText->pStr + pText->CharCount;
+				++pStr;
+				break;
 			}
 		}
-	} while ((ch = *pStr++) != '\0' && ch != '\n' && ch != '\r' && maxchars);
+		else
+		{
+			SetContextForeGroundColor (CommData.AlienTextBColor);
+
+			--pText->baseline.x;
+			font_DrawText (pText);
+
+			++pText->baseline.x;
+			--pText->baseline.y;
+			font_DrawText (pText);
+
+			++pText->baseline.x;
+			++pText->baseline.y;
+			font_DrawText (pText);
+
+			--pText->baseline.x;
+			++pText->baseline.y;
+			font_DrawText (pText);
+
+			SetContextForeGroundColor (CommData.AlienTextFColor);
+
+			--pText->baseline.y;
+			font_DrawText (pText);
+		}
+	} while (!eol && maxchars);
 	pText->pStr = pStr;
 
 	if (status == 1)
@@ -346,7 +310,8 @@ add_text (int status, PTEXT pTextIn)
 		// restores background
 		TFB_DrawScreen_Copy (&r, TFB_SCREEN_EXTRA, TFB_SCREEN_MAIN);
 		// draws cached subtitle
-		TFB_DrawScreen_Image (subtitle_cache, r.corner.x, r.corner.y, 0, 0, TFB_SCREEN_MAIN);
+		TFB_DrawScreen_Image (subtitle_cache, r.corner.x, r.corner.y, 0, 0,
+				TFB_SCREEN_MAIN);
 		
 		last_baseline = pText->baseline.y;
 	}
@@ -355,6 +320,70 @@ add_text (int status, PTEXT pTextIn)
 	return (pText->baseline.y);
 }
 
+static BOOLEAN
+getLineWithinWidth(TEXT *pText, const unsigned char **startNext,
+		SIZE maxWidth, COUNT maxChars) {
+	BOOLEAN eol;
+			// The end of the line of text has been reached.
+	BOOLEAN done;
+			// We cannot add any more words.
+	RECT rect;
+	COUNT oldCount;
+	const unsigned char *ptr;
+	const unsigned char *wordStart;
+	wchar_t ch;
+	COUNT charCount;
+
+	//GetContextClipRect (&rect);
+
+	eol = FALSE;	
+	done = FALSE;
+	oldCount = 1;
+	charCount = 0;
+	ch = '\0';
+	ptr = pText->pStr;
+	for (;;)
+	{
+		wordStart = ptr;
+
+		// Scan one word.
+		for (;;)
+		{
+			if (*ptr == '\0')
+			{
+				eol = TRUE;
+				done = TRUE;
+				break;
+			}
+			ch = getCharFromString (&ptr);
+			eol = ch == '\0' || ch == '\n' || ch == '\r';
+			done = eol || charCount >= maxChars;
+			if (done || ch == ' ')
+				break;
+			charCount++;
+		}
+
+		oldCount = pText->CharCount;
+		pText->CharCount = charCount;
+		TextRect (pText, &rect, NULL_PTR);
+		
+		if (rect.extent.width >= maxWidth)
+		{
+			pText->CharCount = oldCount;
+			*startNext = wordStart;
+			return FALSE;
+		}
+
+		if (done)
+		{
+			*startNext = ptr;
+			return eol;
+		}
+		charCount++;
+				// For the space in between words.
+	}
+}
+
 static void
 DrawSISComWindow (void)
 {
@@ -471,7 +500,7 @@ typedef struct xform_control
 {
 	COLORMAPPTR CMapPtr;
 	SIZE Ticks, TTotal, TOrig;
-	DWORD OldCMap[NUMBER_OF_PLUT_UINT32s];
+	UBYTE OldCMap[PLUT_BYTE_SIZE];
 } XFORM_CONTROL;
 
 #define MAX_XFORMS 32
@@ -479,27 +508,27 @@ static struct {
 	XFORM_CONTROL TaskControl[MAX_XFORMS];
 	volatile int XFormCurrent, XFormInsertPoint;
 	volatile BOOLEAN XFormsPending;
-	Semaphore XFormSem;
+	Mutex XFormLock;
 } XFormControl;
 
-void
+static void
 init_xform_control (void)
 {
 	XFormControl.XFormCurrent = XFormControl.XFormInsertPoint = 0;
 	XFormControl.XFormsPending = FALSE;
-	XFormControl.XFormSem = CreateSemaphore (1, "XForm");
+	XFormControl.XFormLock = CreateMutex ("Transform Lock", SYNC_CLASS_TOPLEVEL | SYNC_CLASS_VIDEO);
 }
 
-void
+static void
 uninit_xform_control (void)
 {
-	DestroySemaphore (XFormControl.XFormSem);
+	DestroyMutex (XFormControl.XFormLock);
 }
 
-void
+static void
 xform_complete (void)
 {
-	SetSemaphore (XFormControl.XFormSem);
+	LockMutex (XFormControl.XFormLock);
 	if (XFormControl.XFormsPending)
 	{
 		SetColorMap (XFormControl.TaskControl[XFormControl.XFormCurrent].CMapPtr);
@@ -512,7 +541,7 @@ xform_complete (void)
 			XFormControl.XFormsPending = FALSE;
 		}
 	}
-	ClearSemaphore (XFormControl.XFormSem);
+	UnlockMutex (XFormControl.XFormLock);
 }
 
 void
@@ -520,13 +549,13 @@ init_communication (void)
 {
 	TFB_Canvas canvas;
 
-	subtitle_mutex = CreateMutex ();
+	subtitle_mutex = CreateMutex ("Subtitle Lock", SYNC_CLASS_TOPLEVEL | SYNC_CLASS_VIDEO);
 	init_xform_control ();
 
 	canvas = TFB_DrawCanvas_New_TrueColor (SIS_SCREEN_WIDTH,
 		SIS_SCREEN_HEIGHT - SLIDER_Y - SLIDER_HEIGHT + 2, FALSE);
 	subtitle_cache = TFB_DrawImage_New (canvas);
-	TFB_DrawCanvas_SetTransparentColor (subtitle_cache->NormalImg, 0, 0, 255, TRUE);
+	TFB_DrawCanvas_SetTransparentColor (subtitle_cache->NormalImg, 0, 0, 128, TRUE);
 }
 
 void
@@ -543,12 +572,11 @@ static volatile BOOLEAN SummaryChange;
 static volatile BOOLEAN ClearSubtitle;
 
 /* Only one thread should ever be allowed to be calling this at any time */
-void
+static void
 xform_PLUT_step (SIZE TDelta)
 {
 	XFORM_CONTROL *control;
 	COLORMAPPTR ColorMapPtr;
-	DWORD *pCurCMap, *pOldCMap;
 	int i;
 
 	if (!XFormControl.XFormsPending)
@@ -561,56 +589,26 @@ xform_PLUT_step (SIZE TDelta)
 	if (TDelta > control->TTotal)
 		TDelta = control->TTotal;
 
-	pCurCMap = (DWORD *)((BYTE *)_varPLUTs + (*ColorMapPtr * PLUT_BYTE_SIZE));
-	pOldCMap = control->OldCMap;
-
-	ColorMapPtr += 2;
-	if (_varPLUTs)
+	if (_varPLUTs && control->TOrig != 0)
 	{
-		for (i = 0; i < NUMBER_OF_PLUT_UINT32s; i++)
+#define XFORM_SCALE 0x10000
+		UBYTE *pCurCMap, *pOldCMap, *pNewCMap;
+		int frac;
+
+		pCurCMap = GET_VAR_PLUT (*ColorMapPtr);
+		pOldCMap = control->OldCMap;
+		pNewCMap = (UBYTE*)ColorMapPtr + 2;
+
+		frac = (int)(control->TOrig - control->TTotal) * XFORM_SCALE
+				/ control->TOrig;
+
+		for (i = 0; i < PLUT_BYTE_SIZE; i++)
 		{
-			SIZE c0, c1;
-			DWORD v0, v1, val;
-			float f = (control->TOrig - control->TTotal) / (float)control->TOrig;
-			COLORMAPPTR oldmap = (COLORMAPPTR) pOldCMap;
-
-			v0 = UQM_SwapBE32 (*(DWORD *)oldmap);
-			v1 = UQM_SwapBE32 (*(DWORD *)ColorMapPtr);
-
-			ColorMapPtr += sizeof (DWORD);
-			
-			c0 = (SIZE)((v0 >> (10 + 16)) & 0x1F);
-			c1 = (SIZE)((v1 >> (10 + 16)) & 0x1F);
-			c0 += (SIZE)((c1 - c0) * f);
-			val = (c0 & 0x1F) | (1 << 5);
-
-			c0 = (SIZE)((v0 >> (5 + 16)) & 0x1F);
-			c1 = (SIZE)((v1 >> (5 + 16)) & 0x1F);
-			c0 += (SIZE)((c1 - c0) * f);
-			val = (val << 5) | (c0 & 0x1F);
-
-			c0 = (SIZE)((v0 >> (0 + 16)) & 0x1F);
-			c1 = (SIZE)((v1 >> (0 + 16)) & 0x1F);
-			c0 += (SIZE)((c1 - c0) * f);
-			val = (val << 5) | (c0 & 0x1F);
-
-			c0 = (SIZE)((v0 >> 10) & 0x1F);
-			c1 = (SIZE)((v1 >> 10) & 0x1F);
-			c0 += (SIZE)((c1 - c0) * f);
-			val = (val << (5 + 1)) | (c0 & 0x1F) | (1 << 5);
-
-			c0 = (SIZE)((v0 >> 5) & 0x1F);
-			c1 = (SIZE)((v1 >> 5) & 0x1F);
-			c0 += (SIZE)((c1 - c0) * f);
-			val = (val << 5) | (c0 & 0x1F);
-			
-			c0 = (SIZE)((v0 >> 0) & 0x1F);
-			c1 = (SIZE)((v1 >> 0) & 0x1F);
-			c0 += (SIZE)((c1 - c0) * f);
-			val = (val << 5) | (c0 & 0x1F);
-
+			*pCurCMap = (UBYTE)(*pOldCMap + ((int)*pNewCMap - *pOldCMap)
+						* frac / XFORM_SCALE);
 			pOldCMap++;
-			*pCurCMap++ = UQM_SwapBE32 (val);
+			pCurCMap++;
+			pNewCMap++;
 		}
 		ColorChange = TRUE;
 	}
@@ -626,7 +624,7 @@ xform_PLUT_step (SIZE TDelta)
  * writes are atomic anyway, and Flush is the only routine that writes
  * XFormFlush. */
 
-void
+static void
 FlushPLUTXForms (void)
 {
 	while (XFormControl.XFormsPending)
@@ -640,27 +638,22 @@ XFormPLUT (COLORMAPPTR ColorMapPtr, SIZE TimeInterval)
 {
 	if (ColorMapPtr)
 	{
-		DWORD *pOldCMap, *pCurCMap;
 		XFORM_CONTROL *control;
-		int i;
 
 		// FlushPLUTXForms ();
 
-		SetSemaphore (XFormControl.XFormSem);
+		LockMutex (XFormControl.XFormLock);
 		while (XFormControl.XFormsPending
 				&& (XFormControl.XFormInsertPoint == XFormControl.XFormCurrent))
 		{
-			ClearSemaphore (XFormControl.XFormSem);
+			UnlockMutex (XFormControl.XFormLock);
 			FlushPLUTXForms ();
-			SetSemaphore (XFormControl.XFormSem);
+			LockMutex (XFormControl.XFormLock);
 		}
 		
 		control = &XFormControl.TaskControl[XFormControl.XFormInsertPoint];
 
-		pCurCMap = (DWORD *)((BYTE *)_varPLUTs + (*ColorMapPtr * PLUT_BYTE_SIZE));
-		pOldCMap = control->OldCMap;
-		for (i = 0; i < NUMBER_OF_PLUT_UINT32s; ++i)
-			*pOldCMap++ = *pCurCMap++;
+		memcpy (control->OldCMap, GET_VAR_PLUT (*ColorMapPtr), PLUT_BYTE_SIZE);
 
 		control->CMapPtr = ColorMapPtr;
 		if ((control->Ticks = TimeInterval) <= 0)
@@ -672,7 +665,7 @@ XFormPLUT (COLORMAPPTR ColorMapPtr, SIZE TimeInterval)
 
 		XFormControl.XFormsPending = TRUE;
 
-		ClearSemaphore (XFormControl.XFormSem);		
+		UnlockMutex (XFormControl.XFormLock);		
 
 	}
 
@@ -845,7 +838,8 @@ UpdateSpeechGraphics (BOOLEAN Initialize)
 	SetContext (OldContext);
 }
 
-int ambient_anim_task(void* data)
+static int
+ambient_anim_task (void *data)
 {
 	SIZE TalkAlarm;
 	FRAME TalkFrame;
@@ -869,18 +863,18 @@ int ambient_anim_task(void* data)
 	while ((CommFrame = CommData.AlienFrame) == 0 && !Task_ReadState (task, TASK_EXIT))
 		TaskSwitch ();
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	memset ((PSTR)&DisplayArray[0], 0, sizeof (DisplayArray));
 	SetUpSequence (Sequencer);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 	ActiveMask = 0;
 	TalkAlarm = 0;
 	TalkFrame = 0;
 	LastTime = GetTimeCounter ();
 	LastOscillTime = LastTime;
-	memset (&FrameChanged[0], 0 , MAX_ANIMATIONS);
-	memset (&AnimFrame[0], 0, sizeof (FRAME) * MAX_ANIMATIONS);
+	memset (FrameChanged, 0, sizeof (FrameChanged));
+	memset (AnimFrame, 0, sizeof (AnimFrame));
 	for (i = 0; i <= CommData.NumAnimations; i++)
 		if (CommData.AlienAmbientArray[i].AnimFlags & YOYO_ANIM)
 			AnimFrame[i] =  SetAbsFrameIndex (
@@ -901,7 +895,7 @@ int ambient_anim_task(void* data)
 
 		SleepThreadUntil (LastTime + ONE_SECOND / 120);
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		BatchGraphics ();
 		CurTime = GetTimeCounter ();
 		ElapsedTicks = CurTime - LastTime;
@@ -1287,7 +1281,7 @@ int ambient_anim_task(void* data)
 			UpdateSpeechGraphics (FALSE);
 		}
 		UnbatchGraphics ();
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 		xform_PLUT_step(ElapsedTicks);
 	}
 	FinishTask (task);
@@ -1319,15 +1313,15 @@ SpewPhrases (COUNT wait_track)
 		if (wait_track == 1 || wait_track == (COUNT)~0)
 		{
 			ResumeTrack ();
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			do
 			{
 				TaskSwitch ();
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 				which_track = PlayingTrack ();
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 			} while (!which_track);
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 		}
 	}
 	else if (which_track <= wait_track)
@@ -1338,8 +1332,8 @@ SpewPhrases (COUNT wait_track)
 		if (GLOBAL (CurrentActivity) & CHECK_ABORT)
 			break;
 		
-		ClearSemaphore (GraphicsSem);
-		/* FIXME: is this a ramnant of 128-tick clock?
+		UnlockMutex (GraphicsLock);
+		/* FIXME: is this a remnant of 128-tick clock?
 		 * with 120-tick clock this will sleep for 1 tick --
 		 * for 1/120th of a second; if ONE_SECOND is upgraded
 		 * it will probably sleep for 2/120th of a second.
@@ -1353,8 +1347,8 @@ SpewPhrases (COUNT wait_track)
 		UpdateInputState ();
 #endif
 
-		SetSemaphore (GraphicsSem);
-		if (CurrentMenuState.cancel)
+		LockMutex (GraphicsLock);
+		if (PulsedInputState.key[KEY_MENU_CANCEL])
 		{
 			SetSliderImage (SetAbsFrameIndex (ActivityFrame, 8));
 			JumpTrack ();
@@ -1368,13 +1362,13 @@ SpewPhrases (COUNT wait_track)
 			BOOLEAN left=FALSE, right=FALSE;
 			if (optSmoothScroll == OPT_PC)
 			{
-				left = CurrentMenuState.left;
-				right = CurrentMenuState.right;
+				left = PulsedInputState.key[KEY_MENU_LEFT];
+				right = PulsedInputState.key[KEY_MENU_RIGHT];
 			}
 			else if (optSmoothScroll == OPT_3DO)
 			{
-				left = ImmediateMenuState.left;
-				right = ImmediateMenuState.right;
+				left = ImmediateInputState.key[KEY_MENU_LEFT];
+				right = ImmediateInputState.key[KEY_MENU_RIGHT];
 			}
 			if (right)
 			{
@@ -1442,7 +1436,7 @@ AlienTalkSegue (COUNT wait_track)
 	if ((GLOBAL (CurrentActivity) & CHECK_ABORT)
 			|| (CommData.AlienTransitionDesc.AnimFlags & TALK_INTRO))
 		return;
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 
 	if (!pCurInputState->Initialized)
 	{
@@ -1504,9 +1498,9 @@ AlienTalkSegue (COUNT wait_track)
 				pCurInputState->AnimTask = AssignTask (ambient_anim_task,
 						3072, "ambient animations");
 
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			SleepThreadUntil (TimeOut);
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 		}
 
 		LastActivity &= ~CHECK_LOAD;
@@ -1525,9 +1519,9 @@ AlienTalkSegue (COUNT wait_track)
 			CommData.AlienTalkDesc.AnimFlags |= WAIT_TALKING;
 		while (CommData.AlienTalkDesc.AnimFlags & TALK_INTRO)
 		{
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			TaskSwitch ();
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 		}
 	}
 
@@ -1543,7 +1537,7 @@ AlienTalkSegue (COUNT wait_track)
 			CommData.AlienTalkDesc.AnimFlags |= PAUSE_TALKING;
 	}
 
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 	FlushInput ();
 	while (AnyButtonPress (TRUE))
 		TaskSwitch ();
@@ -1556,6 +1550,8 @@ AlienTalkSegue (COUNT wait_track)
 static BOOLEAN
 DoCommunication (PENCOUNTER_STATE pES)
 {
+	SetMenuSounds (MENU_SOUND_UP | MENU_SOUND_DOWN, MENU_SOUND_SELECT);
+
 	if (!(CommData.AlienTransitionDesc.AnimFlags & (TALK_INTRO | TALK_DONE)))
 	{
 		AlienTalkSegue ((COUNT)~0);
@@ -1575,10 +1571,10 @@ DoCommunication (PENCOUNTER_STATE pES)
 			TimeIn = GetTimeCounter ();
 			// Warning!  This used to re-gather input data to check for rewind.
 			UpdateInputState ();
-			if (CurrentMenuState.left)
+			if (PulsedInputState.key[KEY_MENU_LEFT])
 			{
 				FadeMusic (BACKGROUND_VOL, ONE_SECOND);
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 				CommData.AlienTransitionDesc.AnimFlags &= ~(TALK_INTRO | TALK_DONE);
 				if (CommData.AlienTalkDesc.NumFrames)
 				{
@@ -1594,9 +1590,9 @@ DoCommunication (PENCOUNTER_STATE pES)
 						CommData.AlienTalkDesc.AnimFlags |= WAIT_TALKING;
 					while (CommData.AlienTalkDesc.AnimFlags & TALK_INTRO)
 					{
-						ClearSemaphore (GraphicsSem);
+						UnlockMutex (GraphicsLock);
 						TaskSwitch ();
-						SetSemaphore (GraphicsSem);
+						LockMutex (GraphicsLock);
 					}
 				}
 				
@@ -1609,7 +1605,7 @@ DoCommunication (PENCOUNTER_STATE pES)
 						CommData.AlienTalkDesc.AnimFlags |= PAUSE_TALKING;
 				}
 
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 				FlushInput ();
 				while (AnyButtonPress (TRUE))
 					TaskSwitch ();
@@ -1631,25 +1627,28 @@ DoCommunication (PENCOUNTER_STATE pES)
 		if (pES->top_response == (BYTE)~0)
 		{
 			pES->top_response = 0;
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			RefreshResponses (pES);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 		}
 
-		if (CurrentMenuState.select)
+		if (PulsedInputState.key[KEY_MENU_SELECT])
 		{
-			pES->phrase_buf_index =
-					pES->response_list[pES->cur_response].response_text.CharCount;
-			wstrncpy (pES->phrase_buf,
-					pES->response_list[pES->cur_response].response_text.pStr,
-					pES->phrase_buf_index);
+			const unsigned char *end;
+			PTEXT response_text =
+					&pES->response_list[pES->cur_response].response_text;
+			end = skipUTF8Chars(response_text->pStr,
+					 response_text->CharCount);
+			pES->phrase_buf_index = end - response_text->pStr;
+			memcpy(pES->phrase_buf, response_text->pStr,
+					 pES->phrase_buf_index);
 			pES->phrase_buf[pES->phrase_buf_index++] = '\0';
 
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			FeedbackPlayerPhrase (pES->phrase_buf);
 			StopTrack ();
 			SetSliderImage (SetAbsFrameIndex (ActivityFrame, 2));
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 
 			FadeMusic (BACKGROUND_VOL, ONE_SECOND);
 
@@ -1658,10 +1657,9 @@ DoCommunication (PENCOUNTER_STATE pES)
 			(*pES->response_list[pES->cur_response].response_func)
 					(pES->response_list[pES->cur_response].response_ref);
 		}
-		else if (CurrentMenuState.cancel && 
+		else if (PulsedInputState.key[KEY_MENU_CANCEL] && 
 				LOBYTE (GLOBAL (CurrentActivity)) != WON_LAST_BATTLE)
 		{
-			extern TFB_SoundChain *first_chain;
 			TFB_SoundChain *curr;
 			RECT r;
 			TEXT t;
@@ -1676,21 +1674,21 @@ DoCommunication (PENCOUNTER_STATE pES)
 			DWORD CurTime;
 			FONT fLast;
 
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			DrawSISComWindow ();
 			FeedbackPlayerPhrase (pES->phrase_buf);
 			summary = TRUE;
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			CurTime = GetTimeCounter ();
 			SleepThreadUntil (CurTime + ONE_SECOND / 30);
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			r.corner.x = 0;
 			r.corner.y = 0;
 			r.extent.width = SIS_SCREEN_WIDTH;
 			r.extent.height = SIS_SCREEN_HEIGHT - SLIDER_Y - SLIDER_HEIGHT + 2;
 			SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x05, 0x00), 0x6E));
 			DrawFilledRectangle (&r);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 
 			SetContextBackGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x05, 0x00), 0x6E));
 			SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x10, 0x00), 0x6B));
@@ -1703,11 +1701,14 @@ DoCommunication (PENCOUNTER_STATE pES)
 
 			for (curr = first_chain; curr != NULL; curr = curr->next)
 			{
-				temp = curr->tag.data;
+				temp = curr->text;
+				if (temp == NULL)
+					continue;
 				// fprintf (stderr, "%s\n", temp);
-				while (wstrlen (temp) > (unsigned int) SUMMARY_CHARS)
+				while (wstrlen (temp) > (unsigned int) SUMMARY_CHARS
+						&& !(GLOBAL (CurrentActivity) & CHECK_ABORT))
 				{
-					int space_index;
+					int space_index = SUMMARY_CHARS;
 					// find last space before it goes over the max chars per line
 					for (i = SUMMARY_CHARS - 1; i > 0; i--)
 					{
@@ -1724,9 +1725,9 @@ DoCommunication (PENCOUNTER_STATE pES)
 					buffer[i] = '\0';
 					temp++;
 					t.pStr = buffer;
-					SetSemaphore (GraphicsSem);
+					LockMutex (GraphicsLock);
 					font_DrawText (&t);
-					ClearSemaphore (GraphicsSem);
+					UnlockMutex (GraphicsLock);
 					t.baseline.y += DELTA_Y_SUMMARY;
 					col++;
 					if (col > MAX_COLS)
@@ -1735,31 +1736,31 @@ DoCommunication (PENCOUNTER_STATE pES)
 						t.align = ALIGN_CENTER;
 						t.pStr = "_MORE_";
 						SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x17, 0x00), 0x01));
-						SetSemaphore (GraphicsSem);
+						LockMutex (GraphicsLock);
 						font_DrawText (&t);
-						ClearSemaphore (GraphicsSem);
+						UnlockMutex (GraphicsLock);
 						col = 0;
 						t.baseline.x = SAFE_X + 2;
 						t.align = ALIGN_LEFT;
 						t.baseline.y = SAFE_Y + DELTA_Y_SUMMARY;
-						while (AnyButtonPress (TRUE))
-							TaskSwitch ();
-				
-						while (!AnyButtonPress (TRUE))
-							TaskSwitch ();
 
-						SetSemaphore (GraphicsSem);
+						WaitAnyButtonOrQuit (TRUE);
+
+						LockMutex (GraphicsLock);
 						SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x05, 0x00), 0x6E));
 						DrawFilledRectangle (&r);
-						ClearSemaphore (GraphicsSem);
+						UnlockMutex (GraphicsLock);
 						SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x10, 0x00), 0x6B));
 					}
 				}
+			
+				if (GLOBAL (CurrentActivity) & CHECK_ABORT)
+					break;				
 				
 				t.pStr = temp;
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 				font_DrawText (&t);
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 				t.baseline.y += DELTA_Y_SUMMARY;
 				col++;
 				if (col > MAX_COLS && curr->next != NULL)
@@ -1769,39 +1770,32 @@ DoCommunication (PENCOUNTER_STATE pES)
 					t.align = ALIGN_CENTER;
 					t.pStr = "_MORE_";
 					SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x17, 0x00), 0x01));
-					SetSemaphore (GraphicsSem);
+					LockMutex (GraphicsLock);
 					font_DrawText (&t);
-					ClearSemaphore (GraphicsSem);
+					UnlockMutex (GraphicsLock);
 					col = 0;
 					t.baseline.x = SAFE_X + 2;
 					t.align = ALIGN_LEFT;
 					t.baseline.y = SAFE_Y + DELTA_Y_SUMMARY;
-					while (AnyButtonPress (TRUE))
-						TaskSwitch ();
-				
-					while (!AnyButtonPress (TRUE))
-						TaskSwitch ();
 
-					SetSemaphore (GraphicsSem);
+					WaitAnyButtonOrQuit (TRUE);
+
+					LockMutex (GraphicsLock);
 					SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x05, 0x00), 0x6E));
 					DrawFilledRectangle (&r);
-					ClearSemaphore (GraphicsSem);
+					UnlockMutex (GraphicsLock);
 					SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x10, 0x00), 0x6B));
 				}
 			}
 
-			while (AnyButtonPress (TRUE))
-				TaskSwitch ();
-				
-			while (!AnyButtonPress (TRUE))
-				TaskSwitch ();
+			WaitAnyButtonOrQuit (TRUE);
 
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			SetContextFont (fLast);
 			RefreshResponses (pES);
 			SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x00, 0x00), 0x00));
 			DrawFilledRectangle (&r);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			SummaryChange = TRUE;
 			summary = FALSE;
 			FlushInput ();
@@ -1811,10 +1805,10 @@ DoCommunication (PENCOUNTER_STATE pES)
 		else
 		{
 			response = pES->cur_response;
-			if (CurrentMenuState.left)
+			if (PulsedInputState.key[KEY_MENU_LEFT])
 			{
 				FadeMusic (BACKGROUND_VOL, ONE_SECOND);
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 				FeedbackPlayerPhrase (pES->phrase_buf);
 
 				CommData.AlienTransitionDesc.AnimFlags &= ~(TALK_INTRO | TALK_DONE);
@@ -1832,9 +1826,9 @@ DoCommunication (PENCOUNTER_STATE pES)
 						CommData.AlienTalkDesc.AnimFlags |= WAIT_TALKING;
 					while (CommData.AlienTalkDesc.AnimFlags & TALK_INTRO)
 					{
-						ClearSemaphore (GraphicsSem);
+						UnlockMutex (GraphicsLock);
 						TaskSwitch ();
-						SetSemaphore (GraphicsSem);
+						LockMutex (GraphicsLock);
 					}
 				}
 
@@ -1851,7 +1845,7 @@ DoCommunication (PENCOUNTER_STATE pES)
 					RefreshResponses (pES);
 					FadeMusic (FOREGROUND_VOL, ONE_SECOND);
 				}
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 				FlushInput ();
 				while (AnyButtonPress (TRUE))
 					TaskSwitch ();
@@ -1859,10 +1853,10 @@ DoCommunication (PENCOUNTER_STATE pES)
 					TaskSwitch ();
 				while (CommData.AlienTalkDesc.AnimFlags & PAUSE_TALKING);
 			}
-			else if (CurrentMenuState.up)
+			else if (PulsedInputState.key[KEY_MENU_UP])
 				response = (BYTE)((response + (BYTE)(pES->num_responses - 1))
 						% pES->num_responses);
-			else if (CurrentMenuState.down)
+			else if (PulsedInputState.key[KEY_MENU_DOWN])
 				response = (BYTE)((BYTE)(response + 1)
 						% pES->num_responses);
 
@@ -1870,7 +1864,7 @@ DoCommunication (PENCOUNTER_STATE pES)
 			{
 				COORD y;
 
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 				BatchGraphics ();
 				add_text (-2, &pES->response_list[pES->cur_response].response_text);
 
@@ -1888,20 +1882,20 @@ DoCommunication (PENCOUNTER_STATE pES)
 					RefreshResponses (pES);
 				}
 				UnbatchGraphics ();
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 			}
 		}
 		
 		return (TRUE);
 	}
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 
 	if (pES->AnimTask)
 	{
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 		ConcludeTask (pES->AnimTask);
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		pES->AnimTask = 0;
 	}
 	CommData.AlienTransitionDesc.AnimFlags &= ~(TALK_INTRO | TALK_DONE);
@@ -1910,7 +1904,7 @@ DoCommunication (PENCOUNTER_STATE pES)
 	DestroyContext (ReleaseContext (TaskContext));
 	TaskContext = 0;
 
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 	FlushPLUTXForms ();
 	ColorChange = FALSE;
@@ -1920,7 +1914,7 @@ DoCommunication (PENCOUNTER_STATE pES)
 	StopMusic ();
 	StopSound ();
 	StopTrack ();
-	SleepThreadUntil (FadeMusic (FOREGROUND_VOL, 0) + ONE_SECOND / 60);
+	SleepThreadUntil (FadeMusic (NORMAL_VOLUME, 0) + ONE_SECOND / 60);
 
 	return (FALSE);
 }
@@ -1930,6 +1924,7 @@ DoResponsePhrase (RESPONSE_REF R, RESPONSE_FUNC response_func,
 		UNICODE *ConstructStr)
 {
 	PENCOUNTER_STATE pES = pCurInputState;
+	PRESPONSE_ENTRY pEntry;
 
 	if (GLOBAL (CurrentActivity) & CHECK_ABORT)
 		return;
@@ -1940,27 +1935,28 @@ DoResponsePhrase (RESPONSE_REF R, RESPONSE_FUNC response_func,
 		pES->top_response = (BYTE)~0;
 	}
 
-	pES->response_list[pES->num_responses].response_ref = R;
-	if ((pES->response_list[pES->num_responses].response_text.pStr = ConstructStr))
-		pES->response_list[pES->num_responses].response_text.CharCount = wstrlen (ConstructStr);
+	pEntry = &pES->response_list[pES->num_responses];
+	pEntry->response_ref = R;
+	pEntry->response_text.pStr = ConstructStr;
+	if (pEntry->response_text.pStr)
+		pEntry->response_text.CharCount = (COUNT)~0;
 	else
 	{
 		STRING locString;
 		
-		locString = SetAbsStringTableIndex (CommData.ConversationPhrases, (COUNT)(R - 1));
-		pES->response_list[pES->num_responses].response_text.pStr =
-				(UNICODE *)GetStringAddress (locString);
-		pES->response_list[pES->num_responses].response_text.CharCount =
-				GetStringLength (locString);
+		locString = SetAbsStringTableIndex (CommData.ConversationPhrases,
+				(COUNT) (R - 1));
+		pEntry->response_text.pStr =
+				(UNICODE *) GetStringAddress (locString);
+		pEntry->response_text.CharCount = GetStringLength (locString);
 //#define BVT_PROBLEM
 #ifdef BVT_PROBLEM
-if (pES->response_list[pES->num_responses].response_text.pStr[
-		pES->response_list[pES->num_responses].response_text.CharCount - 1
-		] == '\0')
-	--pES->response_list[pES->num_responses].response_text.CharCount;
+		if (pEntry->response_text.pStr[pEntry->response_text.CharCount - 1]
+				== '\0')
+			--pEntry->response_text.CharCount;
 #endif /* BVT_PROBLEM */
 	}
-	pES->response_list[pES->num_responses].response_func = response_func;
+	pEntry->response_func = response_func;
 	++pES->num_responses;
 }
 
@@ -1970,6 +1966,7 @@ HailAlien (void)
 	MEM_HANDLE hOldIndex;
 	ENCOUNTER_STATE ES;
 	FONT PlayerFont, OldFont;
+	MUSIC_REF SongRef = 0;
 
 	pCurInputState = &ES;
 	memset (pCurInputState, 0, sizeof (*pCurInputState));
@@ -1988,7 +1985,14 @@ HailAlien (void)
 	CommData.AlienColorMap = CaptureColorMap (
 			LoadColorMapInstance ((RESOURCE)CommData.AlienColorMap)
 			);
-	CommData.AlienSong = LoadMusicInstance ((RESOURCE)CommData.AlienSong);
+	if ((CommData.AlienSongFlags & LDASF_USE_ALTERNATE)
+			&& CommData.AlienAltSong)
+		SongRef = LoadMusicFile (CommData.AlienAltSong);
+	if (SongRef)
+		CommData.AlienSong = SongRef;
+	else
+		CommData.AlienSong = LoadMusicInstance ((RESOURCE)CommData.AlienSong);
+
 	CommData.ConversationPhrases = CaptureStringTable (
 			LoadStringTableInstance ((RESOURCE)CommData.ConversationPhrases)
 			);
@@ -2024,15 +2028,15 @@ HailAlien (void)
 			if (pMenuState == 0)
 			{
 				RepairSISBorder ();
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 				DrawMenuStateStrings ((BYTE)~0, 1);
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 			}
 			else /* in starbase */
 			{
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 				DrawSISFrame ();
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 				if (GET_GAME_STATE (STARBASE_AVAILABLE))
 				{
 					DrawSISMessage (GAME_STRING (STARBASE_STRING_BASE + 1));
@@ -2049,15 +2053,16 @@ HailAlien (void)
 		DrawSISComWindow ();
 	}
 
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 	LastActivity |= CHECK_LOAD; /* prevent spurious input */
 	(*CommData.init_encounter_func) ();
 	DoInput ((PVOID)&ES, FALSE);
 	if (!(GLOBAL (CurrentActivity) & (CHECK_ABORT | CHECK_LOAD)))
-		(*CommData.uninit_encounter_func) ();
+		(*CommData.post_encounter_func) ();
+	(*CommData.uninit_encounter_func) ();
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 
 	DestroyStringTable (ReleaseStringTable (CommData.ConversationPhrases));
 	DestroyMusic ((MUSIC_REF)CommData.AlienSong);
@@ -2082,7 +2087,7 @@ InitCommunication (RESOURCE which_comm)
 
 	last_subtitle = NULL;
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 
 	if (LastActivity & CHECK_LOAD)
 	{
@@ -2091,9 +2096,9 @@ InitCommunication (RESOURCE which_comm)
 		{
 			if (LOBYTE (LastActivity) == 0)
 			{
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 				DrawSISFrame ();
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 			}
 			else
 			{
@@ -2145,27 +2150,32 @@ InitCommunication (RESOURCE which_comm)
 		SetResourceIndex (hIndex);
 
 		LocDataPtr = (LOCDATAPTR)init_race (
-				status != YEHAT_REBEL_SHIP ? which_comm : YEHAT_REBEL_CONVERSATION
+						    status != YEHAT_REBEL_SHIP ? which_comm : (RESOURCE)YEHAT_REBEL_CONVERSATION
 				);
 		if (LocDataPtr)
 			CommData = *LocDataPtr;
 	}
 
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 	if (GET_GAME_STATE (BATTLE_SEGUE) == 0)
+	{
+		// Not offered the chance to attack.
 		status = HAIL;
+	}
 	else if ((status = InitEncounter ()) == HAIL && LocDataPtr)
 	{
+		// The player chose to talk.
 		SET_GAME_STATE (BATTLE_SEGUE, 0);
 	}
 	else
 	{
+		// The player chose to attack.
 		status = ATTACK;
 		SET_GAME_STATE (BATTLE_SEGUE, 1);
 	}
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 
 	if (status == HAIL)
 	{
@@ -2174,12 +2184,17 @@ InitCommunication (RESOURCE which_comm)
 		cur_comm = 0;
 	}
 	else if (LocDataPtr)
-		(*CommData.uninit_encounter_func) ();
+	{	// only when comm initied successfully
+		if (!(GLOBAL (CurrentActivity) & (CHECK_ABORT | CHECK_LOAD)))
+			(*CommData.post_encounter_func) (); // process states
+
+		(*CommData.uninit_encounter_func) (); // cleanup
+	}
 
 	SetResourceIndex (hOldIndex);
 	CloseResourceIndex (hIndex);
 
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 	status = 0;
 	if (!(GLOBAL (CurrentActivity) & (CHECK_ABORT | CHECK_LOAD)))
@@ -2193,6 +2208,7 @@ InitCommunication (RESOURCE which_comm)
 				&& GetHeadLink (&GLOBAL (npc_built_ship_q)));
 		if (status)
 		{
+			// Start combat
 			BuildBattle (0);
 			EncounterBattle ();
 		}
@@ -2389,7 +2405,9 @@ do_subtitles (UNICODE *pStr)
 			CommData.AlienTextTemplate.pStr = pStr;
 			t = CommData.AlienTextTemplate;
 
-			CommData.AlienTextTemplate.CharCount = t.pStr - CommData.AlienTextTemplate.pStr;
+			CommData.AlienTextTemplate.CharCount =
+					utf8StringCountN(CommData.AlienTextTemplate.pStr,
+					t.pStr);
 			subtitle_state = WAIT_SUBTITLE;
 			break;
 		}
