@@ -16,15 +16,17 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "reslib.h"
+#include "comm/commall.h"
 #include "comm/arilou/resinst.h"
-#include "starcon.h"
-#include "commglue.h"
 #include "comm/arilou/strings.h"
 
+#include "gameev.h"
+
+
 static LOCDATA arilou_desc =
 {
 	NULL_PTR, /* init_encounter_func */
+	NULL_PTR, /* post_encounter_func */
 	NULL_PTR, /* uninit_encounter_func */
 	(FRAME)ARILOU_PMAP_ANIM, /* AlienFrame */
 	(FONT)ARILOU_FONT, /* AlienFont */
@@ -33,6 +35,8 @@ static LOCDATA arilou_desc =
 	0, /* SIS_TEXT_WIDTH - 16, */ /* AlienTextWidth */
 	0, /* AlienColorMap */
 	ARILOU_MUSIC, /* AlienSong */
+	0, /* AlienAltSong */
+	0, /* AlienSongFlags */
 	ARILOU_CONVERSATION_PHRASES, /* PlayerPhrases */
 	20, /* NumAnimations */
 	{
@@ -641,13 +645,14 @@ Intro (void)
 	{
 		RESPONSE_FUNC  RespFunc;
 
-		NPCPhrase (INIT_HELLO);
 		if (GET_GAME_STATE (ARILOU_SPACE_SIDE) <= 1)
 		{
+			NPCPhrase (INIT_HELLO);
 			RespFunc = (RESPONSE_FUNC)FriendlySpaceArilou;
 		}
 		else
 		{
+			NPCPhrase (FRDLY_HOMEWORLD_HELLO_1);
 			RespFunc = (RESPONSE_FUNC)ArilouHome;
 			SET_GAME_STATE (ARILOU_HOME_VISITS, 1);
 		}
@@ -783,6 +788,12 @@ ExitIntro:
 
 static COUNT
 uninit_arilou (void)
+{
+	return (0);
+}
+
+static void
+post_arilou_enc (void)
 {
 	BYTE Manner;
 
@@ -809,8 +820,6 @@ uninit_arilou (void)
 			SET_GAME_STATE (ARILOU_MANNER, 3);
 		}
 	}
-
-	return (0);
 }
 
 LOCDATAPTR
@@ -819,6 +828,7 @@ init_arilou_comm (void)
 	LOCDATAPTR retval;
 
 	arilou_desc.init_encounter_func = Intro;
+	arilou_desc.post_encounter_func = post_arilou_enc;
 	arilou_desc.uninit_encounter_func = uninit_arilou;
 
 	arilou_desc.AlienTextTemplate.baseline.x =
