@@ -16,15 +16,18 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "reslib.h"
+#include "comm/commall.h"
 #include "comm/druuge/resinst.h"
-#include "starcon.h"
-#include "commglue.h"
 #include "comm/druuge/strings.h"
 
+#include "build.h"
+#include "setup.h"
+
+
 static LOCDATA druuge_desc =
 {
 	NULL_PTR, /* init_encounter_func */
+	NULL_PTR, /* post_encounter_func */
 	NULL_PTR, /* uninit_encounter_func */
 	(FRAME)DRUUGE_PMAP_ANIM, /* AlienFrame */
 	(FONT)DRUUGE_FONT, /* AlienFont */
@@ -33,6 +36,8 @@ static LOCDATA druuge_desc =
 	0, /* SIS_TEXT_WIDTH - 16, */ /* AlienTextWidth */
 	0, /* AlienColorMap */
 	DRUUGE_MUSIC, /* AlienSong */
+	0, /* AlienAltSong */
+	0, /* AlienSongFlags */
 	DRUUGE_CONVERSATION_PHRASES, /* PlayerPhrases */
 	11,
 	{
@@ -230,9 +235,9 @@ Buy (RESPONSE_REF Response)
 			NPCPhrase (NOT_ENOUGH_ROOM);
 		else
 		{
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			DeltaSISGauges (-SHIP_CREW_COST, 0, 0);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			SlaveryCount += SHIP_CREW_COST;
 			ActivateStarShip (DRUUGE_SHIP, 1);
 
@@ -246,9 +251,9 @@ Buy (RESPONSE_REF Response)
 			NPCPhrase (NOT_ENOUGH_CREW);
 		else
 		{
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			DeltaSISGauges (-ARTIFACT_CREW_COST, 0, 0);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			SlaveryCount += ARTIFACT_CREW_COST;
 			SET_GAME_STATE (ROSY_SPHERE_ON_SHIP, 1);
 			SET_GAME_STATE (ROSY_SPHERE, 1);
@@ -262,9 +267,9 @@ Buy (RESPONSE_REF Response)
 			NPCPhrase (NOT_ENOUGH_CREW);
 		else
 		{
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			DeltaSISGauges (-ARTIFACT_CREW_COST, 0, 0);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			SlaveryCount += ARTIFACT_CREW_COST;
 			SET_GAME_STATE (ARTIFACT_2_ON_SHIP, 1);
 
@@ -277,9 +282,9 @@ Buy (RESPONSE_REF Response)
 			NPCPhrase (NOT_ENOUGH_CREW);
 		else
 		{
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			DeltaSISGauges (-ARTIFACT_CREW_COST, 0, 0);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			SlaveryCount += ARTIFACT_CREW_COST;
 			SET_GAME_STATE (ARTIFACT_3_ON_SHIP, 1);
 
@@ -293,10 +298,10 @@ Buy (RESPONSE_REF Response)
 			NPCPhrase (NOT_ENOUGH_CREW);
 		else
 		{
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			DeltaSISGauges (-FUEL_CREW_COST,
 					FUEL_CREW_COST * FUEL_TANK_SCALE, 0);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			SlaveryCount += FUEL_CREW_COST;
 
 			NPCPhrase (BOUGHT_FUEL);
@@ -452,14 +457,14 @@ DoTransaction (RESPONSE_REF R)
 			capacity -= GLOBAL_SIS (FuelOnBoard);
 			f = (COUNT)((capacity + (FUEL_TANK_SCALE >> 1)) / FUEL_TANK_SCALE);
 
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			while (capacity > 0x3FFFL)
 			{
 				DeltaSISGauges (0, 0x3FFF, 0);
 				capacity -= 0x3FFF;
 			}
 			DeltaSISGauges (0, (SIZE)capacity, 0);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 
 			NPCPhrase (FUEL0);
 			NPCPhrase (-(int)f);
@@ -874,6 +879,12 @@ ExitIntro:
 
 static COUNT
 uninit_druuge (void)
+{
+	return (0);
+}
+
+static void
+post_druuge_enc (void)
 {
 	if (GET_GAME_STATE (BATTLE_SEGUE) == 1
 			&& !GET_GAME_STATE (DRUUGE_MANNER))
@@ -884,8 +895,6 @@ uninit_druuge (void)
 			SET_GAME_STATE (DRUUGE_VISITS, 0);
 		}
 	}
-
-	return (0);
 }
 
 LOCDATAPTR
@@ -894,6 +903,7 @@ init_druuge_comm (void)
 	LOCDATAPTR retval;
 
 	druuge_desc.init_encounter_func = Intro;
+	druuge_desc.post_encounter_func = post_druuge_enc;
 	druuge_desc.uninit_encounter_func = uninit_druuge;
 
 	druuge_desc.AlienTextTemplate.baseline.x =
