@@ -16,12 +16,16 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "reslib.h"
+#include "comm/commall.h"
 #include "comm/melnorm/resinst.h"
-#include "starcon.h"
-#include "commglue.h"
 #include "comm/melnorm/strings.h"
 
+#include "gameev.h"
+#include "setup.h"
+#include "libs/inplib.h"
+#include "libs/mathlib.h"
+
+
 //Added by Chris
 
 void DrawCargoStrings (BYTE OldElement, BYTE NewElement);
@@ -38,6 +42,7 @@ static NUMBER_SPEECH_DESC melnorme_numbers_english;
 static LOCDATA melnorme_desc =
 {
 	NULL_PTR, /* init_encounter_func */
+	NULL_PTR, /* post_encounter_func */
 	NULL_PTR, /* uninit_encounter_func */
 	(FRAME)MELNORME_PMAP_ANIM, /* AlienFrame */
 	(FONT)MELNORME_FONT, /* AlienFont */
@@ -46,6 +51,8 @@ static LOCDATA melnorme_desc =
 	0, /* SIS_TEXT_WIDTH - 16, */ /* AlienTextWidth */
 	MELNORME_COLOR_MAP, /* AlienColorMap */
 	MELNORME_MUSIC, /* AlienSong */
+	0, /* AlienAltSong */
+	0, /* AlienSongFlags */
 	MELNORME_CONVERSATION_PHRASES, /* PlayerPhrases */
 	4, /* NumAnimations */
 	{
@@ -250,9 +257,9 @@ StripShip (COUNT fuel_required)
 	if (fuel_required == 0)
 	{
 		GlobData.SIS_state = SIS_copy;
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		DeltaSISGauges (UNDEFINED_DELTA, rescue_fuel, UNDEFINED_DELTA);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 	}
 	else if (fuel_required == (COUNT)~0)
 	{
@@ -276,9 +283,9 @@ StripShip (COUNT fuel_required)
 				GLOBAL_SIS (ModuleSlots[i]) = EMPTY_SLOT + 2;
 		}
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		DeltaSISGauges (UNDEFINED_DELTA, UNDEFINED_DELTA, UNDEFINED_DELTA);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 	}
 	else if (fuel_required)
 	{
@@ -384,9 +391,9 @@ StripShip (COUNT fuel_required)
 		if (total == 0)
 		{
 			NPCPhrase (CHARITY);
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			DeltaSISGauges (0, fuel_required, 0);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			return (FALSE);
 		}
 		else
@@ -567,9 +574,9 @@ DeltaCredit (SIZE delta_credit)
 		Credit += delta_credit;
 		SET_GAME_STATE (MELNORME_CREDIT0, LOBYTE (Credit));
 		SET_GAME_STATE (MELNORME_CREDIT1, HIBYTE (Credit));
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		DrawStatusMessage ((UNICODE *)~0);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 	}
 	else
 	{
@@ -810,14 +817,14 @@ DoBuy (RESPONSE_REF R)
 				NPCPhrase (GOT_FUEL);
 
 				f = (DWORD)needed_credit * FUEL_TANK_SCALE;
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 				while (f > 0x3FFFL)
 				{
 					DeltaSISGauges (0, 0x3FFF, 0);
 					f -= 0x3FFF;
 				}
 				DeltaSISGauges (0, (SIZE)f, 0);
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 			}
 			needed_credit *= (BIO_CREDIT_VALUE / 2);
 		}
@@ -1204,9 +1211,9 @@ DoSell (RESPONSE_REF R)
 			} while (GLOBAL_SIS (TotalBioMass));
 			SleepThread (ONE_SECOND / 2);
 
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			ClearSISRect (DRAW_SIS_DISPLAY);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 		}
 		else /* if (R == sell_rainbow_locations) */
 		{
@@ -1836,19 +1843,24 @@ Intro (void)
 static COUNT
 uninit_melnorme (void)
 {
-	SetSemaphore (GraphicsSem);
-	DrawStatusMessage (0);
-	ClearSemaphore (GraphicsSem);
-
 	return (0);
 }
 
+static void
+post_melnorme_enc (void)
+{
+	LockMutex (GraphicsLock);
+	DrawStatusMessage (0);
+	UnlockMutex (GraphicsLock);
+}
+
 LOCDATAPTR
 init_melnorme_comm (void)
 {
 	LOCDATAPTR retval;
 
 	melnorme_desc.init_encounter_func = Intro;
+	melnorme_desc.post_encounter_func = post_melnorme_enc;
 	melnorme_desc.uninit_encounter_func = uninit_melnorme;
 
 	melnorme_desc.AlienTextTemplate.baseline.x =
