@@ -16,15 +16,16 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "reslib.h"
+#include "comm/commall.h"
 #include "comm/orz/resinst.h"
-#include "starcon.h"
-#include "commglue.h"
 #include "comm/orz/strings.h"
+#include "build.h"
+
 
 static LOCDATA orz_desc =
 {
 	NULL_PTR, /* init_encounter_func */
+	NULL_PTR, /* post_encounter_func */
 	NULL_PTR, /* uninit_encounter_func */
 	(FRAME)ORZ_PMAP_ANIM, /* AlienFrame */
 	(FONT)ORZ_FONT, /* AlienFont */
@@ -33,6 +34,8 @@ static LOCDATA orz_desc =
 	0, /* SIS_TEXT_WIDTH - 16, */ /* AlienTextWidth */
 	(COLORMAP)ORZ_COLOR_MAP, /* AlienColorMap */
 	ORZ_MUSIC, /* AlienSong */
+	0, /* AlienAltSong */
+	0, /* AlienSongFlags */
 	ORZ_CONVERSATION_PHRASES, /* PlayerPhrases */
 	12 /* 13 */, /* NumAnimations */
 	{
@@ -847,6 +850,12 @@ ExitIntro:
 
 static COUNT
 uninit_orz (void)
+{
+	return (0);
+}
+
+static void
+post_orz_enc (void)
 {
 	BYTE Manner;
 
@@ -861,8 +870,6 @@ uninit_orz (void)
 			SET_GAME_STATE (TAALO_VISITS, 0);
 		}
 	}
-
-	return (0);
 }
 
 LOCDATAPTR
@@ -871,6 +878,7 @@ init_orz_comm (void)
 	LOCDATAPTR retval;
 
 	orz_desc.init_encounter_func = Intro;
+	orz_desc.post_encounter_func = post_orz_enc;
 	orz_desc.uninit_encounter_func = uninit_orz;
 
 	orz_desc.AlienTextTemplate.baseline.x =
