@@ -16,15 +16,18 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "reslib.h"
+#include "comm/commall.h"
 #include "comm/pkunk/resinst.h"
-#include "starcon.h"
-#include "commglue.h"
 #include "comm/pkunk/strings.h"
 
+#include "build.h"
+#include "gameev.h"
+
+
 static LOCDATA pkunk_desc =
 {
 	NULL_PTR, /* init_encounter_func */
+	NULL_PTR, /* post_encounter_func */
 	NULL_PTR, /* uninit_encounter_func */
 	(FRAME)PKUNK_PMAP_ANIM, /* AlienFrame */
 	(FONT)PKUNK_FONT, /* AlienFont */
@@ -33,6 +36,8 @@ static LOCDATA pkunk_desc =
 	0, /* SIS_TEXT_WIDTH - 16, */ /* AlienTextWidth */
 	0, /* AlienColorMap */
 	PKUNK_MUSIC, /* AlienSong */
+	0, /* AlienAltSong */
+	0, /* AlienSongFlags */
 	PKUNK_CONVERSATION_PHRASES, /* PlayerPhrases */
 	3, /* NumAnimations */
 	{
@@ -271,11 +276,14 @@ DiscussConquer (RESPONSE_REF R)
 	}
 	else if (PLAYER_SAID (R, conquer_because_1))
 	{
+#if 0
 		NPCPhrase (NOT_CONQUER_10);
 		NPCPhrase (GLOBAL_ALLIANCE_NAME + name_1);
 		NPCPhrase (NOT_CONQUER_11);
 		NPCPhrase (GLOBAL_ALLIANCE_NAME + name_1);
 		NPCPhrase (NOT_CONQUER_12);
+#endif
+		NPCPhrase (NOT_CONQUER_1);
 
 		DISABLE_PHRASE (conquer_because_1);
 	}
@@ -288,6 +296,7 @@ DiscussConquer (RESPONSE_REF R)
 
 	if (PHRASE_ENABLED (conquer_because_1))
 	{
+#if 0
 		UNICODE buf[80];
 
 		GetAllianceName (buf, name_1);
@@ -298,6 +307,8 @@ DiscussConquer (RESPONSE_REF R)
 				(RESPONSE_REF)-1
 				);
 		DoResponsePhrase (conquer_because_1, DiscussConquer, shared_phrase_buf);
+#endif
+		Response(conquer_because_1, DiscussConquer);
 	}
 	if (PHRASE_ENABLED (conquer_because_2))
 		Response (conquer_because_2, DiscussConquer);
@@ -783,10 +794,10 @@ PkunkNeutralSpace (RESPONSE_REF R)
 				NPCPhrase (GENERAL_INFO_SPACE_2);
 				break;
 			case 2:
-				NPCPhrase (GENERAL_INFO_SPACE_3);
+				NPCPhrase (GENERAL_INFO_SPACE_6 /* was 3 */);
 				break;
 			case 3:
-				NPCPhrase (GENERAL_INFO_SPACE_4);
+				NPCPhrase (GENERAL_INFO_SPACE_7 /* was 4 */);
 				--NumVisits;
 				break;
 		}
@@ -844,6 +855,7 @@ Intro (void)
 	Manner = GET_GAME_STATE (PKUNK_MANNER);
 	if (Manner == 2)
 	{
+		// Irreparably Pissed off the Pkunk.
 		NumVisits = GET_GAME_STATE (PKUNK_VISITS);
 		switch (NumVisits++)
 		{
@@ -867,6 +879,7 @@ Intro (void)
 	}
 	else if (Manner == 1)
 	{
+		// Bad relations with the Pkunk, but not irreparably.
 		NumVisits = GET_GAME_STATE (PKUNK_VISITS);
 		switch (NumVisits++)
 		{
@@ -890,6 +903,7 @@ Intro (void)
 	}
 	else if (GET_GAME_STATE (GLOBAL_FLAGS_AND_DATA) & (1 << 7))
 	{
+		// Encountering the Pkunk at their home world.
 		if (!GET_GAME_STATE (CLEAR_SPINDLE))
 		{
 			NPCPhrase (GIVE_SPINDLE);
@@ -974,6 +988,8 @@ Intro (void)
 	else if ((NumVisits = GET_GAME_STATE (PKUNK_MISSION)) == 0
 			|| !(NumVisits & 1))
 	{
+		// Encountering a Pkunk ship in space, while they are not
+		// migrating.
 		NumVisits = GET_GAME_STATE (PKUNK_VISITS);
 		if (Manner == 3)
 		{
@@ -1034,6 +1050,8 @@ Intro (void)
 	}
 	else
 	{
+		// Encountering a Pkunk ship in space, while they are
+		// migrating.
 		NumVisits = GET_GAME_STATE (PKUNK_MIGRATE_VISITS);
 		switch (NumVisits++)
 		{
@@ -1071,8 +1089,15 @@ ExitIntro:
 	;
 }
 
+// Called after combat or communications
 static COUNT
 uninit_pkunk (void)
+{
+	return (0);
+}
+
+static void
+post_pkunk_enc (void)
 {
 	BYTE Manner;
 
@@ -1086,8 +1111,6 @@ uninit_pkunk (void)
 			SET_GAME_STATE (PKUNK_HOME_VISITS, 0);
 		}
 	}
-
-	return (0);
 }
 
 LOCDATAPTR
@@ -1096,6 +1119,7 @@ init_pkunk_comm (void)
 	LOCDATAPTR retval;
 
 	pkunk_desc.init_encounter_func = Intro;
+	pkunk_desc.post_encounter_func = post_pkunk_enc;
 	pkunk_desc.uninit_encounter_func = uninit_pkunk;
 
 	pkunk_desc.AlienTextTemplate.baseline.x =
@@ -1107,13 +1131,17 @@ init_pkunk_comm (void)
 	if (GET_GAME_STATE (PKUNK_MANNER) == 3
 			|| LOBYTE (GLOBAL (CurrentActivity)) == WON_LAST_BATTLE)
 	{
+		// Enter communications immediately.
 		SET_GAME_STATE (BATTLE_SEGUE, 0);
 	}
 	else
 	{
+		// Ask the player whether to attack or talk.
 		SET_GAME_STATE (BATTLE_SEGUE, 1);
 	}
 	retval = &pkunk_desc;
 
 	return (retval);
 }
+
+
