@@ -16,15 +16,16 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "reslib.h"
+#include "gameev.h"
+#include "comm/commall.h"
 #include "comm/slyhome/resinst.h"
-#include "starcon.h"
-#include "commglue.h"
 #include "comm/slyhome/strings.h"
 
+
 static LOCDATA slylandro_desc =
 {
 	NULL_PTR, /* init_encounter_func */
+	NULL_PTR, /* post_encounter_func */
 	NULL_PTR, /* uninit_encounter_func */
 	(FRAME)SLYLANDRO_PMAP_ANIM, /* AlienFrame */
 	(FONT)SLYLANDRO_FONT, /* AlienFont */
@@ -33,6 +34,8 @@ static LOCDATA slylandro_desc =
 	0, /* SIS_TEXT_WIDTH, */ /* AlienTextWidth */
 	(COLORMAP)SLYLANDRO_COLOR_MAP, /* AlienColorMap */
 	SLYLANDRO_MUSIC, /* AlienSong */
+	0, /* AlienAltSong */
+	0, /* AlienSongFlags */
 	SLYLANDRO_CONVERSATION_PHRASES, /* PlayerPhrases */
 	13, /* NumAnimations */
 	{
@@ -889,12 +892,19 @@ uninit_slylandro (void)
 	return (0);
 }
 
+static void
+post_slylandro_enc (void)
+{
+	// nothing defined so far
+}
+
 LOCDATAPTR
 init_slylandro_comm (void)
 {
 	LOCDATAPTR retval;
 
 	slylandro_desc.init_encounter_func = Intro;
+	slylandro_desc.post_encounter_func = post_slylandro_enc;
 	slylandro_desc.uninit_encounter_func = uninit_slylandro;
 
 	slylandro_desc.AlienTextTemplate.baseline.x =
