@@ -16,15 +16,18 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "reslib.h"
+#include "comm/commall.h"
 #include "comm/spathi/resinst.h"
-#include "starcon.h"
-#include "commglue.h"
 #include "comm/spahome/strings.h"
 
+#include "build.h"
+#include "gameev.h"
+
+
 static LOCDATA spahome_desc =
 {
 	NULL_PTR, /* init_encounter_func */
+	NULL_PTR, /* post_encounter_func */
 	NULL_PTR, /* uninit_encounter_func */
 	(FRAME)SPATHI_HOME_PMAP_ANIM, /* AlienFrame */
 	(FONT)SPATHI_FONT, /* AlienFont */
@@ -33,6 +36,8 @@ static LOCDATA spahome_desc =
 	0, /* SIS_TEXT_WIDTH - 16, */ /* AlienTextWidth */
 	(COLORMAP)SPATHI_HOME_COLOR_MAP, /* AlienColorMap */
 	SPATHI_MUSIC, /* AlienSong */
+	0, /* AlienAltSong */
+	0, /* AlienSongFlags */
 	SPATHI_HOME_CONVERSATION_PHRASES, /* PlayerPhrases */
 	14, /* NumAnimations */
 	{
@@ -956,6 +961,12 @@ Intro (void)
 
 static COUNT
 uninit_spahome (void)
+{
+	return (0);
+}
+
+static void
+post_spahome_enc (void)
 {
 	BYTE Manner;
 
@@ -969,8 +980,6 @@ uninit_spahome (void)
 			SET_GAME_STATE (SPATHI_HOME_VISITS, 0);
 		}
 	}
-
-	return (0);
 }
 
 LOCDATAPTR
@@ -979,6 +988,7 @@ init_spahome_comm ()
 	LOCDATAPTR retval;
 
 	spahome_desc.init_encounter_func = Intro;
+	spahome_desc.post_encounter_func = post_spahome_enc;
 	spahome_desc.uninit_encounter_func = uninit_spahome;
 
 	spahome_desc.AlienTextTemplate.baseline.x =
@@ -987,6 +997,10 @@ init_spahome_comm ()
 	spahome_desc.AlienTextTemplate.align = ALIGN_CENTER;
 	spahome_desc.AlienTextWidth = SIS_TEXT_WIDTH - 16;
 
+	// use alternate "Safe Ones" track if available
+	spahome_desc.AlienAltSong = "comm/spahome/spahome.mod";
+	spahome_desc.AlienSongFlags |= LDASF_USE_ALTERNATE;
+
 	if (GET_GAME_STATE (SPATHI_MANNER) == 3)
 	{
 		SET_GAME_STATE (BATTLE_SEGUE, 0);
