@@ -16,15 +16,17 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "reslib.h"
+#include "comm/commall.h"
 #include "comm/spathi/resinst.h"
-#include "starcon.h"
-#include "commglue.h"
 #include "comm/spathi/strings.h"
 
+#include "build.h"
+
+
 static LOCDATA spathi_desc =
 {
 	NULL_PTR, /* init_encounter_func */
+	NULL_PTR, /* post_encounter_func */
 	NULL_PTR, /* uninit_encounter_func */
 	(FRAME)SPATHI_PMAP_ANIM, /* AlienFrame */
 	(FONT)SPATHI_FONT, /* AlienFont */
@@ -33,6 +35,8 @@ static LOCDATA spathi_desc =
 	0, /* SIS_TEXT_WIDTH - 16, */ /* AlienTextWidth */
 	0, /* AlienColorMap */
 	SPATHI_MUSIC, /* AlienSong */
+	0, /* AlienAltSong */
+	0, /* AlienSongFlags */
 	SPATHI_CONVERSATION_PHRASES, /* PlayerPhrases */
 	8, /* NumAnimations */
 	{
@@ -753,6 +757,12 @@ Intro (void)
 
 static COUNT
 uninit_spathi (void)
+{
+	return (0);
+}
+
+static void
+post_spathi_enc (void)
 {
 	BYTE Manner;
 
@@ -774,8 +784,6 @@ uninit_spathi (void)
 			}
 		}
 	}
-
-	return (0);
 }
 
 LOCDATAPTR
@@ -784,6 +792,7 @@ init_spathi_comm (void)
 	LOCDATAPTR retval;
 
 	spathi_desc.init_encounter_func = Intro;
+	spathi_desc.post_encounter_func = post_spathi_enc;
 	spathi_desc.uninit_encounter_func = uninit_spathi;
 
 	spathi_desc.AlienTextTemplate.baseline.x =
@@ -792,6 +801,16 @@ init_spathi_comm (void)
 	spathi_desc.AlienTextTemplate.align = ALIGN_CENTER;
 	spathi_desc.AlienTextWidth = SIS_TEXT_WIDTH - 16;
 
+	if (GET_GAME_STATE (FOUND_PLUTO_SPATHI) == 1)
+	{	// use alternate Fwiffo track if available
+		spathi_desc.AlienAltSong = "comm/spathi/spafwiff.mod";
+		spathi_desc.AlienSongFlags |= LDASF_USE_ALTERNATE;
+	}
+	else
+	{	// regular track -- let's make sure
+		spathi_desc.AlienSongFlags &= ~LDASF_USE_ALTERNATE;
+	}
+
 	if (GET_GAME_STATE (FOUND_PLUTO_SPATHI) == 1
 			|| GET_GAME_STATE (SPATHI_MANNER) == 3
 			|| LOBYTE (GLOBAL (CurrentActivity)) == WON_LAST_BATTLE)
