@@ -16,12 +16,18 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "reslib.h"
+#include "comm/commall.h"
 #include "comm/comandr/resinst.h"
-#include "starcon.h"
-#include "commglue.h"
 #include "comm/starbas/strings.h"
+
+#include "build.h"
+#include "setup.h"
 #include "libs/graphics/gfx_common.h"
+#include "libs/inplib.h"
+#include "libs/mathlib.h"
+#include "libs/inplib.h"
+#include "libs/sound/sound.h"
+
 
 //Added by Chris
 
@@ -31,9 +37,14 @@ void DrawCargoStrings (BYTE OldElement, BYTE NewElement);
 
 //End Added by Chris
 
+static void TellMission (RESPONSE_REF R);
+static void SellMinerals (RESPONSE_REF R);
+
+
 static LOCDATA commander_desc =
 {
 	NULL_PTR, /* init_encounter_func */
+	NULL_PTR, /* post_encounter_func */
 	NULL_PTR, /* uninit_encounter_func */
 	(FRAME)COMMANDER_PMAP_ANIM, /* AlienFrame */
 	(FONT)COMMANDER_FONT, /* AlienFont */
@@ -42,6 +53,8 @@ static LOCDATA commander_desc =
 	0, /* SIS_TEXT_WIDTH, */ /* AlienTextWidth */
 	(COLORMAP)COMMANDER_COLOR_MAP, /* AlienColorMap */
 	COMMANDER_MUSIC, /* AlienSong */
+	0, /* AlienAltSong */
+	0, /* AlienSongFlags */
 	STARBASE_CONVERSATION_PHRASES, /* PlayerPhrases */
 	10, /* NumAnimations */
 	{
@@ -214,7 +227,11 @@ ByeBye (RESPONSE_REF R)
 		}
 
 		NPCPhrase (pStr0);
-		NPCPhrase (GLOBAL_PLAYER_NAME);
+		if (speechVolumeScale == 0.0f)
+		{
+			NPCPhrase (SPACE);
+			NPCPhrase (GLOBAL_PLAYER_NAME);
+		}
 		NPCPhrase (pStr1);
 	}
 }
@@ -560,7 +577,6 @@ DefeatUrquan (RESPONSE_REF R)
 #define HOW_ALLY_AGAINST_URQUAN (1 << 2)
 #define HOW_STRONG_AGAINST_URQUAN (1 << 3)
 	static BYTE DefeatMask = 0;
-	static void TellMission (RESPONSE_REF R);
 
 	if (PLAYER_SAID (R, how_defeat))
 	{
@@ -889,7 +905,12 @@ TellStarBase (RESPONSE_REF R)
 	else if (PLAYER_SAID (R, tell_me_about_crew))
 	{
 		NPCPhrase (ABOUT_CREW0);
-		NPCPhrase (GLOBAL_SHIP_NAME);
+		if (speechVolumeScale > 0.0f)
+			NPCPhrase (YOUR_FLAGSHIP_3DO2);
+		else {
+			NPCPhrase (YOUR_FLAGSHIP_PC);
+			NPCPhrase (GLOBAL_SHIP_NAME);
+		}
 		NPCPhrase (ABOUT_CREW1);
 
 		stack2 = 2;
@@ -1649,14 +1670,21 @@ NormalStarbase (RESPONSE_REF R)
 		if (GET_GAME_STATE (MOONBASE_ON_SHIP))
 		{
 			NPCPhrase (STARBASE_IS_READY_A);
-			NPCPhrase (GLOBAL_SHIP_NAME);
+			if (speechVolumeScale > 0.0f)
+				NPCPhrase (YOUR_FLAGSHIP_3DO1);
+			else {
+				NPCPhrase (YOUR_FLAGSHIP_PC);
+				NPCPhrase (GLOBAL_SHIP_NAME);
+			}
 			NPCPhrase (STARBASE_IS_READY_B);
-			NPCPhrase (GLOBAL_SHIP_NAME);
+			if (speechVolumeScale > 0.0f)
+				NPCPhrase (YOUR_FLAGSHIP_3DO0);
+			else
+				NPCPhrase (GLOBAL_SHIP_NAME);
 			NPCPhrase (STARBASE_IS_READY_C);
-
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			DeltaSISGauges (0, 0, 2500);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			SET_GAME_STATE (STARBASE_MONTH,
 					GLOBAL (GameClock.month_index));
 			SET_GAME_STATE (STARBASE_DAY,
@@ -1706,7 +1734,11 @@ NormalStarbase (RESPONSE_REF R)
 					break;
 			}
 			NPCPhrase (pStr0);
-			NPCPhrase (GLOBAL_PLAYER_NAME);
+			if (speechVolumeScale == 0.0f)
+			{
+				NPCPhrase (SPACE);
+				NPCPhrase (GLOBAL_PLAYER_NAME);
+			}
 			NPCPhrase (pStr1);
 			CheckBulletins (FALSE);
 		}
@@ -1715,11 +1747,7 @@ NormalStarbase (RESPONSE_REF R)
 	}
 
 	if (GLOBAL_SIS (TotalElementMass))
-	{
-		static void SellMinerals (RESPONSE_REF R);
-
 		Response (have_minerals, SellMinerals);
-	}
 	if (DiscussDevices (FALSE))
 		Response (new_devices, NormalStarbase);
 	if (CurBulletinMask)
@@ -1765,9 +1793,9 @@ SellMinerals (RESPONSE_REF R)
 					Sleepy = FALSE;
 					GLOBAL_SIS (ElementAmounts[i]) = 0;
 					GLOBAL_SIS (TotalElementMass) -= amount;
-					SetSemaphore (GraphicsSem);
+					LockMutex (GraphicsLock);
 					DeltaSISGauges (0, 0, amount * GLOBAL (ElementWorth[i]));
-					ClearSemaphore (GraphicsSem);
+					UnlockMutex (GraphicsLock);
 					break;
 				}
 				
@@ -1776,10 +1804,10 @@ SellMinerals (RESPONSE_REF R)
 				TaskSwitch ();
 				TimeIn = GetTimeCounter ();
 				DrawCargoStrings ((BYTE)i, (BYTE)i);
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 				ShowRemainingCapacity ();
 				DeltaSISGauges (0, 0, GLOBAL (ElementWorth[i]));
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 			} while (--amount);
 		}
 		if (Sleepy) {
@@ -1789,9 +1817,9 @@ SellMinerals (RESPONSE_REF R)
 	}
 	SleepThread (ONE_SECOND / 2);
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	ClearSISRect (DRAW_SIS_DISPLAY);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 // DrawStorageBays (FALSE);
 
 	if (total < 1000)
@@ -1907,7 +1935,11 @@ SellMinerals (RESPONSE_REF R)
 	}
 
 	NPCPhrase (pStr1);
-	NPCPhrase (GLOBAL_PLAYER_NAME);
+	if (speechVolumeScale == 0.0f)
+	{
+		NPCPhrase (SPACE);
+		NPCPhrase (GLOBAL_PLAYER_NAME);
+	}
 	NPCPhrase (pStr2);
 
 	NormalStarbase (R);
@@ -1921,14 +1953,18 @@ Intro (void)
 
 static COUNT
 uninit_starbase (void)
+{
+	return (0);
+}
+
+static void
+post_starbase_enc (void)
 {
 	SET_GAME_STATE (MOONBASE_ON_SHIP, 0);
 	if (GET_GAME_STATE (CHMMR_BOMB_STATE) == 2)
 	{
 		SET_GAME_STATE (CHMMR_BOMB_STATE, 3);
 	}
-
-	return (0);
 }
 
 LOCDATAPTR
@@ -1937,6 +1973,7 @@ init_starbase_comm ()
 	LOCDATAPTR retval;
 
 	commander_desc.init_encounter_func = Intro;
+	commander_desc.post_encounter_func = post_starbase_enc;
 	commander_desc.uninit_encounter_func = uninit_starbase;
 
 	commander_desc.AlienTextWidth = 143;
@@ -1945,6 +1982,10 @@ init_starbase_comm ()
 	commander_desc.AlienTextTemplate.align = ALIGN_CENTER;
 	commander_desc.AlienTextTemplate.valign = VALIGN_MIDDLE;
 
+	// use alternate Starbase track if available
+	commander_desc.AlienAltSong = "comm/starbas/starbas.mod";
+	commander_desc.AlienSongFlags |= LDASF_USE_ALTERNATE;
+
 	CurBulletinMask = 0;
 	SET_GAME_STATE (BATTLE_SEGUE, 0);
 	retval = &commander_desc;
