@@ -16,15 +16,18 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "reslib.h"
+#include "comm/commall.h"
 #include "comm/syreen/resinst.h"
-#include "starcon.h"
-#include "commglue.h"
 #include "comm/syreen/strings.h"
+#include "libs/sound/sound.h"
+
+#include "build.h"
+
 
 static LOCDATA syreen_desc =
 {
 	NULL_PTR, /* init_encounter_func */
+	NULL_PTR, /* post_encounter_func */
 	NULL_PTR, /* uninit_encounter_func */
 	(FRAME)SYREEN_PMAP_ANIM, /* AlienFrame */
 	(FONT)SYREEN_FONT, /* AlienFont */
@@ -33,6 +36,8 @@ static LOCDATA syreen_desc =
 	0, /* SIS_TEXT_WIDTH - 16, */ /* AlienTextWidth */
 	(COLORMAP)SYREEN_COLOR_MAP, /* AlienColorMap */
 	SYREEN_MUSIC, /* AlienSong */
+	0, /* AlienAltSong */
+	0, /* AlienSongFlags */
 	SYREEN_CONVERSATION_PHRASES, /* PlayerPhrases */
 	15, /* NumAnimations */
 	{
@@ -644,10 +649,13 @@ InitialSyreen (RESPONSE_REF R)
 	else if (PLAYER_SAID (R, we_are_vindicator0))
 	{
 		NPCPhrase (WELCOME_VINDICATOR0);
-		NPCPhrase (GLOBAL_PLAYER_NAME);
-		NPCPhrase (WELCOME_VINDICATOR1);
-		NPCPhrase (GLOBAL_SHIP_NAME);
-		NPCPhrase (WELCOME_VINDICATOR2);
+		if (speechVolumeScale == 0.0f)
+		{
+			NPCPhrase (GLOBAL_PLAYER_NAME);
+			NPCPhrase (WELCOME_VINDICATOR1);
+			NPCPhrase (GLOBAL_SHIP_NAME);
+			NPCPhrase (WELCOME_VINDICATOR2);
+		}
 		NPCPhrase (HOW_CAN_YOU_BE_HERE);
 	}
 	else if (PLAYER_SAID (R, we_are_impressed))
@@ -834,12 +842,19 @@ uninit_syreen (void)
 	return (0);
 }
 
+static void
+post_syreen_enc (void)
+{
+	// nothing defined so far
+}
+
 LOCDATAPTR
 init_syreen_comm (void)
 {
 	LOCDATAPTR retval;
 
 	syreen_desc.init_encounter_func = Intro;
+	syreen_desc.post_encounter_func = post_syreen_enc;
 	syreen_desc.uninit_encounter_func = uninit_syreen;
 
 	syreen_desc.AlienTextTemplate.baseline.x =
