@@ -16,15 +16,17 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "reslib.h"
+#include "comm/commall.h"
 #include "comm/talkpet/resinst.h"
-#include "starcon.h"
-#include "commglue.h"
 #include "comm/talkpet/strings.h"
 
+#include "build.h"
+
+
 static LOCDATA talkpet_desc =
 {
 	NULL_PTR, /* init_encounter_func */
+	NULL_PTR, /* post_encounter_func */
 	NULL_PTR, /* uninit_encounter_func */
 	(FRAME)TALKING_PET_PMAP_ANIM, /* AlienFrame */
 	(FONT)TALKING_PET_FONT, /* AlienFont */
@@ -33,6 +35,8 @@ static LOCDATA talkpet_desc =
 	0, /* SIS_TEXT_WIDTH - 16, */ /* AlienTextWidth */
 	(COLORMAP)TALKING_PET_COLOR_MAP, /* AlienColorMap */
 	TALKING_PET_MUSIC, /* AlienSong */
+	0, /* AlienAltSong */
+	0, /* AlienSongFlags */
 	TALKING_PET_CONVERSATION_PHRASES, /* PlayerPhrases */
 	16, /* NumAnimations */
 	{
@@ -789,12 +793,19 @@ uninit_talkpet (void)
 	return (0);
 }
 
+static void
+post_talkpet_enc (void)
+{
+	// nothing defined so far
+}
+
 LOCDATAPTR
 init_talkpet_comm (void)
 {
 	LOCDATAPTR retval;
 
 	talkpet_desc.init_encounter_func = Intro;
+	talkpet_desc.post_encounter_func = post_talkpet_enc;
 	talkpet_desc.uninit_encounter_func = uninit_talkpet;
 
 	talkpet_desc.AlienTextTemplate.baseline.x =
