@@ -16,15 +16,18 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "reslib.h"
+#include "comm/commall.h"
 #include "comm/thradd/resinst.h"
-#include "starcon.h"
-#include "commglue.h"
 #include "comm/thradd/strings.h"
 
+#include "build.h"
+#include "gameev.h"
+
+
 static LOCDATA thradd_desc =
 {
 	NULL_PTR, /* init_encounter_func */
+	NULL_PTR, /* post_encounter_func */
 	NULL_PTR, /* uninit_encounter_func */
 	(FRAME)THRADD_PMAP_ANIM, /* AlienFrame */
 	(FONT)THRADD_FONT, /* AlienFont */
@@ -33,6 +36,8 @@ static LOCDATA thradd_desc =
 	0, /* SIS_TEXT_WIDTH - 16, */ /* AlienTextWidth */
 	0, /* AlienColorMap */
 	THRADD_MUSIC, /* AlienSong */
+	0, /* AlienAltSong */
+	0, /* AlienSongFlags */
 	THRADD_CONVERSATION_PHRASES, /* PlayerPhrases */
 	8, /* NumAnimations */
 	{
@@ -430,7 +435,7 @@ ThraddDemeanor (RESPONSE_REF R)
 
 		SET_GAME_STATE (THRADD_CULTURE, 2);
 	}
-	else if (PLAYER_SAID (R, the_slave_empire))
+	else if (PLAYER_SAID (R, the_slave_empire0))
 	{
 		SET_GAME_STATE (THRADD_CULTURE, 3);
 
@@ -473,9 +478,16 @@ ThraddCulture (RESPONSE_REF R)
 	}
 	NPCPhrase (WHAT_NAME_FOR_CULTURE);
 
+	construct_response (
+			shared_phrase_buf,
+			the_slave_empire0,
+			GLOBAL_SIS (CommanderName),
+			the_slave_empire1,
+			0);
+
 	Response (you_decide, ThraddDemeanor);
 	Response (fat, ThraddDemeanor);
-	Response (the_slave_empire, ThraddDemeanor);
+	DoResponsePhrase (the_slave_empire0, ThraddDemeanor, shared_phrase_buf);
 }
 
 static void
@@ -902,12 +914,19 @@ uninit_thradd (void)
 	return (0);
 }
 
+static void
+post_thradd_enc (void)
+{
+	// nothing defined so far
+}
+
 LOCDATAPTR
 init_thradd_comm (void)
 {
 	LOCDATAPTR retval;
 
 	thradd_desc.init_encounter_func = Intro;
+	thradd_desc.post_encounter_func = post_thradd_enc;
 	thradd_desc.uninit_encounter_func = uninit_thradd;
 
 	thradd_desc.AlienTextTemplate.baseline.x =
