@@ -16,15 +16,18 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "reslib.h"
+#include "comm/commall.h"
 #include "comm/utwig/resinst.h"
-#include "starcon.h"
-#include "commglue.h"
 #include "comm/utwig/strings.h"
 
+#include "build.h"
+#include "gameev.h"
+
+
 static LOCDATA utwig_desc =
 {
 	NULL_PTR, /* init_encounter_func */
+	NULL_PTR, /* post_encounter_func */
 	NULL_PTR, /* uninit_encounter_func */
 	(FRAME)UTWIG_PMAP_ANIM, /* AlienFrame */
 	(FONT)UTWIG_FONT, /* AlienFont */
@@ -33,6 +36,8 @@ static LOCDATA utwig_desc =
 	0, /* SIS_TEXT_WIDTH - 16, */ /* AlienTextWidth */
 	0, /* AlienColorMap */
 	UTWIG_MUSIC, /* AlienSong */
+	0, /* AlienAltSong */
+	0, /* AlienSongFlags */
 	UTWIG_CONVERSATION_PHRASES, /* PlayerPhrases */
 	16, /* NumAnimations */
 	{
@@ -937,12 +942,19 @@ uninit_utwig (void)
 	return (0);
 }
 
+static void
+post_utwig_enc (void)
+{
+	// nothing defined so far
+}
+
 LOCDATAPTR
 init_utwig_comm (void)
 {
 	LOCDATAPTR retval;
 
 	utwig_desc.init_encounter_func = Intro;
+	utwig_desc.post_encounter_func = post_utwig_enc;
 	utwig_desc.uninit_encounter_func = uninit_utwig;
 
 	utwig_desc.AlienTextTemplate.baseline.x =
@@ -952,6 +964,16 @@ init_utwig_comm (void)
 	utwig_desc.AlienTextTemplate.valign = VALIGN_MIDDLE;
 	utwig_desc.AlienTextWidth = SIS_TEXT_WIDTH - 16;
 
+	if (GET_GAME_STATE (UTWIG_HAVE_ULTRON))
+	{	// use alternate 'Happy Utwig!' track
+		utwig_desc.AlienAltSong = "comm/utwig/utwigult.mod";
+		utwig_desc.AlienSongFlags |= LDASF_USE_ALTERNATE;
+	}
+	else
+	{	// regular track -- let's make sure
+		utwig_desc.AlienSongFlags &= ~LDASF_USE_ALTERNATE;
+	}
+
 	if (GET_GAME_STATE (UTWIG_HAVE_ULTRON)
 			|| LOBYTE (GLOBAL (CurrentActivity)) == WON_LAST_BATTLE)
 	{
