@@ -16,15 +16,14 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "reslib.h"
+#include "comm/commall.h"
 #include "comm/vux/resinst.h"
-#include "starcon.h"
-#include "commglue.h"
 #include "comm/vux/strings.h"
 
 static LOCDATA vux_desc =
 {
 	NULL_PTR, /* init_encounter_func */
+	NULL_PTR, /* post_encounter_func */
 	NULL_PTR, /* uninit_encounter_func */
 	(FRAME)VUX_PMAP_ANIM, /* AlienFrame */
 	(FONT)VUX_FONT, /* AlienFont */
@@ -33,6 +32,8 @@ static LOCDATA vux_desc =
 	0, /* (SIS_TEXT_WIDTH - 16) >> 1, */ /* AlienTextWidth */
 	VUX_COLOR_MAP, /* AlienColorMap */
 	VUX_MUSIC, /* AlienSong */
+	0, /* AlienAltSong */
+	0, /* AlienSongFlags */
 	VUX_CONVERSATION_PHRASES, /* PlayerPhrases */
 	17, /* NumAnimations */
 	{
@@ -756,12 +757,19 @@ uninit_vux (void)
 	return (0);
 }
 
+static void
+post_vux_enc (void)
+{
+	// nothing defined so far
+}
+
 LOCDATAPTR
 init_vux_comm (void)
 {
 	LOCDATAPTR retval;
 
 	vux_desc.init_encounter_func = Intro;
+	vux_desc.post_encounter_func = post_vux_enc;
 	vux_desc.uninit_encounter_func = uninit_vux;
 
 	vux_desc.AlienTextTemplate.baseline.x =
