@@ -16,12 +16,14 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "reslib.h"
+#include "comm/commall.h"
 #include "comm/zoqfot/resinst.h"
-#include "starcon.h"
-#include "commglue.h"
 #include "comm/zoqfot/strings.h"
 
+#include "build.h"
+#include "gameev.h"
+
+
 #define ZOQ_FG_COLOR WHITE_COLOR
 #define ZOQ_BG_COLOR BLACK_COLOR
 #define ZOQ_BASE_X (TEXT_X_OFFS + ((SIS_TEXT_WIDTH >> 1) >> 1))
@@ -41,6 +43,7 @@
 static LOCDATA zoqfot_desc =
 {
 	NULL_PTR, /* init_encounter_func */
+	NULL_PTR, /* post_encounter_func */
 	NULL_PTR, /* uninit_encounter_func */
 	(FRAME)ZOQFOTPIK_PMAP_ANIM, /* AlienFrame */
 	(FONT)ZOQFOTPIK_FONT, /* AlienFont */
@@ -49,6 +52,8 @@ static LOCDATA zoqfot_desc =
 	0, /* SIS_SCREEN_WIDTH, */ /* AlienTextWidth */
 	0, /* AlienColorMap */
 	ZOQFOTPIK_MUSIC, /* AlienSong */
+	0, /* AlienAltSong */
+	0, /* AlienSongFlags */
 	ZOQFOTPIK_CONVERSATION_PHRASES, /* PlayerPhrases */
 	3, /* NumAnimations */
 	{
@@ -923,15 +928,23 @@ uninit_zoqfot (void)
 	return (0);
 }
 
+static void
+post_zoqfot_enc (void)
+{
+	// nothing defined so far
+}
+
 LOCDATAPTR
 init_zoqfot_comm (void)
 {
 	LOCDATAPTR retval;
 
 	zoqfot_desc.init_encounter_func = Intro;
+	zoqfot_desc.post_encounter_func = post_zoqfot_enc;
 	zoqfot_desc.uninit_encounter_func = uninit_zoqfot;
 
 	zoqfot_desc.AlienTextTemplate.align = ALIGN_CENTER;
+	zoqfot_desc.AlienTextTemplate.valign = VALIGN_MIDDLE;
 	zoqfot_desc.AlienTextWidth = (SIS_TEXT_WIDTH >> 1) - TEXT_X_OFFS;
 
 	if ((ActivateStarShip (ZOQFOTPIK_SHIP, CHECK_ALLIANCE) & GOOD_GUY)
