@@ -16,14 +16,26 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "starcon.h"
+#include "battle.h"
+#include "colors.h"
+#include "collide.h"
+#include "element.h"
+#include "globdata.h"
+#include "intel.h"
+#include "setup.h"
+#include "units.h"
+#include "libs/inplib.h"
+#include "libs/mathlib.h"
+
+
+//#define DEBUG_CYBORG
 
 COUNT
 PlotIntercept (ELEMENTPTR ElementPtr0, ELEMENTPTR ElementPtr1, COUNT
 		max_turns, COUNT margin_of_error)
 {
-	register SIZE dy;
-	register SIZE time_y_0, time_y_1;
+	SIZE dy;
+	SIZE time_y_0, time_y_1;
 	POINT dst[2];
 	RECT r0, r1;
 	SIZE dx_0, dy_0, dx_1, dy_1;
@@ -89,8 +101,8 @@ PlotIntercept (ELEMENTPTR ElementPtr0, ELEMENTPTR ElementPtr1, COUNT
 			|| (time_y_0 > 0 && dy >= time_y_0)
 			|| (time_y_1 < 0 && dy <= time_y_1))
 	{
-		register SIZE dx;
-		register SIZE time_x_0, time_x_1;
+		SIZE dx;
+		SIZE time_x_0, time_x_1;
 
 		if (margin_of_error)
 		{
@@ -279,19 +291,11 @@ PlotIntercept (ELEMENTPTR ElementPtr0, ELEMENTPTR ElementPtr1, COUNT
 
 STARSHIPPTR CyborgDescPtr;
 
-#ifdef DEBUG
-static COUNT turn_counter[NUM_PLAYERS];
-#endif /* DEBUG */
-
 static void
 InitCyborg (STARSHIPPTR StarShipPtr)
 {
 	COUNT Index, Divisor;
 
-#ifdef DEBUG
-	//turn_counter[which_ship] = 0;
-#endif /* DEBUG */
-
 	Index = StarShipPtr->RaceDescPtr->characteristics.max_thrust
 			* StarShipPtr->RaceDescPtr->characteristics.thrust_increment;
 	if ((Divisor = StarShipPtr->RaceDescPtr->characteristics.turn_wait
@@ -311,9 +315,6 @@ InitCyborg (STARSHIPPTR StarShipPtr)
 	}
 #endif /* PRINT_MI */
 	StarShipPtr->RaceDescPtr->cyborg_control.ManeuverabilityIndex = Index;
-#ifdef DEBUG
-	//turn_counter[which_ship] = 0;
-#endif /* DEBUG */
 }
 
 static void
@@ -378,7 +379,7 @@ ship_weapons (ELEMENTPTR ShipPtr, ELEMENTPTR OtherPtr, COUNT
 			w = *WeaponPtr;
 			if (w)
 			{
-				EPtr = LockElement (w, &EPtr);
+				LockElement (w, &EPtr);
 				if (EPtr->state_flags & APPEARING)
 				{
 					EPtr->next = EPtr->current;
@@ -1021,10 +1022,6 @@ tactical_intelligence (void)
 	STARSHIPPTR StarShipPtr, EnemyStarShipPtr;
 	RACE_DESCPTR RDPtr, EnemyRDPtr;
 
-#ifdef DEBUG
-++turn_counter[cur_player];
-#endif /* DEBUG */
-
 	StarShipPtr = CyborgDescPtr;
 	RDPtr = StarShipPtr->RaceDescPtr;
 
