@@ -16,13 +16,35 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "starcon.h"
+#include "encount.h"
+
+#include "battle.h"
+#include "build.h"
+#include "colors.h"
 #include "controls.h"
+#include "gameopt.h"
+#include "gamestr.h"
+#include "globdata.h"
+#include "init.h"
+#include "intel.h"
+#include "nameref.h"
+#include "resinst.h"
+#include "settings.h"
+#include "setup.h"
+#include "sounds.h"
 #include "libs/graphics/gfx_common.h"
+#include "libs/mathlib.h"
+#include "libs/inplib.h"
+
+
+static void DrawFadeText (const UNICODE *str1, const UNICODE *str2,
+		BOOLEAN fade_in, PRECT pRect);
+
 
 static BOOLEAN
 DoSelectAction (PMENU_STATE pMS)
 {
+	SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
 	if (GLOBAL (CurrentActivity) & CHECK_ABORT)
 	{
 		pMS->CurState = ATTACK + 1;
@@ -33,7 +55,7 @@ DoSelectAction (PMENU_STATE pMS)
 		pMS->Initialized = TRUE;
 		pMS->InputFunc = DoSelectAction;
 	}
-	else if (CurrentMenuState.select)
+	else if (PulsedInputState.key[KEY_MENU_SELECT])
 	{
 		switch (pMS->CurState)
 		{
@@ -47,9 +69,9 @@ DoSelectAction (PMENU_STATE pMS)
 				if (pMS->Initialized)
 				{
 					DrawMenuStateStrings (PM_CONVERSE, pMS->CurState);
-					SetSemaphore (GraphicsSem);
+					LockMutex (GraphicsLock);
 					SetFlashRect ((PRECT)~0L, (FRAME)0);
-					ClearSemaphore (GraphicsSem);
+					UnlockMutex (GraphicsLock);
 				}
 				return ((BOOLEAN)pMS->Initialized);
 			default:
@@ -162,6 +184,12 @@ BuildBattle (COUNT which_player)
 	}
 }
 
+/*
+ * Encountering an alien.
+ * Draws the encounter screen, plays the red alert music, and
+ * waits for a decision of the player on how to handle the situation.
+ * Returns either HAIL or ATTACK.
+ */
 COUNT
 InitEncounter (void)
 {
@@ -173,7 +201,7 @@ InitEncounter (void)
 	extern FRAME planet[];
 	MUSIC_REF MR;
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 
 	hOldIndex = SetResourceIndex (hResIndex);
 
@@ -183,12 +211,12 @@ InitEncounter (void)
 	MR = LoadMusicInstance (REDALERT_MUSIC);
 	PlayMusic (MR, FALSE, 1);
 	SegueFrame = CaptureDrawable (LoadGraphic (SEGUE_PMAP_ANIM));
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 	while (PLRPlaying (MR))
 		TaskSwitch ();
 	StopMusic ();
 	DestroyMusic (MR);
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	s.origin.x = s.origin.y = 0;
 
 	SetTransitionSource (NULL);
@@ -300,7 +328,7 @@ InitEncounter (void)
 
 	SetResourceIndex (hOldIndex);
 
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 	{
 		MENU_STATE MenuState;
@@ -309,23 +337,23 @@ InitEncounter (void)
 		MenuState.Initialized = FALSE;
 
 		DrawMenuStateStrings (PM_CONVERSE, MenuState.CurState = HAIL);
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		SetFlashRect ((PRECT)~0L, (FRAME)0);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 
 		DoInput ((PVOID)&MenuState, TRUE);
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		SetFlashRect (NULL_PTR, (FRAME)0);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 
 		return (MenuState.CurState);
 	}
 }
 
 static void
-DrawFadeText (UNICODE *str1, UNICODE *str2, BOOLEAN fade_in, PRECT
-		pRect)
+DrawFadeText (const UNICODE *str1, const UNICODE *str2, BOOLEAN fade_in,
+		PRECT pRect)
 {
 	SIZE i;
 	DWORD TimeIn;
@@ -357,10 +385,10 @@ DrawFadeText (UNICODE *str1, UNICODE *str2, BOOLEAN fade_in, PRECT
 	{
 		for (i = 0; i < (SIZE) NUM_FADES; ++i)
 		{
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			if (AnyButtonPress (TRUE))
 				i = NUM_FADES - 1;
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 
 			SetContextForeGroundColor (fade_cycle[i]);
 			font_DrawText (&t1);
@@ -373,10 +401,10 @@ DrawFadeText (UNICODE *str1, UNICODE *str2, BOOLEAN fade_in, PRECT
 	{
 		for (i = NUM_FADES - 1; i >= 0; --i)
 		{
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			if (AnyButtonPress (TRUE))
 				i = 0;
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 
 			SetContextForeGroundColor (fade_cycle[i]);
 			font_DrawText (&t1);
@@ -384,7 +412,8 @@ DrawFadeText (UNICODE *str1, UNICODE *str2, BOOLEAN fade_in, PRECT
 			SleepThreadUntil (TimeIn + (ONE_SECOND / 20));
 			TimeIn = GetTimeCounter ();
 		}
-		SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0xA, 0xA, 0xA), 0x08));
+		SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0xA, 0xA, 0xA),
+				0x08));
 		font_DrawText (&t1);
 		font_DrawText (&t2);
 	}
@@ -397,7 +426,7 @@ UninitEncounter (void)
 
 	ships_killed = 0;
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	free_gravity_well ();
 
 	if ((GLOBAL (CurrentActivity) & (CHECK_ABORT | CHECK_LOAD))
@@ -416,12 +445,14 @@ UninitEncounter (void)
 		BOOLEAN Sleepy;
 		DWORD Time;
 		SIZE VictoryState;
-		COUNT RecycleAmount;
+		COUNT RecycleAmount = 0;
 		SIZE i;
 		RECT r, scavenge_r;
 		TEXT t;
 		STAMP ship_s;
-		UNICODE *str1, *str2, buf[20];
+		const UNICODE *str1 = NULL;
+		const UNICODE *str2 = NULL;
+		UNICODE buf[20];
 		HSTARSHIP hStarShip;
 		SHIP_FRAGMENTPTR FragPtr;
 		COLOR fade_ship_cycle[] =
@@ -437,7 +468,9 @@ UninitEncounter (void)
 			BUILD_COLOR (MAKE_RGB15 (0x1B, 0x00, 0x00), 0x2A),
 			BUILD_COLOR (MAKE_RGB15 (0x17, 0x00, 0x00), 0x2B),
 		};
-#define NUM_SHIP_FADES (sizeof (fade_ship_cycle) / sizeof (fade_ship_cycle[0]))
+#define NUM_SHIP_FADES (sizeof (fade_ship_cycle) / \
+		sizeof (fade_ship_cycle[0]))
+
 		COUNT race_bounty[] =
 		{
 			RACE_SHIP_COST
@@ -452,12 +485,11 @@ UninitEncounter (void)
 				) ? 0 : 1;
 
 		hStarShip = GetHeadLink (&GLOBAL (npc_built_ship_q));
-		FragPtr = (SHIP_FRAGMENTPTR)LockStarShip (&GLOBAL (npc_built_ship_q), hStarShip);
+		FragPtr = (SHIP_FRAGMENTPTR)LockStarShip (&GLOBAL (npc_built_ship_q),
+				hStarShip);
 		battle_counter = GET_RACE_ID (FragPtr) + 1;
-		if (GetStarShipFromIndex (
-				&GLOBAL (avail_race_q),
-				(COUNT)(battle_counter - 1)
-				) == 0)
+		if (GetStarShipFromIndex (&GLOBAL (avail_race_q),
+				(COUNT)(battle_counter - 1)) == 0)
 		{
 			VictoryState = -1;
 			InitSISContexts ();
@@ -478,15 +510,13 @@ UninitEncounter (void)
 					VictoryState = 0;
 				else
 				{
-					ClearSemaphore (GraphicsSem);
+					UnlockMutex (GraphicsLock);
 					DrawSISFrame ();
-					SetSemaphore (GraphicsSem);
+					LockMutex (GraphicsLock);
 
 					DrawSISMessage (NULL_PTR);
 					if (LOBYTE (GLOBAL (CurrentActivity)) == IN_HYPERSPACE)
-						DrawHyperCoords (
-								GLOBAL (ShipStamp.origin)
-								);
+						DrawHyperCoords (GLOBAL (ShipStamp.origin));
 					else if (HIWORD (GLOBAL (ShipStamp.frame)) == 0)
 						DrawHyperCoords (CurStarDescPtr->star_pt);
 					else
@@ -501,7 +531,8 @@ UninitEncounter (void)
 				
 			ReinitQueue (&race_q[(NUM_SIDES - 1) - i]);
 
-			for (hStarShip = GetHeadLink (pQueue); hStarShip; hStarShip = hNextShip)
+			for (hStarShip = GetHeadLink (pQueue); hStarShip;
+					hStarShip = hNextShip)
 			{
 				FragPtr = (SHIP_FRAGMENTPTR)LockStarShip (pQueue, hStarShip);
 				hNextShip = _GetSuccLink (FragPtr);
@@ -525,14 +556,16 @@ UninitEncounter (void)
 								
 								ship_s.origin.x = scavenge_r.corner.x + 32;
 								ship_s.origin.y = scavenge_r.corner.y + 56;
-								ship_s.frame = IncFrameIndex (FragPtr->ShipInfo.icons);
+								ship_s.frame = IncFrameIndex (
+										FragPtr->ShipInfo.icons);
 								DrawStamp (&ship_s);
-								SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x8, 0x8, 0x8), 0x1F));
+								SetContextForeGroundColor (BUILD_COLOR (
+										MAKE_RGB15 (0x8, 0x8, 0x8), 0x1F));
 								SetContextFont (TinyFont);
 
 								GetStringContents (
-										FragPtr->ShipInfo.race_strings, (STRINGPTR)buf, FALSE
-										);
+										FragPtr->ShipInfo.race_strings,
+										(STRINGPTR)buf, FALSE);
 								wstrupr (buf);
 
 								t.baseline.x = scavenge_r.corner.x + 100;
@@ -542,19 +575,19 @@ UninitEncounter (void)
 								t.CharCount = (COUNT)~0;
 								font_DrawText (&t);
 								t.baseline.y += 6;
-								t.pStr = GAME_STRING (ENCOUNTER_STRING_BASE + 3);
+								t.pStr = GAME_STRING (
+										ENCOUNTER_STRING_BASE + 3);
 								t.CharCount = (COUNT)~0;
 								font_DrawText (&t);
 
 								ship_s.frame = FragPtr->ShipInfo.icons;
 
 								SetContextFont (MicroFont);
-								DrawFadeText (
-										str1 = GAME_STRING (ENCOUNTER_STRING_BASE + 4),
-										str2 = GAME_STRING (ENCOUNTER_STRING_BASE + 5),
-										TRUE,
-										&scavenge_r
-										);
+								str1 = GAME_STRING (
+										ENCOUNTER_STRING_BASE + 4);
+								str2 = GAME_STRING (
+										ENCOUNTER_STRING_BASE + 5),
+								DrawFadeText (str1, str2, TRUE, &scavenge_r);
 							}
 
 							r.corner.y = scavenge_r.corner.y + 9;
@@ -563,11 +596,12 @@ UninitEncounter (void)
 							SetContextForeGroundColor (BLACK_COLOR);
 
 							r.extent.width = 34;
-							r.corner.x = scavenge_r.corner.x + scavenge_r.extent.width
+							r.corner.x = scavenge_r.corner.x +
+									scavenge_r.extent.width
 									- (10 + r.extent.width);
 							DrawFilledRectangle (&r);
 
-									/* collect bounty ResUnits */
+							/* collect bounty ResUnits */
 							j = race_bounty[battle_counter - 1] >> 3;
 							RecycleAmount += j;
 							wsprintf (buf, "%u", RecycleAmount);
@@ -576,9 +610,8 @@ UninitEncounter (void)
 							t.align = ALIGN_RIGHT;
 							t.pStr = buf;
 							t.CharCount = (COUNT)~0;
-							SetContextForeGroundColor (
-									BUILD_COLOR (MAKE_RGB15 (0x00, 0x00, 0x18), 0x50)
-									);
+							SetContextForeGroundColor (BUILD_COLOR (
+										MAKE_RGB15 (0x00, 0x00, 0x18), 0x50));
 							font_DrawText (&t);
 							DeltaSISGauges (0, 0, j);
 
@@ -601,9 +634,9 @@ UninitEncounter (void)
 								Time = GetTimeCounter ();
 								for (j = 0; j < NUM_SHIP_FADES; ++j)
 								{
-									ClearSemaphore (GraphicsSem);
+									UnlockMutex (GraphicsLock);
 									Sleepy = (BOOLEAN)!AnyButtonPress (TRUE);
-									SetSemaphore (GraphicsSem);
+									LockMutex (GraphicsLock);
 									if (!Sleepy)
 										break;
 
@@ -637,14 +670,14 @@ UninitEncounter (void)
 
 			FlushInput ();
 			Time = GetTimeCounter () + (ONE_SECOND * 3);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			while (!(AnyButtonPress (TRUE)) && GetTimeCounter () < Time)
 				TaskSwitch ();
-			SetSemaphore (GraphicsSem);
-			if (!CurrentInputState.p1_escape)
+			LockMutex (GraphicsLock);
+			if (!CurrentInputState.key[KEY_P1_ESCAPE])
 			{
 				DrawFadeText (str1, str2, FALSE, &scavenge_r);
-				if (!CurrentInputState.p1_escape)
+				if (!CurrentInputState.key[KEY_P1_ESCAPE])
 				{
 					SetContextForeGroundColor (BLACK_COLOR);
 					r.corner.x = scavenge_r.corner.x + 10;
@@ -657,23 +690,19 @@ UninitEncounter (void)
 					t.pStr = buf;
 					t.CharCount = (COUNT)~0;
 					SetContextForeGroundColor (
-							BUILD_COLOR (MAKE_RGB15 (0x00, 0x00, 0x18), 0x50)
-							);
+							BUILD_COLOR (MAKE_RGB15 (0x00, 0x00, 0x18), 0x50));
 					font_DrawText (&t);
 
-					DrawFadeText (
-							str1 = GAME_STRING (ENCOUNTER_STRING_BASE + 6),
-							str2 = GAME_STRING (ENCOUNTER_STRING_BASE + 7),
-							TRUE,
-							&scavenge_r
-							);
+					str1 = GAME_STRING (ENCOUNTER_STRING_BASE + 6);
+					str2 = GAME_STRING (ENCOUNTER_STRING_BASE + 7);
+					DrawFadeText (str1, str2, TRUE, &scavenge_r);
 					Time = GetTimeCounter () + ONE_SECOND * 2;
-					ClearSemaphore (GraphicsSem);
+					UnlockMutex (GraphicsLock);
 					while (!(AnyButtonPress (TRUE))
 							&& GetTimeCounter () < Time)
 						TaskSwitch ();
-					SetSemaphore (GraphicsSem);
-					if (!CurrentInputState.p1_escape)
+					LockMutex (GraphicsLock);
+					if (!CurrentInputState.key[KEY_P1_ESCAPE])
 						DrawFadeText (str1, str2, FALSE, &scavenge_r);
 				}
 			}
@@ -691,7 +720,7 @@ UninitEncounter (void)
 		}
 	}
 ExitUninitEncounter:
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 	return (ships_killed);
 }
@@ -702,10 +731,8 @@ EncounterBattle (void)
 	MEM_HANDLE hLastIndex;
 	ACTIVITY OldActivity;
 	extern UWORD nth_frame;
-	extern BOOLEAN LoadSC2Data (void);
-	extern BOOLEAN FreeSC2Data (void);
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 
 	SET_GAME_STATE (BATTLE_SEGUE, 1);
 
@@ -729,11 +756,11 @@ EncounterBattle (void)
 		cur_speed = (BYTE)(GLOBAL (glob_flags) & COMBAT_SPEED_MASK)
 				>> COMBAT_SPEED_SHIFT;
 		if (cur_speed == 1)
-			cur_speed = 0;
+			cur_speed = 0; /* normal speed */
 		else if (cur_speed == 2)
-			++cur_speed;
+			++cur_speed;   /* 4x speed, 3 of 4 frames skipped */
 		else /* if (cur_speed == 3) */
-			cur_speed = (BYTE)~0;
+			cur_speed = (BYTE)~0; /* maximum speed - no rendering */
 		nth_frame = MAKE_WORD (1, cur_speed);
 		PlayerControl[0] = CYBORG_CONTROL | AWESOME_RATING;
 		PlayerInput[0] = ComputerInput;
@@ -741,9 +768,9 @@ EncounterBattle (void)
 
 	GameSounds = CaptureSound (LoadSound (GAME_SOUNDS));
 
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 	Battle ();
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 
 	DestroySound (ReleaseSound (GameSounds));
 	GameSounds = 0;
@@ -769,6 +796,6 @@ EncounterBattle (void)
 
 	SetResourceIndex (hLastIndex);
 
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 }
 
