@@ -16,27 +16,40 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include <ctype.h>
-#include "starcon.h"
+#include "gameopt.h"
+
+#include "build.h"
+#include "colors.h"
 #include "commglue.h"
-#include "options.h"
-#include "libs/graphics/gfx_common.h"
-#include "inplib.h"
 #include "controls.h"
+#include "encount.h"
+#include "gamestr.h"
+#include "inplib.h"
+#include "load.h"
+#include "options.h"
+#include "save.h"
+#include "settings.h"
+#include "setup.h"
+#include "sounds.h"
+#include "state.h"
+#include "util.h"
+#include "libs/graphics/gfx_common.h"
 
-//Added by Chris
+#include <ctype.h>
 
-void SaveProblem (void);
-
-//End Added by Chris
 
 #define MAX_SAVED_GAMES 50
 #define SUMMARY_X_OFFS 14
 #define SUMMARY_SIDE_OFFS 7
 #define MAX_NAME_CHARS 15
-#define SAVES_PER_PAGE 5  //Should divide evenly into MAX_SAVED_GAMES for 'Per-Page' Scrolling
+#define SAVES_PER_PAGE 5
+
+static BOOLEAN DoSettings (PMENU_STATE pMS);
+static BOOLEAN DoNaming (PMENU_STATE pMS);
 
 static PMENU_STATE pLocMenuState;
+static BYTE prev_save; //keeps track of the last slot that was saved or loaded
+
 
 void
 ConfirmSaveLoad (STAMP *MsgStamp)
@@ -162,11 +175,12 @@ FeedbackSetting (BYTE which_setting)
 			break;
 	}
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	DrawStatusMessage (buf);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 }
-static void 
+
+static void
 FeedbackQuit (BYTE which_setting)
 {
 	UNICODE buf[20];
@@ -181,13 +195,11 @@ FeedbackQuit (BYTE which_setting)
 			break;
 	}
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	DrawStatusMessage (buf);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 }
 
-static BOOLEAN DoSettings (PMENU_STATE pMS);
-
 static BOOLEAN
 DrawDescriptionString (PMENU_STATE pMS, COUNT which_string, SIZE state)
 {
@@ -196,9 +208,8 @@ DrawDescriptionString (PMENU_STATE pMS, COUNT which_string, SIZE state)
 	TEXT lf;
 	COLOR BackGround, ForeGround;
 	FONT Font;
-	static BOOLEAN DoNaming (PMENU_STATE pMS);
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 
 	rel_index = (COUNT)(which_string - pMS->first_item.y);
 
@@ -275,7 +286,7 @@ DrawDescriptionString (PMENU_STATE pMS, COUNT which_string, SIZE state)
 		TextRect (&lf, &text_r, char_deltas);
 		if ((text_r.extent.width + 2) >= r.extent.width)
 		{
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			return (FALSE);
 		}
 
@@ -309,7 +320,7 @@ DrawDescriptionString (PMENU_STATE pMS, COUNT which_string, SIZE state)
 		SetFlashRect (&r, (FRAME)0);
 	}
 
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 	return (TRUE);
 }
 
@@ -435,14 +446,16 @@ DoNaming (PMENU_STATE pMS)
 		pMS->CurString = (STRING)&GD[0];
 		DrawDescriptionString (pMS, pMS->CurState, 1);
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		DrawStatusMessage (GAME_STRING (NAMING_STRING_BASE + 0));
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 
 		EnableCharacterMode ();
 		DoInput (pMS, TRUE);
 		DisableCharacterMode ();
 
+		SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
+
 		pMS->InputFunc = DoSettings;
 		pMS->CurState = (BYTE)pMS->delta_item;
 		pMS->delta_item = 0;
@@ -501,6 +514,7 @@ DoSettings (PMENU_STATE pMS)
 		return (FALSE);
 
 	cur_speed = (BYTE)(GLOBAL (glob_flags) & COMBAT_SPEED_MASK) >> COMBAT_SPEED_SHIFT;
+	SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
 	if (!pMS->Initialized)
 	{
 		DrawMenuStateStrings (PM_SOUND_ON, pMS->CurState);
@@ -508,19 +522,19 @@ DoSettings (PMENU_STATE pMS)
 		pMS->Initialized = TRUE;
 		pMS->InputFunc = DoSettings;
 	}
-	else if (CurrentMenuState.cancel
-			|| (CurrentMenuState.select
+	else if (PulsedInputState.key[KEY_MENU_CANCEL]
+			|| (PulsedInputState.key[KEY_MENU_SELECT]
 			&& pMS->CurState == EXIT_MENU_SETTING))
 	{
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		DrawStatusMessage (NULL_PTR);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 
 		pMS->CurState = SETTINGS;
 		pMS->InputFunc = DoGameOptions;
 		pMS->Initialized = 0;
 	}
-	else if (CurrentMenuState.select)
+	else if (PulsedInputState.key[KEY_MENU_SELECT])
 	{
 		switch (pMS->CurState)
 		{
@@ -570,6 +584,8 @@ DoQuitMenu (PMENU_STATE pMS)
 	if (GLOBAL (CurrentActivity) & CHECK_ABORT)
 		return (FALSE);
 
+	SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
+
 	if (!pMS->Initialized)
 	{
 		DrawMenuStateStrings (PM_NO_QUIT, pMS->CurState);
@@ -577,19 +593,19 @@ DoQuitMenu (PMENU_STATE pMS)
 		pMS->Initialized = TRUE;
 		pMS->InputFunc = DoQuitMenu;
 	}
-	else if (CurrentMenuState.cancel
-			|| (CurrentMenuState.select
+	else if (PulsedInputState.key[KEY_MENU_CANCEL]
+			|| (PulsedInputState.key[KEY_MENU_SELECT]
 			&& pMS->CurState == NO_QUIT_MENU))
 	{
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		DrawStatusMessage (NULL_PTR);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 
 		pMS->CurState = QUIT_GAME;
 		pMS->InputFunc = DoGameOptions;
 		pMS->Initialized = 0;
 	}
-	else if (CurrentMenuState.select)
+	else if (PulsedInputState.key[KEY_MENU_SELECT])
 	{
 		switch (pMS->CurState)
 		{
@@ -857,9 +873,9 @@ ShowSummary (SUMMARY_DESC *pSD)
 			r.corner.y = SIS_ORG_Y + 84;
 			r.extent = OldRect.extent;
 			SetContextClipRect (&r);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			InitLander ((unsigned char)(pSD->Flags | OVERRIDE_LANDER_FLAGS));
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			SetContextClipRect (&OldRect);
 			SetContext (SpaceContext);
 
@@ -961,9 +977,6 @@ LoadGameDescriptions (SUMMARY_DESC *pSD)
 
 	for (i = 0; i < MAX_SAVED_GAMES; ++i, ++pSD)
 	{
-		extern BOOLEAN LoadGame (COUNT which_game,
-				SUMMARY_DESC *summary_desc);
-
 		if (!LoadGame (i, pSD))
 			pSD->year_index = 0;
 	}
@@ -983,18 +996,18 @@ DoPickGame (PMENU_STATE pMS)
 		return (FALSE);
 	}
 	first_time = (BOOLEAN)(pMS->Initialized == 0);
+	SetMenuSounds (MENU_SOUND_ARROWS | MENU_SOUND_PAGEUP | MENU_SOUND_PAGEDOWN, MENU_SOUND_SELECT);
 
 	if (!pMS->Initialized)
 	{
 		pMS->delta_item = (SIZE)pMS->CurState;
-		pMS->CurState = NewState = 0;
-
+		pMS->CurState = NewState = prev_save;
 		pMS->InputFunc = DoPickGame;
 		SleepThreadUntil ((DWORD)pMS->CurFrame);
 		pMS->CurFrame = 0;
 		PauseMusic ();
 		StopSound ();
-		FadeMusic (128, 0);
+		FadeMusic (NORMAL_VOLUME, 0);
 
 		{
 			extern FRAME PlayFrame;
@@ -1002,7 +1015,7 @@ DoPickGame (PMENU_STATE pMS)
 			pMS->ModuleFrame = SetAbsFrameIndex (PlayFrame, 41);
 		}
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		SetTransitionSource (NULL);
 		BatchGraphics ();
 Restart:
@@ -1012,8 +1025,11 @@ Restart:
 		pMS->Initialized = TRUE;
 		goto ChangeGameSelection;
 	}
-	else if (CurrentMenuState.cancel)
+	else if (PulsedInputState.key[KEY_MENU_CANCEL])
 	{
+		LockMutex (GraphicsLock);
+		SetFlashRect ((PRECT)~0L, (FRAME)0);
+		UnlockMutex (GraphicsLock);
 		pMS->ModuleFrame = 0;
 		pMS->CurState = (BYTE)pMS->delta_item;
 		ResumeMusic ();
@@ -1030,18 +1046,16 @@ Restart:
 		}
 		return (FALSE);
 	}
-	else if (CurrentMenuState.select)
+	else if (PulsedInputState.key[KEY_MENU_SELECT])
 	{
 		pSD = &((SUMMARY_DESC *)pMS->CurString)[pMS->CurState];
+		prev_save = pMS->CurState;
 		if (pMS->delta_item == SAVE_GAME || pSD->year_index)
 		{
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			if (pMS->delta_item == SAVE_GAME)
 			{
 				STAMP MsgStamp;
-				extern BOOLEAN SaveGame (COUNT
-						which_game, SUMMARY_DESC
-						*summary_desc);
 
 				ConfirmSaveLoad (&MsgStamp);
 				if (SaveGame ((COUNT)pMS->CurState, pSD))
@@ -1054,13 +1068,13 @@ Restart:
 					SetFlashRect (NULL_PTR, (FRAME)0);
 					DrawStamp (&MsgStamp);
 					DestroyDrawable (ReleaseDrawable (MsgStamp.frame));
-					ClearSemaphore (GraphicsSem);
+					UnlockMutex (GraphicsLock);
 					
 					SaveProblem ();
 
 					pMS->Initialized = FALSE;
 					NewState = pMS->CurState;
-					SetSemaphore (GraphicsSem);
+					LockMutex (GraphicsLock);
 					BatchGraphics ();
 					goto Restart;
 				}
@@ -1079,14 +1093,12 @@ Restart:
 			}
 			else
 			{
-				extern BOOLEAN LoadGame (COUNT which_game, SUMMARY_DESC *summary_desc);
-
 				ConfirmSaveLoad (0);
 				if (LoadGame ((COUNT)pMS->CurState, NULL_PTR))
 					GLOBAL (CurrentActivity) |= CHECK_LOAD;
 			}
 			SetFlashRect (NULL_PTR, (FRAME)0);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 
 			pMS->ModuleFrame = 0;
 			pMS->CurState = (BYTE)pMS->delta_item;
@@ -1096,7 +1108,7 @@ Restart:
 	else
 	{
 		NewState = pMS->CurState;
-		if (CurrentMenuState.left || CurrentMenuState.page_up)
+		if (PulsedInputState.key[KEY_MENU_LEFT] || PulsedInputState.key[KEY_MENU_PAGE_UP])
 		{
 			if (NewState == 0)
 				NewState = MAX_SAVED_GAMES - 1;
@@ -1105,7 +1117,7 @@ Restart:
 			else 
 				NewState = 0;
 		}
-		else if (CurrentMenuState.right || CurrentMenuState.page_down)
+		else if (PulsedInputState.key[KEY_MENU_RIGHT] || PulsedInputState.key[KEY_MENU_PAGE_DOWN])
 		{
 			if (NewState == MAX_SAVED_GAMES - 1)
 				NewState = 0;
@@ -1114,14 +1126,14 @@ Restart:
 			else 
 				NewState = MAX_SAVED_GAMES - 1;
 		}
-		else if (CurrentMenuState.up)
+		else if (PulsedInputState.key[KEY_MENU_UP])
 		{
 			if (NewState == 0)
 				NewState = MAX_SAVED_GAMES - 1;
 			else
 				NewState--;
 		}
-		else if (CurrentMenuState.down)
+		else if (PulsedInputState.key[KEY_MENU_DOWN])
 		{
 			if (NewState == MAX_SAVED_GAMES - 1)
 				NewState = 0;
@@ -1135,7 +1147,7 @@ Restart:
 			TEXT t;
 			BYTE i, SHIFT;
 			UNICODE buf[80],buf2[15];
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 
 			BatchGraphics ();
 			if (((SUMMARY_DESC *)pMS->CurString)[NewState].year_index != 0)
@@ -1157,30 +1169,8 @@ ChangeGameSelection:
 			pMS->CurState = NewState;
 			ShowSummary (&((SUMMARY_DESC *)pMS->CurString)[pMS->CurState]);
 
-			if (LastActivity == CHECK_LOAD)
-			{
-				BYTE clut_buf[] = {FadeAllToColor};
-
-				UnbatchGraphics ();
-
-				LastActivity = 0;
-				XFormColorMap ((COLORMAPPTR)clut_buf, ONE_SECOND / 2);
-			}
-			else
-			{
-				if (first_time)
-				{
-					r.corner.x = SIS_ORG_X;
-					r.corner.y = SIS_ORG_Y;
-					r.extent.width = SIS_SCREEN_WIDTH;
-					r.extent.height = SIS_SCREEN_HEIGHT;
-
-					ScreenTransition (3, &r);
-				}
-				UnbatchGraphics ();
-			}
 			SetContextFont (TinyFont);
-			r.extent.width = 230;
+			r.extent.width = 240;
 			r.extent.height = 65;
 			r.corner.x = 1;
 			r.corner.y = 160;
@@ -1205,7 +1195,7 @@ ChangeGameSelection:
 			else         // 'Per-Page'  Scrolling
 #endif
 				SHIFT = NewState - ((NewState / SAVES_PER_PAGE) * SAVES_PER_PAGE);
-			for (i = 0; i < SAVES_PER_PAGE; i++)
+			for (i = 0; i < SAVES_PER_PAGE && NewState - SHIFT + i < MAX_SAVED_GAMES; i++)
 			{
 				SetContextForeGroundColor ((i == SHIFT) ?
 						(BUILD_COLOR (MAKE_RGB15 (0x1B, 0x00, 0x1B), 0x33)):
@@ -1213,14 +1203,16 @@ ChangeGameSelection:
 				r.extent.width = 15;
 				if (MAX_SAVED_GAMES > 99)
 					r.extent.width += 5;
-				r.extent.height = 9;
+				r.extent.height = 11;
 				r.corner.x = 8;
-				r.corner.y = 161 + (i * 13);
+				r.corner.y = 160 + (i * 13);
 				DrawRectangle (&r);
 
 				t.baseline.x = r.corner.x + 3;
-				t.baseline.y = r.corner.y + 7;
+				t.baseline.y = r.corner.y + 8;
 				wsprintf (buf, "%02i", NewState - SHIFT + i);
+				if (MAX_SAVED_GAMES > 99)
+					wsprintf (buf, "%03i", NewState - SHIFT + i);
 				font_DrawText (&t);
 
 				r.extent.width = 204 - SAFE_X;
@@ -1240,7 +1232,30 @@ ChangeGameSelection:
 				}
 				font_DrawText (&t);
 			}
-			ClearSemaphore (GraphicsSem);
+			if (LastActivity == CHECK_LOAD)
+			{
+				BYTE clut_buf[] = {FadeAllToColor};
+
+				UnbatchGraphics ();
+
+				LastActivity = 0;
+				XFormColorMap ((COLORMAPPTR)clut_buf, ONE_SECOND / 2);
+			}
+			else
+			{
+				if (first_time)
+				{
+					r.corner.x = SIS_ORG_X;
+					r.corner.y = SIS_ORG_Y;
+					r.extent.width = SIS_SCREEN_WIDTH;
+					r.extent.height = SIS_SCREEN_HEIGHT;
+
+					ScreenTransition (3, &r);
+				}
+				UnbatchGraphics ();
+			}
+			SetFlashRect (0, (FRAME)0);
+			UnlockMutex (GraphicsLock);
 		}
 	}
 
@@ -1263,7 +1278,7 @@ PickGame (PMENU_STATE pMS)
 		TaskSwitch ();
 	}
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	OldContext = SetContext (SpaceContext);
 
 	DlgStamp.origin.x = 0;
@@ -1279,11 +1294,10 @@ PickGame (PMENU_STATE pMS)
 	pMS->Initialized = FALSE;
 	pMS->InputFunc = DoPickGame;
 	pMS->CurString = (STRING)&desc_array[0];
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
-	DoInput (pMS, TRUE);
-
-	SetSemaphore (GraphicsSem);
+	DoInput (pMS, TRUE); 
+	LockMutex (GraphicsLock);
 	pMS->Initialized = -1;
 	pMS->InputFunc = DoGameOptions;
 
@@ -1311,8 +1325,16 @@ PickGame (PMENU_STATE pMS)
 
 		if (pSolarSysState)
 		{
+			/* We're in interplanetary, so we let the IP
+			 * functions know we're ready to draw stuff
+			 * again and then update the frame twice; once
+			 * for the screen transition, and once to draw
+			 * the ships afterwards. */
 			--pSolarSysState->MenuState.Initialized;
 			pSolarSysState->PauseRotate = 0;
+			IP_frame ();
+			IP_frame ();
+
 
 			if (CommData.ConversationPhrases == 0 && !PLRPlaying ((MUSIC_REF)~0))
 			{
@@ -1335,7 +1357,7 @@ PickGame (PMENU_STATE pMS)
 	DestroyDrawable (ReleaseDrawable (DlgStamp.frame));
 
 	SetContext (OldContext);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 	return (retval);
 }
@@ -1350,6 +1372,8 @@ DoGameOptions (PMENU_STATE pMS)
 	if (LastActivity == CHECK_LOAD)
 		force_select = TRUE;
 
+	SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
+
 	if (pMS->Initialized <= 0)
 	{
 		if (LastActivity == CHECK_LOAD)
@@ -1359,21 +1383,23 @@ DoGameOptions (PMENU_STATE pMS)
 		pMS->Initialized = 1;
 		pMS->InputFunc = DoGameOptions;
 	}
-	else if (CurrentMenuState.cancel
-			|| (CurrentMenuState.select
+	else if (PulsedInputState.key[KEY_MENU_CANCEL]
+			|| (PulsedInputState.key[KEY_MENU_SELECT]
 			&& pMS->CurState == SETTINGS + 1))
 	{
 		pMS->CurState = SETTINGS + 1;
 		return (FALSE);
 	}
-	else if (CurrentMenuState.select || force_select)
+	else if (PulsedInputState.key[KEY_MENU_SELECT] || force_select)
 	{
 		switch (pMS->CurState)
 		{
 			case SAVE_GAME:
 			case LOAD_GAME:
 				pMS->CurFrame = (FRAME)FadeMusic (0, ONE_SECOND >> 1);
-				return (PickGame (pMS));
+						// XXX: what is going on here? A DWORD is cast
+						//      to a FRAME.
+				return PickGame (pMS);
 			case QUIT_GAME:
 				pMS->Initialized = FALSE;
 				pMS->CurState = NO_QUIT_MENU;
@@ -1403,6 +1429,7 @@ GameOptions (void)
 
 	MenuState.InputFunc = DoGameOptions;
 	MenuState.CurState = SAVE_GAME;
+	SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
 	DoInput ((PVOID)&MenuState, TRUE);
 
 	pLocMenuState = 0;
