@@ -20,6 +20,10 @@
 #define _GLOBDATA_H
 
 #include "clock.h"
+#include "libs/gfxlib.h"
+#include "sis.h"
+#include "velocity.h"
+
 
 #define RANDOM_ANIM (1 << 0)
 #define CIRCULAR_ANIM (1 << 1)
@@ -107,10 +111,18 @@ typedef NUMBER_SPEECH_DESC *PNUMBER_SPEECH_DESC;
 #define NUMBER_SPEECH_DESCPTR PNUMBER_SPEECH_DESC
 typedef NUMBER_SPEECH_DESC *NUMBER_SPEECH;
 
+typedef DWORD LDAS_FLAGS;
+#define LDASF_NONE           ((LDAS_FLAGS)      0 )
+#define LDASF_USE_ALTERNATE  ((LDAS_FLAGS)(1 << 0))
+
 typedef struct
 {
 	void (*init_encounter_func) (void);
+			/* Called when entering communications */
+	void (*post_encounter_func) (void);
+			/* Called when leaving communications or combat normally */
 	COUNT (*uninit_encounter_func) (void);
+			/* Called when encounter is done for cleanup */
 
 	FRAME AlienFrame;
 	FONT AlienFont;
@@ -118,6 +130,8 @@ typedef struct
 	COUNT AlienTextWidth;
 	COLORMAP AlienColorMap;
 	DWORD AlienSong;
+	PVOID AlienAltSong;
+	LDAS_FLAGS AlienSongFlags;
 
 	STRING ConversationPhrases;
 
@@ -185,7 +199,15 @@ START_GAME_STATE
 
 	ADD_GAME_STATE (SHOFIXTI_MAIDENS, 1) /* Did you find the babes yet? */
 	ADD_GAME_STATE (MAIDENS_ON_SHIP, 1)
-	ADD_GAME_STATE (BATTLE_SEGUE, 1) /* Does encounter require battle segue? */
+	ADD_GAME_STATE (BATTLE_SEGUE, 1)
+			/* Set to 0 in init_xxx_comm() if communications directly
+			 * follows an encounter.
+			 * Set to 1 in init_xxx_comm() if the player gets to decide
+			 * whether to attack or talk.
+			 * Set to 1 in communication when battle follows the
+			 * communication. It is still valid when uninit_xxx_comm() gets
+			 * called after combat or communication.
+			 */
 	ADD_GAME_STATE (PLANETARY_LANDING, 1)
 	ADD_GAME_STATE (PLANETARY_CHANGE, 1)
 
@@ -193,6 +215,13 @@ START_GAME_STATE
 	ADD_GAME_STATE (SPATHI_VISITS, 3)
 	ADD_GAME_STATE (SPATHI_HOME_VISITS, 3)
 	ADD_GAME_STATE (FOUND_PLUTO_SPATHI, 2)
+			/* 0 - Haven't met Fwiffo.
+			 * 1 - Met Fwiffo on Pluto, now talking to him.
+			 * 2 - Met Fwiffo on Pluto, after dialog.
+			 * 3 - Met Fwiffo, and have reported to the Safe Ones on
+			 *     the Spathi moon that he was either killed, or that
+			 *     you have him on board.
+			 */
 	ADD_GAME_STATE (SPATHI_SHIELDED_SELVES, 1)
 	ADD_GAME_STATE (SPATHI_CREATURES_EXAMINED, 1)
 	ADD_GAME_STATE (SPATHI_CREATURES_ELIMINATED, 1)
@@ -414,6 +443,13 @@ START_GAME_STATE
 	ADD_GAME_STATE (MYCON_FELL_FOR_AMBUSH, 1)
 
 	ADD_GAME_STATE (GLOBAL_FLAGS_AND_DATA, 8)
+			/* This state seems to be used to distinguish between different
+			 * places where one may have an conversation with an alien.
+			 * Like home world, other world, space.
+			 * Why this needs 8 bits I don't know. Only specific
+			 * combinations of bits seem to be used (0, 1, or all bits).
+			 * A closer investigation is desirable. - SvdB
+			 */
 
 	ADD_GAME_STATE (ORZ_VISITS, 3)
 	ADD_GAME_STATE (TAALO_VISITS, 3)
@@ -492,8 +528,14 @@ START_GAME_STATE
 	ADD_GAME_STATE (PKUNK_VISITS, 3)
 	ADD_GAME_STATE (PKUNK_HOME_VISITS, 3)
 	ADD_GAME_STATE (PKUNK_SHIP_MONTH, 4)
+			/* The month in PKUNK_SHIP_YEAR that new ships are available
+			 * from the Pkunk. */
 	ADD_GAME_STATE (PKUNK_SHIP_DAY, 5)
+			/* The day of the month in PKUNK_SHIP_MONTH in PKUNK_SHIP_YEAR
+			 * that new ships are available. */
 	ADD_GAME_STATE (PKUNK_SHIP_YEAR, 5)
+			/* The year that new ships are available from the Pkunk
+			 * (stored as an offset from the year the game starts). */
 	ADD_GAME_STATE (PKUNK_MISSION, 3)
 
 	ADD_GAME_STATE (SUPOX_VISITS, 3)
@@ -505,6 +547,12 @@ START_GAME_STATE
 	ADD_GAME_STATE (HELIX_UNPROTECTED, 1)
 	ADD_GAME_STATE (THRADD_CULTURE, 2)
 	ADD_GAME_STATE (THRADD_MISSION, 3)
+			/* 0 if the Thraddash fleet hasn't left the Thraddash home world.
+			 * 1 if the Thraddash are heading towards Kohr-Ah territory.
+			 * 2 if the Thraddash are fighting the Kohr-Ah.
+			 * 3 if the Thraddash are returning from Kohr-Ah territory.
+			 * 4 if the Thraddash fleet is back at the Thraddash home world.
+			 */
 
 	ADD_GAME_STATE (DRUUGE_VISITS, 3)
 	ADD_GAME_STATE (DRUUGE_HOME_VISITS, 3)
@@ -520,6 +568,11 @@ START_GAME_STATE
 	ADD_GAME_STATE (ZOQFOT_HOME_VISITS, 3)
 	ADD_GAME_STATE (MET_ZOQFOT, 1)
 	ADD_GAME_STATE (ZOQFOT_DISTRESS, 2)
+			/* 0 if the Zoq-Fot-Pik aren't in distress
+			 * 1 if the Zoq-Fot-Pik are under attack by the Kohr-Ah
+			 * 2 if the Zoq-Fot-Pik have been destroyed because of this
+			 *   attack (not by the Kohr-Ah final victory cleansing)
+			 */
 
 	ADD_GAME_STATE (EGG_CASE1_ON_SHIP, 1)
 	ADD_GAME_STATE (EGG_CASE2_ON_SHIP, 1)
@@ -536,8 +589,17 @@ START_GAME_STATE
 	ADD_GAME_STATE (MELNORME_HISTORY_INFO_STACK, 5)
 
 	ADD_GAME_STATE (RAINBOW_WORLD0, 8)
+			/* Low byte of a bit array, one bit per rainbow world.
+			 * Each bit is set if the rainbow world has been visited.
+			 * The lowest bit is for the first star in the star_array
+			 * with RAINBOW_DEFINED, and so on.
+			 */
 	ADD_GAME_STATE (RAINBOW_WORLD1, 2)
+			/* High byte of the bit array of which RAINBOW_WORLD0
+			 * is the low byte.
+			 */
 	ADD_GAME_STATE (MELNORME_RAINBOW_COUNT, 4)
+			/* The number of rainbow world locations sold to the Melnorme. */
 
 	ADD_GAME_STATE (USED_BROADCASTER, 1)
 	ADD_GAME_STATE (BROADCASTER_RESPONSE, 1)
@@ -578,6 +640,14 @@ START_GAME_STATE
 	ADD_GAME_STATE (THRADDASH_BODY_COUNT, 5)
 
 	ADD_GAME_STATE (UTWIG_SUPOX_MISSION, 3)
+			/* 0 if the Utwig and Supox fleet haven't left their home world.
+			 * 1 if the U&S are on their way towards the Kohr-Ah
+			 * 2 if the U&S are fighting the Kohr-Ah (first 80 days)
+			 * 3 does not occur
+             * 4 if the U&S are fighting the Kohr-Ah (second 80 days)
+			 * 5 if the U&S are returning home.
+			 * 6 if the U&S are back at their home world.
+			 */
 	ADD_GAME_STATE (SPATHI_INFO, 3)
 
 	ADD_GAME_STATE (ILWRATH_INFO, 2)
@@ -599,6 +669,14 @@ START_GAME_STATE
 	ADD_GAME_STATE (MYCON_INFO, 4)
 	ADD_GAME_STATE (MYCON_RAMBLE, 5)
 	ADD_GAME_STATE (KNOW_ABOUT_SHATTERED, 2)
+			/* 0 if the player doesn't known about shattered worlds
+			 * 1 if the player has encountered a shattered world
+			 * 2 if the player knows that shatterred worlds are caused
+			 *   by Mycon deep children.
+			 * 3 if the player has told the Syreen that Mycon Deep Children
+			 *   cause shattered worlds. Proof doesn't have to be presented
+			 *   yet at this time.
+			 */
 	ADD_GAME_STATE (MYCON_INSULTS, 3)
 	ADD_GAME_STATE (MYCON_KNOW_AMBUSH, 1)
 
@@ -618,6 +696,11 @@ START_GAME_STATE
 	ADD_GAME_STATE (REFUSED_ORZ_ALLIANCE, 1)
 
 	ADD_GAME_STATE (PKUNK_MANNER, 2)
+			/* 0 not met the Pkunk
+			 * 1 fought the Pkunk, but relations are still salvagable.
+			 * 2 hostile relations with the Pkunk, no way back.
+			 * 3 friendly relations with the Pkunk
+			 */
 	ADD_GAME_STATE (PKUNK_ON_THE_MOVE, 1)
 	ADD_GAME_STATE (PKUNK_FLEET, 2)
 	ADD_GAME_STATE (PKUNK_MIGRATE, 2)
@@ -777,7 +860,7 @@ END_GAME_STATE
 
 enum
 {
-	SUPER_MELEE = 0,
+	SUPER_MELEE = 0, /* Is also used while in the main menu */
 	IN_LAST_BATTLE,
 	IN_ENCOUNTER,
 	IN_HYPERSPACE /* in Hyperspace or Quasispace */,
@@ -802,6 +885,7 @@ typedef UWORD ACTIVITY;
 
 typedef struct
 {
+	// XXX: cur_state is obsolete now and was replaced by process.c:zoom_out
 	BYTE cur_state, glob_flags;
 
 	BYTE CrewCost, FuelCost;
@@ -820,7 +904,9 @@ typedef struct
 
 	DWORD BattleGroupRef;
 	QUEUE avail_race_q, npc_built_ship_q;
-	QUEUE encounter_q, built_ship_q;
+	QUEUE encounter_q;
+	QUEUE built_ship_q;
+			/* Queue of SIS escort ships */
 
 	BYTE GameState[(NUM_GAME_STATE_BITS + 7) >> 3];
 } GAME_STATE;
@@ -841,41 +927,12 @@ extern GLOBDATA GlobData;
 
 //#define STATE_DEBUG
 	
-#ifdef STATE_DEBUG
-#	include <stdio.h>
-#endif
-	
-static inline BYTE
-getGameState (int startBit, int endBit) {
-	return (BYTE) (((startBit >> 3) == (endBit >> 3)
-			? (GLOBAL (GameState[startBit >> 3]) >> (startBit & 7))
-			: ((GLOBAL (GameState[startBit >> 3]) >> (startBit & 7))
-			  | (GLOBAL (GameState[endBit >> 3])
-			  << (endBit - startBit - (endBit & 7)))))
-			& ((1 << (endBit - startBit + 1)) - 1));
-}
-
-static inline
-void setGameState (int startBit, int endBit, BYTE val
+extern BYTE getGameState (int startBit, int endBit);
+extern void setGameState (int startBit, int endBit, BYTE val
 #ifdef STATE_DEBUG
 		, const char *name
 #endif
-		) {
-	GLOBAL (GameState[startBit >> 3]) =
-			(GLOBAL (GameState[startBit >> 3])
-			& (BYTE) ~(((1 << (endBit - startBit + 1)) - 1) << (startBit & 7)))
-			| (BYTE)((val) << (startBit & 7));
-
-	if ((startBit >> 3) < (endBit >> 3)) {
-		GLOBAL (GameState[endBit >> 3]) =
-				(GLOBAL (GameState[endBit >> 3])
-				& (BYTE)~((1 << ((endBit & 7) + 1)) - 1))
-				| (BYTE)((val) >> (endBit - startBit - (endBit & 7)));
-	}
-#ifdef STATE_DEBUG
-	fprintf (stderr, "State '%s' set to %d.\n", name, val);
-#endif
-}
+		);
 
 #define GET_GAME_STATE(SName) getGameState ((SName), (END_##SName))
 #ifdef STATE_DEBUG
@@ -886,38 +943,17 @@ void setGameState (int startBit, int endBit, BYTE val
 			setGameState ((SName), (END_##SName), (val))
 #endif
 
+	
+extern CONTEXT RadarContext;
 
 extern void DiscardStarMap (PVOID CodeRef);
 extern void RetrieveStarMap (void);
 
-#define STAR_STRING_BASE 0
-#define DEVICE_STRING_BASE 133
-#define CARGO_STRING_BASE 162
-#define ELEMENTS_STRING_BASE 172
-#define SCAN_STRING_BASE 305
-#define STAR_NUMBER_BASE 361
-#define PLANET_NUMBER_BASE 375
-#define MONTHS_STRING_BASE 407
-#define FEEDBACK_STRING_BASE 419
-#define STARBASE_STRING_BASE 420
-#define ENCOUNTER_STRING_BASE 425
-#define NAVIGATION_STRING_BASE 433
-#define NAMING_STRING_BASE 439
-#define MELEE_STRING_BASE 443
-#define SAVEGAME_STRING_BASE 452
-#define OPTION_STRING_BASE 455
+extern void FreeSC2Data (void);
+extern BOOLEAN LoadSC2Data (void);
 
-#define GAME_STRING(i) ((UNICODE *)GetStringAddress (SetAbsStringTableIndex (GameStrings, (i))))
+void InitGlobData (void);
 
-#define wsprintf sprintf
-#define wstrlen strlen
-#define wstrcpy strcpy
-#define wstrcat strcat
-#define wstrupr strupr
-#define wstrncpy strncpy
-#define wstricmp stricmp
-#define wstrtoul strtoul
-#define wstrcspn strcspn
 
 #endif /* _GLOBDATA_H */
 
