@@ -16,13 +16,22 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "starcon.h"
+#include "build.h"
+#include "encount.h"
+#include "file.h"
+#include "globdata.h"
+#include "intel.h"
+#include "sis.h"
+#include "state.h"
+
+#include "libs/mathlib.h"
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include "file.h"
 
-//#define DEBUG
+
+//#define DEBUG_GROUPS
 
 typedef struct
 {
@@ -73,30 +82,30 @@ BuildGroups (void)
 	HSTARSHIP hTemplate, hNextShip;
 	BYTE HomeWorld[] =
 	{
-		0, /* ARILOU_SHIP */
-		0, /* CHMMR_SHIP */
-		0, /* HUMAN_SHIP */
-		ORZ_DEFINED, /* ORZ_SHIP */
-		PKUNK_DEFINED, /* PKUNK_SHIP */
-		0, /* SHOFIXTI_SHIP */
-		SPATHI_DEFINED, /* SPATHI_SHIP */
-		SUPOX_DEFINED, /* SUPOX_SHIP */
-		THRADD_DEFINED, /* THRADDASH_SHIP */
-		UTWIG_DEFINED, /* UTWIG_SHIP */
-		VUX_DEFINED, /* VUX_SHIP */
-		YEHAT_DEFINED, /* YEHAT_SHIP */
-		0, /* MELNORME_SHIP */
-		DRUUGE_DEFINED, /* DRUUGE_SHIP */
-		ILWRATH_DEFINED,/* ILWRATH_SHIP */
-		MYCON_DEFINED, /* MYCON_SHIP */
-		0, /* SLYLANDRO_SHIP */
-		UMGAH_DEFINED, /* UMGAH_SHIP */
-		0, /* URQUAN_SHIP */
-		ZOQFOT_DEFINED, /* ZOQFOTPIK_SHIP */
+		0,                /* ARILOU_SHIP */
+		0,                /* CHMMR_SHIP */
+		0,                /* HUMAN_SHIP */
+		ORZ_DEFINED,      /* ORZ_SHIP */
+		PKUNK_DEFINED,    /* PKUNK_SHIP */
+		0,                /* SHOFIXTI_SHIP */
+		SPATHI_DEFINED,   /* SPATHI_SHIP */
+		SUPOX_DEFINED,    /* SUPOX_SHIP */
+		THRADD_DEFINED,   /* THRADDASH_SHIP */
+		UTWIG_DEFINED,    /* UTWIG_SHIP */
+		VUX_DEFINED,      /* VUX_SHIP */
+		YEHAT_DEFINED,    /* YEHAT_SHIP */
+		0,                /* MELNORME_SHIP */
+		DRUUGE_DEFINED,   /* DRUUGE_SHIP */
+		ILWRATH_DEFINED,  /* ILWRATH_SHIP */
+		MYCON_DEFINED,    /* MYCON_SHIP */
+		0,                /* SLYLANDRO_SHIP */
+		UMGAH_DEFINED,    /* UMGAH_SHIP */
+		0,                /* URQUAN_SHIP */
+		ZOQFOT_DEFINED,   /* ZOQFOTPIK_SHIP */
 
-		0, /* SYREEN_SHIP */
-		0, /* BLACK_URQUAN_SHIP */
-		0, /* YEHAT_REBEL_SHIP */
+		0,                /* SYREEN_SHIP */
+		0,                /* BLACK_URQUAN_SHIP */
+		0,                /* YEHAT_REBEL_SHIP */
 	};
 	BYTE EncounterPercent[] =
 	{
@@ -117,9 +126,7 @@ BuildGroups (void)
 		EXTENDED_SHIP_FRAGMENTPTR TemplatePtr;
 
 		TemplatePtr = (EXTENDED_SHIP_FRAGMENTPTR)LockStarShip (
-				&GLOBAL (avail_race_q),
-				hTemplate
-				);
+				&GLOBAL (avail_race_q), hTemplate);
 		hNextShip = _GetSuccLink (TemplatePtr);
 
 		if ((encounter_radius = TemplatePtr->ShipInfo.actual_strength)
@@ -128,6 +135,10 @@ BuildGroups (void)
 			SIZE dx, dy;
 			DWORD d_squared;
 
+			// XXX: BUG: i is redefined in the line below, so the assignment
+			//      to i a few lines ago is useless.
+			//      EncounterPercent is never used in practice.
+			//      BestIndex may be used uninitialised. - SvdB
 			if ((i = HomeWorld[Index]) && CurStarDescPtr->Index == (BYTE)i)
 			{
 				BestIndex = Index;
@@ -167,7 +178,11 @@ BuildGroups (void)
 						|| (HIWORD (rand_val) % (i + BestPercent)) < i))
 				{
 					if (TemplatePtr->ShipInfo.actual_strength == (COUNT)~0)
+					{
+						// XXX: BUG: This code is never reached.
+						//      When actual_strength == ~0, i is always 0.
 						i = 4;
+					}
 
 					BestPercent = i;
 					BestIndex = Index;
@@ -176,10 +191,7 @@ BuildGroups (void)
 		}
 
 FoundHome:
-		UnlockStarShip (
-				&GLOBAL (avail_race_q),
-				hTemplate
-				);
+		UnlockStarShip (&GLOBAL (avail_race_q), hTemplate);
 	}
 
 	if (BestPercent)
@@ -200,13 +212,13 @@ FoundHome:
 			num_groups = 5;
 		do
 		{
-			for (Index = HINIBBLE (EncounterMakeup[BestIndex]); Index; --Index)
+			for (Index = HINIBBLE (EncounterMakeup[BestIndex]); Index;
+					--Index)
 			{
 				if (Index <= LONIBBLE (EncounterMakeup[BestIndex])
 						|| (COUNT)TFB_Random () % 100 < 50)
-					CloneShipFragment (
-							BestIndex, &GLOBAL (npc_built_ship_q), 0
-							);
+					CloneShipFragment (BestIndex, &GLOBAL (npc_built_ship_q),
+							0);
 			}
 
 			PutGroupInfo (0L, ++which_group);
@@ -218,8 +230,7 @@ FoundHome:
 }
 
 static void
-FlushGroupInfo (GROUP_HEADER *pGH, DWORD offset, BYTE which_group, PVOID
-		fp)
+FlushGroupInfo (GROUP_HEADER *pGH, DWORD offset, BYTE which_group, PVOID fp)
 {
 	BYTE RaceType, NumShips;
 	HSTARSHIP hStarShip;
@@ -243,8 +254,7 @@ FlushGroupInfo (GROUP_HEADER *pGH, DWORD offset, BYTE which_group, PVOID
 				BYTE crew_level;
 
 				FragPtr = (SHIP_FRAGMENTPTR)LockStarShip (
-						&temp_q, hStarShip
-						);
+						&temp_q, hStarShip);
 				hNextShip = _GetSuccLink (FragPtr);
 				crew_level = FragPtr->ShipInfo.crew_level;
 				which_group = GET_GROUP_ID (FragPtr);
@@ -259,12 +269,8 @@ FlushGroupInfo (GROUP_HEADER *pGH, DWORD offset, BYTE which_group, PVOID
 					else
 						FlushGroupInfo (pGH, 0L, which_group, fp);
 					pGH->GroupOffset[which_group] = 0;
-					RemoveQueue (
-							&temp_q, hStarShip
-							);
-					FreeStarShip (
-							&temp_q, hStarShip
-							);
+					RemoveQueue (&temp_q, hStarShip);
+					FreeStarShip (&temp_q, hStarShip);
 				}
 			}
 			GLOBAL (npc_built_ship_q) = temp_q;
@@ -279,21 +285,19 @@ FlushGroupInfo (GROUP_HEADER *pGH, DWORD offset, BYTE which_group, PVOID
 
 		hStarShip = GetHeadLink (&GLOBAL (npc_built_ship_q));
 		FragPtr = (SHIP_FRAGMENTPTR)LockStarShip (
-				&GLOBAL (npc_built_ship_q), hStarShip
-				);
+				&GLOBAL (npc_built_ship_q), hStarShip);
 		RaceType = GET_RACE_ID (FragPtr);
 		SeekResFile (fp, pGH->GroupOffset[which_group], SEEK_SET);
 		WriteResFile ((PBYTE)&RaceType, sizeof (RaceType), 1, fp);
-		UnlockStarShip (
-				&GLOBAL (npc_built_ship_q), hStarShip
-				);
+		UnlockStarShip (&GLOBAL (npc_built_ship_q), hStarShip);
 	}
 	SeekResFile (fp, offset, SEEK_SET);
 	WriteResFile (pGH, sizeof (*pGH), 1, fp);
-#ifdef DEBUG
-	fprintf (stderr, "1)FlushGroupInfo(%lu): WG = %u(%lu), NG = %u, SI = %u\n",
-	offset, which_group, pGH->GroupOffset[which_group], pGH->NumGroups, pGH->star_index);
-#endif /* DEBUG */
+#ifdef DEBUG_GROUPS
+	fprintf (stderr, "1)FlushGroupInfo(%lu): WG = %u(%lu), NG = %u, "
+			"SI = %u\n", offset, which_group, pGH->GroupOffset[which_group],
+			pGH->NumGroups, pGH->star_index);
+#endif /* DEBUG_GROUPS */
 
 	NumShips = (BYTE)CountLinks (&GLOBAL (npc_built_ship_q));
 
@@ -322,7 +326,7 @@ FlushGroupInfo (GROUP_HEADER *pGH, DWORD offset, BYTE which_group, PVOID
 		RaceType = GET_RACE_ID (FragPtr);
 		WriteResFile ((PBYTE)&RaceType, sizeof (RaceType), 1, fp);
 
-#ifdef DEBUG
+#ifdef DEBUG_GROUPS
 		if (which_group == 0)
 			fprintf (stderr, "F) type %u, loc %u<%d, %d>, task 0x%02x:%u\n",
 					RaceType,
@@ -331,14 +335,11 @@ FlushGroupInfo (GROUP_HEADER *pGH, DWORD offset, BYTE which_group, PVOID
 					FragPtr->ShipInfo.loc.y,
 					GET_GROUP_MISSION (FragPtr),
 					GET_GROUP_DEST (FragPtr));
-#endif /* DEBUG */
+#endif /* DEBUG_GROUPS */
 		Ptr = (PBYTE)&FragPtr->RaceDescPtr;
 		WriteResFile ((PBYTE)Ptr,
-				((PBYTE)&FragPtr->ShipInfo.race_strings) - Ptr,
-				1, fp);
-		UnlockStarShip (
-				&GLOBAL (npc_built_ship_q), hStarShip
-				);
+				((PBYTE)&FragPtr->ShipInfo.race_strings) - Ptr, 1, fp);
+		UnlockStarShip (&GLOBAL (npc_built_ship_q), hStarShip);
 		hStarShip = hNextShip;
 	}
 }
@@ -360,17 +361,19 @@ GetGroupInfo (DWORD offset, BYTE which_group)
 
 		SeekResFile (fp, offset, SEEK_SET);
 		ReadResFile (&GH, sizeof (GH), 1, fp);
-#ifdef DEBUG
-		fprintf (stderr, "GetGroupInfo(%lu): %u(%lu) out of %u\n", offset, which_group, GH.GroupOffset[which_group], GH.NumGroups);
-#endif /* DEBUG */
+#ifdef DEBUG_GROUPS
+		fprintf (stderr, "GetGroupInfo(%lu): %u(%lu) out of %u\n", offset,
+				which_group, GH.GroupOffset[which_group], GH.NumGroups);
+#endif /* DEBUG_GROUPS */
 		if (which_group == (BYTE)~0)
 		{
 			COUNT month_index, day_index, year_index;
 
 			ReinitQueue (&GLOBAL (npc_built_ship_q));
-#ifdef DEBUG
-			fprintf (stderr, "%u == %u\n", GH.star_index, (COUNT)(CurStarDescPtr - star_array));
-#endif /* DEBUG */
+#ifdef DEBUG_GROUPS
+			fprintf (stderr, "%u == %u\n", GH.star_index,
+					(COUNT)(CurStarDescPtr - star_array));
+#endif /* DEBUG_GROUPS */
 			day_index = GH.day_index;
 			month_index = GH.month_index;
 			year_index = GH.year_index;
@@ -381,11 +384,12 @@ GetGroupInfo (DWORD offset, BYTE which_group)
 			{
 				res_CloseResFile (fp);
 
-#ifdef DEBUG
-			if (GH.star_index == (COUNT)(CurStarDescPtr - star_array))
-				fprintf (stderr, "GetGroupInfo: battle groups out of date %u/%u/%u!\n",
-						month_index, day_index, year_index);
-#endif /* DEBUG */
+#ifdef DEBUG_GROUPS
+				if (GH.star_index == (COUNT)(CurStarDescPtr - star_array))
+					fprintf (stderr, "GetGroupInfo: battle groups out of "
+							"date %u/%u/%u!\n", month_index, day_index,
+							year_index);
+#endif /* DEBUG_GROUPS */
 				fp = res_OpenResFile (tempDir, RANDGRPINFO_FILE, "wb");
 				GH.NumGroups = 0;
 				GH.star_index = (COUNT)~0;
@@ -394,7 +398,8 @@ GetGroupInfo (DWORD offset, BYTE which_group)
 			}
 			else
 			{
-				for (which_group = 1; which_group <= GH.NumGroups; ++which_group)
+				for (which_group = 1; which_group <= GH.NumGroups;
+						++which_group)
 				{
 					if (GH.GroupOffset[which_group] == 0)
 						continue;
@@ -411,17 +416,17 @@ GetGroupInfo (DWORD offset, BYTE which_group)
 						hStarShip = CloneShipFragment (RaceType,
 								&GLOBAL (npc_built_ship_q), 0);
 						FragPtr = (SHIP_FRAGMENTPTR)LockStarShip (
-								&GLOBAL (npc_built_ship_q), hStarShip
-								);
+								&GLOBAL (npc_built_ship_q), hStarShip);
 						OwnStarShip (FragPtr, BAD_GUY, 0);
 						SET_GROUP_ID (FragPtr, which_group);
 
 						rand_val = TFB_Random ();
-						task = (BYTE)(LOBYTE (LOWORD (rand_val)) % ON_STATION);
+						task = (BYTE)(LOBYTE (LOWORD (rand_val))
+								% ON_STATION);
 						if (task == FLEE)
 							task = ON_STATION;
-						SET_ORBIT_LOC (FragPtr,
-								NORMALIZE_FACING (LOBYTE (HIWORD (rand_val))));
+						SET_ORBIT_LOC (FragPtr, NORMALIZE_FACING (
+								LOBYTE (HIWORD (rand_val))));
 
 						group_loc = pSolarSysState->SunDesc[0].NumPlanets;
 						if (group_loc == 1 && task == EXPLORE)
@@ -438,8 +443,7 @@ GetGroupInfo (DWORD offset, BYTE which_group)
 								(HIWORD (rand_val) % 10000) - 5000;
 						if (task == EXPLORE)
 							FragPtr->ShipInfo.group_counter =
-									((COUNT)TFB_Random ()
-									% MAX_REVOLUTIONS)
+									((COUNT)TFB_Random () % MAX_REVOLUTIONS)
 									<< FACING_SHIFT;
 						else
 						{
@@ -449,16 +453,11 @@ GetGroupInfo (DWORD offset, BYTE which_group)
 								COUNT angle;
 								POINT org;
 
-								XFormIPLoc (
-										&pSolarSysState->PlanetDesc[
-												group_loc - 1
-												].image.origin,
-										&org,
-										FALSE
-										);
-								angle = FACING_TO_ANGLE (
-										GET_ORBIT_LOC (FragPtr) + 1
-										);
+								XFormIPLoc (&pSolarSysState->PlanetDesc[
+										group_loc - 1].image.origin, &org,
+										FALSE);
+								angle = FACING_TO_ANGLE (GET_ORBIT_LOC (
+										FragPtr) + 1);
 								FragPtr->ShipInfo.loc.x = org.x
 										+ COSINE (angle, STATION_RADIUS);
 								FragPtr->ShipInfo.loc.y = org.y
@@ -470,9 +469,9 @@ GetGroupInfo (DWORD offset, BYTE which_group)
 						SET_GROUP_MISSION (FragPtr, task);
 						SET_GROUP_LOC (FragPtr, group_loc);
 
-#ifdef DEBUG
-						fprintf (stderr, "battle group %u(0x%04x) strength %u, "
-								"type %u, loc %u<%d, %d>, task %u\n",
+#ifdef DEBUG_GROUPS
+						fprintf (stderr, "battle group %u(0x%04x) strength "
+								"%u, type %u, loc %u<%d, %d>, task %u\n",
 								which_group,
 								hStarShip,
 								NumShips,
@@ -481,10 +480,9 @@ GetGroupInfo (DWORD offset, BYTE which_group)
 								FragPtr->ShipInfo.loc.x,
 								FragPtr->ShipInfo.loc.y,
 								task);
-#endif /* DEBUG */
-						UnlockStarShip (
-								&GLOBAL (npc_built_ship_q), hStarShip
-								);
+#endif /* DEBUG_GROUPS */
+						UnlockStarShip (&GLOBAL (npc_built_ship_q),
+								hStarShip);
 					}
 				}
 
@@ -544,30 +542,30 @@ GetGroupInfo (DWORD offset, BYTE which_group)
 						&GLOBAL (npc_built_ship_q), 0);
 
 				FragPtr = (SHIP_FRAGMENTPTR)LockStarShip (
-						&GLOBAL (npc_built_ship_q), hStarShip
-						);
+						&GLOBAL (npc_built_ship_q), hStarShip);
 				Ptr = (PBYTE)&FragPtr->RaceDescPtr;
 				ReadResFile ((PBYTE)Ptr,
-						((PBYTE)&FragPtr->ShipInfo.race_strings) - Ptr,
-						1, fp);
+						((PBYTE)&FragPtr->ShipInfo.race_strings) - Ptr, 1,
+						fp);
 
-#ifdef DEBUG
+#ifdef DEBUG_GROUPS
 				if (which_group == 0)
-					fprintf (stderr, "G) type %u, loc %u<%d, %d>, task 0x%02x:%u\n",
+					fprintf (stderr, "G) type %u, loc %u<%d, %d>, "
+							"task 0x%02x:%u\n",
 							RaceType,
 							GET_GROUP_LOC (FragPtr),
 							FragPtr->ShipInfo.loc.x,
 							FragPtr->ShipInfo.loc.y,
 							GET_GROUP_MISSION (FragPtr),
 							GET_GROUP_DEST (FragPtr));
-#endif /* DEBUG */
+#endif /* DEBUG_GROUPS */
 				if (GET_GROUP_ID (FragPtr) != BaseGroup
 						|| which_group
 						|| ShipsLeft)
 				{
-#ifdef DEBUG
+#ifdef DEBUG_GROUPS
 					fprintf (stderr, "\n");
-#endif /* DEBUG */
+#endif /* DEBUG_GROUPS */
 					if (RaceType == SHOFIXTI_SHIP
 							&& which_group
 							&& !GET_GAME_STATE (SHOFIXTI_RECRUITED))
@@ -588,18 +586,12 @@ GetGroupInfo (DWORD offset, BYTE which_group)
 				}
 				else
 				{
-#ifdef DEBUG
+#ifdef DEBUG_GROUPS
 					fprintf (stderr, " -- REMOVING\n");
-#endif /* DEBUG */
-					UnlockStarShip (
-							&GLOBAL (npc_built_ship_q), hStarShip
-							);
-					RemoveQueue (
-							&GLOBAL (npc_built_ship_q), hStarShip
-							);
-					FreeStarShip (
-							&GLOBAL (npc_built_ship_q), hStarShip
-							);
+#endif /* DEBUG_GROUPS */
+					UnlockStarShip (&GLOBAL (npc_built_ship_q), hStarShip);
+					RemoveQueue (&GLOBAL (npc_built_ship_q), hStarShip);
+					FreeStarShip (&GLOBAL (npc_built_ship_q), hStarShip);
 				}
 			}
 		}
@@ -652,17 +644,18 @@ PutGroupInfo (DWORD offset, BYTE which_group)
 			month_index = 0;
 			day_index = 7;
 			year_index = 0;
-			ValidateEvent (RELATIVE_EVENT, &month_index, &day_index, &year_index);
+			ValidateEvent (RELATIVE_EVENT, &month_index, &day_index,
+					&year_index);
 			GH.day_index = (BYTE)day_index;
 			GH.month_index = (BYTE)month_index;
 			GH.year_index = year_index;
 		}
 		GH.star_index = CurStarDescPtr - star_array;
-#ifdef DEBUG
+#ifdef DEBUG_GROUPS
 		fprintf (stderr, "PutGroupInfo(%lu): %u out of %u -- %u/%u/%u\n",
-offset, which_group, GH.NumGroups,
-GH.month_index, GH.day_index, GH.year_index);
-#endif /* DEBUG */
+				offset, which_group, GH.NumGroups,
+				GH.month_index, GH.day_index, GH.year_index);
+#endif /* DEBUG_GROUPS */
 
 		FlushGroupInfo (&GH, offset, which_group, fp);
 
@@ -672,3 +665,4 @@ GH.month_index, GH.day_index, GH.year_index);
 	return (offset);
 }
 
+
