@@ -16,8 +16,22 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "starcon.h"
+#include "hyper.h"
+
+#include "build.h"
+#include "collide.h"
+#include "colors.h"
+#include "controls.h"
+#include "encount.h"
+#include "globdata.h"
+#include "init.h"
+#include "nameref.h"
+#include "resinst.h"
+#include "setup.h"
+#include "sounds.h"
 #include "libs/graphics/gfx_common.h"
+#include "libs/mathlib.h"
+
 
 //Added by Chris
 
@@ -25,6 +39,9 @@ void WaitForNoInput (SIZE Duration);
 
 //End Added by Chris
 
+static void LoadHyperData (void);
+
+
 #define XOFFS ((RADAR_SCAN_WIDTH + (UNIT_SCREEN_WIDTH << 2)) >> 1)
 #define YOFFS ((RADAR_SCAN_HEIGHT + (UNIT_SCREEN_HEIGHT << 2)) >> 1)
 
@@ -180,11 +197,7 @@ check_hyperspace_encounter (void)
 					if (!GET_GAME_STATE (STARBASE_AVAILABLE))
 						percent = 100;
 					else
-#ifdef TESTING
-percent = 0;
-#else /* TESTING */
 						percent *= GET_GAME_STATE (SLYLANDRO_MULTIPLIER);
-#endif /* TESTING */
 				}
 				else if (Type == MELNORME_SHIP
 						&& (GLOBAL_SIS (FuelOnBoard) == 0
@@ -229,14 +242,7 @@ percent = 0;
 					(DWORD)encounter_radius * encounter_radius
 					&& ((COUNT)TFB_Random () % 100) < percent)
 			{
-#ifndef DEBUG
-				char buf[20];
-
-				GetStringContents (SetAbsStringTableIndex (
-						TemplatePtr->ShipInfo.race_strings, 1
-						), (STRINGPTR)buf, FALSE);
-//				fprintf (stderr, "%s encounter\n", buf);
-#endif /* DEBUG */
+				// Ship spawned for encounter.
 				hEncounter = AllocEncounter ();
 				if (hEncounter)
 				{
@@ -278,22 +284,19 @@ FreeHyperData (void)
 	hypercmaps[1] = 0;
 }
 
-void
+static void
 LoadHyperData (void)
 {
 	if (hyperstars[0] == 0)
 	{
 		hyperstars[0] = CaptureDrawable (
-				LoadGraphic (AMBIENT_MASK_PMAP_ANIM)
-				);
+				LoadGraphic (AMBIENT_MASK_PMAP_ANIM));
 		hyperstars[1] = CaptureDrawable (
-				LoadGraphic (HYPERSTARS_MASK_PMAP_ANIM)
-				);
+				LoadGraphic (HYPERSTARS_MASK_PMAP_ANIM));
 		hypercmaps[0] = CaptureColorMap (LoadColorMap (HYPER_COLOR_TAB));
 
 		hyperstars[2] = CaptureDrawable (
-				LoadGraphic (ARISPACE_MASK_PMAP_ANIM)
-				);
+				LoadGraphic (ARISPACE_MASK_PMAP_ANIM));
 		hypercmaps[1] = CaptureColorMap (LoadColorMap (ARISPACE_COLOR_TAB));
 	}
 }
@@ -322,9 +325,9 @@ LoadHyperspace (void)
 	{
 		if (LOBYTE (LastActivity) == 0)
 		{
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			DrawSISFrame ();
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 		}
 		else
 		{
@@ -360,9 +363,9 @@ LoadHyperspace (void)
 BOOLEAN
 FreeHyperspace (void)
 {
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 	SuspendGameClock ();
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 
 	{
 		FRAME F;
@@ -443,14 +446,15 @@ unhyper_transition (PELEMENT ElementPtr)
 	{
 		cleanup_hyperspace ();
 
-		GLOBAL (ShipStamp.frame) = 0;
 		GLOBAL (CurrentActivity) &= ~IN_BATTLE;
 		switch (ElementPtr->turn_wait)
 		{
 			case RANDOM_ENCOUNTER_TRANSITION:
+				SaveFlagshipState ();
 				GLOBAL (CurrentActivity) |= START_ENCOUNTER;
 				break;
 			case INTERPLANETARY_TRANSITION:
+				GLOBAL (ShipStamp.frame) = 0;
 				SET_GAME_STATE (USED_BROADCASTER, 0);
 				if (GET_GAME_STATE (ARILOU_SPACE_SIDE) <= 1)
 				{
@@ -509,6 +513,7 @@ unhyper_transition (PELEMENT ElementPtr)
 				 * or HyperSpace from QuasiSpace through the periodically
 				 * opening portal.
 				 */
+				GLOBAL (ShipStamp.frame) = 0;
 				SET_GAME_STATE (USED_BROADCASTER, 0);
 				GLOBAL (autopilot.x) =
 						GLOBAL (autopilot.y) = ~0;
@@ -964,17 +969,12 @@ AddEncounterElement (ENCOUNTERPTR EncounterPtr,
 			HSTARSHIP hStarShip;
 			SHIP_FRAGMENTPTR TemplatePtr;
 
-			hStarShip = GetStarShipFromIndex (
-					&GLOBAL (avail_race_q), Type
-					);
+			hStarShip = GetStarShipFromIndex (&GLOBAL (avail_race_q), Type);
 			TemplatePtr = (SHIP_FRAGMENTPTR)LockStarShip (
-					&GLOBAL (avail_race_q), hStarShip
-					);
+					&GLOBAL (avail_race_q), hStarShip);
 			EncounterPtr->SD.ShipList[i] = TemplatePtr->ShipInfo;
 			EncounterPtr->SD.ShipList[i].var1 = Type;
-			UnlockStarShip (
-					&GLOBAL (avail_race_q), hStarShip
-					);
+			UnlockStarShip (&GLOBAL (avail_race_q), hStarShip);
 		}
 
 
@@ -1025,10 +1025,8 @@ AddEncounterElement (ENCOUNTERPTR EncounterPtr,
 			if (i == 0 || i > NUM_VORTEX_TRANSITIONS)
 				i = NUM_VORTEX_TRANSITIONS;
 
-			ElementPtr->current.image.frame =
-					SetRelFrameIndex (
-					ElementPtr->current.image.farray[0], -i
-					);
+			ElementPtr->current.image.frame = SetRelFrameIndex (
+					ElementPtr->current.image.farray[0], -i);
 			ElementPtr->death_func = encounter_transition;
 		}
 		else
@@ -1067,7 +1065,8 @@ DrawHyperGrid (COORD ux, COORD uy, COORD ox,
 	RECT r;
 
 	ClearDrawable ();
-	SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x10, 0x00), 0x6B));
+	SetContextForeGroundColor (
+			BUILD_COLOR (MAKE_RGB15 (0x00, 0x10, 0x00), 0x6B));
 
 	if ((sx = ux - (RADAR_SCAN_WIDTH >> 1)) < 0)
 		sx = 0;
@@ -1083,22 +1082,20 @@ DrawHyperGrid (COORD ux, COORD uy, COORD ox,
 	if ((ey = uy + (RADAR_SCAN_HEIGHT >> 1)) > MAX_Y_UNIVERSE + 1)
 		ey = MAX_Y_UNIVERSE + 1;
 
-	r.corner.y = (COORD)((long)(MAX_Y_UNIVERSE - ey)
+	r.corner.y = (COORD) ((long)(MAX_Y_UNIVERSE - ey)
 			* RADAR_HEIGHT / RADAR_SCAN_HEIGHT) - oy;
 	r.extent.width = 1;
-	r.extent.height = ((COORD)((long)(MAX_Y_UNIVERSE - sy)
-			* RADAR_HEIGHT / RADAR_SCAN_HEIGHT) - oy)
-			- r.corner.y + 1;
+	r.extent.height = ((COORD) ((long)(MAX_Y_UNIVERSE - sy)
+			* RADAR_HEIGHT / RADAR_SCAN_HEIGHT) - oy) - r.corner.y + 1;
 	for (ux = sx; ux <= ex; ux += GRID_OFFSET)
 	{
-		r.corner.x = (COORD)((long)ux
+		r.corner.x = (COORD) ((long)ux
 				* RADAR_WIDTH / RADAR_SCAN_WIDTH) - ox;
 		DrawFilledRectangle (&r);
 	} while ((ux += GRID_OFFSET) < ex);
 
-	r.corner.x = (COORD)((long)sx
-			* RADAR_WIDTH / RADAR_SCAN_WIDTH) - ox;
-	r.extent.width = ((COORD)((long)ex
+	r.corner.x = (COORD) ((long)sx * RADAR_WIDTH / RADAR_SCAN_WIDTH) - ox;
+	r.extent.width = ((COORD) ((long)ex
 			* RADAR_WIDTH / RADAR_SCAN_WIDTH) - ox)
 			- r.corner.x + 1;
 	r.extent.height = 1;
@@ -1129,9 +1126,7 @@ ProcessEncounters (PPOINT puniverse, COORD ox,
 		hNextEncounter = GetSuccEncounter (EncounterPtr);
 
 		if (EncounterPtr->hElement == 0
-				&& AddEncounterElement (
-				EncounterPtr, puniverse
-				) == 0)
+				&& AddEncounterElement (EncounterPtr, puniverse) == 0)
 		{
 DeleteEncounter:
 			UnlockEncounter (hEncounter);
@@ -1181,13 +1176,11 @@ DeleteEncounter:
 					COUNT cur_facing, delta_facing;
 
 					cur_facing = ANGLE_TO_FACING (
-							GetVelocityTravelAngle (&ElementPtr->velocity)
-							);
+							GetVelocityTravelAngle (&ElementPtr->velocity));
 					delta_facing = NORMALIZE_FACING (cur_facing
 							- ANGLE_TO_FACING (ARCTAN (
 							puniverse->x - EncounterPtr->SD.star_pt.x,
-							puniverse->y - EncounterPtr->SD.star_pt.y
-							)));
+							puniverse->y - EncounterPtr->SD.star_pt.y)));
 					if (delta_facing || (delta_x == 0 && delta_y == 0))
 					{
 						SIZE s, RaceHyperSpeed[] =
@@ -1601,7 +1594,7 @@ UnbatchGraphics ();
 	OldContext = SetContext (SpaceContext);
 	OldColor = SetContextBackGroundColor (BLACK_COLOR);
 
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 	SuspendGameClock ();
 
 	memset ((PMENU_STATE)&MenuState, 0, sizeof (MenuState));
@@ -1610,13 +1603,14 @@ UnbatchGraphics ();
 	MenuState.CurState = STARMAP + 1;
 
 	DrawMenuStateStrings (PM_STARMAP, STARMAP);
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	SetFlashRect ((PRECT)~0L, (FRAME)0);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
+	SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
 	DoInput ((PVOID)&MenuState, TRUE);
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	SetFlashRect (NULL_PTR, (FRAME)0);
 
 	SetContext (SpaceContext);
@@ -1624,10 +1618,10 @@ UnbatchGraphics ();
 	if (!(GLOBAL (CurrentActivity) & (CHECK_ABORT | CHECK_LOAD)))
 	{
 		ClearSISRect (CLEAR_SIS_RADAR);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 		WaitForNoInput (ONE_SECOND / 2);
 		ResumeGameClock ();
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 	}
 
 	SetContextBackGroundColor (OldColor);
