@@ -16,8 +16,14 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "starcon.h"
+#include "intel.h"
+
 #include "controls.h"
+#include "globdata.h"
+#include "setup.h"
+
+#include <stdio.h>
+
 
 SIZE cur_player;
 
@@ -25,13 +31,20 @@ BATTLE_INPUT_STATE
 computer_intelligence (void)
 {
 	BATTLE_INPUT_STATE InputState;
+
 	if (LOBYTE (GLOBAL (CurrentActivity)) == IN_LAST_BATTLE)
-		return (0);
+		return 0;
 
 	if (CyborgDescPtr)
 	{
 		if (PlayerControl[cur_player] & CYBORG_CONTROL)
+		{
 			InputState = tactical_intelligence ();
+
+			// allow a player to warp-escape in cyborg mode
+			if (cur_player == 0)
+				InputState |= (*(HumanInput[cur_player])) () & BATTLE_ESCAPE;
+		}
 		else
 			InputState = (*(HumanInput[cur_player])) ();
 	}
@@ -47,8 +60,15 @@ computer_intelligence (void)
 				InputState = BATTLE_WEAPON; /* pick a random ship */
 				break;
 			}
+			default:
+				// Should not happen. Satisfying compiler.
+				fprintf (stderr, "Warning: Unexpected state in "
+						"computer_intelligence().");
+				InputState = 0;
+				break;
 		}
 	}
-	return (InputState);
+	return InputState;
 }
 
+
