@@ -1,9 +1,12 @@
-#include "SDL.h"
+#include "port.h"
+#include SDL_INCLUDE(SDL.h)
 #include "sdl_common.h"
 #include "libs/graphics/gfx_common.h"
 #include "libs/graphics/sdl/primitives.h"
 #include "libs/graphics/tfb_draw.h"
+#include "rotozoom.h"
 #include "options.h"
+#include "types.h"
 
 typedef SDL_Surface *NativeCanvas;
 
@@ -147,7 +150,8 @@ TFB_DrawCanvas_FilledImage (TFB_Image *img, int x, int y, int scale, int r, int
 	UnlockMutex (img->mutex);
 }
 
-TFB_Canvas TFB_DrawCanvas_New_TrueColor (int w, int h, BOOLEAN hasalpha)
+TFB_Canvas
+TFB_DrawCanvas_New_TrueColor (int w, int h, BOOLEAN hasalpha)
 {
 	SDL_Surface *new_surf;
 	new_surf = SDL_CreateRGBSurface (SDL_SWSURFACE, w, h, 32, 0x000000ff, 0x0000ff00, 0x00ff0000, hasalpha ? 0xff000000 : 0);
@@ -158,6 +162,37 @@ TFB_Canvas TFB_DrawCanvas_New_TrueColor (int w, int h, BOOLEAN hasalpha)
 	return new_surf;
 }
 
+TFB_Canvas
+TFB_DrawCanvas_New_ForScreen (int w, int h, BOOLEAN withalpha)
+{
+	SDL_Surface *new_surf;
+	SDL_PixelFormat* fmt = SDL_Screen->format;
+
+	if (fmt->palette)
+	{
+		fprintf(stderr, "TFB_DrawCanvas_New_ForScreen() WARNING:"
+				"Paletted display format will be slow");
+
+		new_surf = SDL_CreateRGBSurface (SDL_SWSURFACE, w, h,
+				32, 0x000000ff, 0x0000ff00, 0x00ff0000,
+				withalpha ? 0xff000000 : 0);
+	}
+	else
+	{
+		new_surf = SDL_CreateRGBSurface (SDL_SWSURFACE, w, h,
+				fmt->BitsPerPixel, fmt->Rmask, fmt->Gmask, fmt->Bmask,
+				withalpha ? fmt->Amask : 0);
+	}
+
+	if (!new_surf)
+	{
+		fprintf(stderr, "TFB_DrawCanvas_New_ForScreen() INTERNAL PANIC:"
+				"Failed to create TFB_Canvas: %s", SDL_GetError());
+		exit (-1);
+	}
+	return new_surf;
+}
+
 TFB_Canvas
 TFB_DrawCanvas_New_Paletted (int w, int h, TFB_Palette *palette, int transparent_index)
 {
@@ -207,6 +242,12 @@ TFB_DrawCanvas_New_ScaleTarget (TFB_Canvas canvas, TFB_Canvas oldcanvas, int typ
 						src->format->Amask);
 			if (src->format->palette)
 				TFB_DrawCanvas_SetTransparentIndex (newsurf, TFB_DrawCanvas_GetTransparentIndex (src), FALSE);
+			else
+			{
+				int r, g, b;
+				if (TFB_DrawCanvas_GetTransparentColor (src, &r, &g, &b))
+					TFB_DrawCanvas_SetTransparentColor (newsurf, r, g, b, FALSE);
+			}
 		}
 	}
 	else
@@ -226,6 +267,40 @@ TFB_DrawCanvas_New_ScaleTarget (TFB_Canvas canvas, TFB_Canvas oldcanvas, int typ
 	return newsurf;
 }
 
+TFB_Canvas
+TFB_DrawCanvas_New_RotationTarget (TFB_Canvas src_canvas, int angle)
+{
+	SDL_Surface *src = (SDL_Surface *)src_canvas;
+	SDL_Surface *newsurf;
+	EXTENT size;
+
+	TFB_DrawCanvas_GetRotatedExtent (src_canvas, angle, &size);
+	
+	newsurf = SDL_CreateRGBSurface (SDL_SWSURFACE,
+				size.width, size.height,
+				src->format->BitsPerPixel,
+				src->format->Rmask,
+				src->format->Gmask,
+				src->format->Bmask,
+				src->format->Amask);
+	if (!newsurf)
+	{
+		fprintf(stderr, "TFB_DrawCanvas_New_RotationTarget() INTERNAL PANIC:"
+				"Failed to create TFB_Canvas: %s", SDL_GetError());
+		exit (-1);
+	}
+	if (src->format->palette)
+		TFB_DrawCanvas_SetTransparentIndex (newsurf, TFB_DrawCanvas_GetTransparentIndex (src), FALSE);
+	else
+	{
+		int r, g, b;
+		if (TFB_DrawCanvas_GetTransparentColor (src, &r, &g, &b))
+			TFB_DrawCanvas_SetTransparentColor (newsurf, r, g, b, FALSE);
+	}
+	
+	return newsurf;
+}
+
 void
 TFB_DrawCanvas_Delete (TFB_Canvas canvas)
 {
@@ -324,6 +399,23 @@ TFB_DrawCanvas_SetTransparentIndex (TFB_Canvas canvas, int index, BOOLEAN rleacc
 	}		
 }
 
+BOOLEAN
+TFB_DrawCanvas_GetTransparentColor (TFB_Canvas canvas, int *r, int *g, int *b)
+{
+	if (!TFB_DrawCanvas_IsPaletted (canvas)
+			&& (((SDL_Surface *)canvas)->flags & SDL_SRCCOLORKEY) )
+	{
+		Uint8 ur, ug, ub;
+		int colorkey = ((SDL_Surface *)canvas)->format->colorkey;
+		SDL_GetRGB (colorkey, ((SDL_Surface *)canvas)->format, &ur, &ug, &ub);
+		*r = ur;
+		*g = ug;
+		*b = ub;
+		return TRUE;
+	}
+	return FALSE;
+}
+
 void
 TFB_DrawCanvas_SetTransparentColor (TFB_Canvas canvas, int r, int g, int b, BOOLEAN rleaccel)
 {
@@ -346,8 +438,8 @@ TFB_DrawCanvas_GetScaledExtent (TFB_Canvas src_canvas, TFB_Canvas src_mipmap, in
 	
 	if (!src_mipmap)
 	{
-		size->width = (int) (src->w * scale / (float)GSCALE_IDENTITY);
-		size->height = (int) (src->h * scale / (float)GSCALE_IDENTITY);
+		size->width = (int) (src->w * scale / (float)GSCALE_IDENTITY + 0.5);
+		size->height = (int) (src->h * scale / (float)GSCALE_IDENTITY + 0.5);
 	}
 	else
 	{
@@ -361,8 +453,8 @@ TFB_DrawCanvas_GetScaledExtent (TFB_Canvas src_canvas, TFB_Canvas src_mipmap, in
 		else if (ratio > 1.0f)
 			ratio = 1.0f;
 
-		size->width = (int)((src->w - mipmap->w) * ratio + mipmap->w);
-		size->height = (int)((src->h - mipmap->h) * ratio + mipmap->h);
+		size->width = (int)((src->w - mipmap->w) * ratio + mipmap->w + 0.5);
+		size->height = (int)((src->h - mipmap->h) * ratio + mipmap->h + 0.5);
 	}
 		
 	if (!size->width && src->w)
@@ -371,17 +463,28 @@ TFB_DrawCanvas_GetScaledExtent (TFB_Canvas src_canvas, TFB_Canvas src_mipmap, in
 		size->height = 1;
 }
 
+void
+TFB_DrawCanvas_GetExtent (TFB_Canvas canvas, PEXTENT size)
+{
+	SDL_Surface *src = (SDL_Surface *)canvas;
+
+	size->width = src->w;
+	size->height = src->h;
+}
+
 void
 TFB_DrawCanvas_Rescale_Nearest (TFB_Canvas src_canvas, TFB_Canvas dest_canvas, EXTENT size)
 {
+#define NNS_MAX_DIMS 600	
 	SDL_Surface *src = (SDL_Surface *)src_canvas;
 	SDL_Surface *dst = (SDL_Surface *)dest_canvas;
 	int x, y, sx, sy, *csax, *csay, csx, csy;
-	int sax[321], say[241];
+	int saspace[NNS_MAX_DIMS];
+	int *sax, *say;
 
-	if (size.width > 320 || size.height > 240)
+	if (size.width + size.height > NNS_MAX_DIMS)
 	{
-		fprintf (stderr, "TFB_DrawCanvas_Scale: Tried to zoom an image to be larger than the screen!  Failing.\n");
+		fprintf (stderr, "TFB_DrawCanvas_Scale: Tried to zoom an image to unreasonable size!  Failing.\n");
 		return;
 	}
 	if (size.width > dst->w || size.height > dst->h) 
@@ -391,23 +494,28 @@ TFB_DrawCanvas_Rescale_Nearest (TFB_Canvas src_canvas, TFB_Canvas dest_canvas, E
 		return;
 	}
 
-	sx = (int) (65536.0 * (float) src->w / (float) size.width);
-	sy = (int) (65536.0 * (float) src->h / (float) size.height);
+	sx = sy = 0;
+	if (size.width  > 1)
+		sx = 65536 * (src->w - 1) / (size.width  - 1);
+	if (size.height > 1)
+		sy = 65536 * (src->h - 1) / (size.height - 1);
 
+	sax = saspace;
+	say = saspace + size.width;
 	/*
 	 * Precalculate row increments 
+	 * We start with a value in 0..0.5 range to shift the bigger
+	 * jumps towards the center of the image
 	 */
-	csx = 0;
 	csax = sax;
-	for (x = 0; x <= size.width; x++) {
+	for (x = 0, csx = 0x6000; x < size.width; x++) {
 		*csax = csx >> 16;
 		csax++;
 		csx &= 0xffff;
 		csx += sx;
 	}
-	csy = 0;
 	csay = say;
-	for (y = 0; y <= size.height; y++) {
+	for (y = 0, csy = 0x6000; y < size.height; y++) {
 		*csay = csy >> 16;
 		csay++;
 		csy &= 0xffff;
@@ -426,17 +534,17 @@ TFB_DrawCanvas_Rescale_Nearest (TFB_Canvas src_canvas, TFB_Canvas dest_canvas, E
 
 		csay = say;
 		for (y = 0; y < size.height; ++y) {
+			csp += (*csay) * src->pitch;
 			sp = csp;
 			dp = cdp;
 			csax = sax;
 			for (x = 0; x < size.width; ++x) {
+				sp += *csax;
 				*dp = *sp;
 				++csax;
-				sp += *csax;
 				++dp;
 			}
 			++csay;
-			csp += (*csay) * src->pitch;
 			cdp += dst->pitch;
 		}
 	}	
@@ -453,17 +561,17 @@ TFB_DrawCanvas_Rescale_Nearest (TFB_Canvas src_canvas, TFB_Canvas dest_canvas, E
 
 		csay = say;
 		for (y = 0; y < size.height; ++y) {
+			csp += (*csay) * sgap;
 			sp = csp;
 			dp = cdp;
 			csax = sax;
 			for (x = 0; x < size.width; ++x) {
+				sp += *csax;
 				*dp = *sp;
 				++csax;
-				sp += *csax;
 				++dp;
 			}
 			++csay;
-			csp += (*csay) * sgap;
 			cdp += dgap;
 		}
 	}
@@ -701,6 +809,14 @@ TFB_DrawCanvas_Rescale_Trilinear (TFB_Canvas src_canvas, TFB_Canvas dest_canvas,
 		int fsy1 = (int)(65536.0f * (float)mipmap->h / h);
 		int sx0 = 0, sy0 = 0, sx1 = 0, sy1 = 0;
 		int x, y;
+		Uint32 ck0 = 0; // 0 means alpha=0 too, so it's safe
+		Uint32 ck1 = 0;
+
+		// use colorkeys where appropriate
+		if (src->flags & SDL_SRCCOLORKEY)
+			ck0 = src->format->colorkey;
+		if (mipmap->flags & SDL_SRCCOLORKEY)
+			ck1 = mipmap->format->colorkey;
 
 		SDL_LockSurface(src);
 		SDL_LockSurface(dst);
@@ -736,10 +852,19 @@ TFB_DrawCanvas_Rescale_Trilinear (TFB_Canvas src_canvas, TFB_Canvas dest_canvas,
 				{						
 					SDL_GetRGBA (*src_p0, src->format, &r0[0], &g0[0], &b0[0], &a0[0]);
 					SDL_GetRGBA (*(src_p0 + 1), src->format, &r0[1], &g0[1], &b0[1], &a0[1]);
+					if (src_p0[0] == ck0)
+						a0[0] = 0;
+					if (src_p0[1] == ck0)
+						a0[1] = 0;
+
 					if ((sy0 >> 16) <= src->h - 2)
 					{
 						SDL_GetRGBA (*(src_p0 + src->w), src->format, &r0[2], &g0[2], &b0[2], &a0[2]);
 						SDL_GetRGBA (*(src_p0 + src->w + 1), src->format, &r0[3], &g0[3], &b0[3], &a0[3]);
+						if (src_p0[src->w] == ck0)
+							a0[2] = 0;
+						if (src_p0[src->w + 1] == ck0)
+							a0[3] = 0;
 					}
 					else
 					{
@@ -756,6 +881,8 @@ TFB_DrawCanvas_Rescale_Trilinear (TFB_Canvas src_canvas, TFB_Canvas dest_canvas,
 				else
 				{
 					SDL_GetRGBA (*src_p0, src->format, &r0[0], &g0[0], &b0[0], &a0[0]);
+					if (src_p0[0] == ck0)
+						a0[0] = 0;
 					r0[1] = r0[0];
 					g0[1] = g0[0];
 					b0[1] = b0[0];
@@ -763,6 +890,8 @@ TFB_DrawCanvas_Rescale_Trilinear (TFB_Canvas src_canvas, TFB_Canvas dest_canvas,
 					if ((sy0 >> 16) <= src->h - 2)
 					{
 						SDL_GetRGBA (*(src_p0 + src->w), src->format, &r0[2], &g0[2], &b0[2], &a0[2]);
+						if (src_p0[src->w] == ck0)
+							a0[2] = 0;
 						r0[3] = r0[2];
 						g0[3] = g0[2];
 						b0[3] = b0[2];
@@ -781,10 +910,19 @@ TFB_DrawCanvas_Rescale_Trilinear (TFB_Canvas src_canvas, TFB_Canvas dest_canvas,
 				{						
 					SDL_GetRGBA (*src_p1, mipmap->format, &r1[0], &g1[0], &b1[0], &a1[0]);
 					SDL_GetRGBA (*(src_p1 + 1), mipmap->format, &r1[1], &g1[1], &b1[1], &a1[1]);
+					if (src_p1[0] == ck1)
+						a1[0] = 0;
+					if (src_p1[1] == ck1)
+						a1[1] = 0;
+
 					if ((sy1 >> 16) <= mipmap->h - 2)
 					{
 						SDL_GetRGBA (*(src_p1 + mipmap->w) , mipmap->format, &r1[2], &g1[2], &b1[2], &a1[2]);
 						SDL_GetRGBA (*(src_p1 + mipmap->w + 1), mipmap->format, &r1[3], &g1[3], &b1[3], &a1[3]);
+						if (src_p1[mipmap->w] == ck1)
+							a1[2] = 0;
+						if (src_p1[mipmap->w + 1] == ck1)
+							a1[3] = 0;
 					}
 					else
 					{
@@ -801,6 +939,8 @@ TFB_DrawCanvas_Rescale_Trilinear (TFB_Canvas src_canvas, TFB_Canvas dest_canvas,
 				else
 				{
 					SDL_GetRGBA (*src_p1, mipmap->format, &r1[0], &g1[0], &b1[0], &a1[0]);
+					if (src_p1[0] == ck1)
+						a1[0] = 0;
 					r1[1] = r1[0];
 					g1[1] = g1[0];
 					b1[1] = b1[0];
@@ -808,6 +948,8 @@ TFB_DrawCanvas_Rescale_Trilinear (TFB_Canvas src_canvas, TFB_Canvas dest_canvas,
 					if ((sy1 >> 16) <= mipmap->h - 2)
 					{
 						SDL_GetRGBA (*(src_p1 + mipmap->w), mipmap->format, &r1[2], &g1[2], &b1[2], &a1[2]);
+						if (src_p1[mipmap->w] == ck1)
+							a1[2] = 0;
 						r1[3] = r1[2];
 						g1[3] = g1[2];
 						b1[3] = b1[2];
@@ -864,3 +1006,92 @@ TFB_DrawCanvas_Rescale_Trilinear (TFB_Canvas src_canvas, TFB_Canvas dest_canvas,
 			src->format->BitsPerPixel, dst->format->BitsPerPixel, mipmap->format->BitsPerPixel);
 	}
 }
+
+void
+TFB_DrawCanvas_GetScreenFormat (TFB_PixelFormat *fmt)
+{
+	SDL_PixelFormat *sdl = SDL_Screen->format;
+	
+	if (sdl->palette)
+	{
+		fprintf(stderr, "TFB_DrawCanvas_GetScreenFormat() WARNING:"
+				"Paletted display format will be slow");
+
+		fmt->BitsPerPixel = 32;
+		fmt->Rmask = 0x000000ff;
+		fmt->Gmask = 0x0000ff00;
+		fmt->Bmask = 0x00ff0000;
+		fmt->Amask = 0xff000000;
+	}
+	else
+	{
+		fmt->BitsPerPixel = sdl->BitsPerPixel;
+		fmt->Rmask = sdl->Rmask;
+		fmt->Gmask = sdl->Gmask;
+		fmt->Bmask = sdl->Bmask;
+		fmt->Amask = sdl->Amask;
+	}
+}
+
+void*
+TFB_DrawCanvas_GetLine (TFB_Canvas canvas, int line)
+{
+	SDL_Surface* surf = (SDL_Surface *)canvas;
+	return (uint8*)surf->pixels + surf->pitch * line;
+}
+
+void
+TFB_DrawCanvas_GetPixel (TFB_Canvas canvas, int x, int y, int *r, int *g, int *b)
+{
+	SDL_Surface* surf = (SDL_Surface *)canvas;
+	Uint8 ur, ug, ub;
+	Uint32 pixel;
+	GetPixelFn getpixel;
+
+	getpixel = getpixel_for(surf);
+	pixel = (*getpixel)(surf, x, y);
+	SDL_GetRGB (pixel, surf->format, &ur, &ug, &ub);
+	*r = ur;
+	*g = ug;
+	*b = ub;
+}
+
+void
+TFB_DrawCanvas_Rotate (TFB_Canvas src_canvas, TFB_Canvas dst_canvas, int angle, EXTENT size)
+{
+	SDL_Surface *src = (SDL_Surface *)src_canvas;
+	SDL_Surface *dst = (SDL_Surface *)dst_canvas;
+	int ret;
+	int r, g, b;
+
+	if (size.width > dst->w || size.height > dst->h) 
+	{
+		fprintf (stderr, "TFB_DrawCanvas_Rotate: Tried to rotate image to size %d %d when dst_canvas has only dimensions of %d %d!  Failing.\n",
+			size.width, size.height, dst->w, dst->h);
+		return;
+	}
+
+	if (TFB_DrawCanvas_GetTransparentColor (src, &r, &g, &b))
+	{
+		TFB_DrawCanvas_SetTransparentColor (dst, r, g, b, FALSE);
+		/* fill destination with transparent color before rotating */
+		SDL_FillRect(dst, NULL, SDL_MapRGB(dst->format, r, g, b));
+	}
+
+	ret = rotateSurface (src, dst, angle, 0);
+	if (ret != 0)
+	{
+		fprintf (stderr, "TFB_DrawCanvas_Rotate: WARNING: actual rotation func returned failure\n");
+	}
+}
+
+void
+TFB_DrawCanvas_GetRotatedExtent (TFB_Canvas src_canvas, int angle, PEXTENT size)
+{
+	int dstw, dsth;
+	SDL_Surface *src = (SDL_Surface *)src_canvas;
+	
+	rotozoomSurfaceSize (src->w, src->h, angle, 1, &dstw, &dsth);
+	size->height = dsth;
+	size->width = dstw;
+}
