@@ -22,6 +22,9 @@
 #ifndef MISC_H
 #define MISC_H
 
+#include <sys/types.h>
+
+
 extern void *HMalloc (int size);
 extern void HFree (void *p);
 extern void *HCalloc (int size);
@@ -29,8 +32,5 @@ extern void *HRealloc (void *p, int size);
 
 extern int TFB_DEBUG_HALT;
 
-// from misc.c:
-size_t vstrncpy (char *dest, size_t destLen, ...);
-
 #endif
 
