@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: mikmod.h,v 1.2 2003/07/08 18:17:40 meep-eep Exp $
+  $Id: mikmod.h,v 1.3 2005/04/23 23:57:17 meep-eep Exp $
 
   MikMod sound library include file
 
@@ -525,7 +525,8 @@ MIKMODAPI extern struct MLOADER load_xm;  /* FastTracker 2 (by Triton) */
 
 /* SDL_RWops compatability */
 #ifdef USE_RWOPS
-#include "SDL_rwops.h"
+#include "port.h"
+#include SDL_INCLUDE(SDL_rwops.h)
 MIKMODAPI extern MODULE* Player_LoadRW(SDL_RWops*,int,BOOL);
 #endif /* USE_RWOPS */
 /* End SDL_RWops compatability */
