@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: mikmod_internals.h,v 1.3 2003/07/19 01:20:50 meep-eep Exp $
+  $Id: mikmod_internals.h,v 1.4 2005/04/22 01:40:55 meep-eep Exp $
 
   MikMod sound library internal definitions
 
@@ -33,9 +33,9 @@
 #include <malloc.h>
 #endif
 #include <stdarg.h>
-#if defined(__OS2__)||defined(__EMX__)||defined(WIN32)
-#define strcasecmp(s,t) stricmp(s,t)
-#endif
+//#if defined(__OS2__)||defined(__EMX__)||defined(WIN32)
+//#define strcasecmp(s,t) stricmp(s,t)
+//#endif
 
 #include "mikmod_build.h"
 #include "port.h"
