@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: virtch2.c,v 1.1 2003/01/29 19:51:05 avolkov Exp $
+  $Id: virtch2.c,v 1.2 2004/04/07 04:39:17 meep-eep Exp $
 
   High-quality sample mixing routines, using a 32 bits mixing buffer,
   interpolation, and sample smoothing to improve sound quality and remove
@@ -119,7 +119,8 @@ typedef struct VINFO {
 
 static	SWORD **Samples;
 static	VINFO *vinf=NULL,*vnf;
-static	long tickleft,samplesthatfit,vc_memory=0;
+static	unsigned long tickleft,samplesthatfit;
+static	long vc_memory=0;
 static	int vc_softchn;
 static	SLONGLONG idxsize,idxlpos,idxlend;
 static	SLONG *vc_tickbuf=NULL;
@@ -710,7 +711,7 @@ static void AddChannel(SLONG* ptr,NATIVE todo)
 
 void VC2_WriteSamples(SBYTE* buf,ULONG todo)
 {
-	int left,portion=0;
+	unsigned int left,portion=0;
 	SBYTE *buffer;
 	int t,pan,vol;
 
