@@ -14,23 +14,25 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include <assert.h>
-#include <ctype.h>
 #include "sound.h"
 #include "libs/sound/trackplayer.h"
+#include "libs/sound/trackint.h"
 #include "comm.h"
+#include "sis.h"
 #include "options.h"
+#include <assert.h>
+#include <ctype.h>
+#include <stdlib.h>
+
 
 static int tct;               //total number of subtitle tracks
 static int tcur;              //currently playing subtitle track
 static UNICODE *cur_page = 0; //current page of subtitle track 
 static int no_page_break = 0;
 static int track_pos_changed = 0; // set whenever  ff, frev is enabled
-static TFB_SoundTag *last_tag = NULL;
+static TFB_SoundChain *cur_text_chain = NULL;  //current link w/ subbies
 
 #define MAX_CLIPS 50
-#define is_sample_playing(samp) (((samp)->read_chain_ptr && (samp)->play_chain_ptr) ||\
-			(!(samp)->read_chain_ptr && (samp)->decoder))
 static TFB_SoundSample *sound_sample = NULL;
 TFB_SoundChain *first_chain = NULL; //first decoder in linked list
 static TFB_SoundChain *last_chain = NULL;  //last decoder in linked list
@@ -39,6 +41,22 @@ static TFB_SoundChain *last_ts_chain = NULL; //last element in the chain with a
 static Mutex track_mutex; //protects tcur and tct
 void recompute_track_pos (TFB_SoundSample *sample, 
 						  TFB_SoundChain *first_chain, sint32 offset);
+bool is_sample_playing(TFB_SoundSample* samp);
+
+// stream callbacks
+static bool OnTrackStart (TFB_SoundSample* sample);
+static bool OnChunkEnd (TFB_SoundSample* sample, audio_Object buffer);
+static void OnTrackEnd (TFB_SoundSample* sample);
+static void OnTrackTag (TFB_SoundSample* sample, TFB_SoundTag* tag);
+
+static TFB_SoundCallbacks trackCBs =
+{
+	OnTrackStart,
+	OnChunkEnd,
+	OnTrackEnd,
+	OnTrackTag,
+	NULL
+};
 
 //JumpTrack currently aborts the current track. However, it doesn't clear the
 //data-structures as StopTrack does.  this allows for rewind even after the
@@ -47,23 +65,26 @@ void recompute_track_pos (TFB_SoundSample *sample,
 void
 JumpTrack (void)
 {
-	if (sound_sample)
-	{
-		uint32 cur_time;
-		sint32 total_length = (sint32)((last_chain->start_time + 
-				last_chain->decoder->length) * (float)ONE_SECOND);
-		LockMutex (soundSource[SPEECH_SOURCE].stream_mutex);
-		PauseStream (SPEECH_SOURCE);
-		cur_time = GetTimeCounter();
-		soundSource[SPEECH_SOURCE].start_time = 
-				(sint32)cur_time - total_length;
-		track_pos_changed = 1;
-		sound_sample->play_chain_ptr = last_chain;
-		recompute_track_pos(sound_sample, first_chain, total_length);
-		UnlockMutex (soundSource[SPEECH_SOURCE].stream_mutex);
-		PlayingTrack();
+	TFB_SoundChainData* scd;
+	uint32 cur_time;
+	sint32 total_length = (sint32)((last_chain->start_time + 
+			last_chain->decoder->length) * (float)ONE_SECOND);
+
+	if (!sound_sample)
 		return;
-	}
+
+	scd = (TFB_SoundChainData*) sound_sample->data;
+
+	LockMutex (soundSource[SPEECH_SOURCE].stream_mutex);
+	PauseStream (SPEECH_SOURCE);
+	cur_time = GetTimeCounter();
+	soundSource[SPEECH_SOURCE].start_time = 
+			(sint32)cur_time - total_length;
+	track_pos_changed = 1;
+	scd->play_chain_ptr = last_chain;
+	recompute_track_pos(sound_sample, first_chain, total_length + 1);
+	UnlockMutex (soundSource[SPEECH_SOURCE].stream_mutex);
+	PlayingTrack();
 }
 
 //advance to the next track and start playing
@@ -72,7 +93,14 @@ JumpTrack (void)
 void
 PlayTrack (void)
 {
-	if (sound_sample && (sound_sample->read_chain_ptr || sound_sample->decoder))
+	TFB_SoundChainData* scd;
+
+	if (!sound_sample)
+		return;
+
+	scd = (TFB_SoundChainData*) sound_sample->data;
+
+	if (scd->read_chain_ptr || sound_sample->decoder)
 	{
 		LockMutex (soundSource[SPEECH_SOURCE].stream_mutex);
 		PlayStream (sound_sample,
@@ -88,16 +116,23 @@ PlayTrack (void)
 void
 ResumeTrack ()
 {
-	if (sound_sample && (sound_sample->read_chain_ptr || sound_sample->decoder))
+	TFB_SoundChainData* scd;
+
+	if (!sound_sample)
+		return;
+
+	scd = (TFB_SoundChainData*) sound_sample->data;
+
+	if (scd->read_chain_ptr || sound_sample->decoder)
 	{
 		// Only try to start the track if there is something to play
-		TFBSound_IntVal state;
+		audio_IntVal state;
 		BOOLEAN playing;
 
 		LockMutex (soundSource[SPEECH_SOURCE].stream_mutex);
-		TFBSound_GetSourcei (soundSource[SPEECH_SOURCE].handle, TFBSOUND_SOURCE_STATE, &state);
+		audio_GetSourcei (soundSource[SPEECH_SOURCE].handle, audio_SOURCE_STATE, &state);
 		playing = PlayingStream (SPEECH_SOURCE);
-		if (!track_pos_changed && !playing && state == TFBSOUND_PAUSED)
+		if (!track_pos_changed && !playing && state == audio_PAUSED)
 		{
 			/*adjust start time so the slider doesn't go crazy*/
 			soundSource[SPEECH_SOURCE].start_time += GetTimeCounter () - soundSource[SPEECH_SOURCE].pause_time;
@@ -166,9 +201,10 @@ StopTrack ()
 	if (sound_sample)
 	{
 		DestroyMutex (track_mutex);
-		TFBSound_DeleteBuffers (sound_sample->num_buffers, sound_sample->buffer);
+		audio_DeleteBuffers (sound_sample->num_buffers, sound_sample->buffer);
 		HFree (sound_sample->buffer);
 		HFree (sound_sample->buffer_tag);
+		HFree (sound_sample->data);
 		HFree (sound_sample);
 		sound_sample = NULL;
 	}
@@ -177,18 +213,90 @@ StopTrack ()
 	do_subtitles ((void *)~0);
 }
 
-void 
-DoTrackTag (TFB_SoundTag *tag)
+bool is_sample_playing(TFB_SoundSample* sample)
 {
-	if (tag->type == MIX_BUFFER_TAG_TEXT)
+	TFB_SoundChainData* scd = (TFB_SoundChainData*) sample->data;
+
+	return (scd->read_chain_ptr && scd->play_chain_ptr)
+			|| (!scd->read_chain_ptr && sample->decoder);
+}
+
+static void
+DoTrackTag (TFB_SoundChain *chain)
+{
+	LockMutex (track_mutex);
+	if (chain->callback)
+		chain->callback ();
+	tcur = chain->track_num;
+	cur_page = (UNICODE *) chain->text;
+	UnlockMutex (track_mutex);
+}
+
+static bool
+OnTrackStart (TFB_SoundSample* sample)
+{
+	TFB_SoundChainData* scd = (TFB_SoundChainData*) sample->data;
+
+	if (!scd->read_chain_ptr && !sample->decoder)
+		return false;
+
+	if (scd->read_chain_ptr)
 	{
-		LockMutex (track_mutex);
-		if (tag->callback)
-			tag->callback ();
-		tcur = tag->value;
-		cur_page = (UNICODE *)tag->data;
-		UnlockMutex (track_mutex);
+		sample->decoder = scd->read_chain_ptr->decoder;
+		sample->offset = (sint32) (scd->read_chain_ptr->start_time * (float)ONE_SECOND);
 	}
+	else
+		sample->offset = 0;
+
+	scd->play_chain_ptr = scd->read_chain_ptr;
+
+	if (scd->read_chain_ptr && scd->read_chain_ptr->tag_me)
+		DoTrackTag(scd->read_chain_ptr);
+
+	return true;
+}
+
+static bool
+OnChunkEnd (TFB_SoundSample* sample, audio_Object buffer)
+{
+	TFB_SoundChainData* scd = (TFB_SoundChainData*) sample->data;
+
+	if (!scd->read_chain_ptr || !scd->read_chain_ptr->next)
+		return false;
+
+	scd->read_chain_ptr = scd->read_chain_ptr->next;
+	sample->decoder = scd->read_chain_ptr->decoder;
+	SoundDecoder_Rewind (sample->decoder);
+	fprintf (stderr, "Switching to stream %s at pos %d\n",
+			sample->decoder->filename, sample->decoder->start_sample);
+	
+	if (sample->buffer_tag && scd->read_chain_ptr->tag_me)
+	{
+		TFB_TagBuffer (sample, buffer, scd->read_chain_ptr);
+	}
+
+	return true;
+}
+
+static void
+OnTrackEnd (TFB_SoundSample* sample)
+{
+	TFB_SoundChainData* scd = (TFB_SoundChainData*) sample->data;
+
+	sample->decoder = NULL;
+	scd->read_chain_ptr = NULL;
+}
+
+static void
+OnTrackTag (TFB_SoundSample* sample, TFB_SoundTag* tag)
+{
+	TFB_SoundChainData* scd = (TFB_SoundChainData*) sample->data;
+	TFB_SoundChain* chain = (TFB_SoundChain*) tag->data;
+
+	TFB_ClearBufferTag (tag);
+	DoTrackTag (chain);
+
+	scd->play_chain_ptr = scd->read_chain_ptr;
 }
 
 int
@@ -215,7 +323,7 @@ GetTimeStamps(UNICODE *TimeStamps, sint32 *time_stamps)
 	return (num);
 }
 
-#define TEXT_SPEED 70
+#define TEXT_SPEED 80
 UNICODE **
 SplitSubPages (UNICODE *text, sint32 *timestamp, int *num_pages)
 {
@@ -239,6 +347,8 @@ SplitSubPages (UNICODE *text, sint32 *timestamp, int *num_pages)
 				wstrncpy (split_text[page] + ellips, text, pos);
 				wstrcpy (split_text[page] + ellips + pos, "...");
 				timestamp[page] = - pos * TEXT_SPEED;
+				if (timestamp[page] > -1000)
+					timestamp[page] = -1000;
 				ellips = 2;
 				text = text + pos;
 				pos = 0;
@@ -252,6 +362,8 @@ SplitSubPages (UNICODE *text, sint32 *timestamp, int *num_pages)
 				wstrncpy (split_text[page] + ellips, text, pos);
 				*(split_text[page] + ellips + pos) = 0;
 				timestamp[page] = - pos * TEXT_SPEED;
+				if (timestamp[page] > -1000)
+					timestamp[page] = -1000;
 				ellips = 0;
 				text = text + pos;
 				pos = 0;
@@ -276,6 +388,9 @@ SplitSubPages (UNICODE *text, sint32 *timestamp, int *num_pages)
 		wstrncpy (split_text[page] + ellips, text, pos);
 		*(split_text[page] + ellips + pos) = 0;
 		timestamp[page] = - pos * TEXT_SPEED;
+		if (timestamp[page] > -1000)
+			timestamp[page] = -1000;
+		timestamp[page] += -1000;
 		page ++;
 	}
 	*num_pages = page;
@@ -324,7 +439,7 @@ SpliceMultiTrack (UNICODE *TrackNames[], UNICODE *TrackText)
 		{
 			fprintf (stderr, "  track: %s, decoder: %s, rate %d format %x\n",
 					*TrackNames,
-					track_decs[tracks]->decoder_info,
+					SoundDecoder_GetName (track_decs[tracks]),
 					track_decs[tracks]->frequency,
 					track_decs[tracks]->format);
 			SoundDecoder_DecodeAll (track_decs[tracks]);
@@ -353,18 +468,19 @@ SpliceMultiTrack (UNICODE *TrackNames[], UNICODE *TrackText)
 	if (tct)
 	{
 		int slen1;
-		slen1 = wstrlen ((UNICODE *)last_tag->data);
-		last_tag->data = HRealloc ((UNICODE *)last_tag->data, slen1 + slen + 1);
-		wstrcpy (&((UNICODE *)last_tag->data)[slen1], TrackText);
+		slen1 = wstrlen ((UNICODE *)cur_text_chain->text);
+		cur_text_chain->text = HRealloc (
+				(UNICODE *)cur_text_chain->text, slen1 + slen + 1);
+		wstrcpy (&((UNICODE *)cur_text_chain->text)[slen1], TrackText);
 	}
 	else
 	{
-		begin_chain->tag.data = HMalloc (slen + 1);
-		wstrcpy ((UNICODE *)begin_chain->tag.data, TrackText);
-		last_tag = &begin_chain->tag;
+		begin_chain->text = HMalloc (slen + 1);
+		wstrcpy ((UNICODE *)begin_chain->text, TrackText);
+		cur_text_chain = begin_chain;
+		begin_chain->tag_me = 1;
+		begin_chain->track_num = tct;
 		tct++;
-		begin_chain->tag.type = MIX_BUFFER_TAG_TEXT;
-		begin_chain->tag.value = tct - 1;
 	}
 	no_page_break = 1;
 	
@@ -377,7 +493,7 @@ SpliceMultiTrack (UNICODE *TrackNames[], UNICODE *TrackText)
 }
 
 void
-SpliceTrack (UNICODE *TrackName, UNICODE *TrackText, UNICODE *TimeStamp, void (*cb) (void))
+SpliceTrack (UNICODE *TrackName, UNICODE *TrackText, UNICODE *TimeStamp, TFB_TrackCB cb)
 {
 	unsigned long startTime;
 	UNICODE **split_text = NULL;
@@ -392,7 +508,7 @@ SpliceTrack (UNICODE *TrackName, UNICODE *TrackText, UNICODE *TimeStamp, void (*
 				int num_pages = 0, page_counter;
 				sint32 time_stamps[50];
 
-				if (! last_ts_chain || !last_ts_chain->tag.data)
+				if (!last_ts_chain || !last_ts_chain->text)
 				{
 					fprintf (stderr, 
 						"SpliceTrack(): Tried to append a subtitle to a NULL string\n");
@@ -404,11 +520,11 @@ SpliceTrack (UNICODE *TrackName, UNICODE *TrackText, UNICODE *TimeStamp, void (*
 					fprintf (stderr, "SpliceTrack(): Failed to parse sutitles\n");
 					return;
 				}
-				oTT = (UNICODE *)last_ts_chain->tag.data;
+				oTT = (UNICODE *)last_ts_chain->text;
 				slen1 = wstrlen (oTT);
 				slen2 = wstrlen (split_text[0]);
-				last_ts_chain->tag.data = HRealloc (oTT, slen1 + slen2 + 1);
-				wstrcpy (&((UNICODE *)last_ts_chain->tag.data)[slen1], split_text[0]);
+				last_ts_chain->text = HRealloc (oTT, slen1 + slen2 + 1);
+				wstrcpy (&((UNICODE *)last_ts_chain->text)[slen1], split_text[0]);
 				HFree (split_text[0]);
 				for (page_counter = 1; page_counter < num_pages; page_counter++)
 				{
@@ -418,7 +534,7 @@ SpliceTrack (UNICODE *TrackName, UNICODE *TrackText, UNICODE *TimeStamp, void (*
 						break;
 					}
 					last_ts_chain = last_ts_chain->next;
-					last_ts_chain->tag.data = split_text[page_counter];
+					last_ts_chain->text = split_text[page_counter];
 				}
 			}
 		}
@@ -442,11 +558,11 @@ SpliceTrack (UNICODE *TrackName, UNICODE *TrackText, UNICODE *TimeStamp, void (*
 				int slen1, slen2;
 				UNICODE *oTT;
 
-				oTT = (UNICODE *)last_tag->data;
+				oTT = (UNICODE *)cur_text_chain->text;
 				slen1 = wstrlen (oTT);
 				slen2 = wstrlen (split_text[0]);
-				last_tag->data = HRealloc (oTT, slen1 + slen2 + 1);
-				wstrcpy (&((UNICODE *)last_tag->data)[slen1], split_text[0]);
+				cur_text_chain->text = HRealloc (oTT, slen1 + slen2 + 1);
+				wstrcpy (&((UNICODE *)cur_text_chain->text)[slen1], split_text[0]);
 				HFree (split_text[0]);
 			}
 			else
@@ -468,19 +584,24 @@ SpliceTrack (UNICODE *TrackName, UNICODE *TrackText, UNICODE *TimeStamp, void (*
 				if (! sound_sample)
 				{
 					TFB_SoundDecoder *decoder;
-					track_mutex = CreateMutex();
+					TFB_SoundChainData* scd;
+
+					track_mutex = CreateMutex("trackplayer mutex", SYNC_CLASS_TOPLEVEL | SYNC_CLASS_AUDIO);
 					sound_sample = (TFB_SoundSample *) HMalloc (sizeof (TFB_SoundSample));
+					scd = (TFB_SoundChainData *) HCalloc (sizeof (TFB_SoundChainData));
+					sound_sample->data = scd;
+					sound_sample->callbacks = trackCBs;
 					sound_sample->num_buffers = 8;
-					sound_sample->buffer_tag = HMalloc (sizeof (TFB_SoundTag *) * sound_sample->num_buffers);
+					sound_sample->buffer_tag = HCalloc (sizeof (TFB_SoundTag) * sound_sample->num_buffers);
 					sound_sample->buffer = HMalloc (sizeof (uint32) * sound_sample->num_buffers);
-					TFBSound_GenBuffers (sound_sample->num_buffers, sound_sample->buffer);
+					audio_GenBuffers (sound_sample->num_buffers, sound_sample->buffer);
 					decoder = SoundDecoder_Load (contentDir, TrackName, 4096,
 							startTime, time_stamps[page_counter]);
-					sound_sample->read_chain_ptr = create_soundchain (decoder, 0.0);
+					scd->read_chain_ptr = create_soundchain (decoder, 0.0);
 					sound_sample->decoder = decoder;
-					first_chain = last_chain = sound_sample->read_chain_ptr;
+					first_chain = last_chain = scd->read_chain_ptr;
 					sound_sample->length = 0;
-					sound_sample->play_chain_ptr = 0;
+					scd->play_chain_ptr = 0;
 				}
 				else
 				{
@@ -496,7 +617,7 @@ SpliceTrack (UNICODE *TrackName, UNICODE *TrackText, UNICODE *TimeStamp, void (*
 				if (last_chain->decoder)
 				{
 					static float old_volume = 0.0f;
-					if (last_chain->decoder->type == SOUNDDECODER_NULL)
+					if (last_chain->decoder->is_null)
 					{
 						if (speechVolumeScale != 0.0f)
 						{
@@ -518,23 +639,23 @@ SpliceTrack (UNICODE *TrackName, UNICODE *TrackText, UNICODE *TimeStamp, void (*
 					sound_sample->length += last_chain->decoder->length;
 					if (! no_page_break)
 					{
-						last_chain->tag.type = MIX_BUFFER_TAG_TEXT;
+						last_chain->tag_me = 1;
 						// last_chain->tag.value = (void *)(((tct - 1) << 8) | page_counter);
-						last_chain->tag.value = tct - 1;
+						last_chain->track_num = tct - 1;
 						if (page_counter < num_pages)
 						{
-							last_chain->tag.data = (void *)split_text[page_counter];
+							last_chain->text = (void *)split_text[page_counter];
 							last_ts_chain = last_chain;
 						}
-						last_chain->tag.callback = cb;
-						last_tag = &last_chain->tag;
+						last_chain->callback = cb;
+						cur_text_chain = last_chain;
 					}
 					no_page_break = 0;
 				}
 				else
 				{
 					fprintf (stderr, "SpliceTrack(): couldn't load %s\n", TrackName);
-					TFBSound_DeleteBuffers (sound_sample->num_buffers, sound_sample->buffer);
+					audio_DeleteBuffers (sound_sample->num_buffers, sound_sample->buffer);
 					destroy_soundchain (first_chain);
 					first_chain = NULL;
 					HFree (sound_sample->buffer);
@@ -561,27 +682,32 @@ PauseTrack ()
 void
 recompute_track_pos (TFB_SoundSample *sample, TFB_SoundChain *first_chain, sint32 offset)
 {
+	TFB_SoundChainData* scd;
 	TFB_SoundChain *cur_chain = first_chain;
+
 	if (! sample)
 		return;
+
+	scd = (TFB_SoundChainData*) sample->data;
+
 	while (cur_chain->next &&
 			(sint32)(cur_chain->next->start_time * (float)ONE_SECOND) < offset)
 	{
-		if (cur_chain->tag.type)
-			DoTrackTag (&cur_chain->tag);
+		if (cur_chain->tag_me)
+			DoTrackTag (cur_chain);
 		cur_chain = cur_chain->next;
 	}
-	if (cur_chain->tag.type)
-		DoTrackTag (&cur_chain->tag);
+	if (cur_chain->tag_me)
+		DoTrackTag (cur_chain);
 	if ((sint32)((cur_chain->start_time + cur_chain->decoder->length) * (float)ONE_SECOND) < offset)
 	{
-		sample->read_chain_ptr = NULL;
+		scd->read_chain_ptr = NULL;
 		sample->decoder = NULL;
 	}
 	else
 	{
-		sample->read_chain_ptr = cur_chain;
-		SoundDecoder_Seek(sample->read_chain_ptr->decoder,
+		scd->read_chain_ptr = cur_chain;
+		SoundDecoder_Seek(scd->read_chain_ptr->decoder,
 				(uint32)(1000 * (offset / (float)ONE_SECOND - cur_chain->start_time)));
 	}
 }
@@ -623,12 +749,14 @@ FastReverse_Page ()
 {
 	if (sound_sample)
 	{
+		TFB_SoundChainData* scd = (TFB_SoundChainData*) sound_sample->data;
 		TFB_SoundChain *prev_chain;
+
 		LockMutex (soundSource[SPEECH_SOURCE].stream_mutex);
-		prev_chain = get_previous_chain(first_chain, sound_sample->play_chain_ptr);
+		prev_chain = get_previous_chain (first_chain, scd->play_chain_ptr);
 		if (prev_chain)
 		{
-			sound_sample->read_chain_ptr = prev_chain;
+			scd->read_chain_ptr = prev_chain;
 			PlayStream (sound_sample,
 					SPEECH_SOURCE, false,
 					speechVolumeScale != 0.0f, true);
@@ -652,7 +780,7 @@ FastForward_Smooth ()
 		soundSource[SPEECH_SOURCE].start_time -= ACCEL_SCROLL_SPEED;
 		if ((sint32)cur_time - soundSource[SPEECH_SOURCE].start_time > total_length)
 			soundSource[SPEECH_SOURCE].start_time = 
-				(sint32)cur_time - total_length;
+				(sint32)cur_time - total_length - 1;
 		offset = cur_time - soundSource[SPEECH_SOURCE].start_time;
 		track_pos_changed = 1;
 		recompute_track_pos(sound_sample, first_chain, offset);
@@ -667,13 +795,15 @@ FastForward_Page ()
 {
 	if (sound_sample)
 	{
-		TFB_SoundChain *cur_ptr = sound_sample->play_chain_ptr;
+		TFB_SoundChainData* scd = (TFB_SoundChainData*) sound_sample->data;
+		TFB_SoundChain *cur_ptr = scd->play_chain_ptr;
+
 		LockMutex (soundSource[SPEECH_SOURCE].stream_mutex);
-		while (cur_ptr->next && ! cur_ptr->tag.type)
+		while (cur_ptr->next && !cur_ptr->next->tag_me)
 			cur_ptr = cur_ptr->next;
 		if (cur_ptr->next)
 		{
-			sound_sample->read_chain_ptr = cur_ptr->next;
+			scd->read_chain_ptr = cur_ptr->next;
 			PlayStream (sound_sample,
 					SPEECH_SOURCE, false,
 					speechVolumeScale != 0.0f, true);
@@ -712,7 +842,7 @@ GetSoundData (void *data)
 			UBYTE *sbuffer = soundSource[SPEECH_SOURCE].sbuffer;
 
 			assert (soundSource[SPEECH_SOURCE].sample->decoder->frequency == 11025);
-			assert (soundSource[SPEECH_SOURCE].sample->decoder->format == TFBSOUND_FORMAT_MONO16);
+			assert (soundSource[SPEECH_SOURCE].sample->decoder->format == audio_FORMAT_MONO16);
 
 			if (delta < 0)
 			{
@@ -781,7 +911,7 @@ GetSoundData (void *data)
 			UBYTE *sbuffer = soundSource[MUSIC_SOURCE].sbuffer;
 
 			assert (soundSource[MUSIC_SOURCE].sample->decoder->frequency >= 11025);
-			assert (soundSource[MUSIC_SOURCE].sample->decoder->format == TFBSOUND_FORMAT_STEREO16);
+			assert (soundSource[MUSIC_SOURCE].sample->decoder->format == audio_FORMAT_STEREO16);
 
 			step = soundSource[MUSIC_SOURCE].sample->decoder->frequency / 11025 * 16;
 			if (step % 2 == 1)
@@ -861,3 +991,51 @@ GetSoundInfo (int max_len)
 		return max_len;
 	return (int)(max_len * offset / length);
 }
+
+TFB_SoundChain *
+create_soundchain (TFB_SoundDecoder *decoder, float startTime)
+{
+	TFB_SoundChain *chain;
+	chain = (TFB_SoundChain *)HMalloc (sizeof (TFB_SoundChain));
+	chain->decoder = decoder;
+	chain->next = NULL;
+	chain->start_time = startTime;
+	chain->tag_me = 0;
+	chain->text = 0;
+	return (chain);
+}
+
+void
+destroy_soundchain (TFB_SoundChain *chain)
+{
+	while (chain->next)
+	{
+		TFB_SoundChain *tmp_chain = chain->next;
+		chain->next = chain->next->next;
+		if (tmp_chain->decoder)
+			SoundDecoder_Free (tmp_chain->decoder);
+		if (tmp_chain->text)
+			HFree (tmp_chain->text);
+		HFree (tmp_chain);
+	}
+	SoundDecoder_Free (chain->decoder);
+	HFree (chain);
+}
+
+TFB_SoundChain *
+get_previous_chain (TFB_SoundChain *first_chain, TFB_SoundChain *current_chain)
+{
+	TFB_SoundChain *prev_chain, *last_valid = NULL;
+	prev_chain = first_chain;
+	if (prev_chain == current_chain)
+		return (prev_chain);
+	while (prev_chain->next)
+	{
+		if (prev_chain->tag_me)
+			last_valid = prev_chain;
+		if (prev_chain->next == current_chain)
+			return (last_valid);
+		prev_chain = prev_chain->next;
+	}
+	return (first_chain);
+}
