@@ -51,9 +51,9 @@ static inline void uio_PDirHandle_free(uio_PDirHandle *pDirHandle);
 static inline uio_PFileHandle *uio_PFileHandle_alloc(void);
 static inline void uio_PFileHandle_free(uio_PFileHandle *pFileHandle);
 
-static uio_DirHandle *uio_newDirHandle(uio_Repository *repository, char *path,
+static uio_DirHandle *uio_DirHandle_new(uio_Repository *repository, char *path,
 		char *rootEnd);
-static inline uio_DirHandle *uio_allocDirHandle(void);
+static inline uio_DirHandle *uio_DirHandle_alloc(void);
 static inline void uio_DirHandle_free(uio_DirHandle *dirHandle);
 
 static inline uio_Handle *uio_Handle_alloc(void);
@@ -208,7 +208,7 @@ uio_mountDir(uio_Repository *destRep, const char *mountPoint,
 	}
 
 	assert(handler->mount != NULL);
-	pRoot = handler->mount(handle, flags);
+	pRoot = (handler->mount)(handle, flags);
 	if (pRoot == NULL) {
 		int savedErrno;
 
@@ -325,6 +325,11 @@ uio_unmountAllDirs(uio_Repository *repository) {
 	return 0;
 }
 
+uio_FileSystemID
+uio_getMountFileSystemType(uio_MountHandle *mountHandle) {
+	return mountHandle->mountInfo->fsID;
+}
+
 int
 uio_close(uio_Handle *handle) {
 	uio_Handle_unref(handle);
@@ -340,19 +345,19 @@ uio_rename(uio_DirHandle *oldDir, const char *oldPath,
 	int retVal;
 
 	if (uio_getPhysicalAccess(oldDir, oldPath, O_RDONLY, 0,
-			&oldReadMountInfo, &oldPReadDir,
-			NULL, NULL, &oldName) == -1) {
+			&oldReadMountInfo, &oldPReadDir, NULL,
+			NULL, NULL, NULL, &oldName) == -1) {
 		// errno is set
 		return -1;
 	}
 
 	if (uio_getPhysicalAccess(newDir, newPath, O_WRONLY | O_CREAT | O_EXCL,
-			uio_GPA_NOWRITE, &newReadMountInfo, &newPReadDir,
-			&newWriteMountInfo, &newPWriteDir, &newName) == -1) {
-		int saveErrno = errno;
+			uio_GPA_NOWRITE, &newReadMountInfo, &newPReadDir, NULL,
+			&newWriteMountInfo, &newPWriteDir, NULL, &newName) == -1) {
+		int savedErrno = errno;
 		uio_PDirHandle_unref(oldPReadDir);
 		uio_free(oldName);
-		errno = saveErrno;
+		errno = savedErrno;
 		return -1;
 	}
 
@@ -385,16 +390,16 @@ uio_rename(uio_DirHandle *oldDir, const char *oldPath,
 		errno = ENOSYS;
 		return -1;
 	}
-	retVal = oldReadMountInfo->pDirHandle->pRoot->handler->rename(
+	retVal = (oldReadMountInfo->pDirHandle->pRoot->handler->rename)(
 			oldPReadDir, oldName, newPReadDir, newName);
 	if (retVal == -1) {
-		int saveErrno = errno;
+		int savedErrno = errno;
 		uio_PDirHandle_unref(oldPReadDir);
 		uio_PDirHandle_unref(newPReadDir);
 		uio_PDirHandle_unref(newPWriteDir);
 		uio_free(oldName);
 		uio_free(newName);
-		errno = saveErrno;
+		errno = savedErrno;
 		return -1;
 	}
 
@@ -412,7 +417,7 @@ uio_fstat(uio_Handle *handle, struct stat *statBuf) {
 		errno = ENOSYS;
 		return -1;
 	}
-	return handle->root->handler->fstat(handle, statBuf);
+	return (handle->root->handler->fstat)(handle, statBuf);
 }
 
 int
@@ -423,8 +428,8 @@ uio_stat(uio_DirHandle *dir, const char *path, struct stat *statBuf) {
 	int result;
 
 	if (uio_getPhysicalAccess(dir, path, O_RDONLY, 0,
-			&readMountInfo, &pReadDir,
-			NULL, NULL, &name) == -1) {
+			&readMountInfo, &pReadDir, NULL,
+			NULL, NULL, NULL, &name) == -1) {
 		if (uio_statDir(dir, path, statBuf) == -1) {
 			// errno is set
 			return -1;
@@ -439,7 +444,7 @@ uio_stat(uio_DirHandle *dir, const char *path, struct stat *statBuf) {
 		return -1;
 	}
 
-	result = pReadDir->pRoot->handler->stat(pReadDir, name, statBuf);
+	result = (pReadDir->pRoot->handler->stat)(pReadDir, name, statBuf);
 	uio_PDirHandle_unref(pReadDir);
 	uio_free(name);
 	return result;
@@ -464,9 +469,9 @@ uio_statDir(uio_DirHandle *dirHandle, const char *path,
 	}
 
 	if (uio_statOneDir(pDirHandles[0], statBuf) == -1) {
-		int saveErrno = errno;
+		int savedErrno = errno;
 		uio_PDirHandles_delete(pDirHandles, numPDirHandles);
-		errno = saveErrno;
+		errno = savedErrno;
 		return -1;
 	}
 	// TODO: atm, fstat'ing a dir will show the info for the topmost
@@ -479,9 +484,9 @@ uio_statDir(uio_DirHandle *dirHandle, const char *path,
 
 		if (statOneDir(pDirHandles[i], &statOne) == -1) {
 			// errno is set
-			int saveErrno = errno;
+			int savedErrno = errno;
 			uio_PDirHandles_delete(pDirHandles, numPDirHandles);
-			errno = saveErrno;
+			errno = savedErrno;
 			return -1;
 		}
 
@@ -501,7 +506,7 @@ uio_statOneDir(uio_PDirHandle *pDirHandle, struct stat *statBuf) {
 		errno = ENOSYS;
 		return -1;
 	}
-	return pDirHandle->pRoot->handler->stat(pDirHandle, ".", statBuf);
+	return (pDirHandle->pRoot->handler->stat)(pDirHandle, ".", statBuf);
 			// sets errno on error
 }
 
@@ -513,8 +518,8 @@ uio_mkdir(uio_DirHandle *dir, const char *path, mode_t mode) {
 	uio_PDirHandle *newDirHandle;
 
 	if (uio_getPhysicalAccess(dir, path, O_WRONLY | O_CREAT | O_EXCL, 0,
-			&readMountInfo, &pReadDir,
-			&writeMountInfo, &pWriteDir, &name) == -1) {
+			&readMountInfo, &pReadDir, NULL,
+			&writeMountInfo, &pWriteDir, NULL, &name) == -1) {
 		// errno is set
 		if (errno == EISDIR)
 			errno = EEXIST;
@@ -529,12 +534,12 @@ uio_mkdir(uio_DirHandle *dir, const char *path, mode_t mode) {
 		return -1;
 	}
 
-	newDirHandle = pWriteDir->pRoot->handler->mkdir(pWriteDir, name, mode);
+	newDirHandle = (pWriteDir->pRoot->handler->mkdir)(pWriteDir, name, mode);
 	if (newDirHandle == NULL) {
-		int saveErrno = errno;
+		int savedErrno = errno;
 		uio_free(name);
 		uio_PDirHandle_unref(pWriteDir);
-		errno = saveErrno;
+		errno = savedErrno;
 		return -1;
 	}
 
@@ -552,8 +557,8 @@ uio_open(uio_DirHandle *dir, const char *path, int flags, mode_t mode) {
 	uio_Handle *handle;
 	
 	if (uio_getPhysicalAccess(dir, path, flags, 0,
-			&readMountInfo, &readPDirHandle,
-			&writeMountInfo, &writePDirHandle, &name) == -1) {
+			&readMountInfo, &readPDirHandle, NULL,
+			&writeMountInfo, &writePDirHandle, NULL, &name) == -1) {
 		// errno is set
 		return NULL;
 	}
@@ -592,11 +597,11 @@ uio_open(uio_DirHandle *dir, const char *path, int flags, mode_t mode) {
 				// file needs to be copied
 				if (uio_copyFilePhysical(readPDirHandle, name, writePDirHandle,
 							name) == -1) {
-					int saveErrno = errno;
+					int savedErrno = errno;
 					uio_free(name);
 					uio_PDirHandle_unref(readPDirHandle);
 					uio_PDirHandle_unref(writePDirHandle);
-					errno = saveErrno;
+					errno = savedErrno;
 					return NULL;
 				}
 			}
@@ -615,13 +620,13 @@ uio_open(uio_DirHandle *dir, const char *path, int flags, mode_t mode) {
 		pDirHandle = writePDirHandle;
 	}
 
-	handle = pDirHandle->pRoot->handler->open(pDirHandle, name, flags, mode);
+	handle = (pDirHandle->pRoot->handler->open)(pDirHandle, name, flags, mode);
 			// Also adds a new entry to the physical dir if appropriate.
 	if (handle == NULL) {
-		int saveErrno = errno;
+		int savedErrno = errno;
 		uio_free(name);
 		uio_PDirHandle_unref(pDirHandle);
-		errno = saveErrno;
+		errno = savedErrno;
 		return NULL;
 	}
 
@@ -635,14 +640,17 @@ uio_openDir(uio_Repository *repository, const char *path, int flags) {
 	uio_DirHandle *dirHandle;
 	const char * const rootStr = "/";
 
-	dirHandle = uio_newDirHandle(repository,
+	dirHandle = uio_DirHandle_new(repository,
 			unconst(rootStr), unconst(rootStr));
+			// dirHandle->path will be replaced before uio_openDir()
+			// exits()
 	if (uio_verifyPath(dirHandle, path, &dirHandle->path) == -1) {
-		int saveErrno = errno;
+		int savedErrno = errno;
 		uio_DirHandle_free(dirHandle);
-		errno = saveErrno;
+		errno = savedErrno;
 		return NULL;
 	}
+	// dirHandle->path is no longer equal to 'path' at this point.
 	// TODO: increase ref in repository?
 	dirHandle->rootEnd = dirHandle->path;
 	if (flags & uio_OD_ROOT)
@@ -660,12 +668,12 @@ uio_openDirRelative(uio_DirHandle *base, const char *path, int flags) {
 		return NULL;
 	}
 	if (flags & uio_OD_ROOT) {
-		dirHandle = uio_newDirHandle(base->repository,
+		dirHandle = uio_DirHandle_new(base->repository,
 				newPath, newPath + strlen(newPath));
 		// TODO: increase ref in base->repository?
 	} else {
 		// use the root of the base dir
-		dirHandle = uio_newDirHandle(base->repository,
+		dirHandle = uio_DirHandle_new(base->repository,
 				newPath, newPath + (base->rootEnd - base->path));
 	}
 	return dirHandle;
@@ -673,14 +681,13 @@ uio_openDirRelative(uio_DirHandle *base, const char *path, int flags) {
 
 int
 uio_closeDir(uio_DirHandle *dirHandle) {
-	uio_free(dirHandle->path);
-	uio_DirHandle_free(dirHandle);
+	uio_DirHandle_unref(dirHandle);
 	return 0;
 }
 
 ssize_t
 uio_read(uio_Handle *handle, void *buf, size_t count) {
-	return handle->root->handler->read(handle, buf, count);
+	return (handle->root->handler->read)(handle, buf, count);
 }
 
 int
@@ -740,7 +747,7 @@ uio_rmdir(uio_DirHandle *dirHandle, const char *path) {
 			goto err;
 		}
 
-		if (pDirHandle->pRoot->handler->rmdir(pDirHandle, name) == -1) {
+		if ((pDirHandle->pRoot->handler->rmdir)(pDirHandle, name) == -1) {
 			// errno is set
 			goto err;
 		}
@@ -760,12 +767,12 @@ uio_rmdir(uio_DirHandle *dirHandle, const char *path) {
 
 err:
 	{
-		int saveErrno = errno;
+		int savedErrno = errno;
 		uio_PDirHandles_delete(pDirHandles, numPDirHandles);
 		uio_free(items);
 		if (entry != NULL)
 			uio_PDirEntryHandle_unref(entry);
-		errno = saveErrno;
+		errno = savedErrno;
 		return -1;
 	}
 }
@@ -783,7 +790,7 @@ uio_lseek(uio_Handle *handle, off_t offset, int whence) {
 		errno = ENOSYS;
 		return -1;
 	}
-	return handle->root->handler->seek(handle, offset, whence);
+	return (handle->root->handler->seek)(handle, offset, whence);
 }
 
 ssize_t
@@ -792,7 +799,7 @@ uio_write(uio_Handle *handle, const void *buf, size_t count) {
 		errno = ENOSYS;
 		return -1;
 	}
-	return handle->root->handler->write(handle, buf, count);
+	return (handle->root->handler->write)(handle, buf, count);
 }
 
 int
@@ -850,7 +857,7 @@ uio_unlink(uio_DirHandle *dirHandle, const char *path) {
 			goto err;
 		}
 
-		if (pDirHandle->pRoot->handler->unlink(pDirHandle, name) == -1) {
+		if ((pDirHandle->pRoot->handler->unlink)(pDirHandle, name) == -1) {
 			// errno is set
 			goto err;
 		}
@@ -870,16 +877,114 @@ uio_unlink(uio_DirHandle *dirHandle, const char *path) {
 
 err:
 	{
-		int saveErrno = errno;
+		int savedErrno = errno;
 		uio_PDirHandles_delete(pDirHandles, numPDirHandles);
 		uio_free(items);
 		if (entry != NULL)
 			uio_PDirEntryHandle_unref(entry);
-		errno = saveErrno;
+		errno = savedErrno;
 		return -1;
 	}
 }
 
+// inPath and *outPath may point to the same location
+int
+uio_getFileLocation(uio_DirHandle *dir, const char *inPath,
+		int flags, uio_MountHandle **mountHandle, char **outPath) {
+	uio_PDirHandle *readPDirHandle, *writePDirHandle;
+	uio_MountInfo *readMountInfo, *writeMountInfo, *mountInfo;
+	char *name;
+	char *readPRootPath, *writePRootPath, *pRootPath;
+	
+	if (uio_getPhysicalAccess(dir, inPath, flags, 0,
+			&readMountInfo, &readPDirHandle, &readPRootPath,
+			&writeMountInfo, &writePDirHandle, &writePRootPath,
+			&name) == -1) {
+		// errno is set
+		return -1;
+	}
+
+	// TODO: This code is partly the same as the code in uio_open().
+	//       probably some code could be put in a seperate function.
+	if ((flags & O_ACCMODE) == O_RDONLY) {
+		// WritePDirHandle is not filled in.
+		uio_PDirHandle_unref(readPDirHandle);
+		pRootPath = readPRootPath;
+		mountInfo = readMountInfo;
+	} else if (readPDirHandle == writePDirHandle) {
+		// In general, the dirs can be the same even when the handles are
+		// not the same. But here it works, because uio_getPhysicalAccess
+		// guarantees it.
+		uio_PDirHandle_unref(readPDirHandle);
+		uio_PDirHandle_unref(writePDirHandle);
+		pRootPath = readPRootPath;
+		mountInfo = readMountInfo;
+		uio_free(writePRootPath);
+	} else {
+		// need to write
+		uio_PDirEntryHandle *entry;
+		
+		entry = uio_getPDirEntryHandle(readPDirHandle, name);
+		if (entry != NULL) {
+			// file already exists
+			uio_PDirEntryHandle_unref(entry);
+			if ((flags & O_CREAT) == O_CREAT &&
+					(flags & O_EXCL) == O_EXCL) {
+				uio_free(name);
+				uio_PDirHandle_unref(readPDirHandle);
+				uio_free(readPRootPath);
+				uio_PDirHandle_unref(writePDirHandle);
+				uio_free(writePRootPath);
+				errno = EEXIST;
+				return -1;
+			}
+			if ((flags & O_TRUNC) == O_TRUNC) {
+				// No use copying the file to the writable dir.
+				// As it doesn't exists there, O_TRUNC needs to be turned off
+				// though.
+				flags &= ~O_TRUNC;
+			} else {
+				// file needs to be copied
+				if (uio_copyFilePhysical(readPDirHandle, name, writePDirHandle,
+							name) == -1) {
+					int savedErrno = errno;
+					uio_free(name);
+					uio_PDirHandle_unref(readPDirHandle);
+					uio_free(readPRootPath);
+					uio_PDirHandle_unref(writePDirHandle);
+					uio_free(writePRootPath);
+					errno = savedErrno;
+					return -1;
+				}
+			}
+		} else {
+			// file does not exist
+			if (((flags & O_ACCMODE) == O_RDONLY) ||
+					(flags & O_CREAT) != O_CREAT) {
+				uio_free(name);
+				uio_PDirHandle_unref(readPDirHandle);
+				uio_free(readPRootPath);
+				uio_PDirHandle_unref(writePDirHandle);
+				uio_free(writePRootPath);
+				errno = ENOENT;
+				return -1;
+			}
+		}
+		uio_PDirHandle_unref(readPDirHandle);
+		uio_PDirHandle_unref(writePDirHandle);
+		pRootPath = writePRootPath;
+		mountInfo = writeMountInfo;
+		uio_free(readPRootPath);
+	}
+	
+	uio_free(name);
+
+	*mountHandle = mountInfo->mountHandle;
+	*outPath = pRootPath;
+	return 0;
+}
+
+
 // *** begin dirList stuff *** //
 
 #define uio_DIR_BUFFER_SIZE 2048
@@ -951,16 +1056,16 @@ uio_getDirList(uio_DirHandle *dirHandle, const char *path, const char *pattern,
 			matchType);
 
 	{
-		int saveErrno;
-		saveErrno = errno;
+		int savedErrno;
+		savedErrno = errno;
 		uio_PDirHandles_delete(pDirHandles, numPDirHandles);
-		errno = saveErrno;
+		errno = savedErrno;
 	}
 	return result;
 }
 
 // Names and newNames may point to the same location.
-// &numNames and numNewNames may point to the same location.
+// numNewNames may point to &numNames.
 static void
 uio_filterDoubleNames(const char * const *names, int numNames,
 		const char **newNames, int *numNewNames) {
@@ -1413,7 +1518,7 @@ uio_Handle_new(uio_PRoot *root, uio_NativeHandle native, int openFlags) {
 
 void
 uio_Handle_delete(uio_Handle *handle) {
-	handle->root->handler->close(handle);
+	(handle->root->handler->close)(handle);
 	uio_PRoot_unrefHandle(handle->root);
 	uio_Handle_free(handle);
 }
@@ -1435,19 +1540,27 @@ uio_Handle_free(uio_Handle *handle) {
 	uio_free(handle);
 }
 
+// ref count set to 1
 static uio_DirHandle *
-uio_newDirHandle(uio_Repository *repository, char *path, char *rootEnd) {
+uio_DirHandle_new(uio_Repository *repository, char *path, char *rootEnd) {
 	uio_DirHandle *result;
 	
-	result = uio_allocDirHandle();
+	result = uio_DirHandle_alloc();
+	result->ref = 1;
 	result->repository = repository;
 	result->path = path;
 	result->rootEnd = rootEnd;
 	return result;
 }
 
+void
+uio_DirHandle_delete(uio_DirHandle *dirHandle) {
+	uio_free(dirHandle->path);
+	uio_DirHandle_free(dirHandle);
+}
+
 static inline uio_DirHandle *
-uio_allocDirHandle(void) {
+uio_DirHandle_alloc(void) {
 	uio_DirHandle *result = uio_malloc(sizeof (uio_DirHandle));
 #ifdef uio_MEM_DEBUG
 	uio_MemDebug_debugAlloc(uio_DirHandle, (void *) result);
