@@ -124,9 +124,9 @@ uio_makePath(uio_PDirHandle *pDirHandle, const char *path, size_t pathLen,
 
 		newPDirHandle = mkdirFun(pDirHandle, buf, mode);
 		if (newPDirHandle == NULL) {
-			int saveErrno = errno;
+			int savedErrno = errno;
 			uio_PDirHandle_unref(pDirHandle);
-			errno = saveErrno;
+			errno = savedErrno;
 			return NULL;
 		}
 		uio_PDirHandle_unref(pDirHandle);
@@ -162,17 +162,17 @@ uio_copyFilePhysical(uio_PDirHandle *fromDir, const char *fromName,
 		return -1;
 	}
 
-	fromHandle = fromHandler->open(fromDir, fromName, O_RDONLY, 0);
+	fromHandle = (fromHandler->open)(fromDir, fromName, O_RDONLY, 0);
 	if (fromHandle == NULL) {
 		// errno is set
 		return -1;
 	}
 
-	if (fromHandler->fstat(fromHandle, &statBuf) == -1)
+	if ((fromHandler->fstat)(fromHandle, &statBuf) == -1)
 		return copyError(errno, fromHandler, fromHandle,
 				toHandler, NULL, NULL, NULL, NULL);
 
-	toHandle = toHandler->open(toDir, toName, O_WRONLY | O_CREAT | O_EXCL,
+	toHandle = (toHandler->open)(toDir, toName, O_WRONLY | O_CREAT | O_EXCL,
 			statBuf.st_mode & (S_IRWXU | S_IRWXG | S_IRWXO));
 	if (toHandle == NULL)
 		return copyError(errno, fromHandler, fromHandle,
@@ -182,7 +182,7 @@ uio_copyFilePhysical(uio_PDirHandle *fromDir, const char *fromName,
 			// not allocated on the stack, as this function may be called
 			// from a thread with little stack space.
 	while (1) {
-		numInBuf = fromHandler->read(fromHandle, buf, BUFSIZE);
+		numInBuf = (fromHandler->read)(fromHandle, buf, BUFSIZE);
 		if (numInBuf == -1)
 		{
 			if (errno == EINTR)
@@ -195,7 +195,7 @@ uio_copyFilePhysical(uio_PDirHandle *fromDir, const char *fromName,
 
 		bufPtr = buf;
 		do {
-			numWritten = toHandler->write(toHandle, bufPtr, numInBuf);
+			numWritten = (toHandler->write)(toHandle, bufPtr, numInBuf);
 			if (numWritten == -1)
 			{
 				if (errno == EINTR)
@@ -209,8 +209,8 @@ uio_copyFilePhysical(uio_PDirHandle *fromDir, const char *fromName,
 	}
 
 	uio_free(buf);
-	toHandler->close(toHandle);
-	fromHandler->close(fromHandle);
+	(toHandler->close)(toHandle);
+	(fromHandler->close)(fromHandle);
 	return 0;
 }
 
@@ -231,13 +231,13 @@ copyError(int error,
 #endif
 
 	if (fromHandle != NULL)
-		fromHandler->close(fromHandle);
+		(fromHandler->close)(fromHandle);
 	
 	if (toHandle != NULL)
-		toHandler->close(toHandle);
+		(toHandler->close)(toHandle);
 	
 	if (toName != NULL)
-		toHandler->unlink(toDir, toName);
+		(toHandler->unlink)(toDir, toName);
 	
 	if (buf != NULL)
 		uio_free(buf);
@@ -260,26 +260,38 @@ copyError(int error,
  *              extraFlags - either 0 or uio_GPA_NOWRITE
  *              		When 0, the path will be created if it doesn't
  *              		exist in the writing location, but does exist
- *                      in the reading location. With uio_GPA_NOWRITE, it
- *                      won't be created, and -1 will be returned and errno
- *                      will be set to ENOENT.
+ *              		in the reading location. With uio_GPA_NOWRITE, it
+ *              		won't be created, and -1 will be returned and errno
+ *              		will be set to ENOENT.
  *              mountInfoReadPtr - pointer to location where the pointer
  *              		to the MountInfo structure for the reading location
  *              		should be stored.
  *              readPDirHandlePtr - pointer to the location where the pointer
  *              		to the PDirHandle used for reading should be stored.
+ *              readPRootPath - pointer to the location where the pointer
+ *              		to the physical path to the reading location 
+ *              		is to be stored.
+ *              		The caller is responsible for freeing this.
+ *              		Ignored if NULL.
  *              mountInfoWritePtr - pointer to location where the pointer
  *              		to the MountInfo structure for the writing location
  *              		should be stored.
  *              writePDirHandlePtr - pointer to the location where the pointer
  *              		to the PDirHandle used for writing should be stored.
  *              		NULL if O_RDONLY was specified.
- *                      If this is the same dir as the one refered
- *                      to by readPDirHandlePtr, the handles will be the
- *                      same too.
- *              restPtr - pointer to a newly created string with as contents
- *              		the last component of 'path'.
+ *              		If this is the same dir as the one refered
+ *              		to by readPDirHandlePtr, the handles will be the
+ *              		same too.
+ *              writePRootPath - pointer to the location where the pointer
+ *              		to the physical path to the writing location 
+ *              		is to be stored.
  *              		The caller is responsible for freeing this.
+ *              		Ignored if NULL.
+ *              restPtr - pointer to the location where a newly created
+ *              		string with as contents the last component of 'path'
+ *              		is to be stored.
+ *              		The caller is responsible for freeing this.
+ *              		Ignored if NULL.
  * Returns:     0 - success
  *              -1 - failure (errno set)
  * NB:          This is the function that would most benefit from
@@ -291,10 +303,13 @@ int
 uio_getPhysicalAccess(uio_DirHandle *dirHandle, const char *path,
 		int flags, int extraFlags,
 		uio_MountInfo **mountInfoReadPtr, uio_PDirHandle **readPDirHandlePtr,
+		char **readPRootPathPtr,
 		uio_MountInfo **mountInfoWritePtr, uio_PDirHandle **writePDirHandlePtr,
+		char **writePRootPathPtr,
 		char **restPtr) {
 	char *fullPath;  // path from dirHandle with 'path' added
 	const char *pRootPath;  // path from the pRoot of a physical tree
+	const char *readPRootPath, *writePRootPath;
 	const char *rest, *readRest;
 	uio_MountTree *tree;
 	uio_MountTreeItem *item;
@@ -317,8 +332,10 @@ uio_getPhysicalAccess(uio_DirHandle *dirHandle, const char *path,
 
 	readItem = NULL;
 	readPDirHandle = NULL;
+	readPRootPath = NULL;
 	writeItem = NULL;
 	writePDirHandle = NULL;
+	writePRootPath = NULL;
 	readRest = NULL;
 			// Satisfy compiler.
 	entryExists = false;
@@ -349,22 +366,26 @@ uio_getPhysicalAccess(uio_DirHandle *dirHandle, const char *path,
 			// component is present in this dir.
 			entry = uio_getPDirEntryHandle(pDirHandle, rest);
 			if (entry != NULL) {
-				// Entry is not a dir, as otherwise uio_getPDirEntryHandle
-				// wouldn't have stopped where it did.
+				// 'rest' exists, and it is not a dir, as otherwise
+				// uio_getPDirEntryHandle wouldn't have stopped where it did.
 				uio_PDirEntryHandle_unref(entry);
 				readItem = item;
 				if (readPDirHandle != NULL)
 					uio_PDirHandle_unref(readPDirHandle);
 				readPDirHandle = pDirHandle;
+				readPRootPath = pRootPath;
 				readRest = rest;
 				entryExists = true;
 				break;
 			} else {
-				if (readItem == NULL) {
+				// 'rest' doesn't exist
+				// We're only interested in this dir if we want to write too.
+				if (((flags & O_ACCMODE) != O_RDONLY) && readItem == NULL) {
 					// Keep the first one.
 					readItem = item;
 					assert(readPDirHandle == NULL);
 					readPDirHandle = pDirHandle;
+					readPRootPath = pRootPath;
 					readRest = rest;
 					continue;
 				}
@@ -399,6 +420,9 @@ uio_getPhysicalAccess(uio_DirHandle *dirHandle, const char *path,
 		// write access is not needed
 		*mountInfoReadPtr = readItem->mountInfo;
 		*readPDirHandlePtr = readPDirHandle;
+		if (readPRootPathPtr != NULL)
+			*readPRootPathPtr = joinPathsAbsolute(
+					readItem->mountInfo->dirName, readPRootPath);
 		// Don't touch mountInfoWritePtr and writePDirHandlePtr.
 		// they'd be NULL.
 		*restPtr = uio_strdup(readRest);
@@ -419,13 +443,20 @@ uio_getPhysicalAccess(uio_DirHandle *dirHandle, const char *path,
 		// The read directory is usable as write directory too.
 		*mountInfoReadPtr = readItem->mountInfo;
 		*readPDirHandlePtr = readPDirHandle;
+		if (readPRootPathPtr != NULL)
+			*readPRootPathPtr = joinPathsAbsolute(
+					readItem->mountInfo->dirName, readPRootPath);
 		*mountInfoWritePtr = writeItem->mountInfo;
 				// writeItem == readItem
 		uio_PDirHandle_ref(readPDirHandle);
 		*writePDirHandlePtr = readPDirHandle;
 				// No copy&paste error, the read PDirHandle is the write
 				// pDirHandle too.
-		*restPtr = uio_strdup(readRest);
+		if (writePRootPathPtr != NULL)
+			*writePRootPathPtr = joinPathsAbsolute(
+					writeItem->mountInfo->dirName, writePRootPath);
+		if (restPtr != NULL)
+			*restPtr = uio_strdup(readRest);
 		uio_free(fullPath);
 		return 0;
 	}
@@ -461,17 +492,17 @@ uio_getPhysicalAccess(uio_DirHandle *dirHandle, const char *path,
 		writePDirHandle = uio_makePath(writeItem->mountInfo->pDirHandle,
 				pRootPath, rest - pRootPath, 0777);
 		if (writePDirHandle == NULL) {
-			int saveErrno;
+			int savedErrno;
 			if (errno == ENOSYS) {
 				// mkdir not supported. We want to report that we failed
 				// because of an error in the underlying layer.
 				// EIO sounds like the best choice.
 				errno = EIO;
 			}
-			saveErrno = errno;
+			savedErrno = errno;
 			uio_PDirHandle_unref(readPDirHandle);
 			uio_free(fullPath);
-			errno = saveErrno;
+			errno = savedErrno;
 			return -1;
 		}
 		rest++;  // skip the '/'
@@ -490,9 +521,16 @@ uio_getPhysicalAccess(uio_DirHandle *dirHandle, const char *path,
 	
 	*mountInfoReadPtr = readItem->mountInfo;
 	*readPDirHandlePtr = readPDirHandle;
+	if (readPRootPathPtr != NULL)
+		*readPRootPathPtr = joinPathsAbsolute(
+				readItem->mountInfo->dirName, readPRootPath);
 	*mountInfoReadPtr = writeItem->mountInfo;
 	*writePDirHandlePtr = writePDirHandle;
-	*restPtr = uio_strdup(rest);
+	if (writePRootPathPtr != NULL)
+		*writePRootPathPtr = joinPathsAbsolute(
+				writeItem->mountInfo->dirName, writePRootPath);
+	if (restPtr != NULL)
+		*restPtr = uio_strdup(rest);
 	uio_free(fullPath);
 	return 0;
 }
@@ -632,6 +670,7 @@ uio_verifyPath(uio_DirHandle *dirHandle, const char *path,
 				return 0;
 			case ENOTDIR:
 				// A component is matched, but not as a dir. Failed.
+				uio_free(*resolvedPath);
 				errno = ENOTDIR;
 				return -1;
 			case ENOENT:
