@@ -20,6 +20,10 @@
 
 // The GPDir structures and functions are used for caching only.
 
+#ifdef __svr4__
+#	define _POSIX_PTHREAD_SEMANTICS
+			// For the POSIX variant of r_readdir()
+#endif
 
 #include "./stdio.h"
 
@@ -137,9 +141,9 @@ stdio_stat(uio_PDirHandle *pDirHandle, const char *name,
 
 	result = stat(path, statBuf);
 	if (result == -1) {
-		int saveErrno = errno;
+		int savedErrno = errno;
 		uio_free(path);
-		errno = saveErrno;
+		errno = savedErrno;
 		return -1;
 	}
 
@@ -159,9 +163,9 @@ stdio_mkdir(uio_PDirHandle *pDirHandle, const char *name, mode_t mode) {
 	}
 
 	if (MKDIR(path, mode) == -1) {
-		int saveErrno = errno;
+		int savedErrno = errno;
 		uio_free(path);
-		errno = saveErrno;
+		errno = savedErrno;
 		return NULL;
 	}
 	uio_free(path);
@@ -244,10 +248,10 @@ stdio_rename(uio_PDirHandle *oldPDirHandle, const char *oldName,
 	
 	result = rename(oldPath, newPath);
 	if (result == -1) {
-		int saveErrno = errno;
+		int savedErrno = errno;
 		uio_free(oldPath);
 		uio_free(newPath);
-		errno = saveErrno;
+		errno = savedErrno;
 		return -1;
 	}
 
@@ -283,9 +287,9 @@ stdio_rmdir(uio_PDirHandle *pDirHandle, const char *name) {
 	
 	result = rmdir(path);
 	if (result == -1) {
-		int saveErrno = errno;
+		int savedErrno = errno;
 		uio_free(path);
-		errno = saveErrno;
+		errno = savedErrno;
 		return -1;
 	}
 
@@ -319,9 +323,9 @@ stdio_unlink(uio_PDirHandle *pDirHandle, const char *name) {
 	
 	result = unlink(path);
 	if (result == -1) {
-		int saveErrno = errno;
+		int savedErrno = errno;
 		uio_free(path);
-		errno = saveErrno;
+		errno = savedErrno;
 		return -1;
 	}
 
@@ -345,7 +349,7 @@ stdio_getPDirEntryHandle(const uio_PDirHandle *pDirHandle, const char *name) {
 #ifdef WIN32
 	if (pDirHandle->extra->extra->upDir == NULL) {
 		// Top dir. Contains only drive letters.
-		if (!isalpha(name[0]) || name[1] != ':' || name[2] != '\0')
+		if (!isDriveLetter(name[0]) || name[1] != ':' || name[2] != '\0')
 			return NULL;
 		driveName[0] = tolower(name[0]);
 		driveName[1] = ':';
@@ -505,6 +509,14 @@ stdio_openEntries(uio_PDirHandle *pDirHandle) {
 	result = stdio_EntriesIterator_new(dirHandle);
 	result->status = readdir_r(dirHandle, result->direntBuffer,
 			&result->entry);
+#ifndef WIN32
+#	ifdef DEBUG
+	if (result->status != 0) {
+		fprintf(stderr, "Warning: readdir_r() failed: %s\n",
+				strerror(result->status));
+	}
+#	endif
+#endif
 	return result;
 }
 #endif
@@ -573,7 +585,7 @@ stdio_readEntries(stdio_EntriesIterator **iteratorPtr,
 #ifndef WIN32
 #	ifdef DEBUG
 	if (iterator->status != 0) {
-		fprintf(stderr, "Warning: readdir() failed: %s\n",
+		fprintf(stderr, "Warning: readdir_r() failed: %s\n",
 				strerror(iterator->status));
 	}
 #	endif
@@ -611,6 +623,7 @@ stdio_EntriesIterator_new(long dirHandle) {
 stdio_EntriesIterator *
 stdio_EntriesIterator_new(DIR *dirHandle) {
 	stdio_EntriesIterator *result;
+	size_t bufferSize;
 
 	result = stdio_EntriesIterator_alloc();
 	result->dirHandle = dirHandle;
@@ -619,18 +632,16 @@ stdio_EntriesIterator_new(DIR *dirHandle) {
 	// maximum d_name field (NAME_MAX).
 	// However, POSIX doesn't require this, and in fact
 	// at least QNX defines struct dirent with an empty d_name field.
+	// Solaris defineds it with a d_name field of length 1.
 	// This should take care of it:
-	result->direntBuffer = uio_malloc(
-			sizeof (struct dirent)
-			- ((sizeof (((struct dirent *) 0)->d_name) + 3) & 3)
-			+ (((NAME_MAX + 1) + 3) & 3));
+	bufferSize = sizeof (struct dirent)
+			- sizeof (((struct dirent *) 0)->d_name) + (NAME_MAX + 1);
 			// Take the length of the dirent structure as it is defined,
 			// subtract the length of the d_name field, and add the length
 			// of the maximum length d_name field (NAME_MAX plus 1 for
 			// the '\0').
-			// The '(... +3) & 3' is to take into account that fields
-			// will be aligned on 4 bytes.
-			// (What about 64 bits machines?)
+			// XXX: Could this give problems with weird alignments?
+	result->direntBuffer = uio_malloc(bufferSize);
 	return result;
 }
 #endif
