@@ -16,32 +16,45 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include <ctype.h>
-#include <assert.h>
-#include "starcon.h"
-#include "libs/graphics/drawable.h"
 #include "melee.h"
-#include "options.h"
-#include "file.h"
+
+#include "build.h"
+#include "colors.h"
 #include "controls.h"
+#include "file.h"
+#include "fmv.h"
+#include "gamestr.h"
+#include "globdata.h"
+#include "intel.h"
+#include "nameref.h"
+#include "options.h"
+#include "races.h"
+#include "resinst.h"
+#include "save.h"
+#include "settings.h"
+#include "setup.h"
+#include "sounds.h"
+#include "util.h"
+#include "libs/graphics/drawable.h"
+#include "libs/gfxlib.h"
+#include "libs/inplib.h"
+#include "libs/mathlib.h"
+
+
+#include <assert.h>
+#include <ctype.h>
+#include <string.h>
+
 
 //Added by Chris
 
-#include "string.h"
-
 void ConfirmSaveLoad (STAMP *MsgStamp);
 
-void SaveProblem (void);
-
-void DoShipSpin (COUNT index, MUSIC_REF hMusic);
-
-BOOLEAN InitSpace (void);
-
-void UninitSpace (void);
-
-void InitGlobData (void);
-
 //End Added by Chris
+static void DrawMeleeShipStrings (PMELEE_STATE pMS, BYTE NewStarShip);
+
+
+static int flash_selection_func (void *data);
 
 enum
 {
@@ -75,32 +88,53 @@ enum
 #define RACE_INFO_ORIGIN_Y (SHIP_INFO_HEIGHT + 6)
 #define RACE_INFO_HEIGHT ((STATUS_HEIGHT - 3) - RACE_INFO_ORIGIN_Y)
 
-#define MELEE_BACKGROUND_COLOR BUILD_COLOR (MAKE_RGB15 (0x14, 0x00, 0x00), 0x04)
-#define MELEE_TITLE_COLOR BUILD_COLOR (MAKE_RGB15 (0x1F, 0x0A, 0x0A), 0x0C)
-#define MELEE_TEXT_COLOR BUILD_COLOR (MAKE_RGB15 (0x1F, 0x0A, 0x0A), 0x0C)
-#define MELEE_TEAM_TEXT_COLOR BUILD_COLOR (MAKE_RGB15 (0x1F, 0x1F, 0x0A), 0x0E)
+#define MELEE_BACKGROUND_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (0x14, 0x00, 0x00), 0x04)
+#define MELEE_TITLE_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (0x1F, 0x0A, 0x0A), 0x0C)
+#define MELEE_TEXT_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (0x1F, 0x0A, 0x0A), 0x0C)
+#define MELEE_TEAM_TEXT_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (0x1F, 0x1F, 0x0A), 0x0E)
 
-#define STATE_BACKGROUND_COLOR BUILD_COLOR (MAKE_RGB15 (0x00, 0x00, 0x14), 0x01)
-#define STATE_TEXT_COLOR BUILD_COLOR (MAKE_RGB15 (0x00, 0x14, 0x14), 0x03)
-#define ACTIVE_STATE_TEXT_COLOR BUILD_COLOR (MAKE_RGB15 (0x0A, 0x1F, 0x1F), 0x0B)
-#define UNAVAILABLE_STATE_TEXT_COLOR BUILD_COLOR (MAKE_RGB15 (0x0A, 0x0A, 0x1F), 0x09)
-#define HI_STATE_TEXT_COLOR BUILD_COLOR (MAKE_RGB15 (0x0A, 0x1F, 0x1F), 0x0B)
-#define HI_STATE_BACKGROUND_COLOR BUILD_COLOR (MAKE_RGB15 (0x0A, 0x0A, 0x1F), 0x09)
+#define STATE_BACKGROUND_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (0x00, 0x00, 0x14), 0x01)
+#define STATE_TEXT_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (0x00, 0x14, 0x14), 0x03)
+#define ACTIVE_STATE_TEXT_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (0x0A, 0x1F, 0x1F), 0x0B)
+#define UNAVAILABLE_STATE_TEXT_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (0x0A, 0x0A, 0x1F), 0x09)
+#define HI_STATE_TEXT_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (0x0A, 0x1F, 0x1F), 0x0B)
+#define HI_STATE_BACKGROUND_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (0x0A, 0x0A, 0x1F), 0x09)
 
-#define LIST_INFO_BACKGROUND_COLOR BUILD_COLOR (MAKE_RGB15 (0x14, 0x00, 0x14), 0x05)
-#define LIST_INFO_TITLE_COLOR BUILD_COLOR (MAKE_RGB15 (0x1F, 0x1F, 0x1F), 0x0F)
-#define LIST_INFO_TEXT_COLOR BUILD_COLOR (MAKE_RGB15 (0x14, 0x14, 0x14), 0x07)
-#define LIST_INFO_CURENTRY_TEXT_COLOR BUILD_COLOR (MAKE_RGB15 (0x1F, 0x1F, 0x1F), 0x0F)
-#define HI_LIST_INFO_TEXT_COLOR BUILD_COLOR (MAKE_RGB15 (0x14, 0x00, 0x00), 0x04)
-#define HI_LIST_INFO_BACKGROUND_COLOR BUILD_COLOR (MAKE_RGB15 (0x1F, 0x0A, 0x1F), 0x0D)
+#define LIST_INFO_BACKGROUND_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (0x14, 0x00, 0x14), 0x05)
+#define LIST_INFO_TITLE_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (0x1F, 0x1F, 0x1F), 0x0F)
+#define LIST_INFO_TEXT_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (0x14, 0x14, 0x14), 0x07)
+#define LIST_INFO_CURENTRY_TEXT_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (0x1F, 0x1F, 0x1F), 0x0F)
+#define HI_LIST_INFO_TEXT_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (0x14, 0x00, 0x00), 0x04)
+#define HI_LIST_INFO_BACKGROUND_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (0x1F, 0x0A, 0x1F), 0x0D)
 
-#define TEAM_NAME_TEXT_COLOR BUILD_COLOR (MAKE_RGB15 (15, 16, 27), 0x00)
-#define TEAM_NAME_EDIT_TEXT_COLOR BUILD_COLOR (MAKE_RGB15 (23, 24, 29), 0x00)
-#define TEAM_NAME_EDIT_RECT_COLOR BUILD_COLOR (MAKE_RGB15 (0x14, 0x00, 0x14), 0x05)
-#define TEAM_NAME_EDIT_CURS_COLOR WHITE_COLOR
+#define TEAM_NAME_TEXT_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (15, 16, 27), 0x00)
+#define TEAM_NAME_EDIT_TEXT_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (23, 24, 29), 0x00)
+#define TEAM_NAME_EDIT_RECT_COLOR \
+		BUILD_COLOR (MAKE_RGB15 (0x14, 0x00, 0x14), 0x05)
+#define TEAM_NAME_EDIT_CURS_COLOR \
+		WHITE_COLOR
 
 FRAME PickMeleeFrame;
 static FRAME MeleeFrame;
+static FRAME BuildPickFrame;
 extern QUEUE master_q;
 extern DWORD InTime;
 PMELEE_STATE volatile pMeleeState;
@@ -149,12 +183,14 @@ DrawShipBox (PMELEE_STATE pMS, BOOLEAN HiLite)
 		DrawStarConBox (&r, 1,
 				BUILD_COLOR (MAKE_RGB15 (0x7, 0x00, 0xC), 0x3E),
 				BUILD_COLOR (MAKE_RGB15 (0xC, 0x00, 0x14), 0x3C),
-				(BOOLEAN)(StarShip != (BYTE)~0), BUILD_COLOR (MAKE_RGB15 (0xA, 0x00, 0x11), 0x3D));
+				(BOOLEAN)(StarShip != (BYTE)~0),
+				BUILD_COLOR (MAKE_RGB15 (0xA, 0x00, 0x11), 0x3D));
 	else
 		DrawStarConBox (&r, 1,
 				BUILD_COLOR (MAKE_RGB15 (0x00, 0x00, 0x9), 0x56),
 				BUILD_COLOR (MAKE_RGB15 (0x00, 0x00, 0xE), 0x54),
-				(BOOLEAN)(StarShip != (BYTE)~0), BUILD_COLOR (MAKE_RGB15 (0x00, 0x00, 0xC), 0x55));
+				(BOOLEAN)(StarShip != (BYTE)~0),
+				BUILD_COLOR (MAKE_RGB15 (0x00, 0x00, 0xC), 0x55));
 
 	if (StarShip != (BYTE)~0)
 	{
@@ -195,6 +231,10 @@ DrawControls (COUNT which_side, BOOLEAN HiLite)
 			case AWESOME_RATING:
 				which_icon = 3;
 				break;
+			default:
+				// Should not happen. Satisfying compiler.
+				which_icon = 0;
+				break;
 		}
 	}
 
@@ -204,17 +244,15 @@ DrawControls (COUNT which_side, BOOLEAN HiLite)
 static void
 DrawPickFrame (PMELEE_STATE pMS)
 {
-	FRAME F;
 	RECT r, r0, r1, ship_r;
-	static void DrawMeleeShipStrings (PMELEE_STATE pMS, BYTE
-			NewStarShip);
+	STAMP s;
 				
 	GetShipBox (&r0, 0, 0, 0),
 	GetShipBox (&r1, 1, NUM_MELEE_ROWS - 1, NUM_MELEE_COLUMNS - 1),
 	BoxUnion (&r0, &r1, &ship_r);
 
-	F = SetAbsFrameIndex (MeleeFrame, 27);
-	GetFrameRect (F, &r);
+	s.frame = SetAbsFrameIndex (BuildPickFrame, 0);
+	GetFrameRect (s.frame, &r);
 	r.corner.x = -(ship_r.corner.x
 			+ ((ship_r.extent.width - r.extent.width) >> 1));
 	if (pMS->side)
@@ -222,11 +260,13 @@ DrawPickFrame (PMELEE_STATE pMS)
 	else
 		r.corner.y = -(ship_r.corner.y
 				+ (ship_r.extent.height - r.extent.height));
-	SetFrameHot (F, MAKE_HOT_SPOT (r.corner.x, r.corner.y));
-	DrawMeleeIcon (27);
-	ClearSemaphore (GraphicsSem);
+	SetFrameHot (s.frame, MAKE_HOT_SPOT (r.corner.x, r.corner.y));
+	s.origin.x = s.origin.y = 0;
+	DrawStamp (&s);
+
+	UnlockMutex (GraphicsLock);
 	DrawMeleeShipStrings (pMS, (BYTE)pMS->CurIndex);
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 }
 
 static void
@@ -297,13 +337,9 @@ DrawTeamString (PMELEE_STATE pMS, COUNT HiLiteState)
 	TEXT lfText;
 
 	r.corner.x = MELEE_X_OFFS - 1;
-	r.corner.y =
-			(pMS->side + 1)
-			* (MELEE_Y_OFFS
-			+ ((MELEE_BOX_HEIGHT + MELEE_BOX_SPACE)
-			* NUM_MELEE_ROWS + 2));
-	r.extent.width = NUM_MELEE_COLUMNS
-			* (MELEE_BOX_WIDTH + MELEE_BOX_SPACE);
+	r.corner.y = (pMS->side + 1) * (MELEE_Y_OFFS
+			+ ((MELEE_BOX_HEIGHT + MELEE_BOX_SPACE) * NUM_MELEE_ROWS + 2));
+	r.extent.width = NUM_MELEE_COLUMNS * (MELEE_BOX_WIDTH + MELEE_BOX_SPACE);
 	r.extent.height = 13;
 	if (HiLiteState == 4)
 	{
@@ -382,6 +418,36 @@ DrawTeamString (PMELEE_STATE pMS, COUNT HiLiteState)
 	UnbatchGraphics ();
 }
 
+static void
+DrawPickIcon (COUNT iship, BYTE DrawErase)
+{
+	STAMP s;
+	HSTARSHIP hStarShip;
+	STARSHIPPTR StarShipPtr;
+	RECT r;
+
+	GetFrameRect (BuildPickFrame, &r);
+
+	hStarShip = GetStarShipFromIndex (&master_q, iship);
+	StarShipPtr = LockStarShip (&master_q, hStarShip);
+	s.origin.x = r.corner.x + 20 + (iship % NUM_PICK_COLS) * 18;
+	s.origin.y = r.corner.y +  5 + (iship / NUM_PICK_COLS) * 18;
+	s.frame = StarShipPtr->RaceDescPtr->ship_info.icons;
+	if (DrawErase)
+	{	// draw icon
+		DrawStamp (&s);
+	}
+	else
+	{	// erase icon
+		COLOR OldColor;
+
+		OldColor = SetContextForeGroundColor (BLACK_COLOR);
+		DrawFilledStamp (&s);
+		SetContextForeGroundColor (OldColor);
+	}
+	UnlockStarShip (&master_q, hStarShip);
+}
+
 static void
 Deselect (BYTE opt)
 {
@@ -422,22 +488,7 @@ Deselect (BYTE opt)
 			}
 			else if (pMeleeState->InputFunc == DoPickShip)
 			{
-				STAMP s;
-				HSTARSHIP hStarShip;
-				STARSHIPPTR StarShipPtr;
-				RECT r;
-			
-				GetFrameRect (SetAbsFrameIndex (MeleeFrame, 27), &r);
-
-				hStarShip = GetStarShipFromIndex (&master_q, pMeleeState->CurIndex);
-				StarShipPtr = LockStarShip (&master_q, hStarShip);
-				s.origin.x = r.corner.x + 20
-						+ (pMeleeState->CurIndex % NUM_PICK_COLS) * 18;
-				s.origin.y = r.corner.y + 5
-						+ (pMeleeState->CurIndex / NUM_PICK_COLS) * 18;
-				s.frame = StarShipPtr->RaceDescPtr->ship_info.icons;
-				DrawStamp (&s);
-				UnlockStarShip (&master_q, hStarShip);
+				DrawPickIcon (pMeleeState->CurIndex, 1);
 			}
 			break;
 	}
@@ -483,31 +534,14 @@ Select (BYTE opt)
 			}
 			else if (pMeleeState->InputFunc == DoPickShip)
 			{
-				COLOR OldColor;
-				STAMP s;
-				HSTARSHIP hStarShip;
-				STARSHIPPTR StarShipPtr;
-				RECT r;
-			
-				GetFrameRect (SetAbsFrameIndex (MeleeFrame, 27), &r);
-
-				hStarShip = GetStarShipFromIndex (&master_q, pMeleeState->CurIndex);
-				StarShipPtr = LockStarShip (&master_q, hStarShip);
-				s.origin.x = r.corner.x + 20
-						+ (pMeleeState->CurIndex % NUM_PICK_COLS) * 18;
-				s.origin.y = r.corner.y + 5
-						+ (pMeleeState->CurIndex / NUM_PICK_COLS) * 18;
-				s.frame = StarShipPtr->RaceDescPtr->ship_info.icons;
-				OldColor = SetContextForeGroundColor (BLACK_COLOR);
-				DrawFilledStamp (&s);
-				SetContextForeGroundColor (OldColor);
-				UnlockStarShip (&master_q, hStarShip);
+				DrawPickIcon (pMeleeState->CurIndex, 0);
 			}
 			break;
 	}
 }
 
-int flash_selection_func(void* data)
+static int
+flash_selection_func (void *data)
 {
 	DWORD TimeIn;
 	Task task = (Task) data;
@@ -518,26 +552,26 @@ int flash_selection_func(void* data)
 #define FLASH_RATE (ONE_SECOND / 8)
 		CONTEXT OldContext;
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		OldContext = SetContext (SpaceContext);
 		Deselect (pMeleeState->MeleeOption);
 		SetContext (OldContext);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 		SleepThreadUntil (TimeIn + FLASH_RATE);
 		TimeIn = GetTimeCounter ();
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		OldContext = SetContext (SpaceContext);
 		Select (pMeleeState->MeleeOption);
 		SetContext (OldContext);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 		SleepThreadUntil (TimeIn + FLASH_RATE);
 		TimeIn = GetTimeCounter ();
 	}
 
 	FinishTask (task);
 
-	return(0);
+	return 0;
 }
 
 static void
@@ -571,7 +605,7 @@ DrawMeleeShipStrings (PMELEE_STATE pMS, BYTE NewStarShip)
 	STARSHIPPTR StarShipPtr;
 	CONTEXT OldContext;
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 
 	OldContext = SetContext (StatusContext);
 	GetContextClipRect (&OldRect);
@@ -630,7 +664,7 @@ DrawMeleeShipStrings (PMELEE_STATE pMS, BYTE NewStarShip)
 	SetContextClipRect (&OldRect);
 	SetContext (OldContext);
 
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 }
 
 static COUNT
@@ -645,11 +679,13 @@ GetTeamValue (TEAM_IMAGE *pTI)
 		{
 			BYTE StarShip;
 
-			if ((StarShip = pTI->ShipList[row][col]) != (BYTE)~0)
+			StarShip = pTI->ShipList[row][col];
+			if (StarShip != (BYTE) ~0)
 			{
 				HSTARSHIP hStarShip;
 
-				if ((hStarShip = GetStarShipFromIndex (&master_q, StarShip)) == 0)
+				hStarShip = GetStarShipFromIndex (&master_q, StarShip);
+				if (hStarShip == 0)
 					pTI->ShipList[row][col] = (BYTE)~0;
 				else
 				{
@@ -747,11 +783,9 @@ DrawFileStrings (PMELEE_STATE pMS, int HiLiteState)
 		Text.baseline.y = y + bot * ENTRY_HEIGHT;
 		Text.pStr = pMS->FileList[bot].TeamName;
 		Text.CharCount = (COUNT)~0;
-		SetContextForeGroundColor (
-				HiLiteState == 0
-				? BUILD_COLOR (MAKE_RGB15 (15, 16, 27), 0x00)
-				: BUILD_COLOR (MAKE_RGB15 (23, 24, 29), 0x00)
-				);
+		SetContextForeGroundColor (HiLiteState == 0 ?
+				BUILD_COLOR (MAKE_RGB15 (15, 16, 27), 0x00)
+				: BUILD_COLOR (MAKE_RGB15 (23, 24, 29), 0x00));
 		font_DrawText (&Text);
 
 		rtText.baseline.y = Text.baseline.y;
@@ -767,8 +801,7 @@ DrawFileStrings (PMELEE_STATE pMS, int HiLiteState)
 
 		Text.baseline.y = y;
 		teams_left = (COUNT)(
-				GetDirEntryTableCount (pMS->TeamDE) + NUM_PREBUILT - top
-				);
+				GetDirEntryTableCount (pMS->TeamDE) + NUM_PREBUILT - top);
 		if (teams_left)
 		{
 			bot = top - 1;
@@ -811,8 +844,8 @@ DrawFileStrings (PMELEE_STATE pMS, int HiLiteState)
 					{
 						BYTE StarShip;
 						
-						if ((StarShip = pMS->FileList[bot - top].ShipList[row][col])
-								!= (BYTE)~0)
+						StarShip = pMS->FileList[bot - top].ShipList[row][col];
+						if (StarShip != (BYTE)~0)
 						{
 							HSTARSHIP hStarShip;
 							STARSHIPPTR StarShipPtr;
@@ -844,9 +877,12 @@ DoLoadTeam (PMELEE_STATE pMS)
 	if (GLOBAL (CurrentActivity) & CHECK_ABORT)
 		return (FALSE);
 
+	SetMenuSounds (MENU_SOUND_UP | MENU_SOUND_DOWN | MENU_SOUND_PAGEUP |
+			MENU_SOUND_PAGEDOWN, MENU_SOUND_SELECT);
+
 	if (!pMS->Initialized)
 	{
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		Select (pMS->MeleeOption);
 		if ((pMS->TopTeamIndex = pMS->CurIndex) == (COUNT)~0)
 			pMS->TopTeamIndex = pMS->CurIndex = 0;
@@ -858,15 +894,15 @@ DoLoadTeam (PMELEE_STATE pMS)
 		DrawFileStrings (pMS, -1);
 		pMS->Initialized = TRUE;
 		pMS->InputFunc = DoLoadTeam;
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 	}
-	else if (CurrentMenuState.select | CurrentMenuState.cancel)
+	else if (PulsedInputState.key[KEY_MENU_SELECT] |
+			PulsedInputState.key[KEY_MENU_CANCEL])
 	{
-		if (!CurrentMenuState.cancel)
+		if (!PulsedInputState.key[KEY_MENU_CANCEL])
 		{
 			pMS->TeamImage[pMS->side] = pMS->FileList[
-					pMS->CurIndex - pMS->TopTeamIndex
-					];
+					pMS->CurIndex - pMS->TopTeamIndex];
 			pMS->star_bucks[pMS->side] =
 					GetTeamValue (&pMS->TeamImage[pMS->side]);
 		}
@@ -876,9 +912,9 @@ DoLoadTeam (PMELEE_STATE pMS)
 			RECT r;
 			
 			GetFrameRect (SetAbsFrameIndex (MeleeFrame, 28), &r);
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			RepairMeleeFrame (&r);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 		}
 		InTime = GetTimeCounter ();
 	}
@@ -888,7 +924,7 @@ DoLoadTeam (PMELEE_STATE pMS)
 
 		NewTop = pMS->TopTeamIndex;
 		index = old_index = pMS->CurIndex;
-		if (CurrentMenuState.up)
+		if (PulsedInputState.key[KEY_MENU_UP])
 		{
 			if (index-- == 0)
 				index = 0;
@@ -896,27 +932,34 @@ DoLoadTeam (PMELEE_STATE pMS)
 			if (index < NewTop && (NewTop -= MAX_VIS_TEAMS) < 0)
 				NewTop = 0;
 		}
-		else if (CurrentMenuState.down)
+		else if (PulsedInputState.key[KEY_MENU_DOWN])
 		{
-			if ((int)index < (int)GetDirEntryTableCount (pMS->TeamDE) + NUM_PREBUILT - 1)
+			if ((int)index < (int)GetDirEntryTableCount (pMS->TeamDE) +
+					NUM_PREBUILT - 1)
 				++index;
 
 			if ((int)index > (int)pMS->BotTeamIndex)
 				NewTop = index;
 		}
-		else if (CurrentMenuState.page_up)
+		else if (PulsedInputState.key[KEY_MENU_PAGE_UP])
 		{
-			if ((index -= MAX_VIS_TEAMS) < 0)
-				index = NewTop = 0;
+			index -= MAX_VIS_TEAMS;
+			if (index < 0)
+			{
+				index = 0;
+				NewTop = 0;
+			}
 			else
 			{
-				if ((NewTop -= MAX_VIS_TEAMS) < 0)
+				NewTop -= MAX_VIS_TEAMS;
+				if (NewTop < 0)
 					NewTop = 0;
 			}
 		}
-		else if (CurrentMenuState.page_down)
+		else if (PulsedInputState.key[KEY_MENU_PAGE_DOWN])
 		{
-			if ((int)(index += MAX_VIS_TEAMS) <
+			index += MAX_VIS_TEAMS;
+			if ((int) index <
 					(int)(GetDirEntryTableCount (pMS->TeamDE) + NUM_PREBUILT))
 				NewTop += MAX_VIS_TEAMS;
 			else
@@ -924,7 +967,8 @@ DoLoadTeam (PMELEE_STATE pMS)
 				index = GetDirEntryTableCount (pMS->TeamDE) + NUM_PREBUILT - 1;
 				if (index - (MAX_VIS_TEAMS - 1) > NewTop)
 				{
-					if ((NewTop = index - (MAX_VIS_TEAMS - 1)) < 0)
+					NewTop = index - (MAX_VIS_TEAMS - 1);
+					if (NewTop < 0)
 						NewTop = 0;
 				}
 			}
@@ -932,7 +976,7 @@ DoLoadTeam (PMELEE_STATE pMS)
 
 		if (index != old_index)
 		{
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			if ((int)NewTop == (int)pMS->TopTeamIndex)
 				Deselect (pMS->MeleeOption);
 			else
@@ -941,13 +985,14 @@ DoLoadTeam (PMELEE_STATE pMS)
 				DrawFileStrings (pMS, -1);
 			}
 			pMS->CurIndex = index;
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 		}
 	}
 
 	return (TRUE);
 }
 
+// XXX: TODO: to be merged with DoTextEntry in gameopt.c
 static UWORD
 DoTextEntry (PMELEE_STATE pMS)
 {
@@ -961,6 +1006,11 @@ DoTextEntry (PMELEE_STATE pMS)
 		pBaseStr = pMS->TeamImage[pMS->side].TeamName;
 		max_chars = MAX_TEAM_CHARS;
 	}
+	else
+	{
+		// Should not happen. Satisfying compiler.
+		return 0;
+	}
 
 	ch = GetCharacter ();
 
@@ -1076,7 +1126,7 @@ DoSaveTeam (PMELEE_STATE pMS)
 
 	sprintf (file, "%s.mle", pMS->TeamImage[pMS->side].TeamName);
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	OldContext = SetContext (ScreenContext);
 	ConfirmSaveLoad (&MsgStamp);
 	save_fp = res_OpenResFile (meleeDir, file, "wb");
@@ -1097,7 +1147,7 @@ DoSaveTeam (PMELEE_STATE pMS)
 		DrawStamp (&MsgStamp);
 		DestroyDrawable (ReleaseDrawable (MsgStamp.frame));
 		SetContext (OldContext);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 
 		DeleteResFile (meleeDir, file);
 		SaveProblem ();
@@ -1110,7 +1160,7 @@ DoSaveTeam (PMELEE_STATE pMS)
 		DrawStamp (&MsgStamp);
 		DestroyDrawable (ReleaseDrawable (MsgStamp.frame));
 		SetContext (OldContext);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 	}
 	
 	return (save_fp != 0);
@@ -1134,13 +1184,13 @@ DeleteCurrentShip (PMELEE_STATE pMS)
 		UnlockStarShip (&master_q, hStarShip);
 	
 		pMS->TeamImage[pMS->side].ShipList[pMS->row][pMS->col] = (BYTE)~0;
-       	}
-	SetSemaphore (GraphicsSem);
+	}
+	LockMutex (GraphicsLock);
 	GetShipBox (&r, pMS->side, pMS->row, pMS->col);
 	RepairMeleeFrame (&r);
 
 	DrawTeamString (pMS, 4);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 }
 
@@ -1164,6 +1214,7 @@ DoEdit (PMELEE_STATE pMS)
 {
 	if (GLOBAL (CurrentActivity) & CHECK_ABORT)
 		return (FALSE);
+	SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT | MENU_SOUND_DELETE);
 	if (!pMS->Initialized)
 	{
 		pMS->CurIndex = pMS->TeamImage[pMS->side].ShipList[pMS->row][pMS->col];
@@ -1173,29 +1224,29 @@ DoEdit (PMELEE_STATE pMS)
 		pMS->InputFunc = DoEdit;
 	}
 	else if ((pMS->row < NUM_MELEE_ROWS || pMS->CurIndex == (BYTE)~0)
-			&& (CurrentMenuState.cancel
-			|| (CurrentMenuState.right
+			&& (PulsedInputState.key[KEY_MENU_CANCEL]
+			|| (PulsedInputState.key[KEY_MENU_RIGHT]
 			&& (pMS->col == NUM_MELEE_COLUMNS - 1 || pMS->row == NUM_MELEE_ROWS))))
 	{
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		Deselect (EDIT_MELEE);
 		pMS->CurIndex = (COUNT)~0;
 		pMS->MeleeOption = START_MELEE;
 		pMS->InputFunc = DoMelee;
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 		InTime = GetTimeCounter ();
 	}
 	else if (pMS->row < NUM_MELEE_ROWS
-			&& (CurrentMenuState.select || CurrentMenuState.special))
+			&& (PulsedInputState.key[KEY_MENU_SELECT] || PulsedInputState.key[KEY_MENU_SPECIAL]))
 	{
-		if (CurrentMenuState.select)
+		if (PulsedInputState.key[KEY_MENU_SELECT])
 			pMS->Initialized = 0;
 		else
 			pMS->Initialized = -1;
 		TFB_ResetControls ();
 		DoPickShip (pMS);
 	}
-	else if (pMS->row < NUM_MELEE_ROWS && CurrentMenuState.del)
+	else if (pMS->row < NUM_MELEE_ROWS && PulsedInputState.key[KEY_MENU_DELETE])
 	{
 		DeleteCurrentShip (pMS);
 		AdvanceCursor (pMS);
@@ -1213,7 +1264,7 @@ DoEdit (PMELEE_STATE pMS)
 			if (pMS->CurIndex != (BYTE)~0)
 			{
 				UNICODE ch;
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 				if (pMS->Initialized == 1)
 				{
 					FlushInput ();
@@ -1226,33 +1277,33 @@ DoEdit (PMELEE_STATE pMS)
 					pMS->Initialized = 1;
 					DisableCharacterMode ();
 				}
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 				return (TRUE);
 			}
-			else if (CurrentMenuState.select)
+			else if (PulsedInputState.key[KEY_MENU_SELECT])
 			{
 				pMS->CurIndex = 0;
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 				DrawTeamString (pMS, 1);
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 				EnableCharacterMode ();
 				return (TRUE);
 			}
 		}
 
 		{
-			if (CurrentMenuState.left)
+			if (PulsedInputState.key[KEY_MENU_LEFT])
 			{
 				if (col-- == 0)
 					col = 0;
 			}
-			else if (CurrentMenuState.right)
+			else if (PulsedInputState.key[KEY_MENU_RIGHT])
 			{
 				if (++col == NUM_MELEE_COLUMNS)
 					col = NUM_MELEE_COLUMNS - 1;
 			}
 
-			if (CurrentMenuState.up)
+			if (PulsedInputState.key[KEY_MENU_UP])
 			{
 				if (row-- == 0)
 				{
@@ -1265,7 +1316,7 @@ DoEdit (PMELEE_STATE pMS)
 					}
 				}
 			}
-			else if (CurrentMenuState.down)
+			else if (PulsedInputState.key[KEY_MENU_DOWN])
 			{
 				if (row++ == NUM_MELEE_ROWS)
 				{
@@ -1282,7 +1333,7 @@ DoEdit (PMELEE_STATE pMS)
 
 		if (col != pMS->col || row != pMS->row || side != pMS->side)
 		{
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			Deselect (EDIT_MELEE);
 			pMS->side = side;
 			pMS->row = row;
@@ -1291,7 +1342,7 @@ DoEdit (PMELEE_STATE pMS)
 				pMS->CurIndex = (BYTE)~0;
 			else
 				pMS->CurIndex = pMS->TeamImage[side].ShipList[row][col];
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 
 			DrawMeleeShipStrings (pMS, (BYTE)(pMS->CurIndex));
 		}
@@ -1308,6 +1359,8 @@ DoPickShip (PMELEE_STATE pMS)
 	if (GLOBAL (CurrentActivity) & CHECK_ABORT)
 		return (FALSE);
 
+	SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
+
 	if (pMS->Initialized <= 0)
 	{
 		if (pMS->CurIndex == (BYTE)~0 && pMS->Initialized == 0)
@@ -1319,30 +1372,30 @@ DoPickShip (PMELEE_STATE pMS)
 
 		if (pMS->Initialized == 0)
 		{
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			Deselect (EDIT_MELEE);
 			pMS->InputFunc = DoPickShip;
 			DrawPickFrame (pMS);
 			pMS->Initialized = TRUE;
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 		}
 		else
 		{
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			Deselect (EDIT_MELEE);
 			pMS->Initialized = TRUE;
 			AdvanceCursor (pMS);
 			pMS->CurIndex = pMS->TeamImage[pMS->side].ShipList[pMS->row][pMS->col];
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 
 			DrawMeleeShipStrings (pMS, (BYTE)(pMS->CurIndex));
 		}
 	}
-	else if (CurrentMenuState.select || CurrentMenuState.cancel)
+	else if (PulsedInputState.key[KEY_MENU_SELECT] || PulsedInputState.key[KEY_MENU_CANCEL])
 	{
 		pMS->InputFunc = 0; /* disable ship flashing */
 
-		if (!CurrentMenuState.cancel)
+		if (!PulsedInputState.key[KEY_MENU_CANCEL])
 		{
 			HSTARSHIP hStarShip;
 			
@@ -1353,20 +1406,20 @@ DoPickShip (PMELEE_STATE pMS)
 			UnlockStarShip (&master_q, hStarShip);
 
 			pMS->TeamImage[pMS->side].ShipList[pMS->row][pMS->col] = pMS->CurIndex;
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			DrawTeamString (pMS, 4);
 			DrawShipBox (pMS, FALSE);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			AdvanceCursor (pMS);
 		}
 
 		{
 			RECT r;
 			
-			GetFrameRect (SetAbsFrameIndex (MeleeFrame, 27), &r);
-			SetSemaphore (GraphicsSem);
+			GetFrameRect (BuildPickFrame, &r);
+			LockMutex (GraphicsLock);
 			RepairMeleeFrame (&r);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 		}
 
 		pMS->CurIndex = pMS->TeamImage[pMS->side].ShipList[pMS->row][pMS->col];
@@ -1376,7 +1429,7 @@ DoPickShip (PMELEE_STATE pMS)
 
 		return (TRUE);
 	}
-	else if (CurrentMenuState.special)
+	else if (PulsedInputState.key[KEY_MENU_SPECIAL])
 	{
 		DoShipSpin (pMS->CurIndex, (MUSIC_REF)0);
 	
@@ -1388,25 +1441,25 @@ DoPickShip (PMELEE_STATE pMS)
 
 		NewStarShip = pMS->CurIndex;
 
-		if (CurrentMenuState.left)
+		if (PulsedInputState.key[KEY_MENU_LEFT])
 		{
 			if (NewStarShip-- % NUM_PICK_COLS == 0)
 				NewStarShip += NUM_PICK_COLS;
 		}
-		else if (CurrentMenuState.right)
+		else if (PulsedInputState.key[KEY_MENU_RIGHT])
 		{
 			if (++NewStarShip % NUM_PICK_COLS == 0)
 				NewStarShip -= NUM_PICK_COLS;
 		}
 		
-		if (CurrentMenuState.up)
+		if (PulsedInputState.key[KEY_MENU_UP])
 		{
 			if (NewStarShip >= NUM_PICK_COLS)
 				NewStarShip -= NUM_PICK_COLS;
 			else
 				NewStarShip += NUM_PICK_COLS * (NUM_PICK_ROWS - 1);
 		}
-		else if (CurrentMenuState.down)
+		else if (PulsedInputState.key[KEY_MENU_DOWN])
 		{
 			if (NewStarShip < NUM_PICK_COLS * (NUM_PICK_ROWS - 1))
 				NewStarShip += NUM_PICK_COLS;
@@ -1416,10 +1469,10 @@ DoPickShip (PMELEE_STATE pMS)
 
 		if (NewStarShip != pMS->CurIndex)
 		{
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			Deselect (EDIT_MELEE);
 			pMS->CurIndex = NewStarShip;
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			DrawMeleeShipStrings (pMS, NewStarShip);
 		}
 	}
@@ -1430,34 +1483,42 @@ DoPickShip (PMELEE_STATE pMS)
 static void
 LoadMeleeInfo (PMELEE_STATE pMS)
 {
-    STAMP	s;
-    CONTEXT	OldContext;
+	STAMP	s;
+	CONTEXT	OldContext;
+	RECT    r;
+	COUNT   i;
 
 	// reverting to original behavior
-    OldContext = SetContext (OffScreenContext);
+	OldContext = SetContext (OffScreenContext);
 
-    if (PickMeleeFrame)
-	    DestroyDrawable (ReleaseDrawable (PickMeleeFrame));
-    PickMeleeFrame = CaptureDrawable (CreateDrawable (
-			WANT_PIXMAP, MELEE_WIDTH, MELEE_HEIGHT, 2
-			));
-    s.origin.x = s.origin.y = 0;
-    s.frame = CaptureDrawable (
-			LoadGraphic (MELEE_PICK_MASK_PMAP_ANIM)
-			);
-    SetContextFGFrame (PickMeleeFrame);
-    DrawStamp (&s);
+	if (PickMeleeFrame)
+		DestroyDrawable (ReleaseDrawable (PickMeleeFrame));
+	PickMeleeFrame = CaptureDrawable (CreateDrawable (
+			WANT_PIXMAP, MELEE_WIDTH, MELEE_HEIGHT, 2));
+	s.origin.x = s.origin.y = 0;
+	s.frame = CaptureDrawable (LoadGraphic (MELEE_PICK_MASK_PMAP_ANIM));
+	SetContextFGFrame (PickMeleeFrame);
+	DrawStamp (&s);
 
-    s.frame = IncFrameIndex (s.frame);
-    SetContextFGFrame (IncFrameIndex (PickMeleeFrame));
-    DrawStamp (&s);
-    DestroyDrawable (ReleaseDrawable (s.frame));
-	
-	MeleeFrame = CaptureDrawable (
-			LoadGraphic (MELEE_SCREEN_PMAP_ANIM)
-			);
-	
-    SetContext (OldContext);
+	s.frame = IncFrameIndex (s.frame);
+	SetContextFGFrame (IncFrameIndex (PickMeleeFrame));
+	DrawStamp (&s);
+	DestroyDrawable (ReleaseDrawable (s.frame));
+
+	MeleeFrame = CaptureDrawable (LoadGraphic (MELEE_SCREEN_PMAP_ANIM));
+
+	// create team building ship selection box
+	s.frame = SetAbsFrameIndex (MeleeFrame, 27);
+	GetFrameRect (s.frame, &r);
+	BuildPickFrame = CaptureDrawable (CreateDrawable (
+			WANT_PIXMAP, r.extent.width, r.extent.height, 1));
+	SetContextFGFrame (BuildPickFrame);
+	SetFrameHot (s.frame, MAKE_HOT_SPOT (0, 0));
+	DrawStamp (&s);
+	for (i = 0; i < NUM_PICK_COLS * NUM_PICK_ROWS; ++i)
+		DrawPickIcon (i, 1);
+
+	SetContext (OldContext);
 
 	InitSpace ();
 
@@ -1469,18 +1530,24 @@ FreeMeleeInfo (PMELEE_STATE pMS)
 {
 	if (pMS->flash_task)
 	{
-		SetSemaphore (GraphicsSem);
-		Task_SetState (pMS->flash_task, TASK_EXIT);
-		ClearSemaphore (GraphicsSem);
+		ConcludeTask (pMS->flash_task);
 		pMS->flash_task = 0;
 	}
 	DestroyDirEntryTable (ReleaseDirEntryTable (pMS->TeamDE));
 	pMS->TeamDE = 0;
 
+	if (pMS->hMusic)
+	{
+		DestroyMusic (pMS->hMusic);
+		pMS->hMusic = 0;
+	}
+
 	UninitSpace ();
 
 	DestroyDrawable (ReleaseDrawable (MeleeFrame));
 	MeleeFrame = 0;
+	DestroyDrawable (ReleaseDrawable (BuildPickFrame));
+	BuildPickFrame = 0;
 }
 
 static void
@@ -1506,8 +1573,7 @@ BuildAndDrawShipList (PMELEE_STATE pMS)
 
 		GetFrameRect (s.frame, &r);
 		t.baseline.x = r.extent.width >> 1;
-		t.baseline.y = r.extent.height
-				- NAME_AREA_HEIGHT + 4;
+		t.baseline.y = r.extent.height - NAME_AREA_HEIGHT + 4;
 		r.corner.x += 2;
 		r.corner.y += 2;
 		r.extent.width -= (2 * 2) + (ICON_WIDTH + 2) + 1;
@@ -1587,13 +1653,23 @@ DoMelee (PMELEE_STATE pMS)
 	if (GLOBAL (CurrentActivity) & CHECK_ABORT)
 		return (FALSE);
 
+	SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
 	if (!pMS->Initialized)
 	{
+		if (pMS->hMusic)
+		{
+			StopMusic ();
+			DestroyMusic (pMS->hMusic);
+			pMS->hMusic = 0;
+		}
+		pMS->hMusic = LoadMusicFile ("melee/melemenu.ogg");
 		pMS->Initialized = TRUE;
+		
 		pMS->MeleeOption = START_MELEE;
-		SetSemaphore (GraphicsSem);
+		PlayMusic (pMS->hMusic, TRUE, 1);
+		LockMutex (GraphicsLock);
 		InitMelee (pMS);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 		{
 			BYTE clut_buf[] = {FadeAllToColor};
 				
@@ -1601,20 +1677,26 @@ DoMelee (PMELEE_STATE pMS)
 		}
 		InTime = GetTimeCounter ();
 	}
-	else if (CurrentMenuState.cancel || CurrentMenuState.left)
+	else if (PulsedInputState.key[KEY_MENU_CANCEL] || PulsedInputState.key[KEY_MENU_LEFT])
 	{
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		InTime = GetTimeCounter ();
 		Deselect (pMS->MeleeOption);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 		pMS->MeleeOption = EDIT_MELEE;
 		pMS->Initialized = FALSE;
-		if (CurrentMenuState.cancel)
-			pMS->side = pMS->row = pMS->col = 0;
+		if (PulsedInputState.key[KEY_MENU_CANCEL])
+		{
+			pMS->side = 0;
+			pMS->row = 0;
+			pMS->col = 0;
+		}
 		else
-			pMS->side = 0,
-			pMS->row = NUM_MELEE_ROWS - 1,
+		{
+			pMS->side = 0;
+			pMS->row = NUM_MELEE_ROWS - 1;
 			pMS->col = NUM_MELEE_COLUMNS - 1;
+		}
 		DoEdit (pMS);
 	}
 	else
@@ -1622,13 +1704,13 @@ DoMelee (PMELEE_STATE pMS)
 		MELEE_OPTIONS NewMeleeOption;
 
 		NewMeleeOption = pMS->MeleeOption;
-		if (CurrentMenuState.up)
+		if (PulsedInputState.key[KEY_MENU_UP])
 		{
 			InTime = GetTimeCounter ();
 			if (NewMeleeOption-- == CONTROLS_TOP)
 				NewMeleeOption = CONTROLS_TOP;
 		}
-		else if (CurrentMenuState.down)
+		else if (PulsedInputState.key[KEY_MENU_DOWN])
 		{
 			InTime = GetTimeCounter ();
 			if (NewMeleeOption++ == CONTROLS_BOT)
@@ -1644,14 +1726,14 @@ DoMelee (PMELEE_STATE pMS)
 
 		if (NewMeleeOption != pMS->MeleeOption)
 		{
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			Deselect (pMS->MeleeOption);
 			pMS->MeleeOption = NewMeleeOption;
 			Select (pMS->MeleeOption);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 		}
 
-		if (CurrentMenuState.select || force_select)
+		if (PulsedInputState.key[KEY_MENU_SELECT] || force_select)
 		{
 			switch (pMS->MeleeOption)
 			{
@@ -1665,25 +1747,32 @@ DoMelee (PMELEE_STATE pMS)
 
 					if (pMS->flash_task)
 					{
-						SetSemaphore (GraphicsSem);
-						Task_SetState (pMS->flash_task, TASK_EXIT);
-						ClearSemaphore (GraphicsSem);
+						ConcludeTask (pMS->flash_task);
 						pMS->flash_task = 0;
 					}
 					
 					{
 						BYTE black_buf[] = {FadeAllToBlack};
 						
+						FadeMusic (0, ONE_SECOND / 2);
 						SleepThreadUntil (XFormColorMap (
 								(COLORMAPPTR)black_buf, ONE_SECOND / 2
 								) + ONE_SECOND / 60);
 						FlushColorXForms ();
+						StopMusic ();
 					}
+					FadeMusic (NORMAL_VOLUME, 0);
+					if (pMS->hMusic)
+					{
+						DestroyMusic (pMS->hMusic);
+						pMS->hMusic = 0;
+					}
+
 					do
 					{
-						SetSemaphore (GraphicsSem);
+						LockMutex (GraphicsLock);
 						BuildAndDrawShipList (pMS);
-						ClearSemaphore (GraphicsSem);
+						UnlockMutex (GraphicsLock);
 
 						WaitForSoundEnd (TFBSOUND_WAIT_ALL);
 
@@ -1692,14 +1781,15 @@ DoMelee (PMELEE_STATE pMS)
 						Battle ();
 						free_gravity_well ();
 
-						if (GLOBAL (CurrentActivity) & CHECK_ABORT) return (FALSE);
+						if (GLOBAL (CurrentActivity) & CHECK_ABORT)
+							return (FALSE);
 
 						{
 							BYTE black_buf[] = {FadeAllToBlack};
 						
 							SleepThreadUntil (XFormColorMap (
-									(COLORMAPPTR)black_buf, ONE_SECOND / 2
-									) + ONE_SECOND / 60);
+									(COLORMAPPTR)black_buf, ONE_SECOND / 2)
+									+ ONE_SECOND / 60);
 							FlushColorXForms ();
 						}
 					} while (0 /* !(GLOBAL (CurrentActivity) & CHECK_ABORT) */);
@@ -2053,8 +2143,10 @@ Melee (void)
 		MenuState.side = 0;
 		MenuState.star_bucks[0] = GetTeamValue (&MenuState.TeamImage[0]);
 		MenuState.star_bucks[1] = GetTeamValue (&MenuState.TeamImage[1]);
+		SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
 		DoInput ((PVOID)&MenuState, TRUE);
 
+		StopMusic ();
 		WaitForSoundEnd (TFBSOUND_WAIT_ALL);
 
 		{
