@@ -16,16 +16,18 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "starcon.h"
-#include "libs/graphics/gfx_common.h"
 #include "options.h"
+#include "colors.h"
 #include "controls.h"
+#include "gameopt.h"
+#include "gamestr.h"
+#include "resinst.h"
+#include "settings.h"
+#include "setup.h"
+#include "sis.h"
+#include "sounds.h"
+#include "libs/graphics/gfx_common.h"
 
-//Added by Chris
-
-void GetGaugeRect (PRECT pRect, BOOLEAN IsCrewRect);
-
-//End Added by Chris
 
 enum
 {
@@ -36,6 +38,7 @@ enum
 	OUTFIT_DOFUEL
 };
 
+
 static void
 DrawModuleStrings (PMENU_STATE pMS, BYTE NewModule)
 {
@@ -165,8 +168,6 @@ DisplayLanders (PMENU_STATE pMS)
 static BOOLEAN
 DoInstallModule (PMENU_STATE pMS)
 {
-	extern void DrawFlagshipStats (void);
-
 	BYTE NewState, new_slot_piece, old_slot_piece;
 	SIZE FirstItem, LastItem;
 	BOOLEAN select, cancel, motion;
@@ -177,9 +178,14 @@ DoInstallModule (PMENU_STATE pMS)
 		return (TRUE);
 	}
 
-	select = CurrentMenuState.select;
-	cancel = CurrentMenuState.cancel;
-	motion = CurrentMenuState.left || CurrentMenuState.right || CurrentMenuState.up || CurrentMenuState.down;
+	select = PulsedInputState.key[KEY_MENU_SELECT];
+	cancel = PulsedInputState.key[KEY_MENU_CANCEL];
+	motion = PulsedInputState.key[KEY_MENU_LEFT] ||
+			PulsedInputState.key[KEY_MENU_RIGHT] ||
+			PulsedInputState.key[KEY_MENU_UP] ||
+			PulsedInputState.key[KEY_MENU_DOWN];
+
+	SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
 
 	FirstItem = 0;
 	switch (NewState = pMS->CurState)
@@ -220,7 +226,7 @@ DoInstallModule (PMENU_STATE pMS)
 
 		pMS->InputFunc = DoInstallModule;
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 
 		SetContext (SpaceContext);
 		ClearSISRect (CLEAR_SIS_RADAR);
@@ -270,7 +276,7 @@ DoInstallModule (PMENU_STATE pMS)
 			}
 		}
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		SetContext (SpaceContext);
 
 		SetFlashRect (NULL_PTR, (FRAME)0);
@@ -354,15 +360,15 @@ DoInstallModule (PMENU_STATE pMS)
 		else
 		{
 			SetContext (StatusContext);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			DrawMenuStateStrings (PM_FUEL, pMS->CurState = OUTFIT_MODULES);
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			SetFlashRect ((PRECT)~0L, (FRAME)0);
 
 			pMS->InputFunc = DoOutfit;
 			ClearSISRect (DRAW_SIS_DISPLAY);
 		}
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 	}
 	else if (motion)
 	{
@@ -371,9 +377,9 @@ DoInstallModule (PMENU_STATE pMS)
 		NewItem = NewState < EMPTY_SLOT ? pMS->CurState : pMS->delta_item;
 		do
 		{
-			if (NewState >= EMPTY_SLOT && (CurrentMenuState.up || CurrentMenuState.down))
+			if (NewState >= EMPTY_SLOT && (PulsedInputState.key[KEY_MENU_UP] || PulsedInputState.key[KEY_MENU_DOWN]))
 			{
-				if (CurrentMenuState.up)
+				if (PulsedInputState.key[KEY_MENU_UP])
 				{
 					if (NewState-- == EMPTY_SLOT)
 						NewState = EMPTY_SLOT + 3;
@@ -387,18 +393,21 @@ DoInstallModule (PMENU_STATE pMS)
 				if (GET_GAME_STATE (CHMMR_BOMB_STATE) == 3)
 				{
 					if (NewState == EMPTY_SLOT + 3)
-						NewState = CurrentMenuState.up ? EMPTY_SLOT + 2 : EMPTY_SLOT;
+						NewState = PulsedInputState.key[KEY_MENU_UP] ?
+								EMPTY_SLOT + 2 : EMPTY_SLOT;
 					if (NewState == EMPTY_SLOT + 2)
 						NewItem = NUM_BOMB_MODULES;
 				}
 				pMS->delta_item = NewItem;
 			}
-			else if (CurrentMenuState.left || CurrentMenuState.up)
+			else if (PulsedInputState.key[KEY_MENU_LEFT] ||
+					PulsedInputState.key[KEY_MENU_UP])
 			{
 				if (NewItem-- == FirstItem)
 					NewItem = LastItem;
 			}
-			else if (CurrentMenuState.right || CurrentMenuState.down)
+			else if (PulsedInputState.key[KEY_MENU_RIGHT] ||
+					PulsedInputState.key[KEY_MENU_DOWN])
 			{
 				if (NewItem++ == LastItem)
 					NewItem = FirstItem;
@@ -408,7 +417,7 @@ DoInstallModule (PMENU_STATE pMS)
 				|| (NewItem >= GUN_WEAPON && NewItem <= CANNON_WEAPON
 				&& pMS->delta_item > 0 && pMS->delta_item < 13)));
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		if (NewState < EMPTY_SLOT)
 		{
 			if (NewItem != pMS->CurState)
@@ -505,21 +514,22 @@ InitFlash:
 			else
 				SetFlashRect (&pMS->flash_rect0, (FRAME)0);
 		}
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 	}
 
 	return (TRUE);
 }
-void
-ChangeFuelQuantity ()
+
+static void
+ChangeFuelQuantity (void)
 {
 	RECT r;
 	
 	r.extent.height = 1;
 	
-	if (CurrentMenuState.up)
+	if (PulsedInputState.key[KEY_MENU_UP])
 	{
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		SetContext (SpaceContext);
 		if (GetFTankCapacity (&r.corner) > GLOBAL_SIS (FuelOnBoard)
 			&& GLOBAL_SIS (ResUnits) >=
@@ -541,11 +551,11 @@ ChangeFuelQuantity ()
 			GetGaugeRect (&r, FALSE);
 			SetFlashRect (&r, (FRAME)0);
 		}
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 	}
-	else if (CurrentMenuState.down)
+	else if (PulsedInputState.key[KEY_MENU_DOWN])
 	{
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		SetContext (SpaceContext);
 		if (GLOBAL_SIS (FuelOnBoard))
 		{
@@ -563,7 +573,7 @@ ChangeFuelQuantity ()
 		SetContext (StatusContext);
 		GetGaugeRect (&r, FALSE);
 		SetFlashRect (&r, (FRAME)0);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 	}
 }
 
@@ -581,26 +591,21 @@ DoOutfit (PMENU_STATE pMS)
 		{
 			COUNT num_frames;
 			STAMP s;
-			extern void DrawFlagshipName (BOOLEAN InStatusArea);
-			extern void DrawFlagshipStats (void);
 
 			pMS->CurFrame = CaptureDrawable (
-					LoadGraphic (MODULES_PMAP_ANIM)
-					);
+					LoadGraphic (MODULES_PMAP_ANIM));
 			pMS->hMusic = LoadMusicInstance (OUTFIT_MUSIC);
 			pMS->CurState = OUTFIT_FUEL;
 			pMS->ModuleFrame = CaptureDrawable (
-					LoadGraphic (SISMODS_MASK_PMAP_ANIM)
-					);
+					LoadGraphic (SISMODS_MASK_PMAP_ANIM));
 			s.origin.x = s.origin.y = 0;
 			s.frame = CaptureDrawable (
-					LoadGraphic (OUTFIT_PMAP_ANIM)
-					);
+					LoadGraphic (OUTFIT_PMAP_ANIM));
 
 			SetTransitionSource (NULL);
 			BatchGraphics ();
 			DrawSISFrame ();
-			SetSemaphore(GraphicsSem);
+			LockMutex (GraphicsLock);
 			DrawSISMessage (GAME_STRING (STARBASE_STRING_BASE + 2));
 			DrawSISTitle (GAME_STRING (STARBASE_STRING_BASE));
 
@@ -613,24 +618,24 @@ DoOutfit (PMENU_STATE pMS)
 			{
 				BYTE which_piece;
 
-				if ((which_piece =
-						GLOBAL_SIS (DriveSlots[num_frames])) < EMPTY_SLOT)
+				which_piece = GLOBAL_SIS (DriveSlots[num_frames]);
+				if (which_piece < EMPTY_SLOT)
 					DrawShipPiece (pMS, which_piece, num_frames, FALSE);
 			}
 			for (num_frames = 0; num_frames < NUM_JET_SLOTS; ++num_frames)
 			{
 				BYTE which_piece;
 
-				if ((which_piece =
-						GLOBAL_SIS (JetSlots[num_frames])) < EMPTY_SLOT)
+				which_piece = GLOBAL_SIS (JetSlots[num_frames]);
+				if (which_piece < EMPTY_SLOT)
 					DrawShipPiece (pMS, which_piece, num_frames, FALSE);
 			}
 			for (num_frames = 0; num_frames < NUM_MODULE_SLOTS; ++num_frames)
 			{
 				BYTE which_piece;
 
-				if ((which_piece =
-						GLOBAL_SIS (ModuleSlots[num_frames])) < EMPTY_SLOT)
+				which_piece = GLOBAL_SIS (ModuleSlots[num_frames]);
+				if (which_piece < EMPTY_SLOT)
 					DrawShipPiece (pMS, which_piece, num_frames, FALSE);
 			}
 			RedistributeFuel ();
@@ -656,13 +661,13 @@ DoOutfit (PMENU_STATE pMS)
 					DrawStamp ((PSTAMP)&s);
 			}
 
-			ClearSemaphore(GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			DrawMenuStateStrings (PM_FUEL, pMS->CurState);
-			SetSemaphore(GraphicsSem);
+			LockMutex (GraphicsLock);
 			DrawFlagshipName (FALSE);
 			if (optWhichFonts == OPT_PC)
 				DrawFlagshipStats ();
-			ClearSemaphore(GraphicsSem);
+			UnlockMutex (GraphicsLock);
 
 			{
 				RECT r;
@@ -676,9 +681,9 @@ DoOutfit (PMENU_STATE pMS)
 			PlayMusic (pMS->hMusic, TRUE, 1);
 			UnbatchGraphics ();
 			
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			SetFlashRect ((PRECT)~0L, (FRAME)0);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 
 			GLOBAL_SIS (FuelOnBoard) =
 					(GLOBAL_SIS (FuelOnBoard)
@@ -688,16 +693,16 @@ DoOutfit (PMENU_STATE pMS)
 
 		SetContext (StatusContext);
 	}
-	else if (CurrentMenuState.cancel
-			|| (CurrentMenuState.select
+	else if (PulsedInputState.key[KEY_MENU_CANCEL]
+			|| (PulsedInputState.key[KEY_MENU_SELECT]
 			&& pMS->CurState == OUTFIT_EXIT))
 	{
 		if (pMS->CurState == OUTFIT_DOFUEL)
 		{
 			pMS->CurState = OUTFIT_FUEL;
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			SetFlashRect ((PRECT)~0L, (FRAME)0);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 		}
 		else
 		{
@@ -710,7 +715,7 @@ ExitOutfit:
 			return (FALSE);
 		}
 	}
-	else if (CurrentMenuState.select)
+	else if (PulsedInputState.key[KEY_MENU_SELECT])
 	{
 		switch (pMS->CurState)
 		{
@@ -719,18 +724,18 @@ ExitOutfit:
 				RECT r;
 
 				pMS->CurState = OUTFIT_DOFUEL;
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 				SetContext (StatusContext);
 				GetGaugeRect (&r, FALSE);
 				SetFlashRect (&r, (FRAME)0);
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 				break;
 			}
 			case OUTFIT_DOFUEL:
 				pMS->CurState = OUTFIT_FUEL;
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 				SetFlashRect ((PRECT)~0L, (FRAME)0);
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 				break;
 			case OUTFIT_MODULES:
 				pMS->CurState = EMPTY_SLOT + 2;
@@ -746,14 +751,24 @@ ExitOutfit:
 				if (GameOptions () == 0)
 					goto ExitOutfit;
 				DrawMenuStateStrings (PM_FUEL, pMS->CurState);
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 				SetFlashRect ((PRECT)~0L, (FRAME)0);
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 				break;
 		}
 	}
 	else
 	{
+		switch (pMS->CurState)
+		{
+			case OUTFIT_DOFUEL:
+				SetMenuSounds (MENU_SOUND_UP | MENU_SOUND_DOWN, MENU_SOUND_SELECT | MENU_SOUND_CANCEL);
+				break;
+			default:
+				SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
+				break;
+		}
+
 		if (pMS->CurState == OUTFIT_DOFUEL)
 			ChangeFuelQuantity ();
 		else
