@@ -16,9 +16,11 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "starcon.h"
+#include "encount.h"
+#include "resinst.h"
 #include "planets/elemdata.h"
 
+
 STAR_DESC starmap_array[] =
 {
 	//                           postfix name index (like 'Normae')
@@ -708,11 +710,6 @@ const BYTE element_array[NUMBER_OF_ELEMENTS] =
 };
 
 /*------------------------------ Global Data ------------------------------ */
-#define NO_TECTONICS 0
-#define LOW_TECTONICS 40
-#define MED_TECTONICS 80
-#define HIGH_TECTONICS 140
-#define SUPER_TECTONICS 200
 
 #define NO_DEPOSIT 0
 
