@@ -16,11 +16,19 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "starcon.h"
+#include "controls.h"
+#include "colors.h"
+#include "encount.h"
+#include "gamestr.h"
 #include "lander.h"
 #include "lifeform.h"
+#include "nameref.h"
+#include "resinst.h"
+#include "setup.h"
+#include "sounds.h"
 #include "libs/graphics/gfx_common.h"
-#include "controls.h"
+#include "libs/mathlib.h"
+
 
 //define SPIN_ON_LAUNCH to let the planet spin while
 // the lander animation is playing
@@ -82,8 +90,11 @@ const LIFEFORM_DESC CreatureData[] =
 	{SPEED_MOTIONLESS | DANGER_WEAK, MAKE_BYTE (8, 5)},
 
 	{SPEED_MOTIONLESS | DANGER_MONSTROUS, MAKE_BYTE (1, 1)},
+			// Evil One
 	{BEHAVIOR_UNPREDICTABLE | SPEED_SLOW | DANGER_HARMLESS, MAKE_BYTE (0, 1)},
+			// Brainbox Bulldozer
 	{BEHAVIOR_HUNT | AWARENESS_HIGH | SPEED_FAST | DANGER_MONSTROUS, MAKE_BYTE (15, 15)},
+			// Zex's Beauty
 };
 
 #define FLASH_WIDTH 9
@@ -204,9 +215,8 @@ object_animation (PELEMENT ElementPtr)
 	{
 		COLOR c;
 
-		if ((c = DamageColorCycle (
-				GetPrimColor (pPrim), 0
-				)) == WHITE_COLOR)
+		c = DamageColorCycle (GetPrimColor (pPrim), 0);
+		if (c == WHITE_COLOR)
 		{
 			SetPrimType (pPrim, STAMP_PRIM);
 			if (ElementPtr->hit_points == 0)
@@ -321,6 +331,16 @@ object_animation (PELEMENT ElementPtr)
 					DetectPercent = (((BYTE)(CreatureData[index].Attributes
 							& AWARENESS_MASK) >> AWARENESS_SHIFT) + 1)
 							* (30 / 6);
+							// XXX: Shouldn't this be dependant on
+							// PLANET_SIDE_RATE somehow? And why is it
+							// written as '30 / 6' instead of 5? Does the 30
+							// specify the (PC) framerate? That doesn't make
+							// sense; I would expect it to be in the
+							// denominator. And even then, it wouldn't give
+							// the same results with different frame rates,
+							// as repeating 'random(x / 30)' 30 times doesn't
+							// generally have the same result as repeating
+							// 'random(x / 35)' 25 times. - SvdB
 					if (((BYTE)TFB_Random () % 100) < DetectPercent)
 					{
 						ElementPtr->thrust_wait = 0;
@@ -383,11 +403,8 @@ object_animation (PELEMENT ElementPtr)
 						break;
 				}
 
-				SetVelocityComponents (
-						&ElementPtr->velocity,
-						COSINE (angle, speed),
-						SINE (angle, speed)
-						);
+				SetVelocityComponents (&ElementPtr->velocity,
+						COSINE (angle, speed), SINE (angle, speed));
 			}
 		}
 	}
@@ -611,7 +628,7 @@ CheckObjectCollision (COUNT index)
 									&& CurStarDescPtr->Index != ANDROSYNTH_DEFINED)
 							{
 								UnbatchGraphics ();
-								DoDiscoveryReport (pPSD->OldMenuSounds);
+								DoDiscoveryReport (MenuSounds);
 								BatchGraphics ();
 							}
 							if (ElementPtr->mass_points == 0)
@@ -705,8 +722,7 @@ CheckObjectCollision (COUNT index)
 								SetPrimColor (pPrim, WHITE_COLOR);
 
 								PlaySound (SetAbsSoundIndex (
-										LanderSounds, LANDER_HITS
-										), NotPositional (), NULL, GAME_SOUND_PRIORITY);
+										LanderSounds, LANDER_HITS), NotPositional (), NULL, GAME_SOUND_PRIORITY);
 							}
 							UnlockElement (hElement);
 							break;
@@ -726,10 +742,8 @@ CheckObjectCollision (COUNT index)
 								{
 									if (pPSD->ElementLevel
 											+ NumRetrieved > pPSD->MaxElementLevel)
-										NumRetrieved = (COUNT)(
-												pPSD->MaxElementLevel
-												- pPSD->ElementLevel
-												);
+										NumRetrieved = (COUNT)(pPSD->MaxElementLevel
+												- pPSD->ElementLevel);
 									FillLanderHold (pPSD, scan, NumRetrieved);
 									if (scan == MINERAL_SCAN)
 									{
@@ -1215,7 +1229,7 @@ ScrollPlanetSide (SIZE dx, SIZE dy, SIZE CountDown)
 		pSolarSysState->MenuState.flash_rect1.corner.y = HIWORD (TimeIn);
 	}
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	OldContext = SetContext (SpaceContext);
 
 	BatchGraphics ();
@@ -1329,7 +1343,7 @@ ScrollPlanetSide (SIZE dx, SIZE dy, SIZE CountDown)
 	UnbatchGraphics ();
 
 	SetContext (OldContext);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 }
 
 static void
@@ -1353,7 +1367,7 @@ AnimateLaunch (FRAME farray, BOOLEAN ShowPlanetSide)
 		RepairBackRect (&r);
 		DrawStamp (&s);
 
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 
 #if 0
 		if (ShowPlanetSide)
@@ -1370,7 +1384,7 @@ AnimateLaunch (FRAME farray, BOOLEAN ShowPlanetSide)
 		SleepThreadUntil (Time + (ONE_SECOND / 22));
 		Time = GetTimeCounter ();
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 	} while (--num_frames);
 
 	GetFrameRect (s.frame, &r);
@@ -1386,9 +1400,9 @@ InitPlanetSide (void)
 	CONTEXT OldContext;
 	DWORD Time;
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	OldContext = SetContext (RadarContext);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 	Time = GetTimeCounter ();
 
@@ -1397,9 +1411,9 @@ InitPlanetSide (void)
 	s.frame = SetAbsFrameIndex (LanderFrame[0],
 			(ANGLE_TO_FACING (FULL_CIRCLE) << 1) + 1);
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	DrawStamp (&s);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 	SleepThread (ONE_SECOND / 15);
 	Time = GetTimeCounter ();
@@ -1409,10 +1423,10 @@ InitPlanetSide (void)
 	{
 		SleepThreadUntil (Time + ONE_SECOND / 30);
 		Time = GetTimeCounter ();
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		DeltaSISGauges (-1, 0, 0);
 		DeltaLanderCrew (1, 0);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 	}
 
 	SleepThreadUntil (Time + (ONE_SECOND / 15));
@@ -1423,9 +1437,9 @@ InitPlanetSide (void)
 	else
 		s.frame = SetAbsFrameIndex (s.frame,
 				(ANGLE_TO_FACING (FULL_CIRCLE) << 1) + 2);
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	DrawStamp (&s);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 	SleepThreadUntil (Time + (ONE_SECOND / 15));
 	Time = GetTimeCounter ();
@@ -1436,18 +1450,18 @@ InitPlanetSide (void)
 	{
 		s.frame = SetAbsFrameIndex (s.frame,
 				(ANGLE_TO_FACING (FULL_CIRCLE) << 1) + 3);
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		DrawStamp (&s);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 
 		SleepThreadUntil (Time + (ONE_SECOND / 15));
 		Time = GetTimeCounter ();
 
 		s.frame = IncFrameIndex (s.frame);
 	}
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	DrawStamp (&s);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 	if (GET_GAME_STATE (IMPROVED_LANDER_CARGO))
 	{
@@ -1455,25 +1469,25 @@ InitPlanetSide (void)
 		Time = GetTimeCounter ();
 
 		s.frame = SetAbsFrameIndex (s.frame, 59);
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		DrawStamp (&s);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 	}
 #ifndef SPIN_ON_LAUNCH
 	pSolarSysState->PauseRotate = 1;
 #endif
 	SleepThreadUntil (Time + (ONE_SECOND / 15));
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	PlaySound (SetAbsSoundIndex (LanderSounds, LANDER_DEPARTS),
 			NotPositional (), NULL, GAME_SOUND_PRIORITY + 1);
 	SetContext (SpaceContext);
 	AnimateLaunch (LanderFrame[5], FALSE);
 #ifdef SPIN_ON_LAUNCH
 	pSolarSysState->PauseRotate = 1;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 	TaskSwitch ();
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 #endif
 	// Adjust pSolarSysState->MenuState.first_item by a random jitter.
 #define RANDOM_MISS 64
@@ -1547,7 +1561,7 @@ InitPlanetSide (void)
 
 	SetContext (OldContext);
 
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 	SET_GAME_STATE (PLANETARY_LANDING, 1);
 }
@@ -1592,7 +1606,7 @@ SetVelocityComponents (
 	}
 	else if (pMS->delta_item == 0
 			|| (HIBYTE (pMS->delta_item) 
-			&& (CurrentInputState.lander_escape
+			&& (CurrentInputState.key[KEY_LANDER_ESCAPE]
 			|| ((PPLANETSIDE_DESC)pMenuState->ModuleFrame)->InTransit)))
 	{
 		if (pMS->delta_item || pMS->CurState > EXPLOSION_LIFE + 60)
@@ -1653,9 +1667,9 @@ SetVelocityComponents (
 			index = GetFrameIndex (LanderFrame[0]);
 			if (LONIBBLE (pMS->CurState))
 				pMS->CurState -= MAKE_BYTE (1, 0);
-			else if (CurrentInputState.lander_left || CurrentInputState.lander_right)
+			else if (CurrentInputState.key[KEY_LANDER_LEFT] || CurrentInputState.key[KEY_LANDER_RIGHT])
 			{
-				if (CurrentInputState.lander_left)
+				if (CurrentInputState.key[KEY_LANDER_LEFT])
 				{
 					dx = -1;
 					--index;
@@ -1695,7 +1709,7 @@ SetVelocityComponents (
 						);
 			}
 
-			if (!CurrentInputState.lander_thrust)
+			if (!CurrentInputState.key[KEY_LANDER_THRUST])
 				dx = dy = 0;
 			else
 				GetNextVelocityComponents (
@@ -1704,7 +1718,7 @@ SetVelocityComponents (
 
 			if (HINIBBLE (pMS->CurState))
 				pMS->CurState -= MAKE_BYTE (0, 1);
-			else if (CurrentInputState.lander_weapon)
+			else if (CurrentInputState.key[KEY_LANDER_WEAPON])
 			{
 				HELEMENT hWeaponElement;
 
@@ -1733,7 +1747,7 @@ SetVelocityComponents (
 							index + ANGLE_TO_FACING (FULL_CIRCLE)
 							);
 
-					if (!CurrentInputState.lander_thrust)
+					if (!CurrentInputState.key[KEY_LANDER_THRUST])
 						wdx = wdy = 0;
 					else
 						GetCurrentVelocityComponents (
@@ -1798,39 +1812,22 @@ LoadLanderData (void)
 	if (LanderFrame[0] == 0)
 	{
 		LanderFrame[0] =
-				CaptureDrawable (
-				LoadGraphic (LANDER_MASK_PMAP_ANIM)
-				);
+				CaptureDrawable (LoadGraphic (LANDER_MASK_PMAP_ANIM));
 		LanderFrame[1] =
-				CaptureDrawable (
-				LoadGraphic (QUAKE_MASK_PMAP_ANIM)
-				);
+				CaptureDrawable (LoadGraphic (QUAKE_MASK_PMAP_ANIM));
 		LanderFrame[2] =
-				CaptureDrawable (
-				LoadGraphic (LIGHTNING_MASK_ANIM)
-				);
+				CaptureDrawable (LoadGraphic (LIGHTNING_MASK_ANIM));
 		LanderFrame[3] =
-				CaptureDrawable (
-				LoadGraphic (LAVA_MASK_PMAP_ANIM)
-				);
-		LanderFrame[4] =
-				CaptureDrawable (
-				LoadGraphic (LANDER_SHIELD_MASK_ANIM)
-				);
+				CaptureDrawable (LoadGraphic (LAVA_MASK_PMAP_ANIM));
+		LanderFrame[4] = CaptureDrawable (LoadGraphic (LANDER_SHIELD_MASK_ANIM));
 		LanderFrame[5] =
-				CaptureDrawable (
-				LoadGraphic (LANDER_LAUNCH_MASK_PMAP_ANIM)
-				);
+				CaptureDrawable (LoadGraphic (LANDER_LAUNCH_MASK_PMAP_ANIM));
 		LanderFrame[6] =
-				CaptureDrawable (
-				LoadGraphic (LANDER_RETURN_MASK_PMAP_ANIM)
-				);
+				CaptureDrawable (LoadGraphic (LANDER_RETURN_MASK_PMAP_ANIM));
 		LanderSounds = CaptureSound (LoadSound (LANDER_SOUNDS));
 
 		LanderFrame[7] =
-				CaptureDrawable (
-				LoadGraphic (ORBIT_VIEW_ANIM)
-				);
+				CaptureDrawable (LoadGraphic (ORBIT_VIEW_ANIM));
 
 		{
 			COUNT i;
@@ -1858,7 +1855,7 @@ ReturnToOrbit (PRECT pRect)
 {
 	CONTEXT OldContext;
 	
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	OldContext = SetContext (SpaceContext);
 	SetContextClipRect (pRect);
 
@@ -1873,7 +1870,7 @@ ReturnToOrbit (PRECT pRect)
 
 	LoadIntoExtraScreen (pRect);
 	SetContext (OldContext);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 //	SetPlanetTilt (0);
 }
@@ -1890,8 +1887,6 @@ PlanetSide (PMENU_STATE pMS)
 
 	memset ((PPLANETSIDE_DESC)&PSD, 0, sizeof (PSD));
 	PSD.InTransit = TRUE;
-	PSD.OldMenuSounds = MenuSounds;
-	MenuSounds = 0;
 
 	PSD.TectonicsChance =
 			TectonicsChanceTab[pSolarSysState->SysInfo.PlanetInfo.Tectonics];
@@ -1960,7 +1955,7 @@ PlanetSide (PMENU_STATE pMS)
 		else
 			TimeIn += ONE_SECOND / 60;
 
-		do 
+		do
 		{
 			ScrollPlanetSide (0, 0, index);
 		} while (GetTimeCounter () < TimeIn);
@@ -1973,6 +1968,7 @@ PlanetSide (PMENU_STATE pMS)
 
 	pMS->Initialized = FALSE;
 	pMS->InputFunc = DoPlanetSide;
+	SetMenuSounds (MENU_SOUND_NONE, MENU_SOUND_NONE);
 	DoInput ((PVOID)pMS, FALSE);
 
 	if (!(GLOBAL (CurrentActivity) & CHECK_ABORT))
@@ -1994,9 +1990,9 @@ PlanetSide (PMENU_STATE pMS)
 #endif /* NEVER */
 
 			--GLOBAL_SIS (NumLanders);
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			DrawLanders ();
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 
 			ReturnToOrbit (&r);
 		}
@@ -2029,7 +2025,7 @@ PlanetSide (PMENU_STATE pMS)
 			SleepThread (1);
 #endif
 
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			SetContext (SpaceContext);
 
 			AnimateLaunch (LanderFrame[6], TRUE);
@@ -2047,7 +2043,7 @@ PlanetSide (PMENU_STATE pMS)
 				DrawStorageBays (FALSE);
 			}
 
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 
 			GLOBAL_SIS (TotalBioMass) += PSD.BiologicalLevel;
 		}
@@ -2079,7 +2075,7 @@ PlanetSide (PMENU_STATE pMS)
 	}
 
 	ZeroVelocityComponents (&GLOBAL (velocity));
-	MenuSounds = PSD.OldMenuSounds;
+	SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
 	
 	pSolarSysState->MenuState.Initialized -= 4;
 	pSolarSysState->PauseRotate = 0;
@@ -2090,7 +2086,7 @@ InitLander (BYTE LanderFlags)
 {
 	RECT r;
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 
 	SetContext (RadarContext);
 	
@@ -2182,5 +2178,5 @@ InitLander (BYTE LanderFlags)
 
 	UnbatchGraphics ();
 
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 }
