@@ -16,7 +16,13 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "starcon.h"
+#include "encount.h"
+#include "planets/planets.h"
+#include "libs/compiler.h"
+#include "libs/mathlib.h"
+
+
+//#define DEBUG_ORBITS
 
 enum
 {
@@ -479,11 +485,11 @@ FillOrbits (BYTE NumPlanets, PPLANET_DESC pBaseDesc, BOOLEAN
 		{GIANT_ROCK_DIST, GIANT_GASG_DIST},
 		{SUPERGIANT_ROCK_DIST, SUPERGIANT_GASG_DIST},
 	};
-#ifdef DEBUG
+#ifdef DEBUG_ORBITS
 UNICODE buf[40];
 char stype[] = {'D', 'G', 'S'};
 char scolor[] = {'B', 'G', 'O', 'R', 'W', 'Y'};
-#endif /* DEBUG */
+#endif /* DEBUG_ORBITS */
 
 	pPD = pBaseDesc;
 	StarSize = pSolarSysState->SunDesc[0].data_index;
@@ -498,12 +504,12 @@ char scolor[] = {'B', 'G', 'O', 'R', 'W', 'Y'};
 		pSolarSysState->SunDesc[0].NumPlanets = NumPlanets;
 	}
 
-#ifdef DEBUG
+#ifdef DEBUG_ORBITS
 	GetClusterName (CurStarDescPtr, buf);
 	fprintf (stderr, "cluster name = %s  color = %c type = %c\n", buf,
 			scolor[STAR_COLOR (CurStarDescPtr->Type)],
 			stype[STAR_TYPE (CurStarDescPtr->Type)]);
-#endif /* DEBUG */
+#endif /* DEBUG_ORBITS */
 	GeneratingMoons = (BOOLEAN) (pBaseDesc == pSolarSysState->MoonDesc);
 	if (GeneratingMoons)
 		MaxPlanet = FIRST_LARGE_ROCKY_WORLD;
