@@ -16,11 +16,22 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "starcon.h"
-#include "libs/graphics/gfx_common.h"
-#include "options.h"
+#include "colors.h"
 #include "controls.h"
+#include "encount.h"
+#include "gameopt.h"
+#include "gamestr.h"
+#include "globdata.h"
+#include "options.h"
+#include "settings.h"
+#include "setup.h"
+#include "sounds.h"
+#include "state.h"
+#include "uqmdebug.h"
 #include "libs/inplib.h"
+#include "libs/graphics/gfx_common.h"
+#include "libs/mathlib.h"
+
 
 //Added by Chris
 
@@ -47,8 +58,8 @@ void RepairBackRect (PRECT pRect);
 
 static BOOLEAN transition_pending;
 
-int
-flash_cursor_func(void *data)
+static int
+flash_cursor_func (void *data)
 {
 	BYTE c, val;
 	POINT universe;
@@ -73,7 +84,7 @@ flash_cursor_func(void *data)
 		CONTEXT OldContext;
 
 		TimeIn = GetTimeCounter ();
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		OldContext = SetContext (SpaceContext);
 
 		if (c == 0x00 || c == 0x1A)
@@ -86,7 +97,7 @@ flash_cursor_func(void *data)
 		SetContextForeGroundColor (OldColor);
 
 		SetContext (OldContext);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 		SleepThreadUntil (TimeIn + (ONE_SECOND >> 4));
 	}
 	FinishTask (task);
@@ -254,7 +265,7 @@ DrawStarMap (COUNT race_update, PRECT pClipRect)
 	}
 	else
 	{
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		draw_cursor = TRUE;
 	}
 
@@ -528,7 +539,7 @@ wprintf ("%s\n", buf);
 	}
 
 	if (draw_cursor)
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 }
 
 static void
@@ -558,9 +569,9 @@ EraseCursor (COORD curs_x, COORD curs_y)
 #else /* NEW */
 	r.extent.height += r.corner.y & 1;
 	r.corner.y &= ~1;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 	DrawStarMap (0, &r);
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 #endif /* OLD */
 }
 
@@ -628,7 +639,7 @@ DoMoveCursor (PMENU_STATE pMS)
 		buf[0] = '\0';
 		goto UpdateCursorInfo;
 	}
-	else if (CurrentMenuState.cancel)
+	else if (PulsedInputState.key[KEY_MENU_CANCEL])
 	{
 		if (pMS->flash_task)
 		{
@@ -638,10 +649,23 @@ DoMoveCursor (PMENU_STATE pMS)
 
 		return (FALSE);
 	}
-	else if (CurrentMenuState.select)
+	else if (PulsedInputState.key[KEY_MENU_SELECT])
 	{
 		GLOBAL (autopilot) = pMS->first_item;
-
+#ifdef DEBUG
+		if (instantMove) {
+			GLOBAL_SIS (log_x) = UNIVERSE_TO_LOGX(pMS->first_item.x);
+			GLOBAL_SIS (log_y) = UNIVERSE_TO_LOGY(pMS->first_item.y);
+			PlaySoundEffect (SetAbsSoundIndex (MenuSounds, 3), 0,
+					NotPositional (), NULL, GAME_SOUND_PRIORITY);
+			if (pMS->flash_task)
+			{
+				ConcludeTask (pMS->flash_task);
+				pMS->flash_task = 0;
+			}
+			return (FALSE);
+		}
+#endif
 		DrawStarMap (0, NULL_PTR);
 	}
 	else
@@ -651,18 +675,18 @@ DoMoveCursor (PMENU_STATE pMS)
 		SIZE ZoomIn, ZoomOut;
 
 		ZoomIn = ZoomOut = 0;
-		if (CurrentMenuState.zoom_in)
+		if (PulsedInputState.key[KEY_MENU_ZOOM_IN])
 			ZoomIn = 1;
-		else if (CurrentMenuState.zoom_out)
+		else if (PulsedInputState.key[KEY_MENU_ZOOM_OUT])
 			ZoomOut = 1;
 
 		ZoomStarMap (ZoomIn - ZoomOut);
 
 		sx = sy = 0;
-		if (CurrentMenuState.left)    sx =   -1;
-		if (CurrentMenuState.right)   sx =    1;
-		if (CurrentMenuState.up)      sy =   -1;
-		if (CurrentMenuState.down)    sy =    1;
+		if (PulsedInputState.key[KEY_MENU_LEFT])    sx =   -1;
+		if (PulsedInputState.key[KEY_MENU_RIGHT])   sx =    1;
+		if (PulsedInputState.key[KEY_MENU_UP])      sy =   -1;
+		if (PulsedInputState.key[KEY_MENU_DOWN])    sy =    1;
 
 		if (sx == 0 && sy == 0)
 		{
@@ -730,11 +754,11 @@ DoMoveCursor (PMENU_STATE pMS)
 			}
 			else
 			{
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 				EraseCursor (pt.x, pt.y);
 				// ClearDrawable ();
 				DrawCursor (s.origin.x, s.origin.y);
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 			}
 
 UpdateCursorInfo:
@@ -797,8 +821,8 @@ UpdateCursorInfo:
 				if (f == 0 || GET_GAME_STATE (ARILOU_SPACE_SIDE) > 1)
 					fuel_required = 0;
 				else
-					fuel_required = square_root (f) + (FUEL_TANK_SCALE >> 1);
-				SetSemaphore (GraphicsSem);
+					fuel_required = square_root (f) + (FUEL_TANK_SCALE / 20);
+				LockMutex (GraphicsLock);
 				DrawHyperCoords (pMenuState->first_item);
 				if (last_buf == (UNICODE) ~0
 						|| (buf[0] && !last_buf) || (!buf[0] && last_buf))
@@ -811,7 +835,7 @@ UpdateCursorInfo:
 						fuel_required / FUEL_TANK_SCALE,
 						(fuel_required % FUEL_TANK_SCALE) / 10);
 				DrawStatusMessage (buf);
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 			}
 		}
 	}
@@ -1078,7 +1102,6 @@ DoneSphereGrowth:
 static BOOLEAN
 DoStarMap (void)
 {
-	SOUND OldMenuSounds;
 	MENU_STATE MenuState;
 	POINT universe;
 	//FRAME OldFrame;
@@ -1106,7 +1129,7 @@ DoStarMap (void)
 	if (MenuState.first_item.x == ~0 && MenuState.first_item.y == ~0)
 		MenuState.first_item = universe;
 
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 	TaskSwitch ();
 
 	MenuState.InputFunc = DoMoveCursor;
@@ -1117,7 +1140,7 @@ DoStarMap (void)
 	if (GET_GAME_STATE (ARILOU_SPACE_SIDE) <= 1)
 		UpdateMap ();
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	
 	DrawStarMap (0, (PRECT)-1);
 	transition_pending = FALSE;
@@ -1132,17 +1155,15 @@ DoStarMap (void)
 			UNIVERSE_TO_DISPY (pMenuState->first_item.y)
 			);
 	UnbatchGraphics ();
-	ClearSemaphore (GraphicsSem);
-
-	OldMenuSounds = MenuSounds;
-	MenuSounds = 0;
+	UnlockMutex (GraphicsLock);
 
+	SetMenuSounds (MENU_SOUND_NONE, MENU_SOUND_NONE);
 	DoInput ((PVOID)&MenuState, FALSE);
-	MenuSounds = OldMenuSounds;
+	SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
 
 	pMenuState = 0;
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 
 	DrawHyperCoords (universe);
 	DrawSISMessage (NULL_PTR);
@@ -1159,32 +1180,40 @@ DoStarMap (void)
 BOOLEAN
 DoFlagshipCommands (PMENU_STATE pMS)
 {
+	/* TODO: Make this carried cleanly by MENU_STATE structure */
+	// static DWORD NextTime;
 	if (!(pMS->Initialized & 1))
 	{
+		/* This has some dependency on the IPtask_func */
 		ChangeSolarSys ();
+		// NextTime = GetTimeCounter ();
 	}
 	else
 	{
-		BOOLEAN select = CurrentMenuState.select;
-		SetSemaphore (GraphicsSem);
-		while (*(volatile BYTE *)&pMS->CurState == 0
+		BOOLEAN select = PulsedInputState.key[KEY_MENU_SELECT];
+		LockMutex (GraphicsLock);
+		if (*(volatile BYTE *)&pMS->CurState == 0
 				&& (*(volatile SIZE *)&pMS->Initialized & 1)
 				&& !(GLOBAL (CurrentActivity)
 				& (START_ENCOUNTER | END_INTERPLANETARY
 				| CHECK_ABORT | CHECK_LOAD))
 				&& GLOBAL_SIS (CrewEnlisted) != (COUNT)~0)
 		{
-			ClearSemaphore (GraphicsSem);
-			TaskSwitch ();
-			SetSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
+			IP_frame ();
+			return TRUE;
 		}
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 
 		if (pMS->CurState)
 		{
 			BOOLEAN DoMenu;
 			BYTE NewState;
-			// For some reason, DoFlagshipCommands uses CurState a bit differently
+
+			/* If pMS->CurState == 0, then we're flying
+			 * around in interplanetary.  This needs to be
+			 * corrected for the MenuChooser, which thinks
+			 * that "0" is the first menu option */
 			pMS->CurState --;
 			DoMenu = DoMenuChooser (pMS, 
 				(BYTE)(pMS->Initialized <= 1 ? PM_STARMAP : PM_SCAN));
@@ -1199,9 +1228,9 @@ DoFlagshipCommands (PMENU_STATE pMS)
 				{
 					if (NewState != SCAN + 1 && NewState != (GAME_MENU) + 1)
 					{
-						SetSemaphore (GraphicsSem);
+						LockMutex (GraphicsLock);
 						SetFlashRect (NULL_PTR, (FRAME)0);
-						ClearSemaphore (GraphicsSem);
+						UnlockMutex (GraphicsLock);
 					}
 
 					switch (NewState - 1)
@@ -1247,18 +1276,19 @@ DoFlagshipCommands (PMENU_STATE pMS)
 						{
 							BOOLEAN AutoPilotSet;
 
-							SetSemaphore (GraphicsSem);
+							LockMutex (GraphicsLock);
 							if (++pMS->Initialized > 3) {
 								pSolarSysState->PauseRotate = 1;
 								RepairSISBorder ();
 							}
 
 							AutoPilotSet = DoStarMap ();
+							SetDefaultMenuRepeatDelay ();
 
 							if (LOBYTE (GLOBAL (CurrentActivity)) == IN_HYPERSPACE
 									|| (GLOBAL (CurrentActivity) & CHECK_ABORT))
 							{
-								ClearSemaphore (GraphicsSem);
+								UnlockMutex (GraphicsLock);
 								return (FALSE);
 							}
 							else if (pMS->Initialized <= 3)
@@ -1266,16 +1296,16 @@ DoFlagshipCommands (PMENU_STATE pMS)
 								ZoomSystem ();
 								--pMS->Initialized;
 							}
-							ClearSemaphore (GraphicsSem);
+							UnlockMutex (GraphicsLock);
 
 							if (!AutoPilotSet && pMS->Initialized >= 3)
 							{
 								LoadPlanet (FALSE);
 								--pMS->Initialized;
 								pSolarSysState->PauseRotate = 0;
-								SetSemaphore (GraphicsSem);
+								LockMutex (GraphicsLock);
 								SetFlashRect ((PRECT)~0L, (FRAME)0);
-								ClearSemaphore (GraphicsSem);
+								UnlockMutex (GraphicsLock);
 								break;
 							}
 						}
@@ -1291,15 +1321,17 @@ DoFlagshipCommands (PMENU_STATE pMS)
 							else if (pMS->flash_task)
 							{
 								FreePlanet ();
-								SetSemaphore (GraphicsSem);
+								LockMutex (GraphicsLock);
 								LoadSolarSys ();
+								ValidateOrbits ();
 								ZoomSystem ();
-								ClearSemaphore (GraphicsSem);
+								UnlockMutex (GraphicsLock);
 							}
 
-							SetSemaphore (GraphicsSem);
+							LockMutex (GraphicsLock);
 							pMS->CurState = 0;
-							ClearSemaphore (GraphicsSem);
+							FlushInput ();
+							UnlockMutex (GraphicsLock);
 							break;
 					}
 				
@@ -1307,9 +1339,9 @@ DoFlagshipCommands (PMENU_STATE pMS)
 						;
 					else if (pMS->CurState)
 					{
-						SetSemaphore (GraphicsSem);
+						LockMutex (GraphicsLock);
 						SetFlashRect ((PRECT)~0L, (FRAME)0);
-						ClearSemaphore (GraphicsSem);
+						UnlockMutex (GraphicsLock);
 						if (select)
 						{
 							if (optWhichMenu != OPT_PC)
@@ -1320,9 +1352,9 @@ DoFlagshipCommands (PMENU_STATE pMS)
 					}
 					else
 					{
-						SetSemaphore (GraphicsSem);
+						LockMutex (GraphicsLock);
 						SetFlashRect (NULL_PTR, (FRAME)0);
-						ClearSemaphore (GraphicsSem);
+						UnlockMutex (GraphicsLock);
 						DrawMenuStateStrings (PM_STARMAP, -NAVIGATION);
 					}
 				}
