@@ -16,11 +16,23 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "starcon.h"
-#include "lifeform.h"
-#include "libs/graphics/gfx_common.h"
-#include "options.h"
+#include "build.h"
 #include "controls.h"
+#include "encount.h"
+#include "gamestr.h"
+#include "lifeform.h"
+#include "nameref.h"
+#include "options.h"
+#include "resinst.h"
+#include "settings.h"
+#include "setup.h"
+#include "sounds.h"
+#include "state.h"
+#include "libs/graphics/gfx_common.h"
+#include "libs/inplib.h"
+#include "libs/mathlib.h"
+
+
 //Added by Chris
 
 void
@@ -68,7 +80,7 @@ EraseCoarseScan (void)
 	const int leftScanOffset  = 5;
 	const int rightScanOffset = 50;
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	SetContext (SpaceContext);
 
 	r.corner.x = (SIS_SCREEN_WIDTH >> 1) - (160 >> 1);
@@ -88,20 +100,22 @@ EraseCoarseScan (void)
 	r.extent.width = rightScanWidth;
 	RepairBackRect (&r);
 
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 }
 
 static void
-PrintScanTitlePC (TEXT *t, RECT *r, char *txt, int xpos)
+PrintScanTitlePC (TEXT *t, RECT *r, const char *txt, int xpos)
 {
 	t->baseline.x = xpos;
-	SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x00, 0x15), 0x3B));
-	wsprintf (t->pStr, txt);
+	SetContextForeGroundColor (
+			BUILD_COLOR (MAKE_RGB15 (0x00, 0x00, 0x15), 0x3B));
+	t->pStr = txt;
 	t->CharCount = (COUNT)~0;
 	font_DrawText (t);
 	TextRect (t, r, NULL_PTR);
 	t->baseline.x += r->extent.width;
-	SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0xF, 0x00, 0x19), 0x3B));
+	SetContextForeGroundColor (
+			BUILD_COLOR (MAKE_RGB15 (0xF, 0x00, 0x19), 0x3B));
 }
 
 static void
@@ -113,7 +127,7 @@ PrintCoarseScanPC (void)
 	RECT r;
 	UNICODE buf[40];
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	SetContext (SpaceContext);
 
 	if (CurStarDescPtr->Index == SOL_DEFINED)
@@ -172,11 +186,12 @@ PrintCoarseScanPC (void)
 	t.pStr = buf;
 	t.CharCount = (COUNT)~0;
 
-	SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x00, 0x15), 0x3B));
+	SetContextForeGroundColor (BUILD_COLOR (
+				MAKE_RGB15 (0x00, 0x00, 0x15), 0x3B));
 	SetContextFont (MicroFont);
 	font_DrawText (&t);
 	SetContextFont (TinyFont);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 #define LEFT_SIDE_BASELINE_X_PC 5
 #define RIGHT_SIDE_BASELINE_X_PC (SIS_SCREEN_WIDTH - 75)
@@ -185,8 +200,7 @@ PrintCoarseScanPC (void)
 	t.baseline.y = SCAN_BASELINE_Y_PC;
 	t.align = ALIGN_LEFT;
 
-	SetSemaphore (GraphicsSem);
-	t.pStr = buf;
+	LockMutex (GraphicsLock);
 	PrintScanTitlePC (&t, &r, "Orbit: ", LEFT_SIDE_BASELINE_X_PC);
 	temp = (SIZE)((pSolarSysState->SysInfo.PlanetInfo.PlanetToSunDist * 100L
 			+ (EARTH_RADIUS >> 1)) / EARTH_RADIUS);
@@ -194,13 +208,13 @@ PrintCoarseScanPC (void)
 		wsprintf (buf, "%u.%u a.u.", temp / 100, (temp / 10) % 10);
 	else
 		wsprintf (buf, "%u.%02u a.u.", temp / 100, temp % 100);
+	t.pStr = buf;
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
 	t.baseline.y += SCAN_LEADING_PC;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
-	SetSemaphore (GraphicsSem);
-	t.pStr = buf;
+	LockMutex (GraphicsLock);
 	PrintScanTitlePC (&t, &r, "Atmo: ", LEFT_SIDE_BASELINE_X_PC);
 	if (pSolarSysState->SysInfo.PlanetInfo.AtmoDensity == GAS_GIANT_ATMOSPHERE)
 		wsprintf (buf, "Super Thick");
@@ -215,83 +229,93 @@ PrintCoarseScanPC (void)
 		else
 			wsprintf (buf, "%u.%02u atm", temp / 100, temp % 100);
 	}
+	t.pStr = buf;
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
 	t.baseline.y += SCAN_LEADING_PC;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
-	SetSemaphore (GraphicsSem);
-	t.pStr = buf;
+	LockMutex (GraphicsLock);
 	PrintScanTitlePC (&t, &r, "Temp: ", LEFT_SIDE_BASELINE_X_PC);
-	wsprintf (buf, "%d^ c", pSolarSysState->SysInfo.PlanetInfo.SurfaceTemperature);
+	wsprintf (buf, "%d^ c",
+			pSolarSysState->SysInfo.PlanetInfo.SurfaceTemperature);
+	t.pStr = buf;
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
 	t.baseline.y += SCAN_LEADING_PC;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
-	SetSemaphore (GraphicsSem);
-	t.pStr = buf;
+	LockMutex (GraphicsLock);
 	PrintScanTitlePC (&t, &r, "Weather: ", LEFT_SIDE_BASELINE_X_PC);
 	if (pSolarSysState->SysInfo.PlanetInfo.AtmoDensity == 0)
-		wsprintf (buf, "None");
+		t.pStr = "None";
 	else
-		wsprintf (buf, "Class %u", pSolarSysState->SysInfo.PlanetInfo.Weather + 1);
+	{
+		wsprintf (buf, "Class %u",
+				pSolarSysState->SysInfo.PlanetInfo.Weather + 1);
+		t.pStr = buf;
+	}
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
 	t.baseline.y += SCAN_LEADING_PC;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
-	SetSemaphore (GraphicsSem);
-	t.pStr = buf;
+	LockMutex (GraphicsLock);
 	PrintScanTitlePC (&t, &r, "Tectonics: ", LEFT_SIDE_BASELINE_X_PC);
-	if (PLANSIZE (pSolarSysState->SysInfo.PlanetInfo.PlanDataPtr->Type) == GAS_GIANT)
-		wsprintf (buf, "None");
+	if (PLANSIZE (pSolarSysState->SysInfo.PlanetInfo.PlanDataPtr->Type) ==
+			GAS_GIANT)
+		t.pStr = "None";
 	else
-	wsprintf (buf, "Class %u", pSolarSysState->SysInfo.PlanetInfo.Tectonics + 1);
+	{
+		wsprintf (buf, "Class %u",
+				pSolarSysState->SysInfo.PlanetInfo.Tectonics + 1);
+		t.pStr = buf;
+	}
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 	t.baseline.y = SCAN_BASELINE_Y_PC;
 
-	SetSemaphore (GraphicsSem);
-	t.pStr = buf;
+	LockMutex (GraphicsLock);
 	PrintScanTitlePC (&t, &r, "Mass: ", RIGHT_SIDE_BASELINE_X_PC);
 	{
 		DWORD tr;
 
 		tr = pSolarSysState->SysInfo.PlanetInfo.PlanetRadius;
-		if ((tr = (tr * tr * tr / 100L
+		tr = (tr * tr * tr / 100L
 				* (DWORD)pSolarSysState->SysInfo.PlanetInfo.PlanetDensity
-				+ (DWORD)((100L * 100L) >> 1))
-				/ (DWORD)(100L * 100L)) == 0)
+				+ (DWORD)((100L * 100L) >> 1)) / (DWORD)(100L * 100L);
+		if (tr == 0)
 			tr = 1;
 		if (tr >= 10 * 100)
 			wsprintf (buf, "%lu.%lu e.s.", tr / 100, (tr / 10) % 10);
 		else
 			wsprintf (buf, "%lu.%02lu e.s.", tr / 100, tr % 100);
 	}
+	t.pStr = buf;
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
 	t.baseline.y += SCAN_LEADING_PC;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
-	SetSemaphore (GraphicsSem);
-	t.pStr = buf;
+	LockMutex (GraphicsLock);
 	PrintScanTitlePC (&t, &r, "Radius: ", RIGHT_SIDE_BASELINE_X_PC);
-	if ((temp = pSolarSysState->SysInfo.PlanetInfo.PlanetRadius) >= 10 * 100)
+	temp = pSolarSysState->SysInfo.PlanetInfo.PlanetRadius;
+	if (temp >= 10 * 100)
 		wsprintf (buf, "%u.%u e.s.", temp / 100, (temp / 10) % 10);
 	else
 		wsprintf (buf, "%u.%02u e.s.", temp / 100, temp % 100);
+	t.pStr = buf;
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
 	t.baseline.y += SCAN_LEADING_PC;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
-	SetSemaphore (GraphicsSem);
-	t.pStr = buf;
+	LockMutex (GraphicsLock);
 	PrintScanTitlePC (&t, &r, "Gravity: ", RIGHT_SIDE_BASELINE_X_PC);
-	if ((temp = pSolarSysState->SysInfo.PlanetInfo.SurfaceGravity) >= 10 * 100)
+	temp = pSolarSysState->SysInfo.PlanetInfo.SurfaceGravity;
+	if (temp >= 10 * 100)
 		wsprintf (buf, "%u.%u g.", temp / 100, (temp / 10) % 10);
 	else
 	{
@@ -299,17 +323,18 @@ PrintCoarseScanPC (void)
 			temp = 1;
 		wsprintf (buf, "%u.%02u g.", temp / 100, temp % 100);
 	}
+	t.pStr = buf;
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
 	t.baseline.y += SCAN_LEADING_PC;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
-	SetSemaphore (GraphicsSem);
-	t.pStr = buf;
+	LockMutex (GraphicsLock);
 	PrintScanTitlePC (&t, &r, "Day: ", RIGHT_SIDE_BASELINE_X_PC);
 	if (pSolarSysState->SysInfo.PlanetInfo.RotationPeriod < 240 * 10)
 	{
-		temp = (SIZE)(pSolarSysState->SysInfo.PlanetInfo.RotationPeriod * 10 / 24);
+		temp = (SIZE)(pSolarSysState->SysInfo.PlanetInfo.RotationPeriod *
+				10 / 24);
 		wsprintf (buf, "%u.%02u days", temp / 100, temp % 100);
 	}
 	else
@@ -318,20 +343,21 @@ PrintCoarseScanPC (void)
 				+ (24 >> 1)) / 24);
 		wsprintf (buf, "%u.%u days", temp / 10, temp % 10);
 	}
+	t.pStr = buf;
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
 	t.baseline.y += SCAN_LEADING_PC;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
-	SetSemaphore (GraphicsSem);
-	t.pStr = buf;
+	LockMutex (GraphicsLock);
 	PrintScanTitlePC (&t, &r, "Tilt: ", RIGHT_SIDE_BASELINE_X_PC);
 	if ((temp = pSolarSysState->SysInfo.PlanetInfo.AxialTilt) < 0)
 		temp = -temp;
+	t.pStr = buf;
 	wsprintf (buf, "%u^", temp);
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 }
 
 static void
@@ -344,7 +370,7 @@ PrintCoarseScan3DO (void)
 	UNICODE buf[40];
 	extern FRAME SpaceJunkFrame;
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	SetContext (SpaceContext);
 
 	if (CurStarDescPtr->Index == SOL_DEFINED)
@@ -411,7 +437,7 @@ PrintCoarseScan3DO (void)
 	s.frame = SetAbsFrameIndex (SpaceJunkFrame, 20);
 	DrawStamp (&s);
 
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 #define LEFT_SIDE_BASELINE_X (27 + (16 - SAFE_X))
 #define RIGHT_SIDE_BASELINE_X (SIS_SCREEN_WIDTH - LEFT_SIDE_BASELINE_X)
@@ -421,7 +447,7 @@ PrintCoarseScan3DO (void)
 	t.baseline.y = SCAN_BASELINE_Y;
 	t.align = ALIGN_LEFT;
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	t.pStr = buf;
 	temp = (SIZE)((pSolarSysState->SysInfo.PlanetInfo.PlanetToSunDist * 100L
 			+ (EARTH_RADIUS >> 1)) / EARTH_RADIUS);
@@ -432,9 +458,9 @@ PrintCoarseScan3DO (void)
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
 	t.baseline.y += SCAN_LEADING;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	t.pStr = buf;
 	if (pSolarSysState->SysInfo.PlanetInfo.AtmoDensity == GAS_GIANT_ATMOSPHERE)
 		wsprintf (buf, "\x7f");
@@ -450,26 +476,26 @@ PrintCoarseScan3DO (void)
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
 	t.baseline.y += SCAN_LEADING;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	t.pStr = buf;
 	wsprintf (buf, "%d^", pSolarSysState->SysInfo.PlanetInfo.SurfaceTemperature);
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
 	t.baseline.y += SCAN_LEADING;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	t.pStr = buf;
 	wsprintf (buf, "<%u>", pSolarSysState->SysInfo.PlanetInfo.AtmoDensity == 0
 			? 0 : (pSolarSysState->SysInfo.PlanetInfo.Weather + 1));
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
 	t.baseline.y += SCAN_LEADING;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	t.pStr = buf;
 	wsprintf (buf, "<%u>",
 			PLANSIZE (
@@ -478,13 +504,13 @@ PrintCoarseScan3DO (void)
 			? 0 : (pSolarSysState->SysInfo.PlanetInfo.Tectonics + 1));
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
 	t.baseline.x = RIGHT_SIDE_BASELINE_X;
 	t.baseline.y = SCAN_BASELINE_Y;
 	t.align = ALIGN_RIGHT;
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	t.pStr = buf;
 	{
 		DWORD tr;
@@ -503,9 +529,9 @@ PrintCoarseScan3DO (void)
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
 	t.baseline.y += SCAN_LEADING;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	t.pStr = buf;
 	if ((temp = pSolarSysState->SysInfo.PlanetInfo.PlanetRadius) >= 10 * 100)
 		wsprintf (buf, "%u.%u&", temp / 100, (temp / 10) % 10);
@@ -514,9 +540,9 @@ PrintCoarseScan3DO (void)
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
 	t.baseline.y += SCAN_LEADING;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	t.pStr = buf;
 	if ((temp = pSolarSysState->SysInfo.PlanetInfo.SurfaceGravity) >= 10 * 100)
 		wsprintf (buf, "%u.%u&", temp / 100, (temp / 10) % 10);
@@ -529,9 +555,9 @@ PrintCoarseScan3DO (void)
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
 	t.baseline.y += SCAN_LEADING;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	t.pStr = buf;
 	if ((temp = pSolarSysState->SysInfo.PlanetInfo.AxialTilt) < 0)
 		temp = -temp;
@@ -539,9 +565,9 @@ PrintCoarseScan3DO (void)
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
 	t.baseline.y += SCAN_LEADING;
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	t.pStr = buf;
 	if (pSolarSysState->SysInfo.PlanetInfo.RotationPeriod < 240 * 10)
 	{
@@ -556,7 +582,7 @@ PrintCoarseScan3DO (void)
 	}
 	t.CharCount = (COUNT)~0;
 	font_DrawText (&t);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 }
 
 static void
@@ -569,7 +595,7 @@ SetPlanetLoc (POINT new_pt)
 	new_pt.x >>= MAG_SHIFT;
 	new_pt.y >>= MAG_SHIFT;
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	SetContext (ScanContext);
 	s.origin.x = pMenuState->flash_rect0.corner.x - (FLASH_WIDTH >> 1);
 	s.origin.y = pMenuState->flash_rect0.corner.y - (FLASH_HEIGHT >> 1);
@@ -583,7 +609,7 @@ SetPlanetLoc (POINT new_pt)
 	r.extent.width = FLASH_WIDTH;
 	r.extent.height = FLASH_HEIGHT;
 	LoadDisplayPixmap (&r, pMenuState->flash_frame0);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 	
 	TaskSwitch ();
 }
@@ -609,7 +635,7 @@ flash_planet_loc_func(void *data)
 		DWORD T;
 		CONTEXT OldContext;
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		T = GetTimeCounter ();
 		if (p.Object.Stamp.origin.x != pMenuState->flash_rect0.corner.x
 				|| p.Object.Stamp.origin.y != pMenuState->flash_rect0.corner.y)
@@ -621,7 +647,7 @@ flash_planet_loc_func(void *data)
 		{
 			if (T < TimeIn)
 			{
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 				TaskSwitch ();
 
 				continue;
@@ -639,7 +665,7 @@ flash_planet_loc_func(void *data)
 		SetContext (OldContext);
 
 		TimeIn = T + (ONE_SECOND >> 4);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 		
 		TaskSwitch ();
 	}
@@ -653,8 +679,8 @@ static BOOLEAN
 PickPlanetSide (PMENU_STATE pMS)
 {
 	BOOLEAN select, cancel;
-	select = CurrentMenuState.select;
-	cancel = CurrentMenuState.cancel;
+	select = PulsedInputState.key[KEY_MENU_SELECT];
+	cancel = PulsedInputState.key[KEY_MENU_CANCEL];
 	if (GLOBAL (CurrentActivity) & CHECK_ABORT)
 	{
 		if (pMenuState->flash_task)
@@ -669,18 +695,14 @@ PickPlanetSide (PMENU_STATE pMS)
 	{
 		pMS->InputFunc = PickPlanetSide;
 		SetMenuRepeatDelay (0, 0, 0, FALSE);
-		if (pMS->CurFrame == 0)
-		{
-			pMS->CurFrame = (FRAME)MenuSounds;
-			MenuSounds = 0;
-		}
+		SetMenuSounds (MENU_SOUND_NONE, MENU_SOUND_NONE);
 		if (!select)
 		{
 			RECT r;
 
 			pMS->Initialized = TRUE;
 
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			SetContext (ScanContext);
 			pMenuState->flash_rect0.corner.x =
 					pSolarSysState->MenuState.first_item.x >> MAG_SHIFT;
@@ -694,7 +716,7 @@ PickPlanetSide (PMENU_STATE pMS)
 			LoadDisplayPixmap (&r, pMenuState->flash_frame0);
 
 			SetFlashRect (NULL_PTR, (FRAME)0);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 
 			InitLander (0);
 
@@ -706,12 +728,11 @@ PickPlanetSide (PMENU_STATE pMS)
 	{
 		STAMP s;
 
-		MenuSounds = (SOUND)pMS->CurFrame;
-		pMS->CurFrame = 0;
+		SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		DrawStatusMessage (NULL_PTR);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 
 		FlushInput ();
 		if (pMenuState->flash_task)
@@ -735,13 +756,13 @@ PickPlanetSide (PMENU_STATE pMS)
 
 			EraseCoarseScan ();
 
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			DeltaSISGauges (0, -(SIZE)fuel_required, 0);
 			SetContext (ScanContext);
 			s.origin = pMenuState->flash_rect0.corner;
 			s.frame = SetAbsFrameIndex (misc_data, FLASH_INDEX);
 			DrawStamp (&s);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 
 			PlanetSide (pMS);
 			if (GLOBAL (CurrentActivity) & CHECK_ABORT)
@@ -751,7 +772,6 @@ PickPlanetSide (PMENU_STATE pMS)
 			{
 				HSTARSHIP hStarShip;
 				extern ACTIVITY NextActivity;
-				extern void SaveFlagshipState (void);
 
 				if (pMenuState->flash_task)
 				{
@@ -796,16 +816,12 @@ PickPlanetSide (PMENU_STATE pMS)
 		}
 
 		DrawMenuStateStrings (PM_MIN_SCAN, DISPATCH_SHUTTLE);
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		SetFlashRect ((PRECT)~0L, (FRAME)0);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 
 ExitPlanetSide:
-		if (pMS->CurFrame)
-		{
-			MenuSounds = (SOUND)pMS->CurFrame;
-			pMS->CurFrame = 0;
-		}
+		SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
 
 		pMS->InputFunc = DoScan;
 		pMS->CurState = DISPATCH_SHUTTLE;
@@ -818,10 +834,10 @@ ExitPlanetSide:
 
 		new_pt = pSolarSysState->MenuState.first_item;
 
-		if (CurrentMenuState.left) dx = -1;
-		if (CurrentMenuState.right) dx = 1;
-		if (CurrentMenuState.up) dy = -1;
-		if (CurrentMenuState.down) dy = 1;
+		if (PulsedInputState.key[KEY_MENU_LEFT]) dx = -1;
+		if (PulsedInputState.key[KEY_MENU_RIGHT]) dx = 1;
+		if (PulsedInputState.key[KEY_MENU_UP]) dy = -1;
+		if (PulsedInputState.key[KEY_MENU_DOWN]) dy = 1;
 
 		dx = dx << MAG_SHIFT;
 		if (dx)
@@ -950,8 +966,8 @@ DoScan (PMENU_STATE pMS)
 {
 	DWORD TimeIn, WaitTime;
 	BOOLEAN select, cancel;
-	select = CurrentMenuState.select;
-	cancel = CurrentMenuState.cancel;
+	select = PulsedInputState.key[KEY_MENU_SELECT];
+	cancel = PulsedInputState.key[KEY_MENU_CANCEL];
 	if (GLOBAL (CurrentActivity) & CHECK_ABORT)
 		return (FALSE);
 
@@ -962,12 +978,12 @@ DoScan (PMENU_STATE pMS)
 	}
 	else if (cancel || (select && pMS->CurState == EXIT_SCAN))
 	{
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		SetContext (SpaceContext);
 		BatchGraphics ();
 		DrawPlanet (SIS_SCREEN_WIDTH - MAP_WIDTH, SIS_SCREEN_HEIGHT - MAP_HEIGHT, 0, 0);
 		UnbatchGraphics ();
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 
 		EraseCoarseScan ();
 // DrawMenuStateStrings (PM_SCAN, SCAN);
@@ -1000,26 +1016,25 @@ DoScan (PMENU_STATE pMS)
 				return (TRUE);
 			}
 
-			wsprintf (buf, "%s%u.%u",
+			wsprintf (buf, "%s%1.1f",
 					GAME_STRING (NAVIGATION_STRING_BASE + 5),
-					fuel_required / FUEL_TANK_SCALE,
-					((fuel_required % FUEL_TANK_SCALE) + 5) / 10);
-			SetSemaphore (GraphicsSem);
+					(float) fuel_required / FUEL_TANK_SCALE);
+			LockMutex (GraphicsLock);
 			ClearSISRect (CLEAR_SIS_RADAR);
 			DrawStatusMessage (buf);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			SetContext (ScanContext);
 			BatchGraphics ();
 			DrawPlanet (0, 0, 0, 0);
 			DrawScannedObjects (FALSE);
 			UnbatchGraphics ();
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 		
 			pMS->Initialized = FALSE;
 			pMS->CurFrame = 0;
-			return (PickPlanetSide (pMS));
+			return PickPlanetSide (pMS);
 		}
 
 		pSolarSysState->MenuState.Initialized += 4;
@@ -1048,7 +1063,7 @@ DoScan (PMENU_STATE pMS)
 		pMS->delta_item = (SIZE)pSolarSysState->CurNode;
 		t.pStr = GAME_STRING (SCAN_STRING_BASE + min_scan);
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		SetContext (SpaceContext);
 		r.corner.x = 0;
 		r.corner.y = t.baseline.y - 10;
@@ -1078,7 +1093,7 @@ DoScan (PMENU_STATE pMS)
 		font_DrawText (&t);
 
 		SetContext (ScanContext);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 
 	{
 		DWORD rgb;
@@ -1096,15 +1111,15 @@ DoScan (PMENU_STATE pMS)
 				break;
 		}
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		BatchGraphics ();
 		DrawPlanet (0, 0, 0, 0);
 		UnbatchGraphics ();
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 
 		PressState = AnyButtonPress (TRUE);
 		WaitTime = (ONE_SECOND << 1) / MAP_HEIGHT;
-//		SetSemaphore (GraphicsSem);
+//		LockMutex (GraphicsLock);
 		TimeIn = GetTimeCounter ();
 		for (i = 0; i < MAP_HEIGHT + NUM_FLASH_COLORS + 1; i++)
 		{
@@ -1116,7 +1131,7 @@ DoScan (PMENU_STATE pMS)
 			}
 			if (ButtonState)
 				i = -i;
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			BatchGraphics ();
 			DrawPlanet (0, 0, i, rgb);
 			if (i < 0) 
@@ -1124,19 +1139,19 @@ DoScan (PMENU_STATE pMS)
 			if (pMS->delta_item)
 				DrawScannedStuff (i, min_scan);
 			UnbatchGraphics ();
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 //			FlushGraphics ();
 			SleepThreadUntil (TimeIn + WaitTime);
 			TimeIn = GetTimeCounter ();
 		}
-//		ClearSemaphore (GraphicsSem);
+//		UnlockMutex (GraphicsLock);
 		pSolarSysState->Tint_rgb = 0;
 
 	}
 
 		} while (++min_scan <= max_scan);
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		SetContext (SpaceContext);
 		r.corner.x = 0;
 		r.corner.y = (SIS_SCREEN_HEIGHT - MAP_HEIGHT - 7) - 10;
@@ -1149,12 +1164,12 @@ DoScan (PMENU_STATE pMS)
 		{
 			DrawPlanet (0, 0, 0, 0);
 			DrawScannedObjects (FALSE);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 
 			DrawMenuStateStrings (PM_MIN_SCAN, pMS->CurState = DISPATCH_SHUTTLE);
 		}
 		else
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			
 		pSolarSysState->MenuState.Initialized -= 4;
 		pSolarSysState->PauseRotate = 0;
@@ -1208,7 +1223,7 @@ ScanSystem (void)
 		pSolarSysState->MenuState.first_item.x = (MAP_WIDTH >> 1) << MAG_SHIFT;
 		pSolarSysState->MenuState.first_item.y = (MAP_HEIGHT >> 1) << MAG_SHIFT;
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		ScanContext = CaptureContext (CreateContext ());
 		SetContext (ScanContext);
 		MenuState.flash_rect0.extent.width = FLASH_WIDTH;
@@ -1224,7 +1239,7 @@ ScanSystem (void)
 		r.extent.height = MAP_HEIGHT;
 		SetContextClipRect (&r);
 		DrawScannedObjects (FALSE);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 	}
 
 	DrawMenuStateStrings (PM_MIN_SCAN, MenuState.CurState);
@@ -1235,17 +1250,18 @@ ScanSystem (void)
 		PrintCoarseScan3DO ();
 
 	pMenuState = &MenuState;
+	SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
 	DoInput ((PVOID)&MenuState, FALSE);
 	pMenuState = 0;
 
 	if (ScanContext)
 	{
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		SetContext (SpaceContext);
 		DestroyDrawable (ReleaseDrawable (MenuState.flash_frame0));
 		DestroyContext (ReleaseContext (ScanContext));
 		ScanContext = 0;
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 	}
 }
 
