@@ -16,47 +16,56 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "starcon.h"
-#include "libs/graphics/gfx_common.h"
+#include "process.h"
+
+#include "build.h"
+#include "collide.h"
+#include "options.h"
+#include "settings.h"
+#include "setup.h"
+#include "sounds.h"
+#include "weapon.h"
 #include "libs/graphics/drawable.h"
 #include "libs/graphics/drawcmd.h"
-#include "options.h"
+#include "libs/graphics/gfx_common.h"
+
+
+//#define DEBUG_PROCESS
 
 COUNT DisplayFreeList;
 PRIMITIVE DisplayArray[MAX_DISPLAY_PRIMS];
 extern POINT SpaceOrg;
 
-#ifndef OLD_ZOOM
 SIZE zoom_out = 1 << ZOOM_SHIFT;
-#endif
+static SIZE opt_max_zoom_out;
 
 #if 0
-#define CALC_ZOOM_STUFF(idx,sc) \
-	do \
-	{ \
-		int i, z; \
-		\
-		z = 1 << ZOOM_SHIFT; \
-		for (i = 0; (z <<= 1) <= zoom_out; i++) \
-			; \
-		*(idx) = i; \
-		*(sc) = ((1 << i) << (ZOOM_SHIFT + 8)) / zoom_out; \
-	} while (0)
+static inline void
+CALC_ZOOM_STUFF (COUNT* idx, COUNT* sc)
+{
+	int i, z;
+
+	z = 1 << ZOOM_SHIFT;
+	for (i = 0; (z <<= 1) <= zoom_out; i++)
+		;
+	*idx = i;
+	*sc = ((1 << i) << (ZOOM_SHIFT + 8)) / zoom_out;
+}
 #else
-#define CALC_ZOOM_STUFF(idx,sc) \
-	do \
-	{ \
-		int i; \
-		\
-		if (zoom_out < 2 << ZOOM_SHIFT) \
-			i = 0; \
-		else if (zoom_out < 4 << ZOOM_SHIFT) \
-			i = 1; \
-		else \
-			i = 2; \
-		*(idx) = i; \
-		*(sc) = (1 << (i + ZOOM_SHIFT + 8)) / zoom_out; \
-	} while (0)
+static inline void
+CALC_ZOOM_STUFF (COUNT* idx, COUNT* sc)
+{
+	int i;
+
+	if (zoom_out < 2 << ZOOM_SHIFT)
+		i = 0;
+	else if (zoom_out < 4 << ZOOM_SHIFT)
+		i = 1;
+	else
+		i = 2;
+	*idx = i;
+	*sc = (1 << (i + ZOOM_SHIFT + 8)) / zoom_out;
+}
 #endif
 
 HELEMENT
@@ -95,7 +104,7 @@ FreeElement (HELEMENT hElement)
 }
 
 void
-SetUpElement (register ELEMENTPTR ElementPtr)
+SetUpElement (ELEMENTPTR ElementPtr)
 {
 	ElementPtr->next = ElementPtr->current;
 	if (CollidingElement (ElementPtr))
@@ -107,9 +116,9 @@ SetUpElement (register ELEMENTPTR ElementPtr)
 }
 
 static void
-PreProcess (register ELEMENTPTR ElementPtr)
+PreProcess (ELEMENTPTR ElementPtr)
 {
-	register ELEMENT_FLAGS state_flags;
+	ELEMENT_FLAGS state_flags;
 
 	if (ElementPtr->life_span == 0)
 	{
@@ -168,7 +177,7 @@ PreProcess (register ELEMENTPTR ElementPtr)
 }
 
 static void
-PostProcess (register ELEMENTPTR ElementPtr)
+PostProcess (ELEMENTPTR ElementPtr)
 {
 	if (ElementPtr->postprocess_func)
 		(*ElementPtr->postprocess_func) (ElementPtr);
@@ -185,34 +194,35 @@ PostProcess (register ELEMENTPTR ElementPtr)
 			| POST_PROCESS;
 }
 
-#ifdef OLD_ZOOM
-static BYTE
-CalcReduction (SIZE dx, SIZE dy)
-#else
 static COUNT
 CalcReduction (SIZE dx, SIZE dy)
-#endif
 {
-#ifdef OLD_ZOOM
-	BYTE next_reduction;
-	SIZE sdx, sdy;
+	COUNT next_reduction;
 
-	if (LOBYTE (GLOBAL (CurrentActivity)) > IN_ENCOUNTER)
-		return (0);
+#ifdef KDEBUG
+	fprintf (stderr, "CalcReduction:\n");
+#endif
 
-	sdx = dx;
-	sdy = dy;
-	for (next_reduction = MAX_VIS_REDUCTION;
-			(dx <<= REDUCTION_SHIFT) <= TRANSITION_WIDTH
-			&& (dy <<= REDUCTION_SHIFT) <= TRANSITION_HEIGHT
-			&& next_reduction > 0;
-			next_reduction -= REDUCTION_SHIFT)
-		;
-
-			/* check for "real" zoom in */
-	if (next_reduction < GLOBAL (cur_state)
-			&& GLOBAL (cur_state) <= MAX_VIS_REDUCTION)
+	if (optMeleeScale == TFB_SCALE_STEP)
 	{
+		SIZE sdx, sdy;
+
+		if (LOBYTE (GLOBAL (CurrentActivity)) > IN_ENCOUNTER)
+			return (0);
+
+		sdx = dx;
+		sdy = dy;
+		for (next_reduction = MAX_VIS_REDUCTION;
+				(dx <<= REDUCTION_SHIFT) <= TRANSITION_WIDTH
+				&& (dy <<= REDUCTION_SHIFT) <= TRANSITION_HEIGHT
+				&& next_reduction > 0;
+				next_reduction -= REDUCTION_SHIFT)
+			;
+
+				/* check for "real" zoom in */
+		if (next_reduction < zoom_out
+				&& zoom_out <= MAX_VIS_REDUCTION)
+		{
 #define HYSTERESIS_X DISPLAY_TO_WORLD(24)
 #define HYSTERESIS_Y DISPLAY_TO_WORLD(20)
 		if (((sdx + HYSTERESIS_X)
@@ -221,56 +231,49 @@ CalcReduction (SIZE dx, SIZE dy)
 				<< (MAX_VIS_REDUCTION - next_reduction)) > TRANSITION_HEIGHT)
 		   /* if we don't zoom in, we want to stay at next+1 */
 		   next_reduction += REDUCTION_SHIFT;
+		}
+
+		if (next_reduction == 0
+				&& LOBYTE (GLOBAL (CurrentActivity)) == IN_LAST_BATTLE)
+			next_reduction += REDUCTION_SHIFT;
+	}
+	else
+	{
+		if (LOBYTE (GLOBAL (CurrentActivity)) > IN_ENCOUNTER)
+			return (1 << ZOOM_SHIFT);
+			
+		dx = (dx * MAX_ZOOM_OUT) / (LOG_SPACE_WIDTH >> 2);
+		if (dx < (1 << ZOOM_SHIFT))
+			dx = 1 << ZOOM_SHIFT;
+		else if (dx > MAX_ZOOM_OUT)
+			dx = MAX_ZOOM_OUT;
+			
+		dy = (dy * MAX_ZOOM_OUT) / (LOG_SPACE_HEIGHT >> 2);
+		if (dy < (1 << ZOOM_SHIFT))
+			dy = 1 << ZOOM_SHIFT;
+		else if (dy > MAX_ZOOM_OUT)
+			dy = MAX_ZOOM_OUT;
+			
+		if (dy > dx)
+			next_reduction = dy;
+		else
+			next_reduction = dx;
+
+		if (next_reduction < (2 << ZOOM_SHIFT)
+				&& LOBYTE (GLOBAL (CurrentActivity)) == IN_LAST_BATTLE)
+			next_reduction = (2 << ZOOM_SHIFT);
 	}
 
-	if (next_reduction == 0
-			&& LOBYTE (GLOBAL (CurrentActivity)) == IN_LAST_BATTLE)
-		next_reduction += REDUCTION_SHIFT;
-#else
-	COUNT next_reduction;
-#ifdef KDEBUG
-	fprintf (stderr, "CalcReduction:\n");
-#endif
-	if (LOBYTE (GLOBAL (CurrentActivity)) > IN_ENCOUNTER)
-		return (1 << ZOOM_SHIFT);
-		
-	dx = (dx * MAX_ZOOM_OUT) / (LOG_SPACE_WIDTH >> 2);
-	if (dx < (1 << ZOOM_SHIFT))
-		dx = 1 << ZOOM_SHIFT;
-	else if (dx > MAX_ZOOM_OUT)
-		dx = MAX_ZOOM_OUT;
-		
-	dy = (dy * MAX_ZOOM_OUT) / (LOG_SPACE_HEIGHT >> 2);
-	if (dy < (1 << ZOOM_SHIFT))
-		dy = 1 << ZOOM_SHIFT;
-	else if (dy > MAX_ZOOM_OUT)
-		dy = MAX_ZOOM_OUT;
-		
-	if (dy > dx)
-		next_reduction = dy;
-	else
-		next_reduction = dx;
-
-	if (next_reduction < (2 << ZOOM_SHIFT)
-			&& LOBYTE (GLOBAL (CurrentActivity)) == IN_LAST_BATTLE)
-		next_reduction = (2 << ZOOM_SHIFT);
 #ifdef KDEBUG
 	fprintf (stderr, "CalcReduction: exit\n");
-#endif
 #endif
 
 	return (next_reduction);
 }
 
-#ifdef OLD_ZOOM
 static VIEW_STATE
-CalcView (PPOINT pNewScrollPt, register BYTE next_reduction,
-		PSIZE pdx, PSIZE pdy)
-#else
-static VIEW_STATE
-CalcView (PPOINT pNewScrollPt, register SIZE next_reduction,
+CalcView (PPOINT pNewScrollPt, SIZE next_reduction,
 		PSIZE pdx, PSIZE pdy, COUNT ships_alive)
-#endif
 {
 	SIZE dx, dy;
 	VIEW_STATE view_state;
@@ -284,8 +287,8 @@ CalcView (PPOINT pNewScrollPt, register SIZE next_reduction,
 	dy = WRAP_DELTA_Y (dy);
 	if (ships_alive == 1)
 	{
-#define ORG_JUMP_X (LOG_SPACE_WIDTH / 75)
-#define ORG_JUMP_Y (LOG_SPACE_HEIGHT / 75)
+#define ORG_JUMP_X ((SIZE)DISPLAY_ALIGN(LOG_SPACE_WIDTH / 75))
+#define ORG_JUMP_Y ((SIZE)DISPLAY_ALIGN(LOG_SPACE_HEIGHT / 75))
 		if (dx > ORG_JUMP_X)
 			dx = ORG_JUMP_X;
 		else if (dx < -ORG_JUMP_X)
@@ -299,39 +302,36 @@ CalcView (PPOINT pNewScrollPt, register SIZE next_reduction,
 	if ((dx || dy) && LOBYTE (GLOBAL (CurrentActivity)) == IN_HYPERSPACE)
 		MoveSIS (&dx, &dy);
 
-#ifdef OLD_ZOOM
-	if (GLOBAL (cur_state) == next_reduction)
-#else
 	if (zoom_out == next_reduction)
-#endif
 		view_state = dx == 0 && dy == 0
 				&& LOBYTE (GLOBAL (CurrentActivity)) != IN_HYPERSPACE
 				? VIEW_STABLE : VIEW_SCROLL;
 	else
 	{
-#ifdef OLD_ZOOM
-		GLOBAL (cur_state) = next_reduction;
-
-		SpaceOrg.x = (COORD)(LOG_SPACE_WIDTH >> 1)
-				- (LOG_SPACE_WIDTH >> ((MAX_REDUCTION + 1) - next_reduction));
-		SpaceOrg.y = (COORD)(LOG_SPACE_HEIGHT >> 1)
-				- (LOG_SPACE_HEIGHT >> ((MAX_REDUCTION + 1) - next_reduction));
-#else
+		if (optMeleeScale == TFB_SCALE_STEP)
+		{
+			SpaceOrg.x = (COORD)(LOG_SPACE_WIDTH >> 1)
+					- (LOG_SPACE_WIDTH >> ((MAX_REDUCTION + 1)
+					- next_reduction));
+			SpaceOrg.y = (COORD)(LOG_SPACE_HEIGHT >> 1)
+					- (LOG_SPACE_HEIGHT >> ((MAX_REDUCTION + 1)
+					- next_reduction));
+		}
+		else
+		{
 #define ZOOM_JUMP ((1 << ZOOM_SHIFT) >> 3)
-		if (ships_alive == 1
-				&& zoom_out > next_reduction
-				&& zoom_out <= MAX_ZOOM_OUT
-				&& zoom_out - next_reduction > ZOOM_JUMP)
-			next_reduction = zoom_out - ZOOM_JUMP;
-			
+			if (ships_alive == 1
+					&& zoom_out > next_reduction
+					&& zoom_out <= MAX_ZOOM_OUT
+					&& zoom_out - next_reduction > ZOOM_JUMP)
+				next_reduction = zoom_out - ZOOM_JUMP;
+				
+			SpaceOrg.x = (int)(LOG_SPACE_WIDTH >> 1)
+					- (LOG_SPACE_WIDTH * next_reduction / (MAX_ZOOM_OUT << 2));
+			SpaceOrg.y = (int)(LOG_SPACE_HEIGHT >> 1)
+					- (LOG_SPACE_HEIGHT * next_reduction / (MAX_ZOOM_OUT << 2));
+		}
 		zoom_out = next_reduction;
-
-		SpaceOrg.x = (int)(LOG_SPACE_WIDTH >> 1)
-				- (LOG_SPACE_WIDTH * next_reduction / (MAX_ZOOM_OUT << 2));
-		SpaceOrg.y = (int)(LOG_SPACE_HEIGHT >> 1)
-				- (LOG_SPACE_HEIGHT * next_reduction / (MAX_ZOOM_OUT << 2));
-#endif
-
 		view_state = VIEW_CHANGE;
 	}
 
@@ -349,7 +349,7 @@ CalcView (PPOINT pNewScrollPt, register SIZE next_reduction,
 
 
 static ELEMENT_FLAGS
-ProcessCollisions (HELEMENT hSuccElement, register ELEMENTPTR ElementPtr,
+ProcessCollisions (HELEMENT hSuccElement, ELEMENTPTR ElementPtr,
 		TIME_VALUE min_time, ELEMENT_FLAGS process_flags)
 {
 	HELEMENT hTestElement;
@@ -372,7 +372,7 @@ ProcessCollisions (HELEMENT hSuccElement, register ELEMENTPTR ElementPtr,
 
 		if (CollisionPossible (TestElementPtr, ElementPtr))
 		{
-			register ELEMENT_FLAGS state_flags, test_state_flags;
+			ELEMENT_FLAGS state_flags, test_state_flags;
 			TIME_VALUE time_val;
 
 			state_flags = ElementPtr->state_flags;
@@ -389,10 +389,10 @@ ProcessCollisions (HELEMENT hSuccElement, register ELEMENTPTR ElementPtr,
 						&TestElementPtr->IntersectControl, min_time)) == 1
 						&& !((state_flags | test_state_flags) & FINITE_LIFE))
 				{
-#ifdef DEBUG
+#ifdef DEBUG_PROCESS
 					fprintf (stderr, "BAD NEWS 0x%x <--> 0x%x\n", ElementPtr,
 							TestElementPtr);
-#endif /* DEBUG */
+#endif /* DEBUG_PROCESS */
 					if (state_flags & COLLISION)
 					{
 						InitIntersectEndPoint (TestElementPtr);
@@ -511,10 +511,10 @@ ProcessCollisions (HELEMENT hSuccElement, register ELEMENTPTR ElementPtr,
 			{
 				POINT SavePt, TestSavePt;
 
-#ifdef DEBUG
+#ifdef DEBUG_PROCESS
 				fprintf (stderr, "0x%x <--> 0x%x at %u\n", ElementPtr,
 						TestElementPtr, time_val);
-#endif /* DEBUG */
+#endif /* DEBUG_PROCESS */
 				SavePt = ElementPtr->IntersectControl.EndPoint;
 				TestSavePt = TestElementPtr->IntersectControl.EndPoint;
 				InitIntersectEndPoint (ElementPtr);
@@ -533,10 +533,10 @@ ProcessCollisions (HELEMENT hSuccElement, register ELEMENTPTR ElementPtr,
 					state_flags = ElementPtr->state_flags;
 					test_state_flags = TestElementPtr->state_flags;
 
-#ifdef DEBUG
+#ifdef DEBUG_PROCESS
 					fprintf (stderr, "PROCESSING 0x%x <--> 0x%x at %u\n",
 							ElementPtr, TestElementPtr, time_val);
-#endif /* DEBUG */
+#endif /* DEBUG_PROCESS */
 					if (test_state_flags & PLAYER_SHIP)
 					{
 						(*TestElementPtr->collision_func) (
@@ -617,22 +617,13 @@ ProcessCollisions (HELEMENT hSuccElement, register ELEMENTPTR ElementPtr,
 	return (ElementPtr->state_flags & COLLISION);
 }
 
-#define MAX_SOUNDS 8
-static BYTE num_sounds;
-static SOUND sound_buf[MAX_SOUNDS];
-static ELEMENTPTR sound_posobj[MAX_SOUNDS];
-
 static VIEW_STATE
 PreProcessQueue (PSIZE pscroll_x, PSIZE pscroll_y)
 {
-#ifdef OLD_ZOOM
-	BYTE min_reduction, max_reduction;
-#else
 	SIZE min_reduction, max_reduction;
-#endif
 	COUNT num_ships;
 	POINT Origin;
-	register HELEMENT hElement;
+	HELEMENT hElement;
 	COUNT ships_alive;
 
 #ifdef KDEBUG
@@ -641,11 +632,11 @@ PreProcessQueue (PSIZE pscroll_x, PSIZE pscroll_y)
 	num_ships = (LOBYTE (battle_counter) ? 1 : 0)
 			+ (HIBYTE (battle_counter) ? 1 : 0);
 
-#ifdef OLD_ZOOM
-	min_reduction = max_reduction = MAX_VIS_REDUCTION + 1;
-#else
-	min_reduction = max_reduction = MAX_ZOOM_OUT + (1 << ZOOM_SHIFT);
-#endif
+	if (optMeleeScale == TFB_SCALE_STEP)
+		min_reduction = max_reduction = MAX_VIS_REDUCTION + 1;
+	else
+		min_reduction = max_reduction = MAX_ZOOM_OUT + (1 << ZOOM_SHIFT);
+
 	Origin.x = (COORD)(LOG_SPACE_WIDTH >> 1);
 	Origin.y = (COORD)(LOG_SPACE_HEIGHT >> 1);
 
@@ -672,13 +663,8 @@ PreProcessQueue (PSIZE pscroll_x, PSIZE pscroll_y)
 			SIZE dx, dy;
 
 			ships_alive++;
-#ifdef OLD_ZOOM
-			if (max_reduction > MAX_VIS_REDUCTION
-					&& min_reduction > MAX_VIS_REDUCTION)
-#else
-			if (max_reduction > MAX_ZOOM_OUT
-					&& min_reduction > MAX_ZOOM_OUT)
-#endif
+			if (max_reduction > opt_max_zoom_out
+					&& min_reduction > opt_max_zoom_out)
 			{
 				Origin.x = DISPLAY_ALIGN (ElementPtr->next.location.x);
 				Origin.y = DISPLAY_ALIGN (ElementPtr->next.location.y);
@@ -700,13 +686,8 @@ PreProcessQueue (PSIZE pscroll_x, PSIZE pscroll_y)
 					dy = -dy;
 				max_reduction = CalcReduction (dx, dy);
 			}
-#ifdef OLD_ZOOM
-			else if (max_reduction > MAX_VIS_REDUCTION
-					&& min_reduction <= MAX_VIS_REDUCTION)
-#else
-			else if (max_reduction > MAX_ZOOM_OUT
-					&& min_reduction <= MAX_ZOOM_OUT)
-#endif
+			else if (max_reduction > opt_max_zoom_out
+					&& min_reduction <= opt_max_zoom_out)
 			{
 				Origin.x = DISPLAY_ALIGN (Origin.x + (dx >> 1));
 				Origin.y = DISPLAY_ALIGN (Origin.y + (dy >> 1));
@@ -719,11 +700,7 @@ PreProcessQueue (PSIZE pscroll_x, PSIZE pscroll_y)
 			}
 			else
 			{
-#ifdef OLD_ZOOM
-				BYTE reduction;
-#else
 				SIZE reduction;
-#endif
 
 				if (dx < 0)
 					dx = -dx;
@@ -731,13 +708,8 @@ PreProcessQueue (PSIZE pscroll_x, PSIZE pscroll_y)
 					dy = -dy;
 				reduction = CalcReduction (dx << 1, dy << 1);
 
-#ifdef OLD_ZOOM
-				if (min_reduction > MAX_VIS_REDUCTION
+				if (min_reduction > opt_max_zoom_out
 						|| reduction < min_reduction)
-#else
-				if (min_reduction > MAX_ZOOM_OUT
-						|| reduction < min_reduction)
-#endif
 					min_reduction = reduction;
 			}
 //			fprintf (stderr, "dx = %d dy = %d min_red = %d max_red = %d\n",
@@ -748,17 +720,15 @@ PreProcessQueue (PSIZE pscroll_x, PSIZE pscroll_y)
 		hElement = hNextElement;
 	}
 
-#ifdef OLD_ZOOM
-	if ((min_reduction > MAX_VIS_REDUCTION || min_reduction <= max_reduction)
-			&& (min_reduction = max_reduction) > MAX_VIS_REDUCTION
-			&& (min_reduction = GLOBAL (cur_state)) > MAX_VIS_REDUCTION)
-		min_reduction = 0;
-#else
-	if ((min_reduction > MAX_ZOOM_OUT || min_reduction <= max_reduction)
-			&& (min_reduction = max_reduction) > MAX_ZOOM_OUT
-			&& (min_reduction = zoom_out) > MAX_ZOOM_OUT)
-		min_reduction = 1 << ZOOM_SHIFT;
-#endif
+	if ((min_reduction > opt_max_zoom_out || min_reduction <= max_reduction)
+			&& (min_reduction = max_reduction) > opt_max_zoom_out
+			&& (min_reduction = zoom_out) > opt_max_zoom_out)
+	{
+		if (optMeleeScale == TFB_SCALE_STEP)
+			min_reduction = 0;
+		else
+			min_reduction = 1 << ZOOM_SHIFT;
+	}
 
 #ifdef KDEBUG
 	fprintf (stderr, "PreProcess: exit\n");
@@ -803,31 +773,39 @@ InsertPrim (PRIM_LINKS *pLinks, COUNT primIndex, COUNT iPI)
 
 PRIM_LINKS DisplayLinks;
 
+static inline COORD
+CalcDisplayCoord (COORD c, COORD orgc, SIZE reduction)
+{
+	if (optMeleeScale == TFB_SCALE_STEP)
+	{	/* old fixed-step zoom style */
+		return (c - orgc) >> reduction;
+	}
+	else
+	{	/* new continuous zoom style */
+		return ((c - orgc) << ZOOM_SHIFT) / reduction;
+	}
+}
+
 static void
-PostProcessQueue (register VIEW_STATE view_state, register SIZE scroll_x,
-		register SIZE scroll_y)
+PostProcessQueue (VIEW_STATE view_state, SIZE scroll_x,
+		SIZE scroll_y)
 {
 	POINT delta;
-#ifdef OLD_ZOOM
-	register BYTE reduction;
-#else
-	register SIZE reduction;
-#endif
-	register HELEMENT hElement;
+	SIZE reduction;
+	HELEMENT hElement;
 
 #ifdef KDEBUG
 	fprintf (stderr, "PostProcess:\n");
 #endif
-#ifdef OLD_ZOOM
-	reduction = (BYTE)(GLOBAL (cur_state) + ONE_SHIFT);
-#else
-	reduction = zoom_out << ONE_SHIFT;
-#endif
+	if (optMeleeScale == TFB_SCALE_STEP)
+		reduction = zoom_out + ONE_SHIFT;
+	else
+		reduction = zoom_out << ONE_SHIFT;
 
 	hElement = GetHeadElement ();
 	while (hElement != 0)
 	{
-		register ELEMENT_FLAGS state_flags;
+		ELEMENT_FLAGS state_flags;
 		ELEMENTPTR ElementPtr;
 		HELEMENT hNextElement;
 
@@ -870,12 +848,12 @@ PostProcessQueue (register VIEW_STATE view_state, register SIZE scroll_x,
 				UnlockElement (hPostElement);
 			} while ((hPostElement = hNextElement) != 0);
 
-			scroll_x = scroll_y = 0;
-			delta.x = delta.y = 0; /* because these are
-										  * newly added elements
-										  * that are already in
-										  * adjusted coordinates
-										  */
+			scroll_x = 0;
+			scroll_y = 0;
+			delta.x = 0;
+			delta.y = 0;
+					/* because these are newly added elements that are
+					 * already in adjusted coordinates */
 			state_flags = ElementPtr->state_flags;
 		}
 
@@ -907,70 +885,39 @@ PostProcessQueue (register VIEW_STATE view_state, register SIZE scroll_x,
 
 					next.x = WRAP_X (ElementPtr->current.location.x + delta.x);
 					next.y = WRAP_Y (ElementPtr->current.location.y + delta.y);
-#ifdef OLD_ZOOM
 					DisplayArray[ElementPtr->PrimIndex].Object.Line.first.x =
-							(next.x - SpaceOrg.x) >> reduction;
+							CalcDisplayCoord (next.x, SpaceOrg.x, reduction);
 					DisplayArray[ElementPtr->PrimIndex].Object.Line.first.y =
-							(next.y - SpaceOrg.y) >> reduction;
-#else
-					DisplayArray[ElementPtr->PrimIndex].Object.Line.first.x =
-							((next.x - SpaceOrg.x) << ZOOM_SHIFT) / reduction;
-					DisplayArray[ElementPtr->PrimIndex].Object.Line.first.y =
-							((next.y - SpaceOrg.y) << ZOOM_SHIFT) / reduction;
-#endif
+							CalcDisplayCoord (next.y, SpaceOrg.y, reduction);
+
 					next.x += dx;
 					next.y += dy;
-#ifdef OLD_ZOOM
 					DisplayArray[ElementPtr->PrimIndex].Object.Line.second.x =
-							(next.x - SpaceOrg.x) >> reduction;
+							CalcDisplayCoord (next.x, SpaceOrg.x, reduction);
 					DisplayArray[ElementPtr->PrimIndex].Object.Line.second.y =
-							(next.y - SpaceOrg.y) >> reduction;
-#else
-					DisplayArray[ElementPtr->PrimIndex].Object.Line.second.x =
-							((next.x - SpaceOrg.x) << ZOOM_SHIFT) / reduction;
-					DisplayArray[ElementPtr->PrimIndex].Object.Line.second.y =
-							((next.y - SpaceOrg.y) << ZOOM_SHIFT) / reduction;
-#endif
+							CalcDisplayCoord (next.y, SpaceOrg.y, reduction);
 				}
 				else
 				{
 					next.x = WRAP_X (ElementPtr->next.location.x + delta.x);
 					next.y = WRAP_Y (ElementPtr->next.location.y + delta.y);
-#ifdef OLD_ZOOM
 					DisplayArray[ElementPtr->PrimIndex].Object.Point.x =
-							(next.x - SpaceOrg.x) >> reduction;
+							CalcDisplayCoord (next.x, SpaceOrg.x, reduction);
 					DisplayArray[ElementPtr->PrimIndex].Object.Point.y =
-							(next.y - SpaceOrg.y) >> reduction;
-#else
-					DisplayArray[ElementPtr->PrimIndex].Object.Point.x =
-							((next.x - SpaceOrg.x) << ZOOM_SHIFT) / reduction;
-					DisplayArray[ElementPtr->PrimIndex].Object.Point.y =
-							((next.y - SpaceOrg.y) << ZOOM_SHIFT) / reduction;
-#endif
+							CalcDisplayCoord (next.y, SpaceOrg.y, reduction);
 
-					if (ObjType == STAMP_PRIM
-							|| ObjType == STAMPFILL_PRIM)
+					if (ObjType == STAMP_PRIM || ObjType == STAMPFILL_PRIM)
 					{
 						if (view_state == VIEW_CHANGE
 								|| (state_flags & (APPEARING | CHANGING)))
 						{
-#ifdef OLD_ZOOM
-							ElementPtr->next.image.frame =
-#ifdef SAFE
-									SetAbsFrameIndex (
-									ElementPtr->next.image.farray
-									[GLOBAL (cur_state)],
-									GetFrameIndex (ElementPtr->next.image.frame));
-#else /* SAFE */
-									SetEquFrameIndex (
-									ElementPtr->next.image.farray
-									[GLOBAL (cur_state)],
-									ElementPtr->next.image.frame);
-#endif /* SAFE */
-#else
-							COUNT index, scale;
-							
-							CALC_ZOOM_STUFF (&index, &scale);
+							COUNT index, scale = GSCALE_IDENTITY;
+						
+							if (optMeleeScale == TFB_SCALE_STEP)
+								index = zoom_out;
+							else
+								CALC_ZOOM_STUFF (&index, &scale);
+
 							ElementPtr->next.image.frame =
 #ifdef SAFE
 									SetAbsFrameIndex (
@@ -983,6 +930,7 @@ PostProcessQueue (register VIEW_STATE view_state, register SIZE scroll_x,
 									[index],
 									ElementPtr->next.image.frame);
 #endif /* SAFE */
+
 							if (optMeleeScale == TFB_SCALE_TRILINEAR && index < 2 && scale != 256)
 							{
 								// enqueues drawcommand to assign next (smaller) zoom
@@ -1006,7 +954,6 @@ PostProcessQueue (register VIEW_STATE view_state, register SIZE scroll_x,
 									TFB_EnqueueDrawCommand (&DC);
 								}
 							}
-#endif
 						}
 						DisplayArray[ElementPtr->PrimIndex].Object.Stamp.frame =
 								ElementPtr->next.image.frame;
@@ -1037,11 +984,16 @@ InitDisplayList (void)
 {
 	COUNT i;
 
-#ifdef OLD_ZOOM
-	GLOBAL (cur_state) = MAX_VIS_REDUCTION + 1;
-#else
-	zoom_out = MAX_ZOOM_OUT + (1 << ZOOM_SHIFT);
-#endif
+	if (optMeleeScale == TFB_SCALE_STEP)
+	{
+		zoom_out = MAX_VIS_REDUCTION + 1;
+		opt_max_zoom_out = MAX_VIS_REDUCTION;
+	}
+	else
+	{
+		zoom_out = MAX_ZOOM_OUT + (1 << ZOOM_SHIFT);
+		opt_max_zoom_out = MAX_ZOOM_OUT;
+	}
 
 	ReinitQueue (&disp_q);
 
@@ -1052,64 +1004,13 @@ InitDisplayList (void)
 	DisplayLinks = MakeLinks (END_OF_LIST, END_OF_LIST);
 }
 
-void
-PlaySound (SOUND S, SoundPosition Pos, ELEMENTPTR PositionalObject,
-		BYTE Priority)
-{
-#define MIN_FX_CHANNEL 1
-#define NUM_FX_CHANNELS 4 /* obviously number of channels
-												   is assumed to be at least 5 */
-	BYTE chan, c;
-	static BYTE lru_channel[NUM_FX_CHANNELS] = {0, 1, 2, 3};
-	static SOUND channel[NUM_FX_CHANNELS];
-
-	if (S == 0) return;
-
-	for (chan = 0; chan < NUM_FX_CHANNELS; ++chan)
-	{
-		if (S == channel[chan])
-			break;
-	}
-
-	if (chan == NUM_FX_CHANNELS)
-	{
-		for (chan = 0; chan < NUM_FX_CHANNELS; ++chan)
-		{
-			if (!ChannelPlaying (chan + MIN_FX_CHANNEL))
-				break;
-		}
-
-		if (chan == NUM_FX_CHANNELS)
-			chan = lru_channel[0];
-	}
-
-	channel[chan] = S;
-
-	for (c = 0; c < NUM_FX_CHANNELS - 1; ++c)
-	{
-		if (lru_channel[c] == chan)
-		{
-			memmove (
-					(PBYTE)&lru_channel[c],
-					(PBYTE)&lru_channel[c + 1],
-					(NUM_FX_CHANNELS - 1) - c
-					);
-			break;
-		}
-	}
-	lru_channel[NUM_FX_CHANNELS - 1] = chan;
-
-	PlaySoundEffect (S, chan + MIN_FX_CHANNEL, Pos, PositionalObject, Priority);
-}
-
-UWORD nth_frame;
+UWORD nth_frame = 0;
 
 void
 RedrawQueue (BOOLEAN clear)
 {
 	SIZE scroll_x, scroll_y;
 	VIEW_STATE view_state;
-	int i;
 
 	SetContext (StatusContext);
 
@@ -1117,21 +1018,7 @@ RedrawQueue (BOOLEAN clear)
 	PostProcessQueue (view_state, scroll_x, scroll_y);
 
 	if (optStereoSFX)
-	{
-		for (i = 0; i <= NUM_FX_CHANNELS; ++i)
-		{
-			/* Updates positional sound effects */
-
-			ELEMENTPTR posobj;
-			if (ChannelPlaying (i) && ((posobj = GetPositionalObject (i)) != NULL))
-			{
-				SoundPosition pos;
-				pos = CalcSoundPosition (posobj);
-				if (pos.positional)
-					UpdateSoundPosition (i, pos);
-			}
-		}
-	}
+		UpdateSoundPositions ();
 
 	SetContext (SpaceContext);
 	if (LOBYTE (GLOBAL (CurrentActivity)) == SUPER_MELEE
@@ -1145,88 +1032,28 @@ RedrawQueue (BOOLEAN clear)
 			nth_frame += skip_frames;
 			if (clear)
 				ClearDrawable (); // this is for BATCH_BUILD_PAGE effect, but not scaled by SetGraphicScale
+
+			if (optMeleeScale != TFB_SCALE_STEP)
 			{
 				COUNT index, scale;
 
 				CALC_ZOOM_STUFF (&index, &scale);
 				SetGraphicScale (scale);
 			}
+
 			DrawBatch (DisplayArray, DisplayLinks, 0);//BATCH_BUILD_PAGE);
 			SetGraphicScale (0);
 		}
 
-		if (num_sounds > 0)
-		{
-			SOUND *pSound;
-			ELEMENTPTR *pSoundPosObj;
-
-			pSound = sound_buf;
-			pSoundPosObj = sound_posobj;
-			do
-			{
-				SoundPosition pos = CalcSoundPosition (*pSoundPosObj);
-				PlaySound (*pSound++, pos, *pSoundPosObj++,
-						GAME_SOUND_PRIORITY);
-			} while (--num_sounds);
-		}
-	}
-	DisplayLinks = MakeLinks (END_OF_LIST, END_OF_LIST);
-}
-
-void
-ProcessSound (SOUND Sound, ELEMENTPTR PositionalObject)
-{
-	if (Sound == (SOUND)~0)
-	{
-		memset (sound_buf, 0, sizeof (sound_buf));
-		memset (sound_posobj, 0, sizeof (sound_posobj));
-		num_sounds = MAX_SOUNDS;
-	}
-	else if (num_sounds < MAX_SOUNDS)
-	{
-		sound_buf[num_sounds] = Sound;
-		sound_posobj[num_sounds++] = PositionalObject;
-	}
-}
-
-SoundPosition
-CalcSoundPosition (ELEMENTPTR ElementPtr)
-{
-	SoundPosition pos;
-
-	if (ElementPtr == NULL)
-	{
-		pos.x = pos.y = 0;
-		pos.positional = FALSE;
+		FlushSounds ();
 	}
 	else
-	{
-		GRAPHICS_PRIM objtype;
-		
-		objtype = GetPrimType (&DisplayArray[ElementPtr->PrimIndex]);
-		if (objtype == LINE_PRIM)
-		{
-			pos.x = DisplayArray[ElementPtr->PrimIndex].Object.Line.first.x;
-			pos.y = DisplayArray[ElementPtr->PrimIndex].Object.Line.first.y;
-		}
-		else
-		{
-			pos.x = DisplayArray[ElementPtr->PrimIndex].Object.Point.x;
-			pos.y = DisplayArray[ElementPtr->PrimIndex].Object.Point.y;
-		}
-
-		pos.x -= (SPACE_WIDTH >> 1);
-		pos.y -= (SPACE_HEIGHT);
-		pos.positional = TRUE;
+	{	// sfx queue needs to be flushed when aborting
+		ProcessSound ((SOUND)~0, NULL);
+		FlushSounds ();
 	}
 
-	return pos;
-}
-
-SoundPosition
-NotPositional (void)
-{
-	return CalcSoundPosition (NULL);
+	DisplayLinks = MakeLinks (END_OF_LIST, END_OF_LIST);
 }
 
 void
@@ -1262,24 +1089,14 @@ RemoveElement (HLINK hLink)
 {
 	if (optStereoSFX)
 	{
-		ELEMENTPTR ElementPtr, PosObj;
+		ELEMENTPTR ElementPtr;
 
 		LockElement (hLink, &ElementPtr);
 		if (ElementPtr != NULL)
-		{
-			int i;
-			for (i = 0; i <= NUM_FX_CHANNELS; ++i)
-			{
-				if ((PosObj = GetPositionalObject (i)) == ElementPtr)
-					SetPositionalObject (i, NULL);
-			}
-			for (i = 0; i < num_sounds; ++i)
-			{
-				if (sound_posobj[i] == ElementPtr)
-					sound_posobj[i] = NULL;
-			}
-		}
+			RemoveSoundsForObject(ElementPtr);
 		UnlockElement (hLink);
 	}
 	RemoveQueue (&disp_q, hLink);
 }
+
+
