@@ -16,9 +16,12 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "reslib.h"
+#include "ships/ship.h"
 #include "ships/melnorme/resinst.h"
-#include "starcon.h"
+
+#include "globdata.h"
+#include "libs/mathlib.h"
+
 
 #define MAX_CREW 20
 #define MAX_ENERGY MAX_ENERGY_SIZE
