@@ -16,9 +16,13 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "reslib.h"
+#include "ships/ship.h"
 #include "ships/orz/resinst.h"
-#include "starcon.h"
+
+#include "colors.h"
+#include "globdata.h"
+#include "libs/mathlib.h"
+
 
 #define MAX_CREW 16
 #define MAX_ENERGY 20
@@ -349,7 +353,7 @@ static void marine_preprocess (PELEMENT ElementPtr);
 
 #define MARINE_WAIT 12
 
-static void
+void
 intruder_preprocess (PELEMENT ElementPtr)
 {
 	HELEMENT hElement, hNextElement;
@@ -706,7 +710,7 @@ PELEMENT ElementPtr;
 	}
 }
 
-static void
+void
 marine_collision (PELEMENT ElementPtr0, PPOINT pPt0, PELEMENT ElementPtr1, PPOINT pPt1)
 {
 	if (ElementPtr0->life_span
