@@ -16,21 +16,24 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "starcon.h"
-#include "melee.h"
+#include "build.h"
+#include "colors.h"
 #include "controls.h"
+#include "fmv.h"
+#include "gameopt.h"
+#include "gamestr.h"
+#include "melee.h"
 #include "options.h"
+#include "races.h"
+#include "resinst.h"
+#include "settings.h"
+#include "setup.h"
+#include "sis.h"
+#include "sounds.h"
+#include "state.h"
 #include "libs/graphics/gfx_common.h"
+#include "libs/inplib.h"
 
-//Added by Chris
-
-#ifdef WANT_SHIP_SPINS
-void DoShipSpin (COUNT index, MUSIC_REF hMusic); // Defined in fmv.c
-#endif
-
-void GetGaugeRect (PRECT pRect, BOOLEAN IsCrewRect); // Defined in sis.c
-
-//End added by Chris
 
 #ifdef USE_3DO_HANGAR
 // 3DO 4x3 hangar layout
@@ -68,7 +71,8 @@ enum
 	SHIPYARD_EXIT
 };
 
-int hangar_anim_func (void* data)
+static int
+hangar_anim_func (void *data)
 {
 	DWORD TimeIn;
 	STAMP s;
@@ -92,7 +96,7 @@ int hangar_anim_func (void* data)
 		CONTEXT OldContext;
 		RECT OldClipRect;
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		OldContext = SetContext (ScreenContext);
 		GetContextClipRect (&OldClipRect);
 		SetContextClipRect (&ClipRect);
@@ -103,14 +107,14 @@ int hangar_anim_func (void* data)
 		
 		SetContextClipRect (&OldClipRect);
 		SetContext (OldContext);
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 		
 		SleepThreadUntil (TimeIn + ONE_SECOND / HANGAR_ANIM_RATE);
 		TimeIn = GetTimeCounter ();
 	}
 
 	FinishTask (task);
-	return(0);
+	return 0;
 }
 
 #ifdef WANT_SHIP_SPINS
@@ -156,18 +160,15 @@ GetAvailableRaceCount (void)
 		SHIP_FRAGMENTPTR StarShipPtr;
 
 		StarShipPtr = (SHIP_FRAGMENTPTR)LockStarShip (
-				&GLOBAL (avail_race_q), hStarShip
-				);
+				&GLOBAL (avail_race_q), hStarShip);
 		if (StarShipPtr->ShipInfo.ship_flags & GOOD_GUY)
 			++Index;
 
 		hNextShip = _GetSuccLink (StarShipPtr);
-		UnlockStarShip (
-				&GLOBAL (avail_race_q), hStarShip
-				);
+		UnlockStarShip (&GLOBAL (avail_race_q), hStarShip);
 	}
 
-	return (Index);
+	return Index;
 }
 
 static HSTARSHIP
@@ -180,24 +181,19 @@ GetAvailableRaceFromIndex (BYTE Index)
 	{
 		SHIP_FRAGMENTPTR StarShipPtr;
 
-		StarShipPtr = (SHIP_FRAGMENTPTR)LockStarShip (
-				&GLOBAL (avail_race_q), hStarShip
-				);
+		StarShipPtr = (SHIP_FRAGMENTPTR)LockStarShip (&GLOBAL (avail_race_q),
+				hStarShip);
 		if ((StarShipPtr->ShipInfo.ship_flags & GOOD_GUY) && Index-- == 0)
 		{
-			UnlockStarShip (
-					&GLOBAL (avail_race_q), hStarShip
-					);
-			return (hStarShip);
+			UnlockStarShip (&GLOBAL (avail_race_q), hStarShip);
+			return hStarShip;
 		}
 
 		hNextShip = _GetSuccLink (StarShipPtr);
-		UnlockStarShip (
-				&GLOBAL (avail_race_q), hStarShip
-				);
+		UnlockStarShip (&GLOBAL (avail_race_q), hStarShip);
 	}
 
-	return (0);
+	return 0;
 }
 
 static void
@@ -207,7 +203,7 @@ DrawRaceStrings (BYTE NewRaceItem)
 	STAMP s;
 	CONTEXT OldContext;
 	
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 
 	OldContext = SetContext (StatusContext);
 	GetContextClipRect (&r);
@@ -219,12 +215,14 @@ DrawRaceStrings (BYTE NewRaceItem)
 	r.extent.height = 11;
 	BatchGraphics ();
 	ClearSISRect (CLEAR_SIS_RADAR);
-	SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0xA, 0xA, 0xA), 0x08));
+	SetContextForeGroundColor (BUILD_COLOR (
+				MAKE_RGB15 (0xA, 0xA, 0xA), 0x08));
 	DrawFilledRectangle (&r);
 	r.corner = s.origin;
 	r.extent.width = RADAR_WIDTH;
 	r.extent.height = RADAR_HEIGHT;
-	SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x00, 0x00), 0x00));
+	SetContextForeGroundColor (BUILD_COLOR (
+				MAKE_RGB15 (0x00, 0x00, 0x00), 0x00));
 	DrawFilledRectangle (&r);
 	if (NewRaceItem != (BYTE)~0)
 	{
@@ -238,8 +236,10 @@ DrawRaceStrings (BYTE NewRaceItem)
 		};
 
 		hStarShip = GetAvailableRaceFromIndex (NewRaceItem);
-		NewRaceItem = GetIndexFromStarShip (&GLOBAL (avail_race_q), hStarShip);
-		s.frame = SetAbsFrameIndex (pMenuState->ModuleFrame, 3 + NewRaceItem);
+		NewRaceItem = GetIndexFromStarShip (&GLOBAL (avail_race_q),
+				hStarShip);
+		s.frame = SetAbsFrameIndex (pMenuState->ModuleFrame,
+				3 + NewRaceItem);
 		DrawStamp (&s);
 		StarShipPtr = LockStarShip (&GLOBAL (avail_race_q), hStarShip);
 		s.frame = StarShipPtr->RaceDescPtr->ship_info.melee_icon;
@@ -255,7 +255,8 @@ DrawRaceStrings (BYTE NewRaceItem)
 		t.pStr = buf;
 		wsprintf (buf, "%u", ShipCost[NewRaceItem]);
 		SetContextFont (TinyFont);
-		SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x1F, 0x00), 0x02));
+		SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (
+						0x00, 0x1F, 0x00), 0x02));
 		font_DrawText (&t);
 	}
 	UnbatchGraphics ();
@@ -272,7 +273,7 @@ DrawRaceStrings (BYTE NewRaceItem)
 	}
 	//SetFlashRect ((PRECT)~0L, (FRAME)0);
 	SetContext (OldContext);
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 }
 
 #define SHIP_WIN_WIDTH 34
@@ -288,13 +289,10 @@ ShowShipCrew (SHIP_FRAGMENTPTR StarShipPtr, PRECT pRect)
 	HSTARSHIP hTemplate;
 	SHIP_FRAGMENTPTR TemplatePtr;
 
-	hTemplate = GetStarShipFromIndex (
-			&GLOBAL (avail_race_q),
-			GET_RACE_ID (StarShipPtr)
-			);
+	hTemplate = GetStarShipFromIndex (&GLOBAL (avail_race_q),
+			GET_RACE_ID (StarShipPtr));
 	TemplatePtr = (SHIP_FRAGMENTPTR)LockStarShip (
-			&GLOBAL (avail_race_q), hTemplate
-			);
+			&GLOBAL (avail_race_q), hTemplate);
 	if (StarShipPtr->ShipInfo.crew_level >=
 			TemplatePtr->RaceDescPtr->ship_info.crew_level)
 		wsprintf (buf, "%u", StarShipPtr->ShipInfo.crew_level);
@@ -304,9 +302,7 @@ ShowShipCrew (SHIP_FRAGMENTPTR StarShipPtr, PRECT pRect)
 		wsprintf (buf, "%u/%u",
 				StarShipPtr->ShipInfo.crew_level,
 				TemplatePtr->RaceDescPtr->ship_info.crew_level);
-	UnlockStarShip (
-			&GLOBAL (avail_race_q), hTemplate
-			);
+	UnlockStarShip (&GLOBAL (avail_race_q), hTemplate);
 
 	r = *pRect;
 	t.baseline.x = r.corner.x + (r.extent.width >> 1);
@@ -389,33 +385,21 @@ ShowCombatShip (COUNT which_window, SHIP_FRAGMENTPTR YankedStarShipPtr)
 						);
 				if (GET_GROUP_LOC (StarShipPtr) > which_window)
 				{
-					UnlockStarShip (
-							&GLOBAL (built_ship_q),
-							hStarShip
-							);
+					UnlockStarShip (&GLOBAL (built_ship_q), hStarShip);
 					break;
 				}
 				hNextShip = _GetSuccLink (StarShipPtr);
-				UnlockStarShip (
-						&GLOBAL (built_ship_q),
-						hStarShip
-						);
+				UnlockStarShip (&GLOBAL (built_ship_q), hStarShip);
 
 				hStarShip = hNextShip;
 			}
-			InsertQueue (&GLOBAL (built_ship_q),
-					hTailShip, hStarShip);
+			InsertQueue (&GLOBAL (built_ship_q), hTailShip, hStarShip);
 
 			hStarShip = hTailShip;
 			StarShipPtr = (SHIP_FRAGMENTPTR)LockStarShip (
-					&GLOBAL (built_ship_q),
-					hStarShip
-					);
+					&GLOBAL (built_ship_q), hStarShip);
 			SET_GROUP_LOC (StarShipPtr, which_window);
-			UnlockStarShip (
-					&GLOBAL (built_ship_q),
-					hStarShip
-					);
+			UnlockStarShip (&GLOBAL (built_ship_q), hStarShip);
 		}
 
 		for (i = 0; i < num_ships; ++i)
@@ -459,7 +443,6 @@ ShowCombatShip (COUNT which_window, SHIP_FRAGMENTPTR YankedStarShipPtr)
 
 	if (num_ships)
 	{
-		BYTE ButtonState;
 		BOOLEAN AllDoorsFinished;
 		DWORD TimeIn;
 		RECT r;
@@ -471,15 +454,7 @@ ShowCombatShip (COUNT which_window, SHIP_FRAGMENTPTR YankedStarShipPtr)
 		r.corner.x = r.corner.y = 0;
 		r.extent.width = SHIP_WIN_WIDTH;
 		r.extent.height = SHIP_WIN_HEIGHT;
-		ButtonState = AnyButtonPress (FALSE) ? 1 : 0;
-		if (ButtonState)
-		{
-			while (ButtonState)
-				if (!AnyButtonPress (FALSE))
-					ButtonState = 0;
-				else
-					TaskSwitch();
-		}
+		FlushInput ();
 		TimeIn = GetTimeCounter ();
 
 		for (j = 0; (j < SHIP_WIN_FRAMES) && !AllDoorsFinished; j++)
@@ -494,10 +469,8 @@ ShowCombatShip (COUNT which_window, SHIP_FRAGMENTPTR YankedStarShipPtr)
 					ship_win_info[0].rtdoor_s.origin.x = 0;
 				}
 				AllDoorsFinished = TRUE;
-				while (AnyButtonPress (FALSE))
-					TaskSwitch();
 			}
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			OldContext = SetContext (OffScreenContext);
 			SetContextFGFrame (Screen);
 			SetContextBackGroundColor (BLACK_COLOR);
@@ -510,8 +483,10 @@ ShowCombatShip (COUNT which_window, SHIP_FRAGMENTPTR YankedStarShipPtr)
 				{
 					RECT ClipRect;
 
-					ClipRect.corner.x = SIS_ORG_X + pship_win_info->finished_s.x;
-					ClipRect.corner.y = SIS_ORG_Y + pship_win_info->finished_s.y;
+					ClipRect.corner.x = SIS_ORG_X +
+							pship_win_info->finished_s.x;
+					ClipRect.corner.y = SIS_ORG_Y +
+							pship_win_info->finished_s.y;
 					ClipRect.extent.width = SHIP_WIN_WIDTH;
 					ClipRect.extent.height = SHIP_WIN_HEIGHT;
 					SetContextClipRect (&ClipRect);
@@ -541,7 +516,7 @@ ShowCombatShip (COUNT which_window, SHIP_FRAGMENTPTR YankedStarShipPtr)
 			UnbatchGraphics ();
 			SetContextClipRect (NULL_PTR);
 			SetContext (OldContext);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 		}
 	}
 }
@@ -555,8 +530,7 @@ CrewTransaction (SIZE crew_delta)
 
 		crew_bought = (SIZE)MAKE_WORD (
 				GET_GAME_STATE (CREW_PURCHASED0),
-				GET_GAME_STATE (CREW_PURCHASED1)
-				) + crew_delta;
+				GET_GAME_STATE (CREW_PURCHASED1)) + crew_delta;
 		if (crew_bought < 0)
 		{
 			if (crew_delta < 0)
@@ -571,9 +545,9 @@ CrewTransaction (SIZE crew_delta)
 			{
 				GLOBAL (CrewCost) += 2;
 
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 				DrawMenuStateStrings (PM_CREW, SHIPYARD_CREW);
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 			}
 		}
 		else
@@ -583,14 +557,12 @@ CrewTransaction (SIZE crew_delta)
 			{
 				GLOBAL (CrewCost) -= 2;
 
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 				DrawMenuStateStrings (PM_CREW, SHIPYARD_CREW);
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 			}
 		}
-		if (!(ActivateStarShip (
-				SHOFIXTI_SHIP, CHECK_ALLIANCE
-				) & GOOD_GUY))
+		if (!(ActivateStarShip (SHOFIXTI_SHIP, CHECK_ALLIANCE) & GOOD_GUY))
 		{
 			SET_GAME_STATE (CREW_PURCHASED0, LOBYTE (crew_bought));
 			SET_GAME_STATE (CREW_PURCHASED1, HIBYTE (crew_bought));
@@ -616,15 +588,15 @@ DoModifyShips (PMENU_STATE pMS)
 #ifdef WANT_SHIP_SPINS
 	BOOLEAN special;
 
-	special = CurrentMenuState.special;
+	special = PulsedInputState.key[KEY_MENU_SPECIAL];
 #endif /* WANT_SHIP_SPINS */
-	select = CurrentMenuState.select;
-	cancel = CurrentMenuState.cancel;
+	select = PulsedInputState.key[KEY_MENU_SELECT];
+	cancel = PulsedInputState.key[KEY_MENU_CANCEL];
 
 	if (GLOBAL (CurrentActivity) & CHECK_ABORT)
 	{
 		pMS->InputFunc = DoShipyard;
-		return (TRUE);
+		return TRUE;
 	}
 
 	if (!pMS->Initialized)
@@ -634,19 +606,25 @@ DoModifyShips (PMENU_STATE pMS)
 		pMS->CurState = MAKE_BYTE (0, 0xF);
 		pMS->delta_item = 0;
 
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		SetContext (SpaceContext);
 		goto ChangeFlashRect;
 	}
 	else
 	{
-		SBYTE dx = 0, dy = 0;
+		SBYTE dx = 0;
+		SBYTE dy = 0;
 		BYTE NewState;
 
-		if (CurrentMenuState.right) dx = 1;
-		if (CurrentMenuState.left) dx = -1;
-		if (CurrentMenuState.up) dy = -1;
-		if (CurrentMenuState.down) dy = 1;
+		if (!(pMS->delta_item & MODIFY_CREW_FLAG))
+		{
+			SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
+		}
+
+		if (PulsedInputState.key[KEY_MENU_RIGHT]) dx = 1;
+		if (PulsedInputState.key[KEY_MENU_LEFT]) dx = -1;
+		if (PulsedInputState.key[KEY_MENU_UP]) dy = -1;
+		if (PulsedInputState.key[KEY_MENU_DOWN]) dy = 1;
 		NewState = pMS->CurState;
 		if (pMS->delta_item & MODIFY_CREW_FLAG)
 		{
@@ -681,10 +659,9 @@ DoModifyShips (PMENU_STATE pMS)
 				NewState = (BYTE)(pMS->CurState - NewState + dx);
 		}
 
-#ifdef WANT_SHIP_SPINS
-		if (select || cancel || special
-#else
 		if (select || cancel
+#ifdef WANT_SHIP_SPINS
+				|| special
 #endif
 				|| NewState != pMS->CurState
 				|| ((pMS->delta_item & MODIFY_CREW_FLAG) && (dx || dy)))
@@ -693,24 +670,28 @@ DoModifyShips (PMENU_STATE pMS)
 					hStarShip; hStarShip = hNextShip)
 			{
 				StarShipPtr = (SHIP_FRAGMENTPTR)LockStarShip (
-						&GLOBAL (built_ship_q), hStarShip
-						);
+						&GLOBAL (built_ship_q), hStarShip);
 
 				if (GET_GROUP_LOC (StarShipPtr) == pMS->CurState)
 				{
-					UnlockStarShip (
-							&GLOBAL (built_ship_q), hStarShip
-							);
+					UnlockStarShip (&GLOBAL (built_ship_q), hStarShip);
 					break;
 				}
 
 				hNextShip = _GetSuccLink (StarShipPtr);
-				UnlockStarShip (
-						&GLOBAL (built_ship_q), hStarShip
-						);
+				UnlockStarShip (&GLOBAL (built_ship_q), hStarShip);
 			}
-
-			SetSemaphore (GraphicsSem);
+			if ((pMS->delta_item & MODIFY_CREW_FLAG) && (hStarShip))
+			{
+				SetMenuSounds (MENU_SOUND_UP | MENU_SOUND_DOWN,
+						MENU_SOUND_SELECT | MENU_SOUND_CANCEL);
+			}
+			else
+			{
+				SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
+			}
+	
+			LockMutex (GraphicsLock);
 
 #ifdef WANT_SHIP_SPINS
 			if (special)
@@ -720,8 +701,7 @@ DoModifyShips (PMENU_STATE pMS)
 				if ((hSpinShip = hStarShip)
 						|| (HINIBBLE (pMS->CurState) == 0
 						&& (hSpinShip = GetAvailableRaceFromIndex (
-								LOBYTE (pMS->delta_item)
-								))))
+								LOBYTE (pMS->delta_item)))))
 				{
 					SetFlashRect (NULL_PTR, (FRAME)0);
 					SpinStarShip (hSpinShip);
@@ -745,38 +725,41 @@ DoModifyShips (PMENU_STATE pMS)
 					COUNT Index;
 
 // SetFlashRect (NULL_PTR, (FRAME)0);
-					ClearSemaphore (GraphicsSem);
+					UnlockMutex (GraphicsLock);
 					if (!(pMS->delta_item & MODIFY_CREW_FLAG))
 					{
 						pMS->delta_item = MODIFY_CREW_FLAG;
 						DrawRaceStrings (0);
-						return (TRUE);
+						return TRUE;
 					}
 					else if (cancel)
 					{
 						pMS->delta_item ^= MODIFY_CREW_FLAG;
+						LockMutex (GraphicsLock);
 						SetFlashRect ((PRECT)~0L, (FRAME)0);
+						UnlockMutex (GraphicsLock);
 						DrawMenuStateStrings (PM_CREW, SHIPYARD_CREW);
+						SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
 					}
 					else if (select)
 					{
-						Index = GetIndexFromStarShip (
-								&GLOBAL (avail_race_q),
-								GetAvailableRaceFromIndex (LOBYTE (pMS->delta_item))
-								);
+						Index = GetIndexFromStarShip (&GLOBAL (avail_race_q),
+								GetAvailableRaceFromIndex (
+								LOBYTE (pMS->delta_item)));
 
 						if (GLOBAL_SIS (ResUnits) >= (DWORD)ShipCost[Index]
 								&& CloneShipFragment (Index,
 								&GLOBAL (built_ship_q), 1))
 						{
-							ShowCombatShip ((COUNT)pMS->CurState, (SHIP_FRAGMENTPTR)0);
+							ShowCombatShip ((COUNT)pMS->CurState,
+									(SHIP_FRAGMENTPTR) 0);
 							//Reset flash rectangle
-							SetSemaphore (GraphicsSem);
+							LockMutex (GraphicsLock);
 							SetFlashRect ((PRECT)~0L, (FRAME)0);
-							ClearSemaphore (GraphicsSem);
+							UnlockMutex (GraphicsLock);
 							DrawMenuStateStrings (PM_CREW, SHIPYARD_CREW);
 
-							SetSemaphore (GraphicsSem);
+							LockMutex (GraphicsLock);
 							DeltaSISGauges (UNDEFINED_DELTA, UNDEFINED_DELTA,
 									-((int)ShipCost[Index]));
 							r.corner.x = pMS->flash_rect0.corner.x;
@@ -786,10 +769,12 @@ DoModifyShips (PMENU_STATE pMS)
 							r.extent.height = 5;
 							SetContext (SpaceContext);
 							SetFlashRect (&r, (FRAME)0);
-							ClearSemaphore (GraphicsSem);
+							UnlockMutex (GraphicsLock);
 						}
+						SetMenuSounds (MENU_SOUND_UP | MENU_SOUND_DOWN,
+								MENU_SOUND_SELECT | MENU_SOUND_CANCEL);
 							
-						return (TRUE);
+						return TRUE;
 					}
 					else
 					{
@@ -812,44 +797,59 @@ DoModifyShips (PMENU_STATE pMS)
 							pMS->delta_item = NewState | MODIFY_CREW_FLAG;
 						}
 						
-						return (TRUE);
+						return TRUE;
 					}
-					SetSemaphore (GraphicsSem);
+					LockMutex (GraphicsLock);
 					goto ChangeFlashRect;
 				}
 				else if (select || cancel)
-				{					
+				{
 					if ((pMS->delta_item & MODIFY_CREW_FLAG)
-							&& hStarShip != 0
-							&& StarShipPtr->ShipInfo.crew_level == 0)
+							&& hStarShip != 0)
 					{
-						SetFlashRect (NULL_PTR, (FRAME)0);
-						ClearSemaphore (GraphicsSem);
-						ShowCombatShip ((COUNT)pMS->CurState, StarShipPtr);
-						SetSemaphore (GraphicsSem);
-						UnlockStarShip (&GLOBAL (built_ship_q), hStarShip);
-						RemoveQueue (&GLOBAL (built_ship_q), hStarShip);
-						FreeStarShip (&GLOBAL (built_ship_q), hStarShip);
-						// refresh SIS display
-						DeltaSISGauges (UNDEFINED_DELTA, UNDEFINED_DELTA,
-								UNDEFINED_DELTA);
-						DrawStatusMessage ((UNICODE *)~0);
-						r.corner.x = pMS->flash_rect0.corner.x;
-						r.corner.y = pMS->flash_rect0.corner.y;
-						r.extent.width = SHIP_WIN_WIDTH;
-						r.extent.height = SHIP_WIN_HEIGHT;
-						SetContext (SpaceContext);
-						SetFlashRect (&r, (FRAME)0);
+						StarShipPtr = (SHIP_FRAGMENTPTR)LockStarShip (
+								&GLOBAL (built_ship_q), hStarShip);
+						if (StarShipPtr->ShipInfo.crew_level == 0)
+						{
+							SetFlashRect (NULL_PTR, (FRAME)0);
+							UnlockMutex (GraphicsLock);
+							ShowCombatShip ((COUNT)pMS->CurState,
+									StarShipPtr);
+							LockMutex (GraphicsLock);
+							UnlockStarShip (&GLOBAL (built_ship_q),
+									hStarShip);
+							RemoveQueue (&GLOBAL (built_ship_q), hStarShip);
+							FreeStarShip (&GLOBAL (built_ship_q), hStarShip);
+							// refresh SIS display
+							DeltaSISGauges (UNDEFINED_DELTA, UNDEFINED_DELTA,
+									UNDEFINED_DELTA);
+							DrawStatusMessage ((UNICODE *)~0);
+							r.corner.x = pMS->flash_rect0.corner.x;
+							r.corner.y = pMS->flash_rect0.corner.y;
+							r.extent.width = SHIP_WIN_WIDTH;
+							r.extent.height = SHIP_WIN_HEIGHT;
+							SetContext (SpaceContext);
+							SetFlashRect (&r, (FRAME)0);
+						}
+						else
+						{
+							UnlockStarShip (&GLOBAL (built_ship_q),
+									hStarShip);
+						}
 					}
 					
 					if (!(pMS->delta_item ^= MODIFY_CREW_FLAG))
+					{
 						goto ChangeFlashRect;
+					}
 					else if (hStarShip == 0)
 					{
 						SetContext (StatusContext);
 						GetGaugeRect (&r, TRUE);
 						SetFlashRect (&r, (FRAME)0);
 						SetContext (SpaceContext);
+						SetMenuSounds (MENU_SOUND_UP | MENU_SOUND_DOWN,
+								MENU_SOUND_SELECT | MENU_SOUND_CANCEL);
 					}
 					else
 					{
@@ -860,6 +860,8 @@ DoModifyShips (PMENU_STATE pMS)
 						r.extent.height = 5;
 						SetContext (SpaceContext);
 						SetFlashRect (&r, (FRAME)0);
+						SetMenuSounds (MENU_SOUND_UP | MENU_SOUND_DOWN,
+								MENU_SOUND_SELECT | MENU_SOUND_CANCEL);
 					}
 				}
 				else if (pMS->delta_item & MODIFY_CREW_FLAG)
@@ -868,9 +870,12 @@ DoModifyShips (PMENU_STATE pMS)
 
 					if (hStarShip)
 						StarShipPtr = (SHIP_FRAGMENTPTR)LockStarShip (
-								&GLOBAL (built_ship_q), hStarShip
-								);
+								&GLOBAL (built_ship_q), hStarShip);
+					else
+						StarShipPtr = NULL;  // Keeping compiler quiet.
 
+					SetMenuSounds (MENU_SOUND_UP | MENU_SOUND_DOWN,
+							MENU_SOUND_SELECT | MENU_SOUND_CANCEL);
 					crew_delta = 0;
 					if (dy < 0)
 					{
@@ -897,24 +902,22 @@ DoModifyShips (PMENU_STATE pMS)
 
 							hTemplate = GetStarShipFromIndex (
 									&GLOBAL (avail_race_q),
-									GET_RACE_ID (StarShipPtr)
-									);
+									GET_RACE_ID (StarShipPtr));
 							TemplatePtr = (SHIP_FRAGMENTPTR)LockStarShip (
-									&GLOBAL (avail_race_q), hTemplate
-									);
-							if (GLOBAL_SIS (ResUnits) >= (DWORD)GLOBAL (CrewCost)
+									&GLOBAL (avail_race_q), hTemplate);
+							if (GLOBAL_SIS (ResUnits) >=
+									(DWORD)GLOBAL (CrewCost)
 									&& StarShipPtr->ShipInfo.crew_level <
 									StarShipPtr->ShipInfo.max_crew &&
 									StarShipPtr->ShipInfo.crew_level <
-									TemplatePtr->RaceDescPtr->ship_info.crew_level)
+									TemplatePtr->RaceDescPtr->ship_info.
+									crew_level)
 							{
 								if (StarShipPtr->ShipInfo.crew_level > 0)
 									DeltaSISGauges (0, 0, -GLOBAL (CrewCost));
 								else
-									DeltaSISGauges (0, 0,
-											-(COUNT)ShipCost[
-											GET_RACE_ID (StarShipPtr)
-											]);
+									DeltaSISGauges (0, 0, -(COUNT)ShipCost[
+											GET_RACE_ID (StarShipPtr) ]);
 								++StarShipPtr->ShipInfo.crew_level;
 								crew_delta = 1;
 								ShowShipCrew (StarShipPtr, &pMS->flash_rect0);
@@ -926,17 +929,15 @@ DoModifyShips (PMENU_STATE pMS)
 								SetContext (SpaceContext);
 								SetFlashRect (&r, (FRAME)0);
 							}
-							UnlockStarShip (
-									&GLOBAL (avail_race_q), hTemplate
-									);
+							UnlockStarShip (&GLOBAL (avail_race_q),
+									hTemplate);
 						}
 					}
 					else if (dy > 0)
 					{
 						crew_bought = (SIZE)MAKE_WORD (
 								GET_GAME_STATE (CREW_PURCHASED0),
-								GET_GAME_STATE (CREW_PURCHASED1)
-								);
+								GET_GAME_STATE (CREW_PURCHASED1));
 						if (hStarShip == 0)
 						{
 							if (GetCrewCount ())
@@ -965,10 +966,8 @@ DoModifyShips (PMENU_STATE pMS)
 											- (crew_bought ==
 											CREW_EXPENSE_THRESHOLD ? 2 : 0));
 								else
-									DeltaSISGauges (0, 0,
-											(COUNT)ShipCost[
-											GET_RACE_ID (StarShipPtr)
-											]);
+									DeltaSISGauges (0, 0, (COUNT)ShipCost[
+											GET_RACE_ID (StarShipPtr)]);
 								crew_delta = -1;
 								--StarShipPtr->ShipInfo.crew_level;
 							}
@@ -985,9 +984,7 @@ DoModifyShips (PMENU_STATE pMS)
 
 					if (hStarShip)
 					{
-						UnlockStarShip (
-								&GLOBAL (built_ship_q), hStarShip
-								);
+						UnlockStarShip (&GLOBAL (built_ship_q), hStarShip);
 						
 						// clear out the bought ship index
 						// so that flash rects work correctly
@@ -998,15 +995,15 @@ DoModifyShips (PMENU_STATE pMS)
 			}
 			else if (cancel)
 			{
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 
 				pMS->InputFunc = DoShipyard;
 				DrawMenuStateStrings (PM_CREW, pMS->CurState = SHIPYARD_CREW);
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 				SetFlashRect ((PRECT)~0L, (FRAME)0);
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 
-				return (TRUE);
+				return TRUE;
 			}
 			else
 			{
@@ -1031,11 +1028,11 @@ ChangeFlashRect:
 				}
 				SetFlashRect (&pMS->flash_rect0, (FRAME)0);
 			}
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 		}
 	}
 
-	return (TRUE);
+	return TRUE;
 }
 
 static void
@@ -1044,7 +1041,7 @@ DrawBluePrint (PMENU_STATE pMS)
 	COUNT num_frames;
 	STAMP s;
 
-	SetSemaphore (GraphicsSem);
+	LockMutex (GraphicsLock);
 	SetContext (SpaceContext);
 
 	pMS->ModuleFrame = CaptureDrawable (
@@ -1055,7 +1052,8 @@ DrawBluePrint (PMENU_STATE pMS)
 
 	s.origin.x = s.origin.y = 0;
 	s.frame = DecFrameIndex (pMS->ModuleFrame);
-	SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x00, 0x16), 0x01));
+	SetContextForeGroundColor (BUILD_COLOR (
+				MAKE_RGB15 (0x00, 0x00, 0x16), 0x01));
 	DrawFilledStamp (&s);
 
 	for (num_frames = 0; num_frames < NUM_DRIVE_SLOTS; ++num_frames)
@@ -1078,7 +1076,8 @@ DrawBluePrint (PMENU_STATE pMS)
 			DrawShipPiece (pMS, which_piece, num_frames, TRUE);
 	}
 
-	SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0xA, 0xA, 0x1F), 0x09));
+	SetContextForeGroundColor (BUILD_COLOR (
+				MAKE_RGB15 (0xA, 0xA, 0x1F), 0x09));
 	for (num_frames = 0; num_frames < NUM_MODULE_SLOTS; ++num_frames)
 	{
 		BYTE which_piece;
@@ -1141,7 +1140,8 @@ DrawBluePrint (PMENU_STATE pMS)
 			r.corner.x += r.extent.width + 1;
 			DrawPoint (&r.corner);
 			r.corner.x -= r.extent.width;
-			SetContextForeGroundColor (SetContextBackGroundColor (BLACK_COLOR));
+			SetContextForeGroundColor (
+					SetContextBackGroundColor (BLACK_COLOR));
 			DrawFilledRectangle (&r);
 			m = FuelVolume < FUEL_VOLUME_PER_ROW ?
 					(COUNT)FuelVolume : FUEL_VOLUME_PER_ROW;
@@ -1155,10 +1155,10 @@ DrawBluePrint (PMENU_STATE pMS)
 	DestroyDrawable (ReleaseDrawable (pMS->ModuleFrame));
 	pMS->ModuleFrame = 0;
 
-	ClearSemaphore (GraphicsSem);
+	UnlockMutex (GraphicsLock);
 }
 
-void
+static void
 BeginHangarAnim (PMENU_STATE pMS)
 {
 #ifdef WANT_HANGAR_ANIMATION
@@ -1179,7 +1179,7 @@ BeginHangarAnim (PMENU_STATE pMS)
 #endif
 }
 
-void
+static void
 EndHangarAnim (PMENU_STATE pMS)
 {
 	if (pMS->flash_task)
@@ -1196,9 +1196,10 @@ DoShipyard (PMENU_STATE pMS)
 	if (GLOBAL (CurrentActivity) & CHECK_ABORT)
 		goto ExitShipyard;
 
-	select = CurrentMenuState.select;
-	cancel = CurrentMenuState.cancel;
+	select = PulsedInputState.key[KEY_MENU_SELECT];
+	cancel = PulsedInputState.key[KEY_MENU_CANCEL];
 
+	SetMenuSounds (MENU_SOUND_ARROWS, MENU_SOUND_SELECT);
 	if (!pMS->Initialized)
 	{
 		pMS->InputFunc = DoShipyard;
@@ -1207,9 +1208,7 @@ DoShipyard (PMENU_STATE pMS)
 			STAMP s;
 			RECT r, old_r;
 
-			s.frame = CaptureDrawable (
-					LoadGraphic (SHIPYARD_PMAP_ANIM)
-					);
+			s.frame = CaptureDrawable (LoadGraphic (SHIPYARD_PMAP_ANIM));
 
 			pMS->CurString = CaptureColorMap (LoadColorMapFile (
 					contentDir, "lbm/dockpani.ct"));
@@ -1220,16 +1219,16 @@ DoShipyard (PMENU_STATE pMS)
 			BatchGraphics ();
 
 			DrawSISFrame ();
-			SetSemaphore(GraphicsSem);
+			LockMutex (GraphicsLock);
 			DrawSISMessage (GAME_STRING (STARBASE_STRING_BASE + 3));
 			DrawSISTitle (GAME_STRING (STARBASE_STRING_BASE));
-			ClearSemaphore(GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			DrawBluePrint (pMS);
 			pMS->ModuleFrame = s.frame;
 
 			DrawMenuStateStrings (PM_CREW, pMS->CurState = SHIPYARD_CREW);
 
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			SetContext (SpaceContext);
 			s.origin.x = s.origin.y = 0;
 #ifdef USE_3DO_HANGAR
@@ -1261,12 +1260,12 @@ DoShipyard (PMENU_STATE pMS)
 			PlayMusic (pMS->hMusic, TRUE, 1);
 			UnbatchGraphics ();
 			BeginHangarAnim (pMS);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 
 			ShowCombatShip ((COUNT)~0, (SHIP_FRAGMENTPTR)0);
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			SetFlashRect ((PRECT)~0L, (FRAME)0);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 		}
 
 		pMS->Initialized = TRUE;
@@ -1275,15 +1274,15 @@ DoShipyard (PMENU_STATE pMS)
 	{
 ExitShipyard:
 		EndHangarAnim (pMS);
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 		DestroyDrawable (ReleaseDrawable (pMS->ModuleFrame));
 		pMS->ModuleFrame = 0;
 		pMS->CurFrame = 0;
 		DestroyColorMap (ReleaseColorMap (pMS->CurString));
 		pMS->CurString = 0;
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 
-		return (FALSE);
+		return FALSE;
 	}
 	else if (select)
 	{
@@ -1298,14 +1297,15 @@ ExitShipyard:
 			if (GameOptions () == 0)
 				goto ExitShipyard;
 			DrawMenuStateStrings (PM_CREW, pMS->CurState);
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			SetFlashRect ((PRECT)~0L, (FRAME)0);
 			BeginHangarAnim (pMS);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 		}
 	}
 	else
 		DoMenuChooser (pMS, PM_CREW);
 
-	return (TRUE);
+	return TRUE;
 }
+
