@@ -16,11 +16,18 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "starcon.h"
+#include "colors.h"
 #include "commglue.h"
+#include "encount.h"
+#include "gamestr.h"
+#include "options.h"
+#include "setup.h"
+#include "state.h"
 #include "libs/graphics/gfx_common.h"
 #include "libs/tasklib.h"
-#include "options.h"
+
+
+static const UNICODE *describeWeapon (BYTE moduleType);
 
 void
 RepairSISBorder (void)
@@ -35,7 +42,8 @@ RepairSISBorder (void)
 	r.corner.y = SIS_ORG_Y - 1;
 	r.extent.width = 1;
 	r.extent.height = SIS_SCREEN_HEIGHT + 2;
-	SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x10, 0x10, 0x10), 0x19));
+	SetContextForeGroundColor (
+			BUILD_COLOR (MAKE_RGB15 (0x10, 0x10, 0x10), 0x19));
 	DrawFilledRectangle (&r);
 
 	SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x8, 0x8, 0x8), 0x1F));
@@ -73,9 +81,9 @@ ClearSISRect (BYTE ClearFlags)
 
 	if (ClearFlags & CLEAR_SIS_RADAR)
 	{
-		ClearSemaphore (GraphicsSem);
+		UnlockMutex (GraphicsLock);
 		DrawMenuStateStrings ((BYTE)~0, 1);
-		SetSemaphore (GraphicsSem);
+		LockMutex (GraphicsLock);
 #ifdef NEVER
 		r.corner.x = RADAR_X - 1;
 		r.corner.y = RADAR_Y - 1;
@@ -410,51 +418,51 @@ DrawFlagshipStats (void)
 	FONT OldFont;
 	COLOR OldColor;
 	CONTEXT OldContext;
-	UNICODE buf[60], guns[16][8];
+	UNICODE buf[60];
 	SIZE leading;
-    BYTE i;
-	DWORD thrust, turning, dynamo, fuel;
+	BYTE i;
+	BYTE energy_regeneration, energy_wait, turn_wait;
+	COUNT max_thrust;
+	DWORD fuel;
 
 	/* collect stats */
-	for (i = 0; i < 16; ++i)
-		wsprintf (guns[i], "none");
-	thrust = 40;
-	turning = 1;
-	dynamo = 0;
-	fuel = 1000;
+#define ENERGY_REGENERATION 1
+#define ENERGY_WAIT 10
+#define MAX_THRUST 10
+#define TURN_WAIT 17
+	energy_regeneration = ENERGY_REGENERATION;
+	energy_wait = ENERGY_WAIT;
+	max_thrust = MAX_THRUST;
+	turn_wait = TURN_WAIT;
+	fuel = 10 * FUEL_TANK_SCALE;
 
-	for (i = 0; i < NUM_MODULE_SLOTS; ++i)
+	for (i = 0; i < NUM_MODULE_SLOTS; i++)
 	{
-		if (GLOBAL_SIS (ModuleSlots[i]) == DYNAMO_UNIT
-				|| GLOBAL_SIS (ModuleSlots[i]) == SHIVA_FURNACE)
-			dynamo += GLOBAL_SIS (ModuleSlots[i]) == DYNAMO_UNIT
-					? 30 : 60;
-		else if (GLOBAL_SIS (ModuleSlots[i]) == FUEL_TANK
-				|| GLOBAL_SIS (ModuleSlots[i]) == HIGHEFF_FUELSYS)
-			fuel += GLOBAL_SIS (ModuleSlots[i]) == FUEL_TANK
-					? FUEL_TANK_CAPACITY : HEFUEL_TANK_CAPACITY;
-		else if (GLOBAL_SIS (ModuleSlots[i]) >= GUN_WEAPON
-				&& GLOBAL_SIS (ModuleSlots[i]) <= CANNON_WEAPON)
-		{
-			if (GLOBAL_SIS (ModuleSlots[i]) == CANNON_WEAPON)
-				wsprintf (guns[i], "cannon");
-			else if (GLOBAL_SIS (ModuleSlots[i]) == BLASTER_WEAPON)
-				wsprintf (guns[i], "blaster");
-			else
-				wsprintf (guns[i], "gun");
+		switch (GLOBAL_SIS (ModuleSlots[i])) {
+			case FUEL_TANK:
+				fuel += FUEL_TANK_CAPACITY;
+				break;
+			case HIGHEFF_FUELSYS:
+				fuel += HEFUEL_TANK_CAPACITY;
+				break;
+			case DYNAMO_UNIT:
+				energy_wait -= 2;
+				if (energy_wait < 4)
+					energy_wait = 4;
+				break;
+			case SHIVA_FURNACE:
+				energy_regeneration++;
+				break;
 		}
-		else if (GLOBAL_SIS (ModuleSlots[i]) >= BOMB_MODULE_0
-				&& GLOBAL_SIS (ModuleSlots[i]) <= BOMB_MODULE_5)
-			wsprintf (guns[i], "n/a");
 	}
 
 	for (i = 0; i < NUM_DRIVE_SLOTS; ++i)
 		if (GLOBAL_SIS (DriveSlots[i]) == FUSION_THRUSTER)
-			thrust += 8;
+			max_thrust += 2;
 
 	for (i = 0; i < NUM_JET_SLOTS; ++i)
 		if (GLOBAL_SIS (JetSlots[i]) == TURNING_JETS)
-			turning += 2;
+			turn_wait -= 2;
 	/* END collect stats */
 
 	OldContext = SetContext (SpaceContext);
@@ -485,10 +493,6 @@ DrawFlagshipStats (void)
 			BUILD_COLOR_RGBA (0x00, 0x00, 0xC0, 0xFF),
 			BUILD_COLOR_RGBA (0x60, 0x6C, 0xFC, 0xFF));
 
-	/* should we pull these strings out of a resource file, to make the
-	   inevitable "let's translate it into Suomi, Klingon and Esperanto"
-	   project easier? */
-
 	wsprintf (buf, "nose:");
 	font_DrawText (&t);
 	t.baseline.y += leading;
@@ -505,16 +509,16 @@ DrawFlagshipStats (void)
 	t.baseline.y = r.corner.y + leading + 3;
 	t.align = ALIGN_LEFT;
 
-	wsprintf (buf, "%-7.7s", guns[15]);
+	wsprintf (buf, "%-7.7s", describeWeapon (GLOBAL_SIS (ModuleSlots[15])));
 	font_DrawText (&t);
 	t.baseline.y += leading;
-	wsprintf (buf, "%-7.7s", guns[14]);
+	wsprintf (buf, "%-7.7s", describeWeapon (GLOBAL_SIS (ModuleSlots[14])));
 	font_DrawText (&t);
 	t.baseline.y += leading;
-	wsprintf (buf, "%-7.7s", guns[13]);
+	wsprintf (buf, "%-7.7s", describeWeapon (GLOBAL_SIS (ModuleSlots[13])));
 	font_DrawText (&t);
 	t.baseline.y += leading;
-	wsprintf (buf, "%-7.7s", guns[0]);
+	wsprintf (buf, "%-7.7s", describeWeapon (GLOBAL_SIS (ModuleSlots[0])));
 	font_DrawText (&t);
 
 	t.baseline.x = r.extent.width - 25;
@@ -540,13 +544,20 @@ DrawFlagshipStats (void)
 	t.baseline.x = r.extent.width - 2;
 	t.baseline.y = r.corner.y + leading + 3;
 
-	wsprintf (buf, "%4lu", thrust);
+	wsprintf (buf, "%4u", max_thrust * 4);
 	font_DrawText (&t);
 	t.baseline.y += leading;
-	wsprintf (buf, "%4lu", turning);
+	wsprintf (buf, "%4u", 1 + TURN_WAIT - turn_wait);
 	font_DrawText (&t);
 	t.baseline.y += leading;
-	wsprintf (buf, "%4lu", dynamo);
+	{
+		unsigned int energy_per_10_sec =
+				(((100 * ONE_SECOND * energy_regeneration) /
+				((1 + energy_wait) * BATTLE_FRAME_RATE)) + 5) / 10;
+		wsprintf (buf, "%2u.%1u",
+				energy_per_10_sec / 10,
+				energy_per_10_sec % 10);
+	}
 	font_DrawText (&t);
 	t.baseline.y += leading;
 	wsprintf (buf, "%4lu", (fuel / FUEL_TANK_SCALE));
@@ -556,8 +567,27 @@ DrawFlagshipStats (void)
 	SetContextForeGroundColor (OldColor);
 	SetContextFont (OldFont);
 	SetContext (OldContext);
+}
 
-	return;
+static const UNICODE *
+describeWeapon (BYTE moduleType) {
+	switch (moduleType) {
+		case GUN_WEAPON:
+			return "gun";
+		case BLASTER_WEAPON:
+			return "blaster";
+		case CANNON_WEAPON:
+			return "cannon";
+		case BOMB_MODULE_0:
+		case BOMB_MODULE_1:
+		case BOMB_MODULE_2:
+		case BOMB_MODULE_3:
+		case BOMB_MODULE_4:
+		case BOMB_MODULE_5:
+			return "n/a";
+		default:
+			return "none";
+	}
 }
 
 void
@@ -675,7 +705,8 @@ GetGaugeRect (PRECT pRect, BOOLEAN IsCrewRect)
 	pRect->corner.y = IsCrewRect ? 117 : 38;
 }
 
-void DrawPC_SIS (void)
+static void
+DrawPC_SIS (void)
 {
 	TEXT t;
 	RECT r;
@@ -812,7 +843,7 @@ DeltaSISGauges (SIZE crew_delta, SIZE fuel_delta, int resunit_delta)
 				SUPPORT_SHIP_PTS
 			};
 
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			for (hStarShip = GetHeadLink (&GLOBAL (built_ship_q)),
 					pship_pos = ship_pos;
 					hStarShip; hStarShip = hNextShip, ++pship_pos)
@@ -828,16 +859,16 @@ DeltaSISGauges (SIZE crew_delta, SIZE fuel_delta, int resunit_delta)
 				s.origin.x = pship_pos->x;
 				s.origin.y = pship_pos->y;
 				s.frame = StarShipPtr->ShipInfo.icons;
-				SetSemaphore (GraphicsSem);
+				LockMutex (GraphicsLock);
 				DrawStamp (&s);
-				ClearSemaphore (GraphicsSem);
+				UnlockMutex (GraphicsLock);
 
 				UnlockStarShip (
 						&GLOBAL (built_ship_q),
 						hStarShip
 						);
 			}
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 		}
 	}
 
@@ -929,14 +960,12 @@ DeltaSISGauges (SIZE crew_delta, SIZE fuel_delta, int resunit_delta)
 		}
 		else
 		{
-			r.corner.x = 1;
-			r.corner.y = 129;
-			r.extent.width = 62;
-			r.extent.height = 9;
-			DrawStarConBox (&r, 1,
-					BUILD_COLOR (MAKE_RGB15 (0x10, 0x10, 0x10), 0x19),
-					BUILD_COLOR (MAKE_RGB15 (0x8, 0x8, 0x8), 0x1F),
-					TRUE, BUILD_COLOR (MAKE_RGB15 (0x00, 0x8, 0x00), 0x6E));
+			r.corner.x = 2;
+			r.corner.y = 130;
+			r.extent.width = STATUS_MESSAGE_WIDTH;
+			r.extent.height = STATUS_MESSAGE_HEIGHT;
+			SetContextForeGroundColor (BUILD_COLOR (MAKE_RGB15 (0x00, 0x8, 0x00), 0x6E));
+			DrawFilledRectangle (&r);
 
 			if ((pMenuState == 0
 					&& CommData.ConversationPhrases /* Melnorme shenanigans */
@@ -1172,10 +1201,11 @@ static FRAME flash_frame;
 static FRAME flash_screen_frame = 0;
 static int flash_changed;
 Mutex flash_mutex = 0;
-void arith_frame_blit (FRAME srcFrame, RECT *rsrc, FRAME dstFrame, 
-					   RECT *rdst, int num,int denom);
+void arith_frame_blit (FRAME srcFrame, RECT *rsrc, FRAME dstFrame,
+	   RECT *rdst, int num, int denom);
 
-int flash_rect_func(void *data)
+static int
+flash_rect_func (void *data)
 {
 #define NORMAL_STRENGTH 4
 #define NORMAL_F_STRENGTH 0
@@ -1183,7 +1213,8 @@ int flash_rect_func(void *data)
 	DWORD TimeIn, WaitTime;
 	SIZE strength, fstrength, incr;
 	RECT cached_rect, framesize_rect;
-	FRAME cached_frame, cached_screen_frame = 0;
+	FRAME cached_frame = 0;
+	FRAME cached_screen_frame = 0;
 	Task task = (Task)data;
 	int cached[CACHE_SIZE];
 	STAMP cached_stamp[CACHE_SIZE];
@@ -1315,7 +1346,7 @@ int flash_rect_func(void *data)
 								&tmp_rect, strength, 4);
 				}
 			}
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 			OldContext = SetContext (ScreenContext);
 			SetContextClipRect (&cached_rect);
 			// flash changed_can't be modified while GraphicSem is held
@@ -1323,7 +1354,7 @@ int flash_rect_func(void *data)
 				DrawStamp (pStamp);
 			SetContextClipRect (NULL_PTR); // this will flush whatever
 			SetContext (OldContext);
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 		}
 		FlushGraphics ();
 		SleepThreadUntil (TimeIn + WaitTime);
@@ -1356,7 +1387,7 @@ SetFlashRect (PRECT pRect, FRAME f)
 	int create_flash = 0;
 
 	if (! flash_mutex)
-		flash_mutex = CreateMutex ();
+		flash_mutex = CreateMutex ("FlashRect Lock", SYNC_CLASS_TOPLEVEL | SYNC_CLASS_VIDEO);
 
 	old_r = flash_rect;
 	old_f = flash_frame;
@@ -1393,9 +1424,9 @@ SetFlashRect (PRECT pRect, FRAME f)
 		flash_rect1.extent.width = 0;
 		if (flash_task)
 		{
-			ClearSemaphore (GraphicsSem);
+			UnlockMutex (GraphicsLock);
 			ConcludeTask (flash_task);
-			SetSemaphore (GraphicsSem);
+			LockMutex (GraphicsLock);
 		}
 	}
 	else
